const { EventEmitter } = require('events');
const https = require('https'); // Added to serve the webUI also via TLS/SSL secured connections
const fs = require('fs');
const ALARMiatorPluginManager = require('./pluginManager/pluginManager');
const express = require('express');
const session = require('express-session');
const fileUpload = require('express-fileupload');
const bodyParser = require('body-parser');
const path = require('path');
const Database = require('./models/database'); // Database Manager
const Configuration = require('./models/configuration'); // Configuration of CORE
const AccessManager = require('./models/accessmanagement'); // Global Access Management of CORE
const CoreCron = require('./models/core/core_cron.js'); // Cron Manager
const TestalarmManager = require('./models/testalarmmanager.js'); // TestalarmManager

const uuid = require('uuid').v1;

var config = require('./config/config.js');
var log4js = require('log4js');
var cookieParser = require('cookie-parser')
var SQLiteStore = require('connect-sqlite3')(session);

const HandlerState = require('./models/core/handlerState');
const HandlerAlarm = require('./models/core/handlerAlarm');
const HandlerExternalIp = require('./models/core/handlerExternalIp');
const HandlerRequestedTestAlarm = require('./models/core/handlerRequestedTestAlarms');
const HandlerMaintenance = require('./models/core/handlerMaintenance');



// Printer-related objects
const Printer = require('./models/core/core_printing');
var printing = new Printer();
global.printManager = printing;


// globally used objects
const FCMTokens = require('./models/fctmtokens');
const operations = require('./routes/operations');
const CoreMaintenanceHandler = require('./models/core/handlerMaintenance');

class Core extends EventEmitter {
    constructor() {
        super();

        // inititalize class propertys
        this.coreDb = null;
        this.logger = null;                         // holds logger instance
        this.plgManager = null;                     // holds instance of plgManager
        // Initialize globals
        global.coreDb = null;                       // holds database instance to coredatabase
        global.sessionsDb = null;                   // holds database instance to sessions database

        global.appRoot = path.resolve(__dirname);   // holds complete path to application root
        global.app = null;                          // holds instance of this app globally
        global.appPort = null;                      // holds port number for UI
        global.cron = null;                         // holds core manager after initialization
        global.testalarmmanager = null;             // holds the testalarmmanager
        global.accessmanager = null;                // holds the accessmanager for CORE

        this.app = express();
        global.app = this.app;

        this.uuid = null;                           // holds the uuid of this server instance

        /**
         * initialize defaults for command line parameters handed over with server start
         */
        this.args_development = false;              // Server runs in development mode
        this.args_startedByPM2 = false;             // Server has been started by PM2
        global.serverStartedByPM2 = false;          // global property to tell if this instance has been started by PM2

    }

    /**
     * Read all arguments from server start
     */
    getStartupArguments() {
        global.developmentMode = false;
        process.argv.forEach(function (val, index, array) {

            if (val === '--development') {
                this.args_development = true;
                global.developmentMode = true;
                console.log('CORE | running in development-mode');
            }
            else if (val.startsWith('--pm2')) {
                this.args_startedByPM2 = true;
                this.args_PM2_Name = val.replace('--pm2=', '');

                global.serverStartedByPM2 = this.args_startedByPM2;
                global.PM2_Name = this.args_PM2_Name;
                console.log('CORE | Start by PM2 : ' + global.serverStartedByPM2);
                console.log('CORE | PM2-Name : ' + global.PM2_Name);
            }

        }.bind(this));
        this.emit('getStartupArguments', true);
    }

    /**
     * Starts the coredatabase connection
     */
    startCoredatabase() {
        this.coreDb = new Database();
        this.coreDb.startDatabase((successDatabase) => {
            if (successDatabase) {
                global.coreDb = this.coreDb.db;
                this.emit('startCoredatabase', true);
                // Database started, start server now
            } else {
                // Database could not be started. -> Log error to console
                console.log('CORE | Error starting up database. Server could not be started. Please check logs for more details.');
                this.emit('startCoredatabase', false);
            }
        });
    }

    /**
     * Starts the coredatabase connection
     */
    startSessionsdatabase() {
        this.sessionsDb = new Database();
        this.sessionsDb.startSessionsDatabase((successSessionsDatabase) => {
            if (successSessionsDatabase) {
                global.sessionsDb = this.sessionsDb.dbSessions;
                this.emit('startSessionsdatabase', true);
                // Database started, start server now
            } else {
                // Database could not be started. -> Log error to console
                console.log('CORE | Error starting up sessions database. Server could not be started. Please check logs for more details.');
                this.emit('startSessionsdatabase', false);
            }
        });
    }

    /**
     * Starts the global logger(s)
     */
    startLoggers() {
        /**
         * start new Logger
         */
        // create a rolling file logger based on date/time that fires process events
        const opts = {
            logDirectory: global.appRoot + '/logs/server', // NOTE: folder must exist and be writable...
            fileNamePattern: 'ALARMiator-Server-<DATE>.log',
            dateFormat: 'YYYY.MM.DD'
        };
        this.logmanager = require('simple-node-logger');
        this.logger = this.logmanager.createRollingFileLogger(opts);
        this.logger.setLevel('all');
        // this.logger should be removed later after the whole project used global.logger
        this.app.logger = this.logger;
        global.logger = this.logger;

        // Emit event
        this.emit('startLoggers', true);
    }

    /**
     * starts the server itself
     */
    setupServer() {
        this.port = 5000;
        try {
            this.port = config.serverOptions.uiport;
            this.emit('setupServer', true);
        } catch (err) {
            this.logger.debug('CONFIG: ' + err);
            this.logger.warn(`CORE | configuration for server port not set. Please check the config.js file in config folder. Using default port 5000 for UI.`);
            this.emit('setupServer', true);
        }
        global.appPort = this.port;

    }

    /**
     * Sets up the middleware (express)
     */
    setupMiddelware() {
        const { isAuthenticated, checkPermission } = require('./routes/auth');
        // create status monitor
        const statusMonitor = require('express-status-monitor')({
            title: 'ALARMiator Server Status Monitor',
            path: "/_status",  // this will silently fail in express, but we manually add the route to /status later
            // more info: https://github.com/RafalWilinski/express-status-monitor/issues/90#issuecomment-337575393
            healthChecks: [{
                protocol: 'http',
                host: 'localhost',
                path: '/health/status',
                port: '5010'
            }]
        })

        // remaining app config
        this.app.set('port', process.env.port || this.port); // set express to use this port
        this.app.set('views', __dirname + '/views'); // set express to look in this folder to render our view
        this.app.set('view engine', 'ejs'); // configure template engine
        this.app.locals.moment = require('moment');
        this.app.use(bodyParser.urlencoded({ extended: true }));
        this.app.use(bodyParser.json({
            limit: '5mb',
        })); // parse form data client
        this.app.use(express.static(path.join(__dirname, 'public'))); // configure express to use public folder
        this.app.use(fileUpload()); // configure fileupload
        //this.app.use(fileUpload({limits: { fileSize: 50 * 1024 * 1024 }})); // configure fileupload

        // Configure Session and Authentication handling (SQLite-Version)
        this.app.use(cookieParser());
        const sessionStore = new SQLiteStore({
            table: 'sessions',
            db: 'sessions.db',
            dir: global.appRoot + '/store'
        })
        global.sessionStore = sessionStore
        this.app.use(session({
            store: sessionStore,
            secret: '9xa&aVPKe18XV;f',
            cookie: { maxAge: 2 * 24 * 60 * 60 * 1000 }, // 1 week
            resave: false,
            saveUninitialized: true,
            key: 'alarmiator',
            secret: 'ALRMiator#2019'
        }));
        // secure endpoint for status monitor
        this.app.use(statusMonitor.middleware)
        this.app.get("/status", isAuthenticated, checkPermission("administration"), statusMonitor.pageRoute)
        this.emit('setupMiddleware', true);
    }

    /**
     * registers core handlers
     */
    startCoreHandlers() {
        this.handlerState = new HandlerState();
        this.handlerAlarm = new HandlerAlarm();
        this.handlerExternalIp = new HandlerExternalIp();
        this.handlerRequestedTestAlarm = new HandlerRequestedTestAlarm();
        this.HandlerMaintenance = new HandlerMaintenance();

        this.emit('startCoreHandlers', true);
    }

    /**
     * Starts the plugin manager and makes it globally available
     */
    startPluginManager() {
        this.plgManager = new ALARMiatorPluginManager();
        global.plgManager = this.plgManager;
        // Read inbound plugins and initialize plugins
        this.plgManager.readInboundPluginList();
        this.plgManager.readOutboundPluginList();
        // reinitialize Plugins in plugin store
        this.plgManager.registerPluginsInStore(this.plgManager.pluginList);
        // Load configuration of plugins from database
        this.plgManager.initializeConfigFromStore((success) => {
            if (success) {
                this.plgManager.initializePlugins();
                this.emit('startPluginManager', true);
            } else {
                // Config could not be loaded from database
                tthis.emit('startPluginManager', false);

            }
        })
    }

    /**
     * Registers core to plugin manager events
     */
    registerPluginManagerEvents() {

        this.plgManager.on('event_new_zvei_alarm', (zveiInfo) => {
            // a new zvei alarm has been fired
            console.log('APP | New ZVEI Alarm Event fired with the following data:');
            console.log(zveiInfo);
        })

        this.plgManager.on('event_new_alarm', (alarmInfo) => {
            // a new zvei alarm has been fired
            console.log('APP | New Alarm Event fired with the following data:');
            this.handlerAlarm.persistNewAlarmInformation(alarmInfo);
        })

        this.plgManager.on('event_new_state', (stateInfo) => {
            // a new zvei alarm has been fired
            console.log('APP | New State Event fired with the following data:' + stateInfo.radioStatusHumanReadable + ' (' + stateInfo.issi + ')');
            this.handlerState.persistNewStateInformation(stateInfo);
        })

        this.plgManager.on('event_refresh_extip', (ip) => {
            // external ip has been refetched
            // console.log('APP | New external ip address state fired with the following ip: ' + ip);
            global.externalIP = ip;
            this.handlerExternalIp.persistNewIPInformation(ip);
        })

        this.plgManager.on('event_print_pdf', (pathToPDF) => {
            // PDF Document should be sent to default printer
            try {
                global.printManager.printPDFToDefaultPrinter(pathToPDF, (resultPrint) => {
                    if (resultPrint) {
                        console.log('CORE | PRINTING | Successfully finished print job for document ' + pathToPDF);
                    } else {
                        console.log('CORE | PRINTING | Error printing document: ' + pathToPDF);
                    }
                })
            } catch (err) {
                console.log('CORE | PRINTING | Error while trying to print PDF Document: ' + err.message);
            }
        })

        this.plgManager.on('event_new_test_alarm_requested', (basedataId, pluginNamespace, token, offset) => {
            this.handlerRequestedTestAlarm.addTestalarmCronjob(basedataId, token, offset);
        })

        this.plgManager.on('event_fcm_sending_report', (failedFcmTokenList, successFcmTokenList) => {
            try {
                global.fcmtokens.updateFailedTokenCounter(failedFcmTokenList, (result) => {
                    // update failed tokens + error counter
                    global.fcmtokens.updateSuccessfullToken(successFcmTokenList, (fullResult) => {
                        // update successfull tokens
                    })
                });
            }
            catch (err) {
                console.log("CORE | SENDING REPORT | Error: " + err.message)
            }
        })

        this.emit('registerPluginManagerEvents', true);
    }

    /**
     * Sets up authentication
     */
    setupAuthentication() {
        // Authentication
        const { authenticate, changePassword, isAuthenticated, checkPermission } = require('./routes/auth');
        const { getLoginPage } = require('./routes/auth');
        const { endSession } = require('./routes/auth');
        const { getChangePasswordPage, getPasswordRecoveryPage, getPasswordResetPage } = require('./routes/auth');
        const { setNewPasswordFromStartPassword, sendPasswordRecovery, resetPassword } = require('./routes/auth');
        const { getLockedAccountPage } = require('./routes/auth');
        const { switchTheme } = require('./routes/ui');


        // routes for the app
        // Login
        this.app.get('/', getLoginPage);
        this.app.post('/auth', authenticate);
        this.app.get("/password-recovery", getPasswordRecoveryPage)
        this.app.post("/password-recovery", sendPasswordRecovery)
        this.app.get("/password-recovery/reset", getPasswordResetPage)
        this.app.post("/password-recovery/reset", resetPassword)
        this.app.get('/changepassword', getChangePasswordPage);
        this.app.post('/changepassword', setNewPasswordFromStartPassword);
        this.app.get('/auth/logoff', endSession);
        this.app.get('/lockedaccount', getLockedAccountPage);
        this.app.get('/core/switchTheme/:theme', switchTheme);
        this.emit('setupAuthentication', true);
    }

    /**
     * Sets up all needed express - routes
     */
    setupRoutes() {
        // Load Route Modules
        var routeSetupWizard = require('./routes/setupwizard');
        var routeBasedata = require('./routes/basedata');
        var routeAdmin = require('./routes/admin');
        var routeTraining = require('./routes/training');
        var routeAssets = require('./routes/assets');
        var routeWallboard = require('./routes/wallboard');
        //var routeReports = require('./routes/reports');
        var routesOperations = require('./routes/operations');
        var routesAlarm = require('./routes/alarm');
        var routesHome = require('./routes/home');
        var routesRespiratory = require('./routes/respiratory');
        var routesUploads = require('./routes/uploads');
        // documentation
        var routesDoc = require("./routes/documentation");

        const { isAuthenticated, checkPermission } = require('./routes/auth');
        const { changePassword } = require('./routes/auth');
        const { getUserGroups } = require('./routes/auth');

        this.app.use('/setup', routeSetupWizard);
        this.app.use('/home', isAuthenticated, getUserGroups, routesHome);
        this.app.use('/basedata', isAuthenticated, getUserGroups, checkPermission("basedata"), routeBasedata);
        this.app.use('/admin', isAuthenticated, getUserGroups, checkPermission("administration"), routeAdmin);
        this.app.use('/training', isAuthenticated, getUserGroups, checkPermission("training"), routeTraining);
        this.app.use('/assets', isAuthenticated, getUserGroups, checkPermission("inventory"), routeAssets);
        // this.app.use('/reports', isAuthenticated,getUserGroups, checkPermission("reporting"), routeReports);
        this.app.use('/operations', isAuthenticated, getUserGroups, checkPermission("operations"), routesOperations);
        this.app.use('/alarm', isAuthenticated, getUserGroups, checkPermission("alarmings"), routesAlarm);
        this.app.use('/respiratory', isAuthenticated, getUserGroups, checkPermission("respiratoryequipment"), routesRespiratory);
        this.app.use('/uploads', isAuthenticated, getUserGroups, routesUploads);
        this.app.use('/doc', routesDoc);

        this.app.use('/auth/changePassword', isAuthenticated, getUserGroups, changePassword);

        if (this.args_development) {
            this.startAPIDocumentation();
        }

        // Static Route to wallboard client in plugins directory
        //this.app.use('/wallboard', express.static('plugins/outbound/wallboard/static'));
        this.emit('setupRoutes', true);
    }

    /**
     * Checks for initial Settings like server UUID etc.
     */
    runServer() {

        // read uuid of this instance and publish globally (serverUUID)
        global.configuration.getValueForKeyword('serveruuid', (uuidFromDb) => {
            if (uuidFromDb === null) {
                var uuidFromDb = uuid();
                global.configuration.addKeywordAndValue('serveruuid', uuidFromDb, (successUUID) => {
                    if (successUUID) {
                        this.logger.info('APP | Set UUID of this Server to ' + uuidFromDb);
                        this.uuid = uuidFromDb;
                    } else {
                        this.logger.error('APP | Error setting UUID of this Server.');
                    }
                });
            } else {
                this.uuid = uuidFromDb;
            }
            global.serverUUID = this.uuid;
        })

        // read serverinstance and publish globally
        global.configuration.getValueForKeyword('instancename', (instanceNameFromDb) => {
            if (instanceNameFromDb === null) {
                this.instancename = 'ALARMiator';
            } else {
                this.instancename = instanceNameFromDb;
            }

            global.serverInstanceName = this.instancename;
        })

        // read (external ip) of this server and publish it globally (wanIP)
        global.configuration.getValueForKeyword('externalip', (externalip) => {
            global.externalIP = externalip;
        })


        // set the app to listen on the port
        this.app.listen(this.port, () => {
            this.logger.info(`CORE | ALARMiator Server is running on port: ${this.port}`);
            console.log(`CORE | ALARMiator HTTP-Server is running on port: ${this.port}`);
            this.app.listenPort = this.port;
        });

        // NOW Start SSL Server if server is configured to start SSL 
        // (webuiSecurityLevel)
        global.configuration.getValueForKeyword('webuiSecurityLevel', (webuiSecurityLevel) => {

            if (webuiSecurityLevel === null) {
                webuiSecurityLevel = 'http';
            } else {
                if ((webuiSecurityLevel === 'https') || (webuiSecurityLevel === 'http-https')) {

                    // Check if certificate files exist
                    var keyCert = __dirname + '/certificates/privkey.pem';
                    var keyCertExists = false;
                    var pemCert = __dirname + '/certificates/cert.pem';
                    var pemCertExists = false;


                    try {
                        if (fs.existsSync(keyCert)) {
                            keyCertExists = true;
                        }
                        if (fs.existsSync(pemCert)) {
                            pemCertExists = true;
                        }
                    } catch (err) {
                        console.log('CORE | ALARMiator Server could not be started with SSL as certificates seem not to exist!');
                    }

                    // if both certificates exist -> start Server on SSL
                    if ((keyCertExists === true) && (pemCertExists === true)) {
                        const httpsOptions = {
                            key: fs.readFileSync('./certificates/privkey.pem', 'utf-8'),
                            cert: fs.readFileSync('./certificates/cert.pem', 'utf-8')
                        }

                        const server = https.createServer(httpsOptions, app)
                            .listen(8443, () => {
                                console.log(`CORE | ALARMiator HTTPS Server is running on port: 8443`);
                            })
                    } else {
                        console.log('CORE | ALARMiator Server could not be started with SSL as certificates seem not to exist!');
                        console.log('CORE | --> ' + keyCert);
                        console.log('CORE | --> ' + pemCert);
                    }
                } else {
                    console.log(`CORE | ALARMiator Server is configured to run only on http`);
                }
            }

        })
        this.emit('runServer', true);
    }

    /**
     * Place to initialize all global objects and properties
     */
    initializeGlobalObjects() {
        // Distribute the FCMTokens Class globally
        global.fcmtokens = new FCMTokens(global.logger);
        global.configuration = new Configuration(global.logger);
        global.accessmanager = new AccessManager(global.app.logger);


        // initialise version info
        const versionInfo = {
            revision: null,
            version: null,
            versiontag: null
        }
        // determine git commit
        try {
            const gitFile = ".git/HEAD"
            if (fs.existsSync(gitFile)) {
                const rev = fs.readFileSync(gitFile).toString().trim()
                if (rev.indexOf(':') === -1) {
                    versionInfo.revision = rev.substring(0, 7)
                } else {
                    versionInfo.revision = fs.readFileSync('.git/' + rev.substring(5)).toString().trim().substring(0, 7)
                }
            }
        } catch (error) {
            console.log(`CORE | Could not determine git commit: ${error}`)
        }
        // determine version number from package.json
        try {
            if (fs.existsSync("package.json")) {
                const version = require('./package.json').version;
                versionInfo.version = version;
                const versionTag = require('./package.json').versiontag;
                versionInfo.versiontag = versionTag;
                console.log('VERSIONTAG');
                console.log(versiontag);
            }
        } catch (error) {
            console.log(`CORE | Could not determine version number: ${error}`)
        }
        // store version info in global
        global.versionInfo = versionInfo

        this.emit('initializeGlobalObjects', true);
    }

    /**
     * Starts the cron manager and makes it available globally
     */
    startCron() {
        this.cron = new CoreCron(global.logger);
        global.cron = this.cron;

        global.cron.on('cronStarted', () => {
            this.emit('CronManagerStarted', true);
        });

        global.cron.startCronManager();
    }

    /**
     * load the core-cronjobs from the coredb and load it to cron manager
     */
    loadCoreCronjobs() {

        // adding a static cronjob not saved in DB as example
        var that = this;

        var cronjob_name = "maintenance_2_min";
        var cronjob_data = {};
        cronjob_data.event_name = "maintenance_2_min";
        cronjob_data.event_data = cronjob_name;
        if (global.cron.AddCronJob(cronjob_name, "core", "*/2 * * * *", cronjob_data, null)) {
            global.cron.on(cronjob_data.event_name, (event_data) => {
                console.log('CORE | CRON | event "' + event_data.event_name + '" fired from job ' + event_data.event_data);
                this.HandlerMaintenance.do2MinMainenance();
            });
            global.cron.StartCronJob(cronjob_name);
        }

        // Cron job for nightly maintenance
        cronjob_name = "maintenance_nightly";
        cronjob_data = {};
        cronjob_data.event_name = "maintenance_nightly";
        cronjob_data.event_data = cronjob_name;
        if (global.cron.AddCronJob(cronjob_name, "core", "0 23 * * *", cronjob_data, null)) {
            global.cron.on(cronjob_data.event_name, (event_data) => {
                console.log('CORE | CRON | event "' + event_data.event_name + '" fired from job ' + event_data.event_data);
                this.HandlerMaintenance.doMaintenance();
            });
            global.cron.StartCronJob(cronjob_name);
        }

        global.cron.GetCronJobsFromCoreDBbyType('core', (rows, error) => {
            if (rows) {
                rows.forEach(job => {
                    var data = JSON.parse(job.data);
                    var created = global.cron.AddCronJob(job.name, job.type, job.cronDefinition, data, job.id);
                    if (created) {
                        global.cron.on(data.event_name, (event_data) => {
                            console.log('CORE | CRON | event fired from job ' + event_data.event_name + ' --> ' + event_data.event_data);
                        });

                        if (job.running) {
                            global.cron.StartCronJob(job.name);
                        } else {
                            global.cron.StopCronJob(job.name);
                        }
                    }
                });

                this.emit('CoreCronjobsLoaded', true);
            }
            else {
                console.log('CORE | CRON | ERROR at getting cronjobs from coredatabase: ' + error);
            }
        })
    }

    /**
    * load the reporting-cronjobs from the coredb and load it to cron manager
    */
    loadMaintenanceCronJobs() {
        global.cron.GetCronJobsFromCoreDBbyType('maintenance', (rows, error) => {
            if (rows) {
                rows.forEach(job => {
                    var data = JSON.parse(job.data);
                    var created = global.cron.AddCronJob(job.name, job.type, job.cronDefinition, job.data, job.id);

                    if (created) {
                        if (job.running) {
                            global.cron.StartCronJob(job.name)
                        } else {
                            global.cron.StopCronJob(job.name)
                        }
                    }
                });

                this.emit('MaintenanceCronjobsLoaded', true);
            }
            else {
                global.testalarmmanager.Log('ERROR at getting cronjobs from coredatabase: ' + error);
            }
        })
    }

    /**
     * starts the testalarmmanager and makes it available globally
     */
    startTestalarmManager() {
        this.testalarmmanager = new TestalarmManager(global.logger);
        global.testalarmmanager = this.testalarmmanager;
        global.testalarmmanager.Log('successfully started');
        this.emit('TestalarmManagerStarted', true);
    }

    /**
     * load the testalarm-cronjobs from the coredb and load it to cron manager
     */
    loadTestalarmCronjobs() {
        global.cron.GetCronJobsFromCoreDBbyType('testalarm', (rows, error) => {
            if (rows) {
                rows.forEach(job => {
                    var data = JSON.parse(job.data);

                    const Operation = require('./models/core/core_operation');
                    let operation = Object.setPrototypeOf(data.event_data, Operation.prototype);
                    data.event_data = operation;

                    var created = global.cron.AddCronJob(job.name, job.type, job.cronDefinition, data, job.id);

                    global.cron.on(data.event_name, (event_data) => {
                        global.testalarmmanager.FireTestalarm(event_data);
                    });

                    if (created) {
                        if (job.running) {
                            global.cron.StartCronJob(job.name)
                        } else {
                            global.cron.StopCronJob(job.name)
                        }
                    }
                });

                this.emit('TestalarmCronjobsLoaded', true);
            }
            else {
                global.testalarmmanager.Log('ERROR at getting cronjobs from coredatabase: ' + error);
            }
        })
    }

    /**
     * Starts the REST API documentation if server is in development mode
     */
    startAPIDocumentation() {
        this.swaggerActice = true;
        this.swaggerUi = require('swagger-ui-express');
        this.YAML = require('yamljs');
        this.swaggerDocument = this.YAML.load('./plugins/inbound/api/api/swagger/swagger.yaml');
        this.app.use('/api-docs', this.swaggerUi.serve, this.swaggerUi.setup(this.swaggerDocument));
        console.log('CORE | successful | startAPIDumentation');
    }

}

// GENERIC Error Handling
process.on('unhandledRejection', (reason, p) => {
    console.log('CORE | Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
    // application specific logging, throwing an error, or other logic here
});

// Start Core-System
const core = new Core();

core.on('getStartupArguments', (state) => {

    if (state) {
        // Database could be started successfully
        console.log('CORE | successful | getStartupArguments');
        core.startCoredatabase();
    } else {
        // Database could not be started successfully
        console.log('CORE | ERROR reading passed startup arguments. Please check logs for more details');
    }
})

core.on('startCoredatabase', (state) => {
    if (state) {
        // Database could be started successfully
        console.log('CORE | successful | startCoredatabase');
        core.startSessionsdatabase();
    } else {
        // Database could not be started successfully
        console.log('CORE | ERROR starting Database. Please check logs for more details');
    }
})

core.on('startSessionsdatabase', (state) => {
    if (state) {
        // Database could be started successfully
        console.log('CORE | successful | startSessionsdatabase');
        core.startLoggers();
    } else {
        // Database could not be started successfully
        console.log('CORE | ERROR starting Database. Please check logs for more details');
    }
})

core.on('startLoggers', (state) => {
    if (state) {
        console.log('CORE | successful | startLoggers');
        // Loggers have been successfully started
        core.initializeGlobalObjects();
    } else {
        console.log('CORE | ERROR starting Loggers. Please check logs for more details.');
    }
})

core.on('initializeGlobalObjects', (state) => {
    if (state) {
        console.log('CORE | successful | initializeGlobalObjects');
        core.setupServer();
    } else {
        console.log('CORE | ERROR initializing global objects. Please check logs for more details.');
    }
})

core.on('setupServer', (state) => {
    if (state) {
        // Server config successfully done
        console.log('CORE | successful | setupServer');
        core.setupMiddelware();
    } else {
        console.log('CORE | ERROR setting Up Server. Please check logs for more details.');
    }
})

core.on('setupMiddleware', (state) => {
    if (state) {
        // Middleware Setup successfully done
        console.log('CORE | successful | startMiddleware');
        core.startCron();
    } else {
        console.log('CORE | ERROR setting Up Middleware. Please check logs for more details.');
    }
})

core.on('CronManagerStarted', (state) => {
    if (state) {
        // Core Handlers Setup successfully done
        console.log('CORE | successful | setupCron');
        core.loadCoreCronjobs();
    } else {
        console.log('CORE | ERROR setting up Cron. Please check logs for more details.');
    }
})

core.on('CoreCronjobsLoaded', (state) => {
    if (state) {
        console.log('CORE | successful | core cronjobs successfully added to CRON MANAGER');
        core.loadMaintenanceCronJobs();
    } else {
        console.log('CORE | ERROR at load of core cronjobs ')
    }
})

core.on('MaintenanceCronjobsLoaded', (state) => {
    if (state) {
        console.log('CORE | successful | maintenance cronjobs successfully added to CRON MANAGER');
        core.startTestalarmManager();
    } else {
        console.log('CORE | ERROR at load of maintenance cronjobs ')
    }
})

core.on('TestalarmManagerStarted', (state) => {
    if (state) {
        // Core Handlers Setup successfully done
        console.log('CORE | successful | started Testalarm-Manager');
        core.loadTestalarmCronjobs();
    } else {
        console.log('CORE | ERROR setting up Testalarm-Manager. Please check logs for more details.');
    }
})

core.on('TestalarmCronjobsLoaded', (state) => {
    if (state) {
        console.log('CORE | successful | testalarm cronjobs successfully added to CRON MANAGER');
        core.startCoreHandlers();
    } else {
        console.log('CORE | ERROR at load of testalarm cronjobs ')
    }
})

core.on('startCoreHandlers', (state) => {
    if (state) {
        // Core Handlers Setup successfully done
        console.log('CORE | successful | startCoreHandlers');
        core.startPluginManager();
    } else {
        console.log('CORE | ERROR setting Up Core handlers. Please check logs for more details.');
    }
})

core.on('startPluginManager', (state) => {
    if (state) {
        // Core Handlers Setup successfully done
        console.log('CORE | successful | startPluginManager');
        core.registerPluginManagerEvents();
    } else {
        console.log('CORE | ERROR starting PluginManager. Please check logs for more details.');
    }
})

core.on('registerPluginManagerEvents', (state) => {
    if (state) {
        // Core Handlers Setup successfully done
        console.log('CORE | successful | registerPluginManagerEvents');
        core.setupAuthentication();
    } else {
        console.log('CORE | ERROR registering Plugin Manager Events. Please check logs for more details.');
    }
})

core.on('setupAuthentication', (state) => {
    if (state) {
        // Core Handlers Setup successfully done
        console.log('CORE | successful | setupAuthentication');
        core.setupRoutes();
    } else {
        console.log('CORE | ERROR setting up Authentication. Please check logs for more details.');
    }
})

core.on('setupRoutes', (state) => {
    if (state) {
        // Core Handlers Setup successfully done
        console.log('CORE | successful | setupRoutes');
        core.runServer();
    } else {
        console.log('CORE | ERROR setting up Routes. Please check logs for more details.');
    }
})

core.on('runServer', (state) => {
    if (state) {
        // Core Handlers Setup successfully done
        console.log('CORE | ALARMiator Server started successfully');
    } else {
        console.log('CORE | ERROR starting ALARMiator Server. Please check logs for more details.');
    }
})




// Starting now
core.getStartupArguments();