/**
 * SDS Encoder.
 *
 * Provides encoding and creation of SDS messages.
 *
 * @link   http://wwww.alarmiator.de
 * @file   This files defines the SdsEncoder class.
 * @author Stefan Bauer.
 * @since  2020.12.24
 */

const EventEmitter = require('events');
const hexToBinary = require('hex-to-binary');
const SDSHelper = require('./sdsHelper');
const moment = require('moment');

class SdsEncoder extends EventEmitter {
    sdsHelper;
    constructor(){
        super();
        this.sdsHelper = new SDSHelper();
    }

    calcBinaryLength(userDataHexString){
        var binary = hexToBinary(userDataHexString);
        return binary.length;
    }

    buildCalloutMessage(message, globalMessageCounter, timestampEnabled, encoding, constantMessageCounter){

        console.log("Using message: ", message);
        var length = 266;
        
        var encodedMessage = this.sdsHelper.encodePayload(message,encoding);
        console.log("Using Encoding: ", this.sdsHelper.getEncodingForValue(encoding));

        var messageParts = encodedMessage.match(new RegExp(".{1," +length + "}","g"));

        var numberOfMessages = messageParts.length;
        var messages = [];
        for (var currentMessage = 0; currentMessage < messageParts.length; currentMessage++){
            var userData = this.generateUserDataHeader(
                globalMessageCounter + currentMessage, 
                timestampEnabled, 
                encoding,  
                constantMessageCounter, 
                currentMessage + 1, 
                numberOfMessages
                ) + messageParts[currentMessage];

            var ctsdsrMessage = this.buildCTSDSMessage(108, userData);
            messages.push(ctsdsrMessage);
        }

        //console.log(messages);
        return messages;
    }

    buildStatusMessage(statusShort){
        var statusTransport = this.sdsHelper.getTransportStatusForStatusShort(statusShort);
        return this.buildCTSDSMessage(109, this.sdsHelper.convertIntToHexString(statusTransport,2));
    }

    buildCTSDSMessage(aiService, encodedUserData){
        return this.generateCTSDSRHeader(aiService, this.calcBinaryLength(encodedUserData)) + encodedUserData;
    }

    generateCTSDSRHeader(aiService, length){
        // +CTSDSR: <AI service>, [<calling party identity>], [<calling party identity type>], <called party identity>, <called party identity type>, <length>, [<end to end encryption>]<CR><LF>user data
        var callingPartyIdentity = 262100105569552;
        var callingPartyIdentityType = 1;
        var calledPartyIdentity = 262100105563034;
        var calledPartyIdentityType = 1;

        var message="+CTSDSR: ";

        var parameter = [];
        parameter.push(aiService);
        parameter.push(callingPartyIdentity);
        parameter.push(callingPartyIdentityType);
        parameter.push(calledPartyIdentity);
        parameter.push(calledPartyIdentityType);
        message += parameter.filter(x => x).join(',');

        message += ",";
        message += length;
        message += "\r\n"

        return message;
    }
    generateUserDataHeader(globalMessageCounter, timeStampEnabled, encoding, constantMessageCounter, currentMessage, numberOfMessages){

        var header="";
        // PDU
        // Reference: Table 29.63: Concatenation-part message with SDS-TL data transfer service
        // EN 300 392-2 - V3.8.1 - Terrestrial Trunked Radio (TETRA); Voice plus Data (V+D); Part 2: Air Interface (AI)

        // Protocol Identifier
        // First Byte

        header += this.sdsHelper.convertIntToHexString(138,1);

        // Message type
        // Length: 4 Bit

        // Delivery report request 
        // Length: 2 Bit

        // Service selection / Short form report 
        // Length: 1 Bit

        // Storage/forward control 
        // Length: 1 Bit

        // For us its 0000 0000
        header += this.sdsHelper.convertIntToHexString(0,1);

        // Message reference
        // Length: 8 Bit
        // Info: Each message has its own reference

        header += this.sdsHelper.convertIntToHexString(globalMessageCounter,1);

        // SDU
        // First Byte is shared between timestamp + text encoding scheme

        // Timestamp Used
        // Length: 1 Bit
        var timestampUsed;

        if (timeStampEnabled){
            timestampUsed = (1 << 7);
        }
        else {
            timestampUsed = (0 << 7);
        }

        // Text Coding Scheme
        // Length: 7 Bit
        header += this.sdsHelper.convertIntToHexString(timestampUsed + encoding,1);

        if (timeStampEnabled){
            // Timestamp
            // Length: 24 Bit

            // Timeframe type
            // Length: 2 Bit
            //      Values:
            //          0 Undefined, application dependent
            //          1 UTC
            //          2 Local time, normal time
            //          3 Local time, daylight saving time

            // Reserved 
            // Length: 2 Bit
            //      Value: 0

            var timenow = moment().format();

            // Month
            // Length: 4 Bit

            var month = parseInt(moment(timenow).format("MM"));
            // Day
            // Length: 5 Bit
            var day = parseInt(moment(timenow).format("DD"));
            // Hour
            // Length: 5 Bit

            var hour = parseInt(moment(timenow).format("HH"));
            // Minute
            // Length: 6 Bit

            var minute = parseInt(moment(timenow).format("mm"));
            var timePart = (minute & 63)
            timePart += (hour & 31) << 6;
            timePart += (day & 31) << (6 + 5);
            timePart += (month & 15) << ( 6 + 5 + 5);
            header += this.sdsHelper.convertIntToHexString(timePart,3);
        }

        // User Data Header Length
        // Length: 8 Bit

        header += "xx";

        // UDH information element ID
        // Length: 8 Bit
        // Values:
        // - 0 : Concatenated text message (8-Bit reference number) 
        // - 8 : Concatenated text message (16-Bit reference number) 
        
        header += this.sdsHelper.convertIntToHexString(0,1); 

        // UDH information element length 
        // Info: The UDH information element length information element shall indicate the number of octets
        // within its associated UDH information element data information element, which follows and shall
        // not include itself in its count value.
        // Length: 8 Bit
        header += "yy";

        // Message reference number
        // Length: 8 / 16 Bit -> Depends on UDH Information Element ID
        // Info:  This reference number shall remain constant for every short message,  

        header +=  this.sdsHelper.convertIntToHexString(constantMessageCounter,1);

        // Maximum number of messages 
        // Length: 8 Bit

        header += this.sdsHelper.convertIntToHexString(numberOfMessages,1);

        // Sequence number of current message
        // Length: 8 Bit

        header += this.sdsHelper.convertIntToHexString(currentMessage,1);

        header = header.replace(/xx/,this.sdsHelper.convertIntToHexString(this.sdsHelper.calcByteLength(header.substr(header.indexOf("xx") + 2)),1));
        header = header.replace(/yy/,this.sdsHelper.convertIntToHexString(this.sdsHelper.calcByteLength(header.substr(header.indexOf("yy") + 2)),1));

        return header;

        // User text (Text segment) 
        // Length: variable
    }
}

module.exports = SdsEncoder;