/**
 * SDS Helper.
 *
 * Provides helper functions for encoding, mapping and convertions.
 *
 * @link   http://wwww.alarmiator.de
 * @file   This files defines the SdsHelper class.
 * @author Stefan Bauer.
 * @since  2020.12.24
 */

const EventEmitter = require('events');
const hexToBinary = require('hex-to-binary');
const gsm7 = require('gsm7');
const iconv = require('iconv-lite');

class SdsHelper extends EventEmitter {
    constructor(){
        super();
    }

    getEncodingForValue(value){
        var encoding;
        switch (value) {
            case 0:
                encoding = '7-bit alphabet';
                break;
            case 1:
                encoding = 'ISO/IEC 8859-1 Latin 1 (8-bit) alphabet';
                break;
            case 2:
                encoding = 'ISO/IEC 8859-2 Latin 2 (8-bit) alphabet';
                break;
            case 3:
                encoding = 'ISO/IEC 8859-3 Latin 3 (8-bit) alphabet';
                break;
            case 4:
                encoding = 'ISO/IEC 8859-4 Latin 4 (8-bit) alphabet';
                break;
            case 5:
                encoding = 'ISO/IEC 8859-5 Latin/Cyrillic (8-bit) alphabet';
                break;
            case 6:
                encoding = 'ISO/IEC 8859-6 Latin/Arabic (8-bit) alphabet';
                break;
            case 7:
                encoding = 'ISO/IEC 8859-7 Latin/Greek (8-bit) alphabet';
                break;
            case 8:
                encoding = 'ISO/IEC 8859-8 Latin/Hebrew (8-bit) alphabet';
                break;
            case 9:
                encoding = 'ISO/IEC 8859-9 Latin 5 (8-bit) alphabet';
                break;
            case 10:
                encoding = 'ISO/IEC 8859-10 Latin 6 (8-bit) alphabet';
                break;
            case 11:
                encoding = 'ISO/IEC 8859-13 Latin 7 (8-bit) alphabet';
                break;
            case 12:
                encoding = 'ISO/IEC 8859-14 Latin 8 (8-bit) alphabet';
                break;
            case 13:
                encoding = 'ISO/IEC 8859-15 Latin 0 (8-bit) alphabet';
                break;
            case 14:
                encoding = 'PC code page 437 (United States)';
                break;
            case 15:
                encoding = 'PC code page 737 (Greek II)';
                break;
            case 16:
                encoding = 'PC code page 850 (Latin I)';
                break;
            case 17:
                encoding = 'PC code page 852 (Eastern Europe/Latin II)';
                break;
            case 18:
                encoding = 'PC code page 855 (Cyrillic I)';
                break;
            case 19:
                encoding = 'PC code page 857 (Turkish)';
                break;
            case 20:
                encoding = 'PC code page 860 (Portuguese)';
                break;
            case 21:
                encoding = 'PC code page 861 (Icelandic)';
                break;
            case 22:
                encoding = 'PC code page 863 (Canadian/French)';
                break;
            case 23:
                encoding = 'PC code page 865 (Nordic)';
                break;
            case 24:
                encoding = 'PC code page 866 (Russian/Cyrillic II)';
                break;
            case 25:
                encoding = 'PC code page 869 (Greek)';
                break;
            case 26:
                encoding = 'ISO/IEC 10646-1 [28] UCS2 (16-bit) alphabet';
                break;
            default:
                encoding = 'reserved or user application specific';
                break;
        }

        return encoding;
    }
    
    encodePayload(payload,encodingInt){
        var encoding;
        switch (encodingInt) {
            case 0:
                return gsm7.encode(payload).toString('hex').toUpperCase();
            case 1:
                encoding = 'ISO-8859-1';
                break;
            case 2:
                encoding = 'ISO-8859-2';
                break;
            case 3:
                encoding = 'ISO-8859-3';
                break;
            case 4:
                encoding = 'ISO-8859-4';
                break;
            case 5:
                encoding = 'ISO-8859-5';
                break;
            case 6:
                encoding = 'ISO-8859-6';
                break;
            case 7:
                encoding = 'ISO-8859-7';
                break;
            case 8:
                encoding = 'ISO-8859-8';
                break;
            case 9:
                encoding = 'ISO-8859-9';
                break;
            case 10:
                encoding = 'ISO-8859-10';
                break;
            case 11:
                encoding = 'ISO-8859-13';
                break;
            case 12:
                encoding = 'ISO-8859-14';
                break;
            case 13:
                encoding = 'ISO-8859-15';
                break;
            case 14:
                encoding = 'cp473';
                break;
            case 15:
                encoding = 'cp737';
                break;
            case 16:
                encoding = 'cp850';
                break;
            case 17:
                encoding = 'cp852';
                break;
            case 18:
                encoding = 'cp855';
                break;
            case 19:
                encoding = 'cp857';
                break;
            case 20:
                encoding = 'cp860';
                break;
            case 21:
                encoding = 'cp861';
                break;
            case 22:
                encoding = 'cp863';
                break;
            case 23:
                encoding = 'cp865';
                break;
            case 24:
                encoding = 'cp866';
                break;
            case 25:
                encoding = 'cp869';
                break;
            case 26:
                encoding = 'UCS2';
                break;
            default:
                encoding = 'ISO-8859-14';
                break;
        }
        return iconv.encode(payload,encoding).toString('hex').toUpperCase();
    }

    getProtocolIdentifierForValue(value){
        var protocolIdentifier;
        switch (value) {
            case 1:		// Over The Air re-Keying for end to end encryption		
                protocolIdentifier = "OTAK";
                break;
            case 2:		// Simple Text Messaging		
                protocolIdentifier = "Simple Text Messaging";
                break;
            case 3:		// Simple location system		
                protocolIdentifier = "Simple location system";
                break;
            case 4:		// Wireless Datagram Protocol WAP
                protocolIdentifier = "Wireless Datagram Protocol WAP";
                break;
            case 5:		// Wireless Control Message Protocol WCMP
                protocolIdentifier = "Wireless Control Message Protocol WCMP";
                break;
            case 6:		// Managed DMO
                protocolIdentifier = "Managed DMO";
                break;
            case 7:		// PIN authentication
                protocolIdentifier = "PIN authentication";
                break;
            case 8:		// End-to-end encrypted message
                protocolIdentifier = "End-to-end encrypted message";
                break;
            case 9:		// Simple immediate text messaging
                protocolIdentifier = "Simple immediate text messaging";
                break;
            case 10:		// Location information protocol		
                protocolIdentifier = "Location Information Protocol";
                break;
            case 11:		// Net Assist Protocol (NAP)
                protocolIdentifier = "Net Assist Protocol (NAP)";
                break;
            case 12:		// Concatenated SDS message
                protocolIdentifier = "Concatenated SDS message";
                break;
            case 13:		// DOTAM
                protocolIdentifier = "DOTAM";
                break;
            case 130:		// Text Messaging
                protocolIdentifier = "Text Messaging";
                //foreach(get_TextPDU($PDUbin,$ret["StringPos"]) as $key => $value) { $ret[$key] = $value; }
                break;
            case 131:		// Location system
                protocolIdentifier = "Location system";
                break;
            case 132:		// Wireless Datagram Protocol WAP
                protocolIdentifier = "Wireless Datagram Protocol WAP";
                break;
            case 133:		// Wireless Control Message Protocol WCMP
                protocolIdentifier = "Wireless Control Message Protocol WCMP";
                break;
            case 134:		// Managed DMO
                protocolIdentifier = "Managed DMO";
                break;
            case 136:		// End-to-end encrypted message
                protocolIdentifier = "End-to-end encrypted message";
                break;
            case 137:		// Immediate text messaging
                protocolIdentifier = "Immediate text messaging";
                break;
            case 138:		// Message with User Data Header
                protocolIdentifier = "Message with User Data Header";
                break;
            case 140:		// Concatenated SDS message
                protocolIdentifier = "Concatenated SDS message";
                break;
            default:
                protocolIdentifier = "reserved or user application specific pid";
        }
        return protocolIdentifier;
    }

    convertIntToHexString(number, numberOfBytes){

        var result = (number).toString(16).toUpperCase();
        if (result.length < numberOfBytes * 2){
            result = result.padStart(numberOfBytes * 2,"0")
        }
        else {
            result = result.slice(-numberOfBytes * 2);
            // overflow
        }
        return result;
    }

    calcByteLength(hexString){
        return hexString.length / 2;
    }


    getTransportStatusForStatusShort(statusShort){
        switch(statusShort) {
            case 0: 
                return 32770;
            case 1: 
                return 32771;
            case 2: 
                return 32772;
            case 3: 
                return 32773;
            case 4: 
                return 32774;
            case 5: 
                return 32775;
            case 6: 
                return 32776;
            case 7: 
                return 32777;
            case 8: 
                return 32778;
            case 9: 
                return 32779;
            default: 
                return 0;
        }
    }
}

module.exports = SdsHelper;