/**
 * Virtual PEI implementation
 *
 * It provides basic features to emulate a PEI for testing purposes
 *
 * @link   http://wwww.alarmiator.de
 * @file   This files defines the vPEI class.
 * @author Stefan Bauer.
 * @since  2020.12.24
 */

const EventEmitter = require('events');
const SerialPort = require('serialport')
const Delimiter = require('@serialport/parser-delimiter')
const SdsEncode = require('./sdsEncode');

class Vpei extends EventEmitter {
    serial = "COM5";
    port;
    parser;
    enableLocationInformationProtocol = false;
    enableStatus = false;
    enableTextMessaging = false;
    enableImmediateTextMessaging = false;
    enableMessageWithUserDataHeader = false;

    sdsEncode;

    portsList = [];
    constructor(){
        super();
        if (global.selectedPort != undefined) {
            this.serial = global.selectedPort;
            this.startvPEI(this.serial);
        }

    }

    initializeEvents(){
        // Open errors will be emitted as an error event
        this.port.on('error', function(err) {
            console.error( err.message)
        })

        this.parser = this.port.pipe(new Delimiter({ delimiter: '\r' }))

        this.parser.on('data', (data) => {
            var str = data.toString();
            console.log('Message received | ' +str);
            this.getCMDMode(str);
            switch (true) {
                case /\+CTSP/.test(str):
                    this.cmdCTSP(str);
                    break;
                case /\+GMI/.test(str):
                    this.cmdGMI();
                    break;
                case /\+GMM/.test(str):
                    this.cmdGMM();
                    break;
                case /\+GMR/.test(str):
                    this.cmdGMR();
                    break;
                case /\+GSN/.test(str):
                    this.cmdGSN();
                    break;
                case /\+CSPBOPTA/.test(str):
                    this.cmdCSPBOPTA(str);
                    break;
                case /\+CREG/.test(str):
                    this.cmdCREG(str);
                    break;
                case /\+CSQ/.test(str):
                    this.cmdCSQ(str);
                    break;
                default:
                    console.log("Unknown command");
                    break;
            }
        }) 
    }

    cmdCTSP(cmd){
        switch(this.getCMDMode(cmd)){
            case 2: {
                // +CTSP=<service profile>, <service layer1>, [<service layer2>], [<AI mode>], [<link identifier>] 
                // Example cmd AT+CTSP=2,2,20
                var parameters = this.getCmdParameters(cmd);
                var serviceProfile = parameters[0];
                var serviceLayer1 = parameters[1];
                var serviceLayer2 = parameters[2];
                var aiMode = parameters[3];
                var linkIdentifier = parameters[4];
            
                var cmdObject = {};
                cmdObject['serviceProfile'] = serviceProfile;
                cmdObject['serviceLayer1'] = serviceLayer1;
                cmdObject['serviceLayer2'] = serviceLayer2;
                cmdObject['aiMode'] = aiMode;
                cmdObject['linkIdentifier'] = linkIdentifier;
            
                if (serviceLayer1 === "2"){
                    switch (serviceLayer2){
                        case '20':
                        // enabled
                        // Status
                        this.enableStatus = true;
                        break;
                        default: 
                        // unknown service layer 2
                    }
                }
                
                if (serviceLayer1 === "3"){
                    switch (serviceLayer2){
                        case '10':
                        // enabled
                        // Location information protocol 
                        this.enableLocationInformationProtocol = true;
                        break;
                        case '137':
                        // enabled
                        // Text Messaging
                        this.enableTextMessaging = true;
                        break;
                        case '137':
                        // enabled
                        // Immediate text messaging 
                        this.enableImmediateTextMessaging = true;
                        break;
                        case '138':
                        // enabled
                        // Message with user data header
                        this.enableMessageWithUserDataHeader = true;
                        break;
                        default: 
                        // unknown service layer 2
                    }
                }
                this.cmdOK();
                //console.log(cmdObject);
                break;
            }
            default: 
                console.log("UNSUPPORTED MODE");
                break;
        }
    }
      
    getCMDMode(cmdString){
        // Return
        // 0: INVALID
        // 1: EXECUTION
        // 2: SET 
        // 3: TEST
        // 4: READ

        var mode = -1;

        switch(true){
            case /\+[A-Z]+$/gm.test(cmdString):
                // EXECUTION
                mode = 1;
                break;
            case /\+[A-Z]+\=/gm.test(cmdString):
                // SET, =
                mode = 2;
                break;
            case /\+[A-Z]+\=\?/gm.test(cmdString):
                // TEST, =?
                mode = 3;
                break;
            case /\+[A-Z]+\?/gm.test(cmdString):
                // READ, ?
                mode = 4;
                break;
            default: 
                // INVALID MODE
                mode = 0;
                break;
        }
        return mode;
    }

    getCmdParameters(cmdString){
        return cmdString.replace(/AT\+[A-Z]+=/gm,'').split(',').filter( x => x);
    }
      
      
    cmdGMI(){
        // Request MT manufacturer identification 
        this.sendMessageToPort("+GMI: ALARMiator DEVs");
    }
        
    cmdGMM(){
        // Request MT model identification  
        this.sendMessageToPort("+GMM: ALARMiator vPEI");
    }
        
    cmdGMR(){
        // Request MT revision identification
        this.sendMessageToPort("+GMR: ALARMiator DEVs");
    }
        
    cmdGSN(){
        // Request MT serial number identification 
        this.sendMessageToPort("+GSN: 1234");
    }
        
    cmdCSPBOPTA(cmd){
        // Request OPTA
        switch(this.getCMDMode(cmd)){
            case 4: {
                this.sendMessageToPort("+CSPBOPTA: ALARMiator_vPEI");
                break;
            }
            default: 
                console.log("UNSUPPORTED MODE");
                break;
        }
    }
        
    cmdCREG(cmd){
        // Network registration
        switch(this.getCMDMode(cmd)){
            case 4: {
                // +CREG: <Reg stat>, [<LA>], [<MNI>] 
                
                var payload = [];
                // This parameter is used to indicate the registration status of the MT to the TEI.
                var regStat = 1; // Registered, Home network
                payload.push(regStat);
                
                // 14-bit location area code of EN 300 392-2
                var LA = "7502";
                payload.push(LA);
                
                // 24 bits Mobile Network Identity, see EN 300 392-1 [1] presented in character set defined by +CSCS. 
                var MNI = "1234123"; 
                payload.push(MNI);
                
                this.sendMessageToPort("+CREG: "+ payload.join(','));

                break;
            }
            default: 
                console.log("UNSUPPORTED MODE");
                break;
        }
    }
        
    cmdCSQ(cmd){
        // Sends RSSI
        // +CSQ: -77,7
        switch(this.getCMDMode(cmd)){
            case 4: {
                var rssi = "-45,6"
                this.sendMessageToPort("+CSQ: "+ rssi);
                break;
            }
            default: 
                console.log("UNSUPPORTED MODE");
                break;
        }
    }
        
    cmdOK(){
        this.sendMessageToPort("OK");
    }

    msleep(n) {
        Atomics.wait(new Int32Array(new SharedArrayBuffer(4)), 0, 0, n);
    }
    
    sleep(n) {
        this.msleep(n*1000);
    }

    sendMessageToPort(cmd){
        this.port.write(cmd+"\n", function(err) {
            if (err) {
            return console.error('Error on write: ', err.message)
            }
            console.log('Message send     | '+cmd);
        });
    }

    sendUserdataMessageToPort(cmd){
        this.port.write(cmd+"\n", function(err) {
            if (err) {
            return console.error('Error on write: ', err.message)
            }
            console.log('Message send     | '+cmd);
        });
    }

    sendCallout(message, globalMessageCounter, timestampEnabled, encoding, constantMessageCounter){
        var messages = this.sdsEncode.buildCalloutMessage(message, globalMessageCounter, timestampEnabled, encoding, constantMessageCounter);

        messages.forEach( (msg) => {
            this.sendUserdataMessageToPort(msg);
            this.sleep(1);
        })
        return messages;
    }

    sendStatus(status){
        var message = this.sdsEncode.buildStatusMessage(status);

        this.sendUserdataMessageToPort(message);
        return message;
    }

    startvPEI(serial){
        try {
            this.port = new SerialPort(serial, { baudRate: 38400 });

            console.log("Connected to serial port: "+ serial);

            this.initializeEvents();
            this.sdsEncode = new SdsEncode();

        }
        catch (exception){
            console.error("Closing application ... ");
            console.error(exception);

        }

    }
    destroyvPEI() {
        this.port.close(function (err) {
            console.log('port closed', err);
        });
        this.port = undefined;
    }
}

module.exports = Vpei;