/**
 * Class handles all methods and models regarding access groups and properties
 * @class
 */
class accessmanagement {

  /**
   * @constructor
   * @param {object} alogger Logger to be used by class methods (usually req.app.logger)
   */
  constructor(alogger) {
    // always initialize all instance properties
    this._logger = alogger;
    this.accessgroups = [];
    this.loadAccessModel((success) => {

    });
  }



  loadAccessModel(callback) {
    this.getAccessGroups((accessGroups) => {
      if (accessGroups != null) { 
        this.accessgroups = accessGroups;
        global.app.logger.log('CORE | ACCESSMANAGER | successfully (re)loaded access groups');
        return callback(true);
      } else {
        this.accessgroups = [];
        global.app.logger.error('CORE | ACCESSMANAGER | Error loading the access model, please see logs. Server might not be in stable mode nor accessable for users');
        return callback(false);
      }
    })
  }


  /**
   * Returns all access groups as rows from core database
   * @param {function} callback row or null
   */
  getAccessGroups(callback) {
    let query = "SELECT * FROM `accessGroupsStore`";
    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-ACCESSMANAGEMENT | Error loading access groups from core database:' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all access endpoints ordered by accessGroup alias descending
   * @param {function} callback rows or null
   */
  getAccessEndpoints(callback) {
    let query = "SELECT " + 
                "    accessEndpointsStore.*, " +
                "    accessGroupsStore.alias AS groupAlias " +
                "FROM " +
                "    accessEndpointsStore " +
                "INNER JOIN " +
                "    accessGroupsStore ON accessGroupsStore.id = accessEndpointsStore.accessGroup " +
                "ORDER BY accessGroupsStore.alias";
    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-ACCESSMANAGEMENT | Error loading access groups from core database:' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });            
  }

  /**
   * Returns all existing defined access endpoints for role with given id
   * @param {integer} id 
   * @param {function} callback 
   */
  getAccessEndpointsForRoleWithId(id, callback) {
    let query = "SELECT * FROM `xt_roles_accessEnpoints` WHERE roleId = ?";
    global.coreDb.all(query, [
      id
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-ACCESSMANAGEMENT | Error loading access endpoints for role with id ' + id + ':' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Removes all access endpoints for role with given id
   * @param {integer} id id of role endpoints shall be deleted for
   * @param {*} callback success (true or false)
   */
  deleteAccessEndpointsForRoleWithId(id, callback) {
    let query = "DELETE FROM `xt_roles_accessEnpoints` WHERE roleId = ?";
    global.coreDb.run(query,
    [
        id
    ],
    (err) => {
        if (err) {
          this._logger.error('MODELS-ACCESSMANAGEMENT | Error deleting access endpoints for role with id ' + id + ':' + err.message);
          callback(false);
        } else {
          callback(true);
        }
    });
  }

  /**
   * Loads access endpoint details for given role id
   * @param {integer} id id of role to load all access endpoint details for
   * @param {*} callback rows or null
   */
  getDefinedAccessEndpointsForRoleWithId(id, callback) {
    let query = "SELECT " +
                  "accessEndpointsStore.*, " +
                  "accessGroupsStore.alias AS groupAlias, " +
                  "xt_roles_accessEndpoints.* " +
                "FROM " +
                  "accessEndpointsStore " +
                "INNER JOIN " +
                  "accessGroupsStore ON accessGroupsStore.id = accessEndpointsStore.accessGroup " +
                "INNER JOIN " +
                  "xt_roles_accessEndpoints ON xt_roles_accessEndpoints.accessEndpointAlias = accessEndpointsStore.alias " +
                "WHERE " +
                  "xt_roles_accessEndpoints.roleId = ?";
    global.coreDb.all(query, [
      id
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-ACCESSMANAGEMENT | Error loading access endpoints for role with id ' + id + ':' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }


  /**
   * Adds a new entry in table accessStore for given roleId
   * @param {integer} roleId id of role, this entry shall be stored for
   * @param {string} endpointAlias alias of endpoint
   * @param {integer} granted 1 or 0 - access granted
   * @param {integer} grantCreate 1 or 0 - role is allowed to create new objects
   * @param {integer} grantRead 1 or 0 - role is allowed to read objects
   * @param {integer} grantModify 1 or 0 - role is allowed to modify existing objects
   * @param {integer} grantRemove 1 or 0 - role is allowed to remove existing objects permanently
   * @param {integer} grantNotification 1 or 0 - role is allowed to recieve notifications
   * @param {integer} grantLimit null or integer - limit for role
   * @param {function} callback success (true/false)
   */
  addEndpointForRole(roleId, endpointAlias, granted, grantCreate, grantRead, grantModify, grantRemove, grantNotification, grantLimit, callback) {
    // first check if there is an existing entry for this role and the given endpoint alias
    // - there is only one allowed
    
    let query = "SELECT * " +
                "FROM " +
                  "xt_roles_accessEndpoints " +
                "WHERE " +
                  "roleId = ? " + 
                "AND " + 
                  "accessEndpointAlias = ?";
    global.coreDb.all(query, [
      roleId,
      endpointAlias
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-ACCESSMANAGEMENT | Error cjecking for existing entry in xt_roles_accessEndpoints for role with id ' + id + ': ' + err.message);
        return callback(false);
      } else {
        if (rows.length > 0) {
          // There seems to exist at least one entry for this combination
        } else {
          // There is no entry for this combination
          let query = "INSERT INTO xt_roles_accessEndpoints (roleId, accessEndpointAlias, granted, grantCreate, grantRead, grantModify, grantRemove, grantNotification, grantLimit) " + 
                      "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)";
          global.coreDb.run(query, [
            roleId,
            endpointAlias,
            granted,
            grantCreate,
            grantRead,
            grantModify,
            grantRemove,
            grantNotification,
            grantLimit
          ], (err) => {
              if (err) {
                  console.log('MODELS-ACCESSMANAGEMENT | Error storing new access entry to database: ' + err.message);
                  callback(false);
              } else {
                  callback(true);
              }
          });
        }
      }
    });
  }
}




// export the class
module.exports = accessmanagement;