const uuid = require('uuid').v4;
const moment = require('moment');

/**
 * Class handles all methods and models regarding access groups and properties
 * @class
 */
class accesstokens {

  /**
   * @constructor
   * @param {object} alogger Logger to be used by class methods (usually req.app.logger)
   */
  constructor(alogger) {
    // always initialize all instance properties
    this._logger = alogger;
  }

  /**
   * Returns all access tokens as rows from core database
   * @param {function} callback row or null
   */
  getAccessTokens(callback) {
    let query = "SELECT * FROM `accessTokensStore`";
    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-ACCESSTOKENS | Error loading access tokens from core database:' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * adds a new access token to core database and returns id of creatd token from core database
   * @param {string} description textual description of this token
   * @param {string} type type of token (module-identifier)
   * @param {integer} state 1 or 0
   * @param {function} callback success (true/false), insertId (if success = true) or null (if success = false)
   */
  addNewAccessToken(description, type, state, callback) {
    let token = uuid();
    let query = "INSERT INTO accessTokensStore (token, description, type, state, registeredAt) VALUES (?, ?, ?, ?, DATETIME('now', 'localtime'))";

    global.coreDb.run(query,
      [
        token,
        description,
        type,
        state
      ],
      function (err) {
        if (err) {
          console.log('MODELS-ACCESSTOKENS | Error adding new access token to core database:' + err.message);
          callback(false, null);
        } else {
          callback(true, this.lastID);
        }
      });
  }

  /**
   * Removes an access token and all role references from core database
   * @param {integer} tokenId id of access token do be removed
   * @param {function} callback success (true or false)
   */
  removeAccessToken(tokenId, callback) {
    let query = "DELETE FROM xt_roles_accesstokensstore WHERE accesstokensstoreId = ?";
    global.coreDb.run(query,
      [
        tokenId
      ],
      function (err) {
        if (err) {
          this._logger.error('MODELS-ACCESSTOKENS | Error removing role to access token assignment from core database ' + tokenId + ':' + err.message);
          callback(false);
        } else {
          let query2 = "DELETE FROM accessTokensStore WHERE id = ?";
          global.coreDb.run(query2,
            [
              tokenId
            ],
            function (err) {
              if (err) {
                this._logger.error('MODELS-ACCESSTOKENS | Error removing access token from core database ' + tokenId + ':' + err.message);
                callback(false);
              } else {
                callback(true);
              }
            });
        }
      });
  }

  /**
   * activates or deactivates an access token
   * @param {integer} tokenId id of access token in core database
   * @param {integer} state 1 or 0
   * @param {function} callback success (true / false)
   * @returns 
   */
  setActiveState(tokenId, state, callback) {
    let query = "UPDATE accessTokensStore SET state = ? WHERE id = ?";
    if ((state === 1) || (state === 0) || (state === '1') || (state === '0'))  {
      global.coreDb.run(query,
        [
          state,
          tokenId
        ],
        function (err) {
          if (err) {
            this._logger.error('MODELS-ACCESSTOKENS | Error removing access token from core database ' + tokenId + ':' + err.message);
            return callback(false);
          } else {
            return callback(true);
          }
        });
    } else {
      return callback(false);
    }
  }

  /**
   * checks if a token is valid and is active to be used
   * @param {string} token 
   * @param {function} callback null in case of error, true if valid and active, false if inactive or not found
   */
  isTokenValidandActive(token, callback) {
    let query = "SELECT * FROM accessTokensStore WHERE token = ?";
    global.coreDb.all(query, [
      token
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-ACCESSTOKENS | Error checking for valid access token at core database:' + err.message);
        return callback(null);
      } else {
        if (rows.length > 0) {
          return callback(true);
        } else {
          return callback(false);
        }
      }
    });
  }

  /**
   * sets a new Token for given access token id
   * @param {integer} atId id of access token to be updated
   * @param {*} callback success
   */
  recalculateToken(atId, callback) {
    let token = uuid();
    console.log(atId);
    let query = "UPDATE accessTokensStore SET token = ? WHERE id = ?";

    global.coreDb.run(query,
      [
        token,
        atId
      ],
      function (err) {
        if (err) {
          console.log('MODELS-ACCESSTOKENS | Error updating access token to core database:' + err.message);
          callback(false);
        } else {
          callback(true);
        }
      });
  }
}




// export the class
module.exports = accesstokens;