const assetsparameterhelper = require('./assetsparametershelper');
const Uploads = require('./core/core_uploads');
const EmailTemplates = require('./emailtemplates');
const EmailQueue = require('./emailqueue');
const Configuration = require('./configuration');
const Moment = require('moment');

/**
 * Class handles all assets related methods and also accessing assets in core database
 * @class
 */
class assets {
  /**
   * @constructor
   * @param {object} alogger Logger to be used in class methods (usually req.app.logger)
   */
  constructor(alogger) {
    // always initialize all instance properties
    this.logger = alogger;
  }

  /**
   * Returns list of assets in given warehouse
   * @param {integer} wId id of warehouse assets shall be loaded for
   * @param {function} callback rows or null
   */
  getAssetsForWarehouse(wId, callback) {
    let query = "SELECT * FROM assets WHERE warehouseId = ?";
    global.coreDb.all(query, [
      wId
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns list of assets for given basedata id
   * @param {integer} bId id of basedata assets shall be loaded for
   * @param {function} callback rows or null
   */
  getAssetsForBasedata(bId, callback) {
    let query = "SELECT * FROM assets WHERE basedataId = ?";
    global.coreDb.all(query, [
      bId
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all assets for display in UI view
   * @param {function} callback rows or null
   */
  getListOfAssets(callback) {
    this.logger.debug('MODELS-ASSETS | Reading assets');

    let query = `SELECT assets.*, 
                  assets_groups.name AS groupname, 
                  assets_states.name AS statename, 
                  assets_warehouses.name AS warehousename, 
                  assets_warehouses.wareHousePath AS warehousepath, 
                  assets_warehouses.description AS warehouseDescription, 
                  assets_states.usable AS usable, 
                  assets_states.maintenance AS maintenance,
                  assets_states.colorBg AS colorBg,
                  assets_states.colorFg AS colorFg,
                  basedata.firstname AS basedataFirstname, 
                  basedata.lastname AS basedataLastname 
                 FROM 'assets' 
                 LEFT JOIN assets_groups ON assets.assetGroupId = assets_groups.id 
                 LEFT JOIN assets_states ON assets.state = assets_states.id 
                 LEFT JOIN assets_warehouses ON assets.warehouseId = assets_warehouses.id 
                 LEFT JOIN basedata ON basedata.id = assets.basedataId`;

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        console.log(err);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all assets of an given assets group for display in UI view
   * @param {function} callback rows or null
   */
  getListOfAssetsForGroup(groupId, callback) {
    this.logger.debug('MODELS-ASSETS | Reading assets for group');

    let query = `SELECT assets.*, 
                  assets_groups.name AS groupname, 
                  assets_states.name AS statename, 
                  assets_warehouses.name AS warehousename, 
                  assets_warehouses.wareHousePath AS warehousepath, 
                  assets_warehouses.description AS warehouseDescription, 
                  assets_states.usable AS usable, 
                  assets_states.maintenance AS maintenance,
                  assets_states.colorBg AS colorBg,
                  assets_states.colorFg AS colorFg,
                  basedata.firstname AS basedataFirstname, 
                  basedata.lastname AS basedataLastname 
                 FROM 'assets' 
                 LEFT JOIN assets_groups ON assets.assetGroupId = assets_groups.id 
                 LEFT JOIN assets_states ON assets.state = assets_states.id 
                 LEFT JOIN assets_warehouses ON assets.warehouseId = assets_warehouses.id 
                 LEFT JOIN basedata ON basedata.id = assets.basedataId
                 WHERE assets.assetGroupId = ?`;

    global.coreDb.all(query, [
      groupId
    ], (err, rows) => {
      if (err) {
        console.log(err);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns list of asset states from core database
   * @param {function} callback rows or null
   */
  getListOfAssetsStates(callback) {
    let query = "SELECT * FROM assets_states WHERE state = 1";

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns a single asset from core database
   * @param {integer} assetId id of asset to be returned out of core database
   * @param {*} callback row or null
   */
  getAssetwithId(assetId, callback) {
    let query = "SELECT * FROM assets WHERE id = ?";
    global.coreDb.all(query, [
      assetId
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns a single asset from core database for given issi - number
   * @param {integer} issi issi to look for
   * @param {function} callback rows or null
   */
  getAssetForISSI(issi, callback) {
    let query = `
        SELECT assets.*,
          organizations.shortName as organizationsShortName, 
          organizations.name as organizationsName 
        FROM assets 
        INNER JOIN organizations 
          ON assets.ownerOrgId = organizations.id 
        WHERE issi = ?`;
    global.coreDb.all(query, [
      issi
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all assets which can be alarmed (isGear === 1)
   * @param {function} callback rows or null
   */
  getAllAssetsWhichCanBeAlarmed(callback) {
    // let query = "SELECT id, title, state, isGear, gearname, issi FROM assets WHERE isGear = 1";
    let query = "SELECT assets.*, " +
      "assets_groups.name AS groupname, " +
      "assets_states.name AS statename, " +
      "assets_warehouses.name AS warehousename, " +
      "assets_warehouses.wareHousePath AS warehousepath, " +
      "assets_warehouses.description AS warehouseDescription, " +
      "assets_states.usable AS usable, " +
      "assets_states.maintenance AS maintenance, " +
      "(" +
      " SELECT radioStatusShort " +
      " FROM `gearStateStore` " +
      " WHERE gearStateStore.issi = assets.issi " +
      " ORDER BY gearStateStore.timestamp DESC " +
      " LIMIT 1" +
      ") gearState " +
      "FROM `assets` " +
      "LEFT JOIN assets_groups ON assets.assetGroupId = assets_groups.id " +
      "LEFT JOIN assets_states ON assets.state = assets_states.id " +
      "LEFT JOIN assets_warehouses ON assets.warehouseId = assets_warehouses.id " +
      "WHERE isGear = 1";

    global.coreDb.all(query, [
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all assets which can be alarmed (isGear === 1) and has a set issi
   * @param {function} callback rows or null
   */
  getAllAssetsWhichCanBeAlarmedWithIssi(callback) {
    // let query = "SELECT id, title, state, isGear, gearname, issi FROM assets WHERE isGear = 1";
    let query = "SELECT assets.*, " +
      "assets_groups.name AS groupname, " +
      "assets_states.name AS statename, " +
      "assets_warehouses.name AS warehousename, " +
      "assets_warehouses.wareHousePath AS warehousepath, " +
      "assets_warehouses.description AS warehouseDescription, " +
      "assets_states.usable AS usable, " +
      "assets_states.maintenance AS maintenance " +
      "FROM `assets` " +
      "LEFT JOIN assets_groups ON assets.assetGroupId = assets_groups.id " +
      "LEFT JOIN assets_states ON assets.state = assets_states.id " +
      "LEFT JOIN assets_warehouses ON assets.warehouseId = assets_warehouses.id " +
      "WHERE isGear = 1 AND assets.issi IS NOT NULL AND length(assets.issi) > 0";

    global.coreDb.all(query, [
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all assets which can be alarmed (isGear === 1) for given Organization
   * @param {function} callback rows or null
   */
  getAllAssetsWhichCanBeAlarmedForOrganization(orgId, callback) {

    let query = "SELECT assets.*, " +
      "assets_groups.name AS groupname, " +
      "assets_states.name AS statename, " +
      "assets_warehouses.name AS warehousename, " +
      "assets_warehouses.wareHousePath AS warehousepath, " +
      "assets_warehouses.description AS warehouseDescription, " +
      "assets_states.usable AS usable, " +
      "assets_states.maintenance AS maintenance " +
      "FROM `assets` " +
      "LEFT JOIN assets_groups ON assets.assetGroupId = assets_groups.id " +
      "LEFT JOIN assets_states ON assets.state = assets_states.id " +
      "LEFT JOIN assets_warehouses ON assets.warehouseId = assets_warehouses.id " +
      "WHERE isGear = 1 AND ownerOrgId = ?";

    global.coreDb.all(query, [
      orgId
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns assets for given Organization
   * @param {integer} orgId id of organization
   * @param {function} callback rows or null
   */
  getAssetsForOrganization(orgId, callback) {
    let query = "SELECT * FROM assets WHERE ownerOrgId = ?";
    global.coreDb.all(query, [
      orgId
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Adds a new asset to core database
   * @param {object} reqBody form data (usually req.body)
   * @param {function} callback success (true / false)
   */
  addAsset(reqBody, callback) {
    let assetGroupId = reqBody.assetgroupid;
    let assetTitle = reqBody.title;
    let assetState = reqBody.state;
    let assetWarehouse = reqBody.warehouse;
    let assetIsGear = reqBody.isGear;                 // is Asset an alarmable gear
    let assetGearName = reqBody.gearname;             // name of asset in alarmings
    let assetIssi = reqBody.issi;                     // ISSI for alarmable assets
    let supplierId = reqBody.supplierId;              // Supplier Id as reference to suppliers table
    let assetOwnerOrgId = reqBody.assetsOwnerOrgId;   // organization id of the owner organization of this asset
    let assetBasedataId = reqBody.basedataId;         // basedata id the asset will be referenced to
    let assetUuid = reqBody.uuid;                          // uuid for referencing attachments in core database to this asset prior persiting of asset initially


    // Make sure we have null values for not set properties
    if (typeof assetBasedataId === "undefined") {
      assetBasedataId = null;
    }
    if (typeof assetWarehouse === "undefined") {
      assetWarehouse = null;
    }

    if (assetIsGear === 'on') {
      assetIsGear = 1;
    } else {
      assetIsGear = 0;
    }



    let assetsparameters = new assetsparameterhelper(this.logger);

    // Transform date for mysql query
    let assetDateBought = reqBody.date_bought;
    if (assetDateBought != '') {
    } else {
      assetDateBought = null;
    }

    let assetnextMaintenance = reqBody.date_nextMaintenance;
    if (assetnextMaintenance != '') {
    } else {
      assetnextMaintenance = null;
    }

    let query = "INSERT INTO assets " +
      "(title, " +
      "assetGroupId, " +
      "warehouseId, " +
      "date_bought, " +
      "dateNextMaintenance, " +
      "isGear, " +
      "issi, " +
      "gearname, " +
      "state, " +
      "supplierId, " +
      "ownerOrgId,  " +
      "basedataId, " +
      "uuid " +
      ") VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

    global.coreDb.run(query, [
      assetTitle,
      assetGroupId,
      assetWarehouse,
      assetDateBought,
      assetnextMaintenance,
      assetIsGear,
      assetIssi,
      assetGearName,
      assetState,
      supplierId,
      assetOwnerOrgId,
      assetBasedataId,
      assetUuid
    ], function (err) {
      if (err) {
        console.log('ROUTES-ASSETS | Error writing new asset to database: ' + err.message);
        global.logger.error('ROUTES-ASSETS | Error writing new asset to database: ' + err.message);
        return callback(false);
      } else {
        var newAssetId = this.lastID;
        //
        // link possible temporary uploads to the asset itself now
        //
        var uploads = new Uploads();
        uploads.linkUploadsWithUUIDToObjectId(assetUuid, newAssetId, (successLink) => {
          assetsparameters.getAssetsParametersForAssetGroup(assetGroupId, (resultAssetParameters) => {
            if (resultAssetParameters === false) {
              // Error reading asset parameters from database
              req.app.logger.error('ROUTES-ASSETS | Error reading asset parameters.');
              return callback(false);
            } else {
              // Recieved assets parameters.
              assetsparameters.persistParameterValuesInDatabase(newAssetId, reqBody, resultAssetParameters, (result) => {
                return callback(true);
              })
            }
          });
        })
      }
    });
  }

  /**
   * updates an asset at core database
   * @param {object} reqBody object holding all form fields (usually req.body)
   * @param {*} callback 
   */
  updateAsset(reqBody, callback) {
    let assetId = reqBody.assetId;
    let assetGroupId = reqBody.assetgroupid;
    let assetTitle = reqBody.title;
    let assetState = reqBody.state;
    let assetWarehouse = reqBody.warehouse;
    let assetIsGear = reqBody.isGear;                 // is Asset an alarmable gear
    let assetGearName = reqBody.gearname;             // name of asset in alarmings
    let assetIssi = reqBody.issi                      // ISSI of asset for alarmable assets
    let supplierId = reqBody.supplierId;              // supplierId as an reference 
    let assetOwnerOrgId = reqBody.assetsOwnerOrgId;   // id of organization this asset is owned by
    let assetBasedataId = reqBody.basedataId;         // basedata id the asset will be referenced to



    // Make sure we have null values for not set properties
    if (typeof assetBasedataId === "undefined") {
      assetBasedataId = null;
    }
    if (typeof assetWarehouse === "undefined") {
      assetWarehouse = null;
    }

    console.log(assetIsGear);
    if (assetIsGear === 'on') {
      assetIsGear = 1;
    } else {
      assetIsGear = 0;
    }

    // Transform date for mysql query
    let assetDateBought = reqBody.date_bought;
    if (assetDateBought != '') {
    } else {
      assetDateBought = null;
    }

    let assetnextMaintenance = reqBody.date_nextMaintenance;
    console.log('1: ' + assetnextMaintenance);
    if (assetnextMaintenance != '') {
    } else {
      assetnextMaintenance = null;
    }

    let assetsparameters = new assetsparameterhelper();

    let query = "UPDATE assets SET " +
      "title =  ?, " +
      "assetGroupId = ?, " +
      "warehouseId = ?, " +
      "date_bought = ?, " +
      "dateNextMaintenance = ?, " +
      "isGear = ?, " +
      "issi = ?, " +
      "gearname = ?, " +
      "state = ?, " +
      "supplierId = ?, " +
      "ownerOrgId = ?,  " +
      "basedataId = ? " +
      "WHERE id = ?";

    global.coreDb.run(query, [
      assetTitle,
      assetGroupId,
      assetWarehouse,
      assetDateBought,
      assetnextMaintenance,
      assetIsGear,
      assetIssi,
      assetGearName,
      assetState,
      supplierId,
      assetOwnerOrgId,
      assetBasedataId,
      assetId
    ], function (err) {
      if (err) {
        console.log(`ERROR Updating asset in core database: ${err.message}`);
        callback(false);
      } else {
        assetsparameters.getAssetsParametersForAssetGroup(assetGroupId, (resultAssetParameters) => {
          if (resultAssetParameters === false) {
            // Error reading asset parameters from database
            callback(false);
          } else {
            // Recieved assets parameters.
            // Now update assets parameters
            assetsparameters.updateParameterValuesInDatabase(assetId, reqBody, resultAssetParameters, (resultUpdate) => {
              console.log('Result Updating Asset: ' + resultUpdate);
              if (resultUpdate) {
                callback(true);
              } else {
                callback(false);
              }
            });
          }
        });
      }
    });
  }

  /**
   * Removes an asset and referenced assets parameters from core database
   * @param {integer} assetId id of asset to be removed from database
   * @param {function} callback success (true or false)
   */
  removeAsset(assetId, callback) {
    let query = "DELETE FROM assets_parameters_store WHERE assetsId = ?";
    global.coreDb.run(query, [
      assetId,
    ], function (err) {
      if (err) {
        console.log(`ERROR Removing Assets parameters for deletion of asset: ${err.message}`);
        callback(false);
      } else {
        let queryDeleteAsset = "DELETE FROM assets WHERE id = ?";
        global.coreDb.run(queryDeleteAsset, [
          assetId,
        ], function (err) {
          if (err) {
            console.log(`ERROR Removing Asset from core database: ${err.message}`);
            callback(false);
          } else {
            callback(true);
          }
        });
      }
    });
  }


  /**
   * Checks for assets which enter in maintenance period within given days from now
   * @param {integer} daysFromNow days from now to check for assets entering the maintenance period
   * @param {*} callback 
   */
  getAssetsEnteringMaintenanceInDays(daysFromNow, callback) {
    let query = "SELECT * FROM assets WHERE assets.dateNextMaintenance <= DATE('now','+? days');"
    global.coreDb.all(query, [
      daysFromNow
    ], (err, rows) => {
      if (err) {
        return null;
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns Assets entered the maintenance window
   * @param {integer} daysFromNow days from now to define the maintenance window
   * @param {function} callback rows or null
   */
  getAssetsInMaintenanceWindow(daysFromNow, callback) {
    if (daysFromNow === null) { daysFromNow = 30 };

    let query = `SELECT 
                    assets.*,
                    assets_groups.name AS groupName,
                    roles.rolename AS roleName,
                    basedata.id AS basedataId,
                    (basedata.firstname || ' ' || basedata.lastname) AS respName,
                    basedata.firstname AS respFirstName,
                    basedata.email AS respEMail,
                    assets_warehouses.name AS warehouseName,
                    date('now', 'localtime', '+30 days') AS windowEnd
                FROM assets 
                LEFT JOIN assets_groups ON assets_groups.id = assets.assetGroupId
                LEFT JOIN roles ON roles.id = assets_groups.responsibleRoleId
                LEFT JOIN xt_basedata_roles ON xt_basedata_roles.roleId = roles.id
                LEFT JOIN basedata ON basedata.id = xt_basedata_roles.basedataId
                LEFT JOIN assets_warehouses ON assets_warehouses.id = assets.warehouseId
                WHERE 
                  	dateNextMaintenance <= date('now', 'localtime', '+${daysFromNow} days') 
                AND
                    dateNextMaintenance != ''
                ORDER BY 
                    respEMail, dateNextMaintenance, assetGroupId`;

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        console.log('MODEL-ASSETS | Error loading assets in maintenance window from core database: ' + err.message);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns Assets entered the maintenance window for organization name
   * @param {integer} daysFromNow days from now to define the maintenance window
   * @param {string} organizationName organization where assets belong to
   * @param {function} callback rows or null
   */
   getAssetsInMaintenanceWindowForOrganizationName(daysFromNow, organizationName, callback) {
    if (daysFromNow === null) { daysFromNow = 30 };

    let query = `SELECT 
                    assets.*,
                    assets_groups.name AS groupName,
                    roles.rolename AS roleName,
                    basedata.id AS basedataId,
                    (basedata.firstname || ' ' || basedata.lastname) AS respName,
                    basedata.firstname AS respFirstName,
                    basedata.email AS respEMail,
                    assets_warehouses.name AS warehouseName,
                    date('now', 'localtime', '+30 days') AS windowEnd
                FROM assets 
                LEFT JOIN assets_groups ON assets_groups.id = assets.assetGroupId
                LEFT JOIN roles ON roles.id = assets_groups.responsibleRoleId
                LEFT JOIN xt_basedata_roles ON xt_basedata_roles.roleId = roles.id
                LEFT JOIN basedata ON basedata.id = xt_basedata_roles.basedataId
                LEFT JOIN assets_warehouses ON assets_warehouses.id = assets.warehouseId
                LEFT JOIN organizations ON assets.ownerOrgId = organizations.id
                WHERE 
                  	dateNextMaintenance <= date('now', 'localtime', '+${daysFromNow} days') 
                AND
                    dateNextMaintenance != '' 
                AND 
                    organizations.name = ? 
                ORDER BY 
                    respEMail, dateNextMaintenance, assetGroupId`;

    global.coreDb.all(query, [organizationName], (err, rows) => {
      if (err) {
        console.log('MODEL-ASSETS | Error loading assets in maintenance window from core database: ' + err.message);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all asset parameters for display in UI view
   * @param {function} callback rows or null
   */
  getListOfAssetParameters(callback) {
    this.logger.debug('MODELS-ASSETS | Reading asset parameters');

    let query = `SELECT 
                    assets_parameters.*,
                    assets_groups.name AS groupName,
                    assets_parameters_tabs.tabtitle AS tabName
                FROM assets_parameters
                INNER JOIN assets_groups ON assets_groups.id = assets_parameters.assetsGroup
                INNER JOIN assets_parameters_tabs ON assets_parameters_tabs.id = assets_parameters.assetsParametersTab`;

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        console.log(err);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all asset parameters tabs for display in UI view
   * @param {function} callback rows or null
   */
  getListOfAssetParametersTabs(callback) {
    this.logger.debug('MODELS-ASSETS | Reading asset parameters tabs');

    let query = `SELECT 
                    assets_parameters_tabs.*,
                    assets_groups.name AS groupName
                FROM assets_parameters_tabs
                INNER JOIN assets_groups ON assets_groups.id = assets_parameters_tabs.assetsGroup`;

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        console.log(err);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * central handler for maintenance tasks to be started for assets
   */
  doMaintenance() {
    var configuration = new Configuration(this._logger);
    configuration.getValueForKeyword('mtncAssetsDaysInFutureWindow', (mtncAssetsDaysInFutureWindow) => {
      console.log('MODELS-ASSETS | Starting doMaintenance operations');
      this.maintenance_checkForAssetsInMaintenanceWindow(mtncAssetsDaysInFutureWindow);
      this.maintenance_checkForExpiredMaintenanceSendNotifications(mtncAssetsDaysInFutureWindow);
    });
  }

  /**
   * Checks for assets which are now in maintenance window (today +30 days) and sends a notification with found assets to pluginManager
   */
  maintenance_checkForAssetsInMaintenanceWindow(daysFromNow) {
    if (daysFromNow !== null) {
      if (isNaN(daysFromNow)) {
          daysFromNow = 1;
      }
    } else {
        daysFromNow = 1;
    }
    this.getAssetsInMaintenanceWindow(daysFromNow, (rows) => {
      if (rows.length > 0) {
        console.log('MODEL-ASSETS | maintenance_checkForAssetsInMaintenanceWindow | found ' + rows.length + ' assets within maintenance window. Firing notification now');
        let notification = {
          type: 'notification',
          subtype: 'maintenance',
          module: 'assets',
          data: rows
        }
        global.plgManager.event_notification(notification);
      } else {
        console.log('MODEL-ASSETS | maintenance_checkForAssetsInMaintenanceWindow | found 0 assets within maintenance window.');
      }
    })
  }

  /**
   * Checks for assets which are entering maintenance window or have an expired maintenance date and creates email reports and puts them into email queue.
   * Method shall be ran once a day during maintenance tasks
   */
  maintenance_checkForExpiredMaintenanceSendNotifications(daysFromNow) {
    let newMailsCreated = false;
    if (daysFromNow !== null) {
      if(isNaN(daysFromNow)) {
          daysFromNow = 1;
          this.logger.error('MODELS-ASSETS | MAINTENANCE | Days to look into future not a number, using default of 1');
      } else {
          this.logger.log('MODELS-ASSETS | MAINTENANCE | Days to look into future: ' + daysFromNow);
      }
    } else {
        this.logger.error('MODELS-ASSETS | MAINTENANCE | Days to look into future not configured, using default of 1');
        daysFromNow = 1;
    }
    // query for expired examinations
    this.getAssetsInMaintenanceWindow(30, (rows) => {
      if (rows.length > 0) {
        console.log('MODELS-ASSETS | MAINTENANCE | Found ' + rows.length + ' assets with expiring maintenance dates. Sending email notifications now');
        var expiredTable = '';
        var expiredTablePlain = '';
        var dataByRecievers = [];
        var recieverInCycle = '';
        var newReciever = true;

        let reciever = {
          firstname: '',
          email: '',
          assets: []
        }
        rows.forEach((expired, index) => {
          if (expired.respEMail != null) {
            console.log('EMail: ' + expired.respEMail);
            // build groups for every recipient now
            // iterate over groups to send one email for all assets where the same person is responsible for
            if (recieverInCycle != expired.respEMail) {
              newReciever = true;
              recieverInCycle = expired.respEMail;
              if (index > 0) {
                dataByRecievers.push(reciever);
              }
            } else {
              newReciever = false;
            }
            console.log('index: ' + index + ' | newReciever: ' + newReciever + ' recieverInCycle: ' + recieverInCycle);
            if (newReciever) {
              reciever = {
                firstname: expired.respFirstName,
                email: expired.respEMail,
                assets: []
              }
            }


            if (Moment(expired.dateNextMaintenance).isValid()) {
              expired.dateNextMaintenance = Moment(expired.dateNextMaintenance).format('DD.MM.YYYY');
            }
            if ((expired.warehouseName === null) || (expired.warehouseName === 'null')) {
              expired.warehouseName = 'nicht zugeordnet';
            }
            let newAsset = {
              title: expired.title,
              dateNextMaintenance: expired.dateNextMaintenance,
              warehouseName: expired.warehouseName
            }
            reciever.assets.push(newAsset);
            // check if this is the last cycle
            if (index === (rows.length - 1)) {
              dataByRecievers.push(reciever);
            }
          }
        });

        console.log('MODEL-ASSETS | MAINTENANCE | compose now emails to ' + dataByRecievers.length + ' recipients');
        console.log(dataByRecievers);

        let emailqueue = new EmailQueue(global.app.logger);
        let addToQueuePromises = [];
        dataByRecievers.forEach((emailData) => {
          if (emailData.email.length > 4) {
            // Compose details table
            var expiredTable = '';
            var expiredTablePlain = '';
            emailData.assets.forEach((asset) => {
              expiredTable = expiredTable + `<tr>
                          <td style="text-align: left; border-bottom-width: 1px; border-bottom-color: #E1E1E1; border-bottom-style: solid; padding: 10px 15px 10px 0;" align="left">${asset.title}</td>
                          <td style="text-align: left; border-bottom-color: #E1E1E1; border-bottom-width: 1px; border-bottom-style: solid; padding: 10px 15px;" align="left">${asset.warehouseName}</td>
                          <td style="text-align: left; border-bottom-width: 1px; border-bottom-color: #E1E1E1; border-bottom-style: solid; padding: 10px 0 10px 15px;" align="left">${asset.dateNextMaintenance}</td>
                      </tr>`;
              expiredTablePlain = expiredTablePlain + `- ` + asset.title + ', ausgelaufen am ' + asset.dateNextMaintenance + '\n';
            })
            // Create Email in Queue now
            let emailtemplates = new EmailTemplates(global.app.logger);
            let data = {
              firstname: emailData.firstname,
              reciever: emailData.email,
              table: expiredTable,
              tablePlain: expiredTablePlain
            }

            let addPromise = new Promise((resolve, reject) => {
              emailtemplates.renderDataToTemplate(data, 'expiringMaintenanceDates', (template) => {
                if (template === null) {
                  console.log('MODEL-ASSETS | MAINTENANCE | Could not create eMail as template could not be loaded');
                } else {
                  let recipients = [];
                  recipients.push(emailData.email);
                  emailqueue.addToQueue('ALARMiator - Wartung anstehend', template.html, template.plain, recipients, (success) => {
                    if (success) {
                      console.log('MODELS-ASSETS | MAINTENANCE | Successfully added email to queue');
                      resolve(true);
                    } else {
                      console.log('MODELS-ASSETS | MAINTENANCE | ERROR | Could not add email to queue, please see logs');
                      reject(false);
                    }
                  })
                }
              });
            });
            addToQueuePromises.push(addPromise);
          } else {
            console.log('MODELS-ASSETS | MAINTENANCE | ERROR | Empty reciever -> could not create eMail Object');
          }
        })
        Promise.allSettled(addToQueuePromises).then((results => {
          emailqueue.getEmailsToBeSent(null, (rowsEmails) => {
            if (rowsEmails != null) {
              if (rowsEmails.length > 0) {
                console.log('MODELS-ASSETS | MAINTENANCE | Informing about new emails by firing event_email_queue_refreshed with ' + rowsEmails.length + ' new emails');
                global.plgManager.event_email_queue_refreshed(rowsEmails);
              }
            }
          })
        }))
      }
    });
  }

}



// export the class
module.exports = assets;