/*
//
//  Class offers helper functions for 
//  rendering and organization of assets parameters
//
*/

const { UUID } = require('sequelize');
const assetgroups = require('./assetsgroups');
const AssetsGroups = require('./assetsgroups');
const Uploads = require('./core/core_uploads');

const uuid = require('uuid').v1;

/**
 * Class for handling of assets parameters for assets groups and their tabs and orders
 * @class
 */
class assetsparametershelper {

    /**
     * @constructor
     */
    constructor() {
        // always initialize all instance properties
    }

    /**
     * reads all assets parameters tabs from core database
     * @param {function} callback rows or null
     */
    getAllAssetsParametersTabs(assetGroup, callback) {

        let query = "SELECT * FROM assets_parameters_tabs";
        global.coreDb.all(query, [
            assetGroup
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * reads assets parameter from core database
     * @param {integer} pId id of asset parameter to read from core database
     * @param {function} callback rows or null
     */
    getAssetParameter(pId, callback) {
        let query = "SELECT * FROM assets_parameters WHERE id = ?";
        global.coreDb.all(query, [
            pId
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all selectable Options for a parameter of type select
     * @param {integer} pId id of parameter to return selectable options for
     * @param {function} callback rows or null
     */
    getSelectableOptionsForParameter(pId, callback) {
        let query = "SELECT * FROM assets_parameters_options WHERE parameterId = ?";
        global.coreDb.all(query, [
            pId
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all selectable Options for a parameter of type select async
     * @param {integer} pId id of parameter to return selectable options for
     * @param {function} callback rows or null
     */
    async getSelectableOptionsForParameterAsync(pId) {
        let query = "SELECT * FROM assets_parameters_options WHERE parameterId = ?";


        return new Promise((resolve, reject) => {

            const queries = [];
            global.coreDb.each(query, [
                pId
            ], (err, row) => {
                if (err) {
                    reject(err); // optional: you might choose to swallow errors.
                } else {
                    queries.push(row); // accumulate the data
                }
            }, (err, n) => {
                if (err) {
                    reject(err); // optional: again, you might choose to swallow this error.
                } else {
                    resolve(queries); // resolve the promise
                }
            });
        });
    }

    /**
     * inserts new seletable option for select parameter into core database after checking for duplicates
     * @param {integer} pId parameter id
     * @param {string} label label of the new selectable option
     * @param {function} callback success (true-> persisted, false->duplicate, null->technical error)
     */
    addSelectableOptionForParameter(pId, label, callback) {
        let query = "SELECT * FROM assets_parameters_options WHERE label = ? and id = ?";
        global.coreDb.all(query, [
            label,
            pId
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                if (rows.length > 0) {
                    // Found duplicate --> not persisting new Option
                    return false;
                } else {
                    let alias = uuid();
                    let query = "INSERT INTO assets_parameters_options (parameterId, label, alias, state, sortorder) VALUES (?, ?, ?, ?, ?)";
                    global.coreDb.run(query,
                        [
                            pId,
                            label,
                            alias,
                            1,
                            0
                        ],
                        function (err) {
                            if (err) {
                                console.log('ERROR inserting new select option for parameter with id ' + pId + ' in core database: ' + err.message);
                                callback(false);
                            } else {
                                callback(true);
                            }
                        });
                }
            }
        });
    }

    /**
     * removes a selectable parameter option from core database
     * @param {integer} oId id of selectable option
     * @param {*} callback success (true / false)
     */
    removeSelectableParameterOption(oId, callback) {
        let query = "DELETE FROM assets_parameters_options WHERE id = ?";
        global.coreDb.run(query,
            [
                oId
            ],
            function (err) {
                if (err) {
                    console.log('ERROR deleting select option with id ' + oId + ' from core database: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * reads defined tabs for assets parameters from database
     * @param {integer} assetGroup id of asset group, asset parameter tabs shall be returned for
     * @param {function} callback rows or null
     */
    getAssetsParametersTabs(assetGroup, callback) {

        let query = "SELECT * FROM assets_parameters_tabs WHERE assetsGroup = ? AND state = 1 ORDER BY `order` ASC";
        global.coreDb.all(query, [
            assetGroup
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                console.log('getAssetsParametersTab Count Tabs: ' + rows.length);
                return callback(rows);
            }
        });
    }

    /**
     * reads one single tab for assets parameters from database
     * @param {integer} tabId id of assets tab
     * @param {function} callback rows or null
     */
    getAssetsParametersTab(tabId, callback) {
        let query = "SELECT * FROM assets_parameters_tabs WHERE id = ?";
        global.coreDb.all(query, [
            tabId
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * returns list of assets parameters for given assetGroup id
     * @param {integer} assetGroup id of assets group, parameters shall be returned for
     * @param {function} callback rows or null
     */
    getAssetsParametersForAssetGroup(assetGroup, callback) {
        console.log('getAssetsParametersForAssetGroup');
        let query = "SELECT assets_parameters.*, " +
            "`assets_parameters_tabs`.`order` AS taborder " +
            "FROM assets_parameters " +
            "LEFT JOIN assets_parameters_tabs ON assets_parameters.assetsParametersTab = assets_Parameters_tabs.id " +
            "WHERE assets_parameters.assetsGroup = ? AND assets_parameters.state = 1 " +
            "ORDER BY `assets_parameters_tabs`.`order` ASC";

        global.coreDb.all(query, [
            assetGroup
        ], (err, rows) => {
            if (err) {
                console.log('getAssetsParametersForAssetGroup: Error reading parameters: ' + err.message);
                return null;
            } else {
                console.log('getAssetsParametersForAssetGroup: found ' + rows.length + ' Parameters');
                return callback(rows);
            }
        });
    }



    /**
     * checks weather a value for the given assetsparameter exists in assets_parameters_store
     * @param {integer} assetsId id of asset the check shall be run for
     * @param {integer} assetsParametersId id of asset parameter the check shall be run for
     * @param {function} callback id of asstets_parameter_store if found, or false if not found
     */
    doesParameterExistInStore(assetsId, assetsParametersId, callback) {
        let query = "SELECT * FROM assets_parameters_store WHERE assetsId = ? AND assetsParametersId = ?";
        global.coreDb.all(query, [
            assetsId,
            assetsParametersId
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                if (rows.length > 0) {
                    console.log('Parameter to be updated exists in core database');
                    return callback(rows[0].id);
                } else {
                    console.log('Parameter to be updated DOES NOT exist in core database');
                    return callback(false);
                }
            }
        });
    }



    /**
     * updates Value in parameters Store
     * @param {integer} storeId id of object in table assets_parameters_store
     * @param {string} newValue new value as string
     * @param {string} interpretAs type of value as string
     * @param {function} callback callback function, success (true or false)
     */
    updateParameterInStore(storeId, fieldType, newValue, callback) {

        let query = '';
        switch (fieldType) {
            case 'string':
                query = "UPDATE assets_parameters_store SET valueStr = ? WHERE id = ?";
                break;
            case 'select':
                query = "UPDATE assets_parameters_store SET valueSelect = ? WHERE id = ?";
                break;
            case 'date':
                query = "UPDATE assets_parameters_store SET valueDate = ? WHERE id = ?";
                break;
            case 'integer':
                query = "UPDATE assets_parameters_store SET valueInt = ? WHERE id = ?";
                break;
            default:
                break;
        }

        global.coreDb.run(query,
            [
                newValue,
                storeId
            ],
            function (err) {
                if (err) {
                    console.log('ERROR updating asset parameter in core database: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });

    }

    /**
     * update parameter value to database table assets_parameters_store
     * @param {integer} assetId ID of asset this value has to be stored for
     * @param {integer} parameterId ID of parameter, this value is for
     * @param {string} parameterType type of parameter (string, select, integer)
     * @param {string} value value to be stored
     * @param {function} callback callback function (false in case of error, true if successfully written)
     */
    updateParameterValuesInDatabase(assetId, formData, parameters, callback) {
        var insertValuePart = '';
        console.log('updateParameterValuesInDatabase - 1');
        var updateQueries = [];
        var inserQueries = [];

        console.log('parameters: --------------------------------------');
        console.log(parameters);
        console.log('formData:-----------------------------------------');
        console.log(formData);
        console.log('--------------------------------------------------');

        global.coreDb.serialize(() => {
            parameters.forEach((parameter, index) => {
                console.log('updateParameterValuesInDatabase - 2');
                let fieldValue = formData['ap_' + parameter.alias];
                let fieldType = parameter.type;
                var interpretAs = '';
                var valueStr = null;
                var valueSelect = null;
                var valueInt = null;
                var valueDate = null;
                console.log('interpret As: ' + interpretAs);
                switch (fieldType) {
                    case 'string':
                        interpretAs = 'valueStr';
                        valueStr = fieldValue;
                        break;
                    case 'select':
                        interpretAs = 'valueSelect';
                        valueSelect = fieldValue;
                        break;
                    case 'integer':
                        interpretAs = 'valueInt';
                        valueInt = fieldValue;
                        break;
                    case 'date':
                        interpretAs = 'valueDate';
                        valueDate = fieldValue;
                        break;
                    case 'datetime':
                        interpretAs = 'valueDate';
                        valueDate = fieldValue;
                        break;
                    case 'multiline':
                        interpretAs = 'valueStr';
                        valueStr = fieldValue;
                        break;
                    default:
                        interpretAs = 'valueStr';
                        valueStr = fieldValue;
                        break;
                }


                let queryInsertOrReplace = "INSERT OR REPLACE INTO assets_parameters_store (id, assetsParametersId, assetsId, valueStr, valueInt, valueSelect, valueDate, interpretAs) " +
                    "VALUES (  " +
                    "(    SELECT id " +
                    "FROM assets_parameters_store " +
                    "WHERE assetsParametersId = ? " +
                    "AND assetsId = ?), " +
                    "?, " +
                    "?,  " +
                    "?, " +
                    "?, " +
                    "?, " +
                    "?, " +
                    "? " +
                    ");"

                global.coreDb.run(queryInsertOrReplace, [
                    parameter.id,
                    assetId,
                    parameter.id,
                    assetId,
                    valueStr,
                    valueInt,
                    valueSelect,
                    valueDate,
                    interpretAs
                ], (err, rows) => {
                    if (err) {
                        console.log('Error updating or inserting parameter: ' + err.message);
                        //return null;
                    } else {

                    }
                });
            })
        })
        console.log('finished Update of parameters');
        callback(true);

    }



    /**
     * writes parameter value to database table assets_parameters_store
     * @param {integer} newAssetId 
     * @param {object} formData 
     * @param {object} parameters 
     * @param {function} callback success (true or false)
     */
    persistParameterValuesInDatabase(newAssetId, formData, parameters, callback) {

        var insertValuePart = '';
        parameters.forEach((parameter, index) => {
            let fieldValue = formData['ap_' + parameter.alias];
            let fieldType = parameter.type;

            var insertStr = '';
            var identifiedParameterType = true;
            switch (fieldType) {
                case 'string':
                    insertStr = '(' + parameter.id + ', ' + newAssetId + ', "' + fieldValue + '", null, null, null, "string")';
                    break;
                case 'multiline':
                    insertStr = '(' + parameter.id + ', ' + newAssetId + ', "' + fieldValue + '", null, null, null, "string")';
                    break;
                case 'select':
                    insertStr = '(' + parameter.id + ', ' + newAssetId + ', null, null,  "' + fieldValue + '", null, "select")';
                    break;
                case 'integer':
                    insertStr = '(' + parameter.id + ', ' + newAssetId + ', null, ' + fieldValue + ',  null, null, "integer")';
                    break;
                case 'date':
                    // set defined null if date is not filled
                    if (fieldValue.length === 0) {
                        fieldValue = null;
                    } else {
                        fieldValue = '"' + fieldValue + '"';
                    }
                    insertStr = '(' + parameter.id + ', ' + newAssetId + ', null, null,  null, ' + fieldValue + ', "date")';
                    break;
                case 'datetime':
                    // set defined null if date is not filled
                    if (fieldValue.length === 0) {
                        fieldValue = null;
                    } else {
                        fieldValue = '"' + fieldValue + '"';
                    }
                    insertStr = '(' + parameter.id + ', ' + newAssetId + ', null, null,  null, ' + fieldValue + ', "date")';
                    break;
                case 'attachments':
                    console.log('ASSETSPARAMETERSHELPER | persistParameterValuesInDatabase | ignoring Attachment-Parameter (Attachments are side loaded by modal');
                    identifiedParameterType = false;
                    break;
                default:
                    console.log('ASSETSPARAMETERSHELPER | persistParameterValuesInDatabase | found unhandled fieldtype: ' + fieldType);
                    identifiedParameterType = false;
                    break;
            }
            var separator = '';
            if ((index > 0) && (identifiedParameterType === true)) {
                separator = ',';
            }
            insertValuePart = insertValuePart + separator + insertStr;
            console.log(index + ' | ' + parameter.label + ' | ' + parameter.alias + ' | ' + fieldValue + ' | ' + fieldType);
        })
        if (insertValuePart.length > 0) {
            let query = "INSERT INTO `assets_parameters_store` " +
                "( " +
                "`assetsParametersId`, " +
                "`assetsId`, " +
                "`valueStr`, " +
                "`valueInt`, " +
                "`valueSelect`, " +
                "`valueDate`, " +
                "`interpretAs`) " +
                "VALUES " + insertValuePart;

            global.coreDb.run(query,
                [],
                function (err) {
                    if (err) {
                        console.log('ERROR persisting new asset parameters to core database: ' + err.message);
                        callback(false);
                    } else {
                        callback(true);
                    }
                });
        } else {
            // No parameters to be stored
            callback(true);
        }
    }

    /**
     * Function loads values for asset parameters for given asset with assetId
     * @param {integer} assetId 
     * @param {function} callback rows or null
     */
    getAssetParameterValuesForAssetId(assetId, callback) {
        console.log('getAssetParameterValuesForAssetId');
        let query = "SELECT * FROM assets_parameters_store WHERE assetsId = ?";
        global.coreDb.all(query, [
            assetId
        ], (err, rows) => {
            if (err) {
                console.log('getAssetParameterValuesForAssetId: ERROR loading from core database: ' + err.message);
                return false;
            } else {
                console.log('getAssetParameterValuesForAssetId: found ' + rows.length + ' asset parameter values rows');
                return callback(rows);
            }
        });
    }


    /**
     * writes parameter value to database table assets_parameters_store
     * @param {*} assetId ID of asset this value has to be stored for
     * @param {*} parameterId ID of parameter, this value is for
     * @param {*} parameterType type of parameter (string, select, integer)
     * @param {*} value value to be stored
     * @param {*} callback callback function (false in case of error, true if successfully written)
     */
    persistParameterValueInDatabase(assetId, parameterId, parameterType, value, callback) {

        var interpretAs = '';
        switch (parameterType) {
            case 'string':
                interpretAs = 'valueStr';
                break;
            case 'multiline':
                interpretAs = 'valueStr';
                break;
            case 'select':
                interpretAs = 'valueSelect';
                break;
            case 'integer':
                interpretAs = 'valueInt';
                break;
            case 'date':
                interpretAs = 'valueDate';
                break;
            case 'datetime':
                interpretAs = 'valueDate';
                break;
            default:
                interpretAs = 'valueStr';
                break;
        }

        let query = "INSERT INTO assets_parameters_store (assetsParametersId, assetsId, " + interpretAs + ") VALUES (" + parameterId + ", " + assetId + ", '" + value + "')";
        global.coreDb.run(query,
            [],
            function (err) {
                if (err) {
                    console.log('MODELS-ASSETPARAMETERSHELPER | Error writing parameter value to database: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * writes parameter value to database table assets_parameters_store
     * @param {*} req 
     * @param {*} res 
     * @param {*} parameterId ID of parameter, this value is for
     * @param {*} parameterType type of parameter (string, select, integer)
     * @param {*} value value to be stored
     * @param {*} callback callback function (false in case of error, true if successfully written)
     */
    updateParameterValueInDatabase(parameterStoreId, parameterType, value, callback) {

        var interpretAs = '';
        switch (parameterType) {
            case 'string':
                interpretAs = 'valueStr';
                break;
            case 'multiline':
                interpretAs = 'valueStr';
                break;
            case 'select':
                interpretAs = 'valueSelect';
                break;
            case 'integer':
                interpretAs = 'valueInt';
                break;
            case 'date':
                interpretAs = 'valueDate';
                break;
            case 'datetime':
                interpretAs = 'valueDate';
                break;
            default:
                interpretAs = 'valueStr';
                break;
        }

        let query = "UPDATE `assets_parameters_store` " +
            "SET ? = ? WHERE id = ?";

        global.coreDb.run(query,
            [
                interpretAs,
                value,
                parameterStoreId
            ],
            function (err) {
                if (err) {
                    console.log('MODELS-ASSETPARAMETERSHELPER | Error updating parameter value to database: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Renders HTML Code for Tab navigation for given assetGroup id
     * @param {*} assetGroups 
     * @param {*} callback 
     */
    renderTabsForAssetGroups(assetGroups, showAlarmingTab, callback) {
        console.log('renderTabsForAssetGroups');
        var returnHTML = '<nav mt-5>' +
            '<div class="nav nav-tabs" id="nav-tab" role="tablist">';
        assetGroups.forEach((group, index) => {
            var activeClass = ' active';
            if (index > 0) {
                activeClass = '';
            }
            var tabId = group.tabalias;
            returnHTML = returnHTML + '<a class="nav-item nav-link ' + activeClass + '" id="nav-' + tabId + '-tab" data-toggle="tab" href="#nav-' + tabId + '" role="tab" aria-controls="nav-' + tabId + '" aria-selected="true" style="font-size: 13px">' + group.tabtitle + '</a>';
        })
        // For Assets in a Group which is declaed as a GearGroup, show alarming tab
        if (showAlarmingTab === 1) {
            var activeClass = '';
            var tabId = 'alarming';
            returnHTML = returnHTML + '<a class="nav-item nav-link ' + activeClass + '" id="nav-' + tabId + '-tab" data-toggle="tab" href="#nav-' + tabId + '" role="tab" aria-controls="nav-' + tabId + '" aria-selected="true" style="font-size: 13px">Alarmierung</a>';
        }
        returnHTML = returnHTML + '</div>' +
            '</nav>' +
            '<div class="mb-5"></div>';
        callback(returnHTML);
    }



    /**
     * builds HTML-form for assets parameters organized in tabs
     * @param {*} req 
     * @param {*} res 
     * @param {*} assetGroup 
     * @param {*} callback 
     */
    getAssetsParametersFormHTMLForAssetGroup(req, res, assetGroup, callback) {
        var promiseListe = [];

        this.getAssetsParametersTabs(assetGroup, (resultTabs) => {
            var assetsgroups = new AssetsGroups(req.app.logger);
            assetsgroups.getAssetGroup(assetGroup, (rowAssetGroup) => {
                this.getAssetsParametersForAssetGroup(assetGroup, (result) => {
                    if (result === false) {
                        // Error reading parameters
                        req.app.logger.error('MODELS-ASSETPARAMETERSHELPER | Error reading asset parameters from database');
                        callback('ERROR');
                    } else {

                        req.app.logger.debug('MODELS-ASSETPARAMETERSHELPER | Successfully read ' + result.length + ' asset parameters from database');
                        console.log('MODELS-ASSETPARAMETERSHELPER | Successfully read ' + result.length + ' asset parameters from database');
                        var retValHTML = '';
                        // Render Navigation Tabs
                        var showAlarming = 0;
                        if (rowAssetGroup.length > 0) {
                            showAlarming = rowAssetGroup[0].isGearGroup;
                        }
                        this.renderTabsForAssetGroups(resultTabs, showAlarming, (tabsHTML) => {
                            //console.log('getAssetsParametersForAssetsGroup: rendered Tabs for Asset Groups: ' + tabsHTML);
                            var tabId = 0;
                            retValHTML = tabsHTML;
                            retValHTML = retValHTML + '<div class="tab-content" id="nav-tabContent">';
                            resultTabs.forEach((tab, tabIndex) => {
                                var activeTab = '';
                                if (tabIndex < 1) {
                                    activeTab = 'active';
                                }

                                promiseListe.push(new Promise((resolve, reject) => {
                                    resolve('<div class="tab-pane fade show ' + activeTab + '" id="nav-' + tab.tabalias + '" role="tabpanel" aria-labelledby="nav-' + tab.tabalias + '-tab"><div class="m-3">')
                                }))

                                result.forEach((assetParameter, index) => {
                                    if (assetParameter.assetsParametersTab === tab.id) {
                                        var mandantory = '';
                                        if (assetParameter.mandantory === 1) { mandantory = ' required'; }
                                        switch (assetParameter.type) {
                                            case 'string':

                                                promiseListe.push(new Promise((resolve, reject) => {
                                                    resolve(
                                                        '<div class="form-group">' +
                                                        '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                        '<input type="text" class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" ' + mandantory + '>' +
                                                        '</div>'
                                                    )
                                                }))
                                                break;
                                            case 'multiline':

                                                promiseListe.push(new Promise((resolve, reject) => {
                                                    resolve(
                                                        '<div class="form-group">' +
                                                        '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                        '<textarea class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" ' + mandantory + ' rows="10" cols="30"></textarea>' +
                                                        '</div>'
                                                    )
                                                }))
                                                break;
                                            case 'select':
                                                var dbRowsOptions = this.getSelectableOptionsForParameterAsync(assetParameter.id);

                                                promiseListe.push(dbRowsOptions.then((rowsOptions) => {

                                                    var html = "";
                                                    html = html + '<div class="form-group">' +
                                                        '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                        '<select class="form-control text-primary" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" ' + mandantory + '>';

                                                    if (rowsOptions === null) {
                                                        html = html + '<option disabled>11Fehler beim laden der Optionen</option>';
                                                    } else if (rowsOptions.length > 0) {
                                                        rowsOptions.forEach(option => {
                                                            //var keys = Object.keys(option);
                                                            html = html + '<option value="' + option.id + '">' + option.label + '</option>';
                                                        })
                                                    } else {
                                                        html = html + '<option disabled>Keine Optionen zur Auswahl</option>';
                                                    }
                                                    html = html + '</select></div>';

                                                    return new Promise((resolve, reject) => {
                                                        resolve(html)
                                                    })
                                                }))

                                                break;
                                            case 'integer':


                                                promiseListe.push(new Promise((resolve, reject) => {
                                                    resolve(
                                                        '<div class="form-group">' +
                                                        '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                        '<input type="text" class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '"  onkeypress="return isNumberKey(event);" ' + mandantory + '>' +
                                                        '</div>'
                                                    )
                                                }))
                                                break;
                                            case 'date':
                                                promiseListe.push(new Promise((resolve, reject) => {
                                                    resolve(
                                                        '<div class="form-group">' +
                                                        '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                        '<input type="date" class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '"  " ' + mandantory + '>' +
                                                        '</div>'
                                                    )
                                                }))
                                                break;
                                            case 'datetime':
                                                promiseListe.push(new Promise((resolve, reject) => {
                                                    resolve(
                                                        '<div class="form-group">' +
                                                        '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                        '<input type="datetime-local" class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '"  " ' + mandantory + '>' +
                                                        '</div>'
                                                    )
                                                }))
                                                break;
                                            case 'attachments':
                                                promiseListe.push(new Promise((resolve, reject) => {
                                                    resolve(
                                                        `<div class="form-group">
                                                            <table id="attachmentsTable" class="table table-striped table-bordered">
                                                                <thead>
                                                                    <tr>
                                                                        <th scope="col">Typ</th>
                                                                        <th scope="col">Dateiname</th>
                                                                        <th scope="col">Beschreibung</th>
                                                                        <th scope="col">Aktion</th>
                                                                    </tr>
                                                                </thead>
                                                                <tbody>
                                                                </tbody>
                                                            </table>
                                                        </div>
                                                        <button type="button" class="btn btn-success" data-toggle="modal" data-target="#addAttachmentModal">hinzufügen</button>`
                                                    )
                                                }))
                                                break;
                                            default:
                                                break;
                                        }
                                    }
                                })
                                promiseListe.push(new Promise((resolve, reject) => {
                                    resolve(
                                        '</div></div>'
                                    )
                                }))
                            })
                            if (showAlarming === 1) {
                                var tempHtml = "";
                                tempHtml = tempHtml + '<div class="tab-pane fade" id="nav-alarming" role="tabpanel" aria-labelledby="nav-alarming-tab"><div class="m-3">';
                                tempHtml = tempHtml +
                                    '<div class="form-group">' +
                                    '<div class="custom-control custom-switch ml-3 mt-3"> ' +
                                    '<input type="checkbox" class="custom-control-input" id="isGear" name="isGear" checked="">' +
                                    '<label class="custom-control-label" for="isGearGroup">alarmierbares Einsatzmittel</label>' +
                                    '</div>' +
                                    '</div>' +
                                    '<div class="form-group">' +
                                    '<label for="isAlarmableGear">Bezeichner in Alarmquellen</label>' +
                                    '<input type="text" class="form-control" id="gearname" name="gearname">' +
                                    '</div>' +
                                    '<div class="form-group">' +
                                    '<label for="issi">ISSI für Statusausleitung</label>' +
                                    '<input type="text" class="form-control" id="issi" name="issi">' +
                                    '</div>';
                                tempHtml = tempHtml + '</div></div>';
                                promiseListe.push(new Promise((resolve, reject) => {
                                    resolve(
                                        tempHtml
                                    )
                                }))
                            }

                            Promise.allSettled(promiseListe).then((result) => {
                                result.forEach((resultHtml) => {
                                    if (resultHtml.status == 'fulfilled') {
                                        retValHTML += resultHtml.value
                                    }
                                    else {
                                        console.log("Error with promise: ", resultHtml.reason);
                                    }

                                })
                                callback(retValHTML);
                            })

                        })
                    }
                })
            })

        })
    }

    /**
     * builds HTML-form for assets parameters organized in tabs for editing mode
     * @param {object} req Request object
     * @param {object} res Response object
     * @param {integer} assetGroup if of asset group of asset
     * @param {integer} assetId  if of asset in core database
     * @param {integer} isGear 1 if its an alarmable asset, 0 if not
     * @param {string} gearname name of gear in alarmings
     * @param {function} callback string if HTML form
     */
    getAssetsParametersFormHTMLForAssetGroupAndAsset(req, res, assetGroup, assetId, isGear, gearname, issi, callback) {

        var promiseListe = [];

        var uploads = new Uploads();
        uploads.getUploadsForObjectInTable(assetId, 'assets', (rowsUploads) => {
            req.app.logger.debug('MODELS-ASSETPARAMETERSHELPER | start rendering HTML SubForm for Editing for asset with id ' + assetId + ' and asset group with id ' + assetGroup);
            this.getAssetsParametersTabs(assetGroup, (resultTabs) => {
                var assetsgroups = new AssetsGroups(req.app.logger);
                assetsgroups.getAssetGroup(assetGroup, (rowAssetGroup) => {
                    console.log('... now get assets parameters');
                    this.getAssetsParametersForAssetGroup(assetGroup, (resultAssetParameters) => {
                        console.log('... got assets parameters');
                        var retValHTML = '';
                        if (resultAssetParameters === false) {
                            // Error reading parameters
                            req.app.logger.error('MODELS-ASSETPARAMETERSHELPER | Error reading asset parameters from database');
                            callback('ERROR');
                        } else {
                            console.log(resultAssetParameters);
                            console.log('... now get assets parameters values for asset');
                            this.getAssetParameterValuesForAssetId(assetId, (resultAssetParametersValues) => {
                                console.log('... got assets parameters values');
                                console.log(resultAssetParametersValues);
                                var showAlarming = 0;
                                if (rowAssetGroup.length > 0) {
                                    showAlarming = rowAssetGroup[0].isGearGroup;
                                }
                                this.renderTabsForAssetGroups(resultTabs, showAlarming, (tabsHTML) => {
                                    console.log('... got tabs HTML');
                                    retValHTML = tabsHTML;
                                    retValHTML = retValHTML + '<div class="tab-content" id="nav-tabContent">';
                                    resultTabs.forEach((tab, index) => {
                                        var activeTab = '';
                                        if (index < 1) {
                                            activeTab = 'active';
                                        }

                                        promiseListe.push(new Promise((resolve, reject) => {
                                            resolve('<div class="tab-pane fade show ' + activeTab + '" id="nav-' + tab.tabalias + '" role="tabpanel" aria-labelledby="nav-' + tab.tabalias + '-tab"><div class="m-3">')
                                        }));

                                        // Now iterate through parameters in tab
                                        var tabId = 0;
                                        resultAssetParameters.forEach((assetParameter, index) => {
                                            var theParameter = null;
                                            if ((resultAssetParametersValues != null) && (resultAssetParametersValues != false)) {
                                                resultAssetParametersValues.forEach(theValue => {
                                                    if (theValue.assetsParametersId === assetParameter.id) {
                                                        theParameter = theValue;
                                                    }
                                                })
                                            } else {
                                                theParameter = null;
                                            }
                                            
                                            if (assetParameter.assetsParametersTab === tab.id) {
                                                // AssetParameter belongs to currently rendered Tab
                                                var mandantory = '';
                                                if (assetParameter.mandantory === 1) { mandantory = ' required'; }
                                                switch (assetParameter.type) {
                                                    case 'string':
                                                        var value = '';
                                                        if (theParameter !== null) {
                                                            value = theParameter.valueStr;
                                                        }

                                                        promiseListe.push(new Promise((resolve, reject) => {
                                                            resolve(
                                                                '<div class="form-group">' +
                                                                '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                                '<input type="text" class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" value="' + value + '" ' + mandantory + '>' +
                                                                '</div>')
                                                        }));
                                                        break;
                                                    case 'multiline':
                                                        var value = '';
                                                        if (theParameter !== null) {
                                                            value = theParameter.valueStr;
                                                        }

                                                        promiseListe.push(new Promise((resolve, reject) => {
                                                            resolve(
                                                                '<div class="form-group">' +
                                                                '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                                '<textarea class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" ' + mandantory + ' rows="10" cols="30">' + value + '</textarea>' +
                                                                '</div>')
                                                        }));
                                                        break;
                                                    case 'select':
                                                        var dbRowsOptions = this.getSelectableOptionsForParameterAsync(assetParameter.id);

                                                        var value = '';
                                                        if (theParameter !== null) {
                                                            value = theParameter.valueSelect;
                                                        }
                                                        promiseListe.push(dbRowsOptions.then((rowsOptions) => {
                                                            var html = '';
                                                            html = html + '<div class="form-group">' +
                                                                '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                                '<select class="form-control text-primary" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" ' + mandantory + '>';

                                                            if (rowsOptions != null) {
                                                                rowsOptions.forEach(option => {
                                                                    var selectedState = '';
                                                                    console.log('option.id: ' + option.id + ' | valueSelect: ' + assetParameter.valueSelect);
                                                                    if (option.id === value) {
                                                                        selectedState = ' selected ';
                                                                    }
                                                                    html = html + '<option value="' + option.id + '" ' + selectedState + '>' + option.label + '</option>';
                                                                })
                                                            } else {
                                                                html = html + '<option disabled>(E22) - Fehler beim laden der Optionen</option>';
                                                            }

                                                            html = html + '</select></div>';
                                                            return new Promise((resolve, reject) => {
                                                                resolve(html)
                                                            })
                                                        }));


                                                        break;
                                                    case 'integer':
                                                        var value = '';
                                                        if (theParameter !== null) {
                                                            value = theParameter.valueInt;
                                                        }
                                                        promiseListe.push(new Promise((resolve, reject) => {
                                                            resolve(
                                                                '<div class="form-group">' +
                                                                '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                                '<input type="text" class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" value="' + value + '" onkeypress="return isNumberKey(event);" ' + mandantory + '>' +
                                                                '</div>'
                                                            );

                                                        }));
                                                        break;
                                                    case 'date':
                                                        var value = '';
                                                        if (theParameter !== null) {
                                                            value = theParameter.valueDate;
                                                        }
                                                        promiseListe.push(new Promise((resolve, reject) => {
                                                            resolve(
                                                                '<div class="form-group">' +
                                                                '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                                '<input type="date" class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" value="' + value + '" ' + mandantory + '>' +
                                                                '</div>'
                                                            );
                                                        }));
                                                        break;
                                                    case 'datetime':
                                                        var value = '';
                                                        if (theParameter !== null) {
                                                            value = theParameter.valueDate;
                                                        }
                                                        promiseListe.push(new Promise((resolve, reject) => {
                                                            resolve(
                                                                '<div class="form-group">' +
                                                                '<label for="ap_' + assetParameter.alias + '">' + assetParameter.label + '</label>' +
                                                                '<input type="datetime-local" class="form-control" id="ap_' + assetParameter.alias + '" name="ap_' + assetParameter.alias + '" value="' + value + '" ' + mandantory + '>' +
                                                                '</div>'
                                                            );
                                                        }));
                                                        break;
                                                    case 'attachments':
                                                        var value = '';
                                                        promiseListe.push(new Promise((resolve, reject) => {
                                                            let body = '';
                                                            rowsUploads.forEach((attachment) => {
                                                                body = body + `<tr>
                                                                                    <td>${attachment.mimetype}</td>
                                                                                    <td><a href="/uploads//getfile/${attachment.id}" target="blank">${attachment.filename}</a></td>
                                                                                    <td>${attachment.title}</td>
                                                                                    <td><a href="/uploads/remove/${attachment.id}" rel="noopener" class="btn btn-sm btn-danger">löschen</a></td>
                                                                                   </tr>`;
                                                            })
                                                            resolve(
                                                                `<div class="form-group">
                                                                        <table id="attachmentsTable" class="table table-striped table-bordered">
                                                                            <thead>
                                                                                <tr>
                                                                                    <th scope="col">Typ</th>
                                                                                    <th scope="col">Dateiname</th>
                                                                                    <th scope="col">Beschreibung</th>
                                                                                    <th scope="col">Aktion</th>
                                                                                </tr>
                                                                            </thead>
                                                                            <tbody>
                                                                                ${body}
                                                                            </tbody>
                                                                        </table>
                                                                    </div>
                                                                    <button type="button" class="btn btn-success" data-toggle="modal" data-target="#addAttachmentModal">+</button>`
                                                            )

                                                            resolve(
                                                                `<div class="form-group">
                                                                        <div class="custom-file">
                                                                            <input type="file" class="custom-file-input" id="ap_${assetParameter.alias}" name="ap_${assetParameter.alias}">
                                                                            <label id="custom-file-label" class="custom-file-label" for="ap_${assetParameter.alias}">Datei wählen</label>
                                                                        </div>
                                                                    </div>
                                                                    <script type="text/javascript">
                                                                        $(document).ready(function(){ 
                                                                            $("#ap_${assetParameter.alias}").on("change",function(){
                                                                                //get the file name 
                                                                                var fileName = $(this).val(); 
                                                                                //replace the "Choose a file" label
                                                                                $("#custom-file-label").html(fileName);
                                                                            })
                                                                        })
                                                                    </script>`
                                                            )
                                                        }))
                                                        break;
                                                    default:
                                                        break;
                                                }
                                            }
                                        })
                                        promiseListe.push(new Promise((resolve, reject) => {
                                            resolve(
                                                '</div></div>'
                                            );
                                        }))
                                    })
                                    if (showAlarming === 1) {
                                        var valIsGear = '';
                                        if (isGear === 1) {
                                            valIsGear = ' checked';
                                        }
                                        if (issi === null) {
                                            issi = '';
                                        }
                                        var tempHtml = '';
                                        tempHtml = tempHtml + '<div class="tab-pane fade" id="nav-alarming" role="tabpanel" aria-labelledby="nav-alarming-tab"><div class="m-3">';
                                        tempHtml = tempHtml +
                                            '<div class="form-group">' +
                                            '<div class="custom-control custom-switch ml-3 mt-3"> ' +
                                            '<input type="checkbox" class="custom-control-input" id="isGear" name="isGear" ' + valIsGear + '>' +
                                            '<label class="custom-control-label" for="isGear">alarmierbares Einsatzmittel</label>' +
                                            '</div>' +
                                            '</div>' +
                                            '<div class="form-group">' +
                                            '<label for="gername">Bezeichner in Alarmquellen</label>' +
                                            '<input type="text" class="form-control" id="gearname" name="gearname" value="' + gearname + '">' +
                                            '</div>' +
                                            '<div class="form-group">' +
                                            '<label for="issi">ISSI für Statusausleitung</label>' +
                                            '<input type="text" class="form-control" id="issi" name="issi" value="' + issi + '">' +
                                            '</div>';
                                        tempHtml = tempHtml + '</div></div>';
                                        tempHtml = tempHtml + '</div></div>';
                                        promiseListe.push(new Promise((resolve, reject) => {
                                            resolve(
                                                tempHtml
                                            )
                                        }))
                                    }
                                    req.app.logger.debug('MODELS-ASSETPARAMETERSHELPER | Successfully read ' + resultAssetParameters.length + ' asset parameters from database');
                                });
                                Promise.allSettled(promiseListe).then((result) => {
                                    result.forEach((resultHtml) => {
                                        if (resultHtml.status == 'fulfilled') {
                                            retValHTML += resultHtml.value
                                        }
                                        else {
                                            console.log("Error with promise: ", resultHtml.reason);
                                        }

                                    })
                                    callback(retValHTML);
                                })
                            })
                        }
                    })
                })
            })
        })


    }

    /**
   * 
   * @param {object} reqBody form body containing the form fields data (usually req.body)
   * @param {integer} tId id of assets parameter tab to be updated
   * @param {function} callback success (true or false)
   */
    editAssetsParameterTab(reqBody, callback) {
        let tabId = reqBody.tabId;
        let tabtitle = reqBody.tabtitle;
        let assetsGroup = reqBody.assetsGroup;

        let query = "UPDATE `assets_parameters_tabs` SET `tabtitle` = ?, assetsGroup = ? WHERE id = ?";
        global.coreDb.run(query,
            [
                tabtitle,
                assetsGroup,
                tabId
            ],
            function (err) {
                if (err) {
                    console.log('ERROR Updating assets parameter tab enty to core database: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Adds a new asset parameter tab to core database
     * @param {object} reqBody form body containing the form fields data (usually req.body)
     * @param {function} callback success (true or false)
     */
    addAssetsParametersTab(reqBody, callback) {
        let tabtitle = reqBody.tabtitle;
        let assetsGroup = reqBody.assetsGroup;
        let tabalias = reqBody.tabalias;

        let query = "INSERT INTO `assets_parameters_tabs` (`tabtitle`, `assetsGroup`, `tabalias`, `order`, `state`) VALUES (?, ?, ?, ?, ?)";
        global.coreDb.run(query,
            [
                tabtitle,
                assetsGroup,
                tabalias,
                0,
                1
            ],
            function (err) {
                if (err) {
                    console.log('ERROR Insertig new assets parameter tab enty to core database: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Removes a asset parameter tab to core database if no parameters reference to the tab anymore
     * @param {integer} tabId form body containing the form fields data (usually req.body)
     * @param {function} callback success (true or false)
     */
    removeAssetsParametersTab(tabId, callback) {
        // check if there are parameters referencing to the tab to be deleted
        let query = "SELECT * FROM `assets_parameters` WHERE assetsParametersTab = ?";
        global.coreDb.all(query, [
            tabId
        ], (err, rows) => {
            if (err) {
                console.log('removeAssetsParametersTab: ERROR checking if parameters reference to tab id in core database: ' + err.message);
                return false;
            } else {
                if (rows.length > 0) {
                    return callback(false);
                } else {
                    // there seem to be no referencing parameters
                    // delete tab now
                    let query = "DELETE FROM `assets_parameters_tabs` WHERE id = ?";
                    global.coreDb.run(query,
                        [
                            tabId
                        ],
                        function (err) {
                            if (err) {
                                console.log('ERROR deleting assets parameter tab with id ' + tabId + ' from core database: ' + err.message);
                                callback(false);
                            } else {
                                callback(true);
                            }
                        });
                }
            }
        });
    }

    /**
     * Adds a new asset parameter to core database
     * @param {object} reqBody req.body coming from form
     * @param {function} callback success (true / false), rowId
     */
    addAssetParameter(reqBody, callback) {
        let label = reqBody.label;
        let assetsGroup = reqBody.assetsGroup;
        let assetsTab = reqBody.assetsTab;
        let type = reqBody.type;
        let mandantory = reqBody.mandantory;
        let alias = reqBody.alias;

        let query = `INSERT INTO assets_parameters 
                        (assetsGroup, label, alias, type, mandantory, assetsParametersTab, state) 
                    VALUES 
                        (?, ?, ?, ?, ?, ?, ?)`;
        global.coreDb.run(query,
            [
                assetsGroup,
                label,
                alias,
                type,
                mandantory,
                assetsTab,
                1
            ],
            function (err) {
                if (err) {
                    console.log('ERROR storing new assets parameter to core database: ' + err.message);
                    callback(false, null);
                } else {
                    var insertId = this.lastID;
                    callback(true, insertId);
                }
            });
    }

    /**
     * Updates an existing asset parameter at core database
     * @param {object} reqBody request body containg form data
     * @param {function} callback success (true / fals)
     */
    editAssetParameter(reqBody, callback) {
        let paramId = reqBody.pId;
        let label = reqBody.label;
        let assetsTab = reqBody.assetsTab;
        let mandantory = reqBody.mandantory;

        let query = `UPDATE assets_parameters SET
                        label = ?,  
                        mandantory = ?, 
                        assetsParametersTab = ? 
                    WHERE 
                        id = ?`;

        global.coreDb.run(query,
            [
                label,
                mandantory,
                assetsTab,
                paramId
            ],
            function (err) {
                if (err) {
                    console.log('ERROR updating assets parameter at core database: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });

    }
}
// export the class
module.exports = assetsparametershelper;
