/**
 * 
 * Authentication related methods
 * 
 */
const md5 = require('md5');
const { v4: uuidv4 } = require('uuid');


class auth {

    constructor(logger) {
        // always initialize all instance properties
        this._logger = logger;
    }

    /**
     * Returns access groups (users_groups) for given user
     * @param {integer} uId userId of user
     * @param {*} callback rows or null
     */
    getUsersGroupsForUserWithId(uId, callback) {
        let query = "SELECT " +
            "xt_users_users_groups.*, " +
            "users_groups.isSystemGroup, " +
            "users_groups.grantnamespace, " +
            "users_groups.title AS groupTitle " +
            "FROM xt_users_users_groups " +
            "INNER JOIN users_groups ON users_groups.id = xt_users_users_groups.groupId " +
            "WHERE xt_users_users_groups.userId = ?";
        global.coreDb.all(query, [
            uId
        ], (err, rows) => {
            if (err) {
                this._logger.error('AUTH-MODEL | Error loading users group for user from core data: ' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * 
     * @param {string} username username
     * @param {string} passwordHashMD5 hash of password
     * @param {function} callback success (true/false), if true row of user from core database
     */
    authenticateUserMD5(username, passwordHashMD5, callback) {
        let query = "SELECT `users`.*, " +
            "`organizations`.`name` AS organizationname, " +
            "`basedata`.`image` AS image, " +
            "`basedata`.`firstname` AS firstname, " +
            "`basedata`.`id` AS basedataId, " +
            "`basedata`.`uuid` AS basedataUUID, " + 
            "`basedata`.`lastname` AS lastname, " +
            "`organizations`.`id` AS organizationId " +
            "FROM `users` " +
            "LEFT JOIN `basedata` ON `users`.`basedataid` = `basedata`.`id` " +
            "LEFT JOIN `organizations` ON `basedata`.`organizationid` = `organizations`.`id` " +
            "WHERE `username` = ? COLLATE NOCASE";

        global.coreDb.all(query, [
            username
        ], (err, rows) => {
            if (err) {
                console.log('AUTH-MODEL | Error loading user for authentication from core data: ' + err.message);
                return callback(false, null);
            } else {
                if (rows.length > 0) {
                    // user found
                    var userRow = rows[0];
                    // Check if passwords match
                    if (passwordHashMD5 === userRow['password']) {
                        // Password typed in correct --> return user row
                        if (userRow['state'] === 1) {
                            // user is active
                            this.setLastLoginDate(userRow.id, (successLastLogin) => {
                                if (successLastLogin) {
                                    return callback(true, userRow);
                                } else {
                                    return callback(false, userRow);
                                }
                            })
                        } else {
                            return callback(false, userRow);
                        }
                        
                    } else if (passwordHashMD5 === userRow['startPasswordHash']) {
                        // User sent in a startPassword --> inform App that user is enforced to change password now
                        if (userRow['state'] === 1) {
                            // user is active
                            this.setLastLoginDate(userRow.id, (successLastLogin) => {
                                if (successLastLogin) {
                                    return callback(true, userRow);
                                } else {
                                    return callback(false, userRow);
                                }
                            })
                        } else {
                            return callback(false, userRow);
                        }
                    } else {
                        
                        return callback(false, null)
                    }
                } else {
                    // No User for given username found
                    return callback(false, null);
                }

            }
        });
    }

    /**
     * Retrieves all organizations that are sub-organizations of the users primary org ID. This will 
     * recursivly resolve all organizations that are sub-organizations (or sub-organizations of the 
     * sub-organizations) of the users primary organization.
     * @param {Number} usersPrimaryOrgId - the id of the users primary organization
     * @param {Function} callback - a callback with 2 parameters: success (boolean) and the rows containing
     * the organization IDs. Each row has an `id` field that can be extracted.
     */
    getAccessibleOrganizations(usersPrimaryOrgId, callback) {
        // recursively fetch all organizations that are sub-organizations of the current users organization
        const query =   "WITH RECURSIVE " +
                            "sub_org(org_id) AS (" +
                                "VALUES(?) " +
                                "UNION " +
                                "SELECT id FROM `organizations`, sub_org " +
                                "WHERE parent = sub_org.org_id" +
                            ") " +
                        "SELECT id FROM `organizations` " +
                        "WHERE `organizations`.`id` IN sub_org"
        global.coreDb.all(query, [usersPrimaryOrgId], (err, rows) => {
            callback(err == null, rows)
        })
    }

    /**
     * 
     * @param {string} username username caseinsensitive as string
     * @param {string} password password casesensitive as string
     * @param {function} callback success(true or false), userrowsuccess (row or null), passwordchange enforced (true, false)
     */
    authenticateUser(username, password, callback) {
        let query = "SELECT `users`.*, " +
            "`organizations`.`name` AS organizationname, " +
            "`organizations`.`id` AS organizationid, " +
            "`basedata`.`image_data` AS image_data, " +
            "`basedata`.`firstname` AS firstname, " +
            "`basedata`.`lastname` AS lastname " +
            "FROM `users` " +
            "LEFT JOIN `basedata` ON `users`.`basedataid` = `basedata`.`id` " +
            "LEFT JOIN `organizations` ON `basedata`.`organizationid` = `organizations`.`id` " +
            "WHERE `username` = ? COLLATE NOCASE";

        global.coreDb.all(query, [
            username
        ], (err, rows) => {
            if (err) {
                this._logger.error('AUTH-MODEL | Error loading user for authentication from core data: ' + err.message);
                return callback(false, null, false);
            } else {
                if (rows.length > 0) {
                    // user found
                    var userRow = rows[0];
                    // Check if passwords match
                    var passwordHashMD5 = md5(password);
                    if (passwordHashMD5 === userRow['password']) {
                        this.setLastLoginDate(userRow.id, (successLastLogin) => {
                            if (successLastLogin) {
                                return callback(true, userRow, false);
                            } else {
                                return callback(false, userRow, false);
                            }

                        })
                    } else if (password === userRow['startPassword']) {
                        // User sent his startPassword
                        this.setLastLoginDate(userRow.id, (successLastLogin) => {
                            if (successLastLogin) {
                                return callback(true, userRow, true);
                            } else {
                                return callback(false, userRow, false);
                            }
                        })
                    } else {
                        // Wrong password typed in
                        return callback(false, null, false)
                    }
                } else {
                    // No User for given username found
                    return callback(false, null, false);
                }

            }
        });
    }

    /**
     * Sets last login timestamp in user record in core database to "now" in local timezone
     * @param {integer} uId 
     * @param {function} callback success, true or false
     */
    setLastLoginDate(uId, callback) {
        let query = "UPDATE users SET lastlogin = DATETIME('now', 'localtime') WHERE id = ?;"
        global.coreDb.all(query, [
            uId
        ], (err, rows) => {
            if (err) {
                this._logger.error('AUTH-MODEL | Error setting last login date in core database for user: ' + err.message);
                return callback(false);
            } else {
                return callback(true);
            }
        });
    }

    /**
     * Retrieves the user.id for the given email address by resolving the basedata entry.
     * @param {String} emailAddress - the email address to search for
     * @param {Function} callback - a callback with two parameters: (1) boolean indicating the 
     * success of the request and (2) a JSON object either containing an error: int corresponding
     * to HTTP response codes or the `userId` and `firstName` under the given key with error: null.
     */
    getUserInfoByEmail(emailAddress, callback) {
        const baseDataQuery = "SELECT id, firstname FROM `basedata` WHERE email = ?"
        global.coreDb.all(baseDataQuery, [emailAddress], (err, rows) => {
            if (err) {
                this._logger.error(`Error fetching basedata ID for email address: '${emailAddress}': ${err.message}`)
                callback(false, { error: 500 })
                return
            }
            if (rows.length === 0) {
                // email not found
                callback(false, { error: 404 })
                return
            }
            if (rows.length > 1) {
                this._logger.error(`Multiple users found for the same email address: '${emailAddress}': ${rows.map(row => row.id).join(", ")}`)
                callback(false, { error: 400 })
                return
            }

            // get user for basedata id
            const firstName = rows[0].firstname
            const basedataId = rows[0].id
            const userQuery = "SELECT id FROM `users` WHERE basedataid = ?"
            global.coreDb.all(userQuery, [basedataId], (err, rows) => {
                if (err) {
                    this._logger.error(`Error fetching user ID for basedata ID: '${basedataId}': ${err.message}`)
                    callback(false, { error: 500 })
                    return
                }
                if (rows.length === 0) {
                    // there is no user for this basedata entry
                    callback(false, { error: 404 })
                    return
                }
                // return the userId
                const userId = rows[0].id
                callback(true, {  userId, firstName })
            })
        })
    }

    setPasswordRecoveryToken(userId, ip, callback) {
        const resetToken = uuidv4()
        const md5ResetToken = md5(resetToken)  // don't store the reset token in plain in the database (treat it as password)
        
        const updateStatement = "UPDATE users SET recoverytoken = ? WHERE id = ?"
        global.coreDb.run(updateStatement, [md5ResetToken, userId], (err) => {
            if (err) {
                // technical error updating the database
                this._logger.error(`AUTH-MODEL | Failed to set recovery token for user '${userId}': ${err.message}`)
                callback(false, null)
                return
            }
            this._logger.info(`'AUTH-MODEL | '${ip}' requested password reset for user ${userId}`)
            // return the unencrypted reset token for the email
            callback(true, resetToken)
        })
    }

    /**
     * Changes a users password
     * @param {integer} basedataId basedata id of user the password should be changed for
     * @param {string} oldPassword the old password
     * @param {string} newPassword the new password
     * @param {function} callback success (true or false)
     */
    changePassword(username, oldPassword, newPassword, callback) {
        let query = "SELECT * FROM users WHERE username = ?;"
        global.coreDb.all(query, [
            username
        ], (err, rows) => {
            if (err) {
                this._logger.error('AUTH-MODEL | Error getting user for password change request: ' + err.message);
                return callback(false);
            } else {
                if (rows.length > 0) {
                    // User found. Check if passwords match
                    var passwordHashMD5 = md5(oldPassword);
                    if (passwordHashMD5 === rows[0]['password']) {
                        // given old password matches existing one in database. now set the new password in database;
                        var newPasswordMD5 = md5(newPassword);
                        let query = "UPDATE users SET password = ? WHERE username = ?";
                        global.coreDb.all(query, [
                            newPasswordMD5,
                            username
                        ], (err, rows) => {
                            if (err) {
                                this._logger.error('AUTH-MODEL | Error setting new password in core database for user: ' + err.message);
                                return callback(false);
                            } else {
                                return callback(true);
                            }
                        });
                    } else {
                        // Wrong password typed in
                        return callback(false)
                    }
                } else {
                    return callback(false);
                }
            }
        });
    }

    /**
     * Checks if session is valid for redirecting user from login page to home dashboard
     * @param {object} session session object
     * @param {*} callback valid (true) or not valid (false)
     */
    checkForValidSession(session, callback) {
        var valid = false;
        if (typeof session != 'undefined') {
            if (session != null) {
                if (typeof session.loggedin != 'undefined') {
                    if (session.loggedin === true) {
                        valid = true;
                    } else {
                        return callback(false);
                    }
                } else {
                    return callback(false);
                }
                if (typeof session.username != 'undefined') {
                    if (session.username.length > 0) {
                        valid = true;
                    } else {
                        return callback(false);
                    }
                } else {
                    return callback(false);
                }
                if (typeof session.userid != 'undefined') {
                    if (session.userid > 0) {
                        valid = true;
                    } else {
                        return callback(false);
                    }
                } else {
                    return callback(false);
                }
                if (typeof session.basedataId != 'undefined') {
                    if (session.basedataId > 0) {
                        valid = true;
                    } else {
                        return callback(false);
                    }
                } else {
                    return callback(false);
                }
                if (typeof session.organization != 'undefined') {
                    if (session.organization.length > 0) {
                        valid = true;
                    } else {
                        return callback(false);
                    }
                } else {
                    return callback(false);
                }
                if (typeof session.name != 'undefined') {
                    if (session.name.length > 0) {
                        valid = true;
                    } else {
                        return callback(false);
                    }
                } else {
                    return callback(false);
                }

                return callback(valid);
            } else {
                return callback(false);
            }
        } else {
            return callback(false);
        }
    }

    /**
     * Checks if user has access to namespace
     * @param {integer} userId user id to check permission for
     * @param {string} grantNamespace namespace to check permissions for
     */
    checkPermission(userId, grantNamespace) {
        return new Promise((resolve, reject) => {
            const userSQL = `SELECT * FROM users WHERE id = ?`;
            global.coreDb.all(userSQL, [
                userId
            ], (err, userRows) => {
                if (err) {
                    reject(err);
                } else {
                    if (userRows == null || userRows.length != 1) {
                        reject("No or multiple users found");
                        return;
                    }
                    const user = userRows[0];

                    if (grantNamespace == null) {
                        // no special permission required for this page
                        resolve(true);
                    } else {
                        if (user.superuser === 1) {
                            // super user has access to everything
                            resolve(true);
                        }                

                        this.getUsersGroupsForUserWithId(userId, (accessRows) => {
                            if (accessRows == null) {
                                // user has no perms
                                reject("Missing permissions");
                                return
                            }

                            if (accessRows.map(row => row.grantnamespace).includes(grantNamespace)) {
                                // user has the required permission
                                resolve(true);
                            } else {
                                // user doesn't have the required permission
                                reject("Missing permissions");
                            }
                        })
                    }
                }
            });
        })
    }
}

module.exports = auth;
