const EventEmitter = require('events');
const glob = require('glob');
const fs = require('fs');
const fs_extra = require('fs-extra');
const path = require('path');
const _7zip = require("7zip-min");
const moment = require('moment');

/**
 * Class offers functionality to manage backups
 * @class
 */
class BackupManager extends EventEmitter {

    /**
     * @constructor
     */
    constructor() {
        super();

        //defne backuplocation
        this.backupfolder = path.join(global.appRoot, `backup`);
        this.tmp_backup = "tmp_backup";
        this.backup_extension = ".7z";
        // check if backup directory exists. If not -> create it
        if(!fs.existsSync(this.backupfolder)){
            fs.mkdirSync(this.backupfolder);
        }

        //define backup-content
        this.content_folders = [
            path.join(global.appRoot, 'store'),
            path.join(global.appRoot, 'public', 'assets', 'img'),
            path.join(global.appRoot, 'plugins','inbound', 'katsys', 'uploads'),
            path.join(global.appRoot, 'plugins','inbound', 'katsys', 'pem')
        ];
    }

    /**
     * creates a new Backup
     * @param {function} callback 
     */
    Create(password, callback) {
        var now = moment();
        var zip_name = `alarmiatorserver_backup_` + now.format("YYYYMMDDHHmmss") + this.backup_extension;    
        var zip_file = path.join(this.backupfolder, zip_name);
        var pwd="secret";

        var tmp_backupfolder = path.join(this.backupfolder, this.tmp_backup);
        if(fs.existsSync(tmp_backupfolder)){
            fs.rmdirSync(tmp_backupfolder, { recursive: true });
        }
        fs.mkdirSync(tmp_backupfolder);

        this.content_folders.forEach((folder) =>{
            if(fs.existsSync(folder)){
                var foldername = path.basename(folder);
                var dst = path.join(tmp_backupfolder, foldername);
                console.log('CORE | MODELS | BACKUP | copy ' + folder + ' to ' + tmp_backupfolder );   
                fs_extra.copySync(folder, dst);
            }
        });

        var command = []
        command.push('a');
        command.push(zip_file);
        if(password){
            command.push('-p'+password);
            command.push('-mhe=on');
        }
        command.push( tmp_backupfolder);

        _7zip.cmd(command, err => {
            if(err){
                console.log('CORE | MODELS | BACKUP | error on adding ' + tmp_backupfolder + ' to ' + zip_name + '!');
                callback(err);
            }
            else{
                console.log('CORE | MODELS | BACKUP | ' + tmp_backupfolder + ' sucessfully added to ' + zip_name );
                fs.rmdirSync(tmp_backupfolder, { recursive: true });
                console.log('CORE | MODELS | BACKUP | ' + tmp_backupfolder + ' removed. ');
                return callback('reloadPage');
            }
        });
        
        return callback(null);
    }

     /**
     * Delete a Backup
     * @param {function} callback 
     */
    Delete(filename, callback) {

        var backupfile = path.join(this.backupfolder, filename)

        if(fs.existsSync(backupfile)){
            fs.unlink(backupfile, (err) => {
                if(err){
                    console.log('CORE | MODELS | BACKUP | error during delete ' + filename + '!');
                    return callback(err);
                }
                else{
                    console.log('CORE | MODELS | BACKUP | ' + filename + ' deleted.');
                    return callback(null);
                }
            });
        }
        else
        {
            return callback(new Error(filename + ' not found!'));
        }
    }

    /**
     * restore a backupfile
     * @param {string} filename 
     * @param {*} callback 
     */
    Restore(filename, password, callback) {

        var backupfile = path.join(this.backupfolder, filename)

        if(fs.existsSync(backupfile)){
            
            // implement restore here
            // 1. unzip

            var restorefolder = path.join(this.backupfolder, 'restore');
            if(fs.existsSync(restorefolder))
            {
                fs.rmdirSync(restorefolder, {recursive : true});    
            }
            fs.mkdirSync(restorefolder);
            console.log('CORE | MODELS | BACKUP | create ' + restorefolder);

            var command = ['x', backupfile, '-o'+restorefolder];
            if(password){
                command.push('-p'+password);
            }
    
            _7zip.cmd(command, (err, result) => {
                if(result){
                    console.log('CORE | MODELS | BACKUP | extract result: ' + result );
                }

                if(err){
                    console.log('CORE | MODELS | BACKUP | error on extracting ' + backupfile + ' to ' + restorefolder + '!');
                    console.log('CORE | MODELS | BACKUP | error: ' + err);
                    fs.rmdirSync(restorefolder, { recursive: true });
                    console.log('CORE | MODELS | BACKUP | ' + restorefolder + ' removed. ');
                    callback(err);
                }
                else{
                    console.log('CORE | MODELS | BACKUP | ' + backupfile + ' sucessfully extracted to ' + restorefolder );
                    // 2 copy unzipped folders to location under global.appRoot
                    var restore_backup_folder = path.join(restorefolder, this.tmp_backup);

                    this.content_folders.forEach((dst_folder) => {
                        var foldername = path.basename(dst_folder)

                        var src_folder = path.join(restore_backup_folder, foldername);
                        if(fs.existsSync(src_folder)){
                            if(foldername === 'store')
                            {
                                dst_folder = path.join(dst_folder, 'restore');
                            }
                            console.log('CORE | MODELS | BACKUP | copy ' + src_folder + ' to ' + dst_folder );
                            fs_extra.copySync(src_folder, dst_folder);
                        }
                    });

                    fs.rmdirSync(restorefolder, { recursive: true });
                    console.log('CORE | MODELS | BACKUP | ' + restorefolder + ' removed. ');
                
                    return callback('finished');
                }
            });
        }
        else
        {
            return callback(new Error(filename + ' not found!'));
        }
    }

    /**
     * returns the complete path of a backupfile
     * @param {string} filename 
     * @param {function} callback null or path to backupfile
     */
    GetBackupfilePath(filename, callback) {
        
        var backupfile = path.join(this.backupfolder, filename)
            
        if (fs.existsSync(backupfile)) {
            return callback(backupfile)
        } else {
            console.log('CORE | MODELS | BACKUP | ' + backupfile + ' not found!');
            return callback(null);
        }
        
    }

    /**
     * returns list of backupfiles in a given folder
     * @param {string} folder folder to look for backupfiles
     * @param {function} callback list of files in folder as array
     */
    getListOfBackupfilesInFolder(callback) {

        var fullbackuppath = path.join(this.backupfolder, '*'+this.backup_extension);

        glob(fullbackuppath, null, (er, files) => {
            if (er) {
                console.log('CORE | MODELS | BACKUP | Error getting list of backupfiles from filesystem: ' + er.message);
                return callback(null);
            } else {
                return callback(files);
            }
        });
    }
}

module.exports = BackupManager;