/**
 * Model for accessing the core configuration
 * 
 * Database table: configuration
 * 
 */
class configurations {
    /**
     * @constructor
     * @param {object} logger logger to be used by class instance
     */
    constructor(logger) {
        // always initialize all instance properties
        this._logger = logger;
        this.getAllConfigurations((rows) => {
            this.serverconfig = rows;
        })
    }

    /**
     * Returns all configurations from core database table configuration
     * @param {function} callback rows or null
     */
    getAllConfigurations(callback) {
        let query = "SELECT * FROM configuration";
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns configuration value for given configuration keyword
     * @param {string} keyword 
     * @param {function} callback returns value for keyword as string
     */
    getValueForKeyword(keyword, callback) {
        let query = "SELECT * FROM `configuration` WHERE keyword = ?";
        global.coreDb.all(query, [
            keyword
        ], (err, rows) => {
            if (err) {
                callback(null);
            } else {
                if (rows.length > 0) {
                    var value = rows[0]['value'];
                    callback(value);
                } else {
                    callback(null);
                }

            }
        });
    }

    /**
     * checks if a keyword exists in configuration table in core database
     * @param {string} keyword keyword for configuration setting
     * @param {*} callback success (true/false), rows (rows or null)
     */
    doesKeywordExists(keyword, callback) {
        let query = "SELECT * FROM configuration WHERE keyword = ?";
        global.coreDb.all(query, [
            keyword
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                if (rows.length > 0) {
                    return callback(true, rows);
                } else {
                    return callback(false, null);
                }
            }
        });
    }

    /**
     * Adds a new keyword/value pair to configuration table in core database
     * @param {string} keyword keyword of pair
     * @param {*} value value of pair
     * @param {*} callback success (true/false)
     */
    addKeywordAndValue(keyword, value, callback) {
        let query = "INSERT INTO configuration (keyword, value) VALUES (?, ?)";

        global.coreDb.run(query, [
            keyword,
            value,
        ], function (err) {
            if (err) {
                console.log('MODELS-CONFIGURATION | Error storing new key value pair to database: ' + err.message);
                callback(false);
            } else {
                callback(true);
            }
        });
    }


    /**
     * Updates a value for given keyword in core database table configuration
     * @param {string} keyword keyword the value shall be updated for
     * @param {string} value value to set
     * @param {function} callback success (true / false)
     */
    updateValuesForKeyword(reqBody, callback) {
        var emitChangedValuesArray = [];
        this.getAllConfigurations((rowsExistingConfiguration) => {
            if (rowsExistingConfiguration != null) {
                // Cycle through existing configuration values from core database to check what changed, has been removed or has been added
                var keyFoundInReqBody = false;
                var valueForKeyHasChanged = true;
                rowsExistingConfiguration.forEach((existingConfig) => {
                    keyFoundInReqBody = false;
                    valueForKeyHasChanged = true;
                    if (reqBody.hasOwnProperty(existingConfig.keyword)) {
                        keyFoundInReqBody = true;
                        var value = reqBody[existingConfig.keyword];
                        if (value === existingConfig.value) {
                            valueForKeyHasChanged = false;
                        }
                    } else {
                        keyFoundInReqBody = false;
                    };

                    if (keyFoundInReqBody === true) {
                        var confElement = {
                            keyword: existingConfig.keyword,
                            oldValue: existingConfig.value,
                            newValue: reqBody[existingConfig.keyword],
                            changed: valueForKeyHasChanged
                        }
                        emitChangedValuesArray.push(confElement);                        
                    }

                   
                    //console.log(existingConfig.keyword + ' | keyFoundInReqBody: ' + keyFoundInReqBody + ' | valueForKeyHasChanged: ' + valueForKeyHasChanged);
                });
            }

            // NOW PERIST THE NEW CONFIGURATION SETTING
            var pairsCount = Object.keys(reqBody).length;
            if (pairsCount > 0) {
                var index = 1;
                global.coreDb.serialize(() => {
                    for (var key in reqBody) {
                        if (reqBody.hasOwnProperty(key)) {
                            var value = reqBody[key];

                            let query = "INSERT OR REPLACE INTO configuration (id, keyword, value) VALUES ( " +
                                "(SELECT id FROM configuration WHERE keyword = ?), ?, ?);";

                            global.coreDb.run(query, [
                                key,
                                key,
                                value
                            ], function (err) {
                                if (err) {
                                    console.log('MODELS-CONFIGURATION | Error updating key value pair to database: ' + err.message);
                                } else {
                                    this.updateGlobals(key, value);
                                    console.log('MODELS-CONFIGURATION | Successfully updated key / value in configuration');
                                }
                            }.bind(this));
                        };
                    };
                });
                if (emitChangedValuesArray.length > 0) {
                    // inform plugin manager about change at server config
                    global.plgManager.event_server_config_changed(emitChangedValuesArray);
                }
                callback(true);
            } else {
                callback(false);
            };
        })

    }



    /**
     * Updates global variables if configuration values change
     * @param {string} key 
     * @param {value} value 
     */
    updateGlobals(key, value) {

        if (key === 'instancename') {
            global.serverInstanceName = value;
            console.log('MODELS-CONFIGURATION | Successfully updated global.serverInstanceName');
        }

    }

    getExternalDomainName(callback) {
        this.getValueForKeyword("externalfqdn", callback)
    }

    getWebuiSecurityLevel(callback) {
        this.getValueForKeyword("webuiSecurityLevel", callback)
    }

    getWebuiBaseUrl(callback) {
        this.getExternalDomainName(domainName => {
            this.getWebuiSecurityLevel(httpOrHttps => {
                const protocolLookup = {
                    http: "http",
                    https: "https",
                    "http-https": "https",  // use https if available
                }
                const protocol = httpOrHttps != null ? protocolLookup[httpOrHttps] : "http"  // fallback for local dev
                const port = protocol === "http" ? 5000 : 8443
                const hostname = domainName != null ? domainName : "localhost"  // fallback to local dev
                callback(`${protocol}://${hostname}:${port}`)
            })
        })
    }

    /**
     * Extracts width and height configuration values for a given prefix. This is used for organizations and basedata image size limits.
     * @param {String} prefix - the config key prefix
     * @param {Object} defaults - a JSON object containing a default width and height
     * @returns {Promise} a promise that fulfills with the effective width and height
     */
    getImageSizeLimits(prefix, defaults) {
        return new Promise(resolve => {
            const limit = {}
            // fetch width and height from config
            this.getValueForKeyword(`${prefix}_width`, width => {
                limit.width = width == null ? defaults.width : parseInt(width, 10)
                this.getValueForKeyword(`${prefix}_height`, height => {
                    limit.height = height == null ? defaults.height : parseInt(height, 10)
                    // ensure width and height are valid numbers, otherwies use default
                    Object.keys(limit).forEach(key => {
                        if (isNaN(limit[key])) {
                            limit[key] = defaults[key]
                        }
                    })
                    resolve(limit)
                })
            })
        })
    }

    getBasedataImageSizeLimits(callback) {
        this.getImageSizeLimits("basedataimagesize", {
            width: 200,
            height: 200,
        }).then(limits => {
            callback(limits)
        })
    }

    getOrganizationImageSizeLimits(callback) {
        this.getImageSizeLimits("organizationimagesize", {
            width: 800,
            height: 800,
        }).then(limits => {
            callback(limits)
        })
    }
}

module.exports = configurations;
