const EventEmitter = require('events');
const Cron = require('cron');
const moment = require('moment');

class CoreCron extends EventEmitter {

    constructor(logger) {
        super();
        this.logger = logger;
        this.jobsArray = [];
    }

    log(message) {
        if (this.logger !== null) {
            this.logger.log('CORE | CRON MANAGER | ' + message);
        } 
        
        console.log('CORE | CRON MANAGER | ' + message);
        
    }

    /**
     * start the CronManager
     */
    startCronManager() {
        // do some initialisation here!
        this.emit('cronStarted');
    }

    /**
     * Adds a new CronJob to cron Scheduler
     * @param {string} name The name of the cronjob to add
     * @param {string} jobtype the type of the job. for example testalarm, backup, ... ( or null )
     * @param {string} cronDefinition Definition of Job like for cron (e.g. '* * * * *' for run once a minute)
     * @param {*} job_data the data the job should work with. (or null if no data for execution is needed)
     * @param {*} callback the callback-function to call at the end of the function
     */
    AddCronJob(name, jobtype, cronDefinition, job_data, dbId) {
        if(this.jobsArray.filter(j => j.name === name).length === 0){
            var job = {};
            job.name = name;
            job.type = jobtype;
            job.dbId = dbId;
            job.cronjob = new Cron.CronJob(
                cronDefinition,
                function(){ this.emit(job_data.event_name, job_data)},
                null,
                true,
                'Europe/Berlin',
                this
            );
            job.data = job_data;

            this.jobsArray.push(job);
            global.cron.log('Job with name ' + name + ' loaded');
            return true;
        } else {
            global.cron.log('a job with name ' + name + ' allready exists! Please use an other name!');
            return false;
        }       
    }

    /**
     * delete a cron job with given name
     * @param {string} name 
     */
    DeleteCronJob(name){
        var job = this.jobsArray.filter(j => j.name === name)[0];
        if(job){

            if(job.cronjob.running === true){
                job.cronjob.stop();
            }

            const IsSearchedElement = (j) => j.name === name;
            var index = this.jobsArray.findIndex(IsSearchedElement);
            global.cron.log('delete cronjob with index ' + index);
            
            this.jobsArray.splice(index, 1);
            global.cron.log('cron job with name ' + name + ' deleted');
            return true;
        }
        else{
            global.cron.log('job with name ' + name + ' not found!');
            return false;
        }
    }

    /**
     * start the cron job which has the given name
     * @param {string} name 
     */
    StartCronJob(name){
        var job = this.jobsArray.filter(j => j.name === name)[0];
        if(job){
            job.cronjob.start();
            global.cron.log('cron job with name ' + name + ' started');
            return true;
        }
        else{
            global.cron.log('job with name ' + name + ' not found!');
            return false;
        }
    }

    /**
     * stop the cron job which has the given name
     * @param {string} name 
     */
    StopCronJob(name){
        var job = this.jobsArray.filter(j => j.name === name)[0];
        if(job){
            job.cronjob.stop();
            global.cron.log('cron job with name ' + name + ' stopped');
            return true;
        }
        else{
            global.cron.log('job with name ' + name + ' not found!');
            return false;
        }
    }

    /**
     * get all Jobs from Jobsarray with specific type
     * @param {*} type 
     */
    GetCronjobOfType(type){
        var jobsoftype = this.jobsArray.filter(j => j.type === type);
        jobsoftype.forEach(j =>{
            var last = j.cronjob.lastDate();
            var next = j.cronjob.nextDates();

            if(last){
                j.last = moment(last).format("DD.MM.YYYY HH:mm:ss");
            } else {
                j.last = '';
            }

            if(j.cronjob.running){
                j.next = next.format("DD.MM.YYYY HH:mm:ss");
            } else {
                j.next = '';
            }

        });    
        return jobsoftype;
    }

    /**
     * get all Jobs from Jobsarray
     */
    GetAllCronjobs(){
        var jobsoftype = this.jobsArray;
        jobsoftype.forEach(j =>{
            var last = j.cronjob.lastDate();
            var next = j.cronjob.nextDates();

            if(last){
                j.last = moment(last).format("DD.MM.YYYY HH:mm:ss");
            } else {
                j.last = '';
            }

            if(j.cronjob.running){
                j.next = next.format("DD.MM.YYYY HH:mm:ss");
            } else {
                j.next = '';
            }
            
        });    
        return jobsoftype;
    }

    /**
     * Get job by type from coredb
     * @param {*} type 
     * @param {*} callback => function(null, err){}
     */
    GetCronJobsFromCoreDBbyType(type, callback){
        let query ="SELECT * FROM `cron_tasks` WHERE type = ?";
        global.coreDb.all(query, [
            type
        ], (err, rows) => {
            if (err) {
                return callback(null, err);
            } else {
                return callback(rows, null);
            }
        });
    }
    
    /**
     * gewt job by name from coredb
     * @param {*} name 
     * @param {*} callback => function(null, err){}
     */
    GetCronJobFromCoreDB(name, callback){
        let query ="SELECT * FROM `cron_tasks` WHERE name = ?";
        global.coreDb.all(query, [
            name
        ], (err, rows) => {
            if (err) {
                return callback(null, err);
            } else {
                return callback(rows[0], null);
            }
        });
    }

    /**
     * get job by id from coredb
     * @param {integer} id id of job 
     * @param {*} callback => function(null, err){}
     */
     GetCronJobWithIdFromCoreDB(id, callback){
        let query ="SELECT * FROM `cron_tasks` WHERE id = ?";
        global.coreDb.all(query, [
            id
        ], (err, rows) => {
            if (err) {
                return callback(null, err);
            } else {
                return callback(rows[0], null);
            }
        });
    }

    /**
     * update a key in the coredb for a entry with the given name
     * @param {*} name 
     * @param {*} key 
     * @param {*} value 
     * @param {*} callback 
     */
    UpdateCronJobInCoreDB(name, key, value, callback){
        let query = `UPDATE cron_tasks SET `+ key +` = ? WHERE name = ?`;
        global.coreDb.run(query, [
            value,
            name
        ],  (err) => {
            if (err) {
                var msg = 'ERROR at set ' + key + ' to ' + value + ' for cronjob ' + name + ' in coredatabase: ' + err.message;
                global.cron.log(msg);
                return callback(msg);
            } else {
                global.cron.log('set ' + key + ' to ' + value + ' for cronjob ' + name + ' in coredatabase');
                return callback(null);
            }
        });
    }

    /**
     * Stores a new cron job to core database
     * @param {object} reqBody form body
     * @param {*} callback success (true/false), null or ID of new object in core database
     */
    addCronJob(reqBody, callback) {
        let name = reqBody.name;
        let type = reqBody.crontype;
        let report = reqBody.report;
        let dayofweek = reqBody.dayofweek;
        let hour = reqBody.hour;
        let min = reqBody.min;

        // calculate CRON-Like schedule payload
        var cron_schedule = "";
        cron_schedule += '0';
        cron_schedule += " ";
        cron_schedule += min;
        cron_schedule += " ";
        cron_schedule += hour;
        cron_schedule += " ";
        cron_schedule += '*';
        cron_schedule += " ";
        cron_schedule += '*';
        cron_schedule += " ";
        cron_schedule += dayofweek;

        // calculate data depending on chosen details
        let data = {};
        if (type === 'reports') {
            data = {
                event: 'generateReport',
                reports : true,
                report : report
            }
        }

        let query = `INSERT INTO cron_tasks 
                        (
                            name, 
                            type,
                            running,
                            cronDefinition, 
                            data
                        ) VALUES (
                            ?,
                            ?,
                            ?,
                            ?,
                            ?
                        )`;
        global.coreDb.run(query, [
            name,
            type,
            0,
            cron_schedule,
            JSON.stringify(data)
            ], (err) => {
                if (err) {
                    var msg = 'ERROR Adding new cronjob to core database: ' + err.message;
                    global.cron.log(msg);
                    return callback(false, null);
                } else {
                    global.cron.log('Job with name ' + name + ' saved in coredatabase');
                    return callback(true, this.lastID);
                }
            });
    }

    /**
     * Edits a cron job at core database
     * @param {id} integer id of cron in core database to be edited
     * @param {object} reqBody form body
     * @param {*} callback success (true/false)
     */
     editCronJob(id, reqBody, callback) {
        let name = reqBody.name;
        let event_name = reqBody.event_name;
        let type = reqBody.crontype;
        let report = reqBody.report;
        let dayofweek = reqBody.dayofweek;
        let hour = reqBody.hour;
        let min = reqBody.min;

        // calculate CRON-Like schedule payload
        var cron_schedule = "";
        cron_schedule += '0';
        cron_schedule += " ";
        cron_schedule += min;
        cron_schedule += " ";
        cron_schedule += hour;
        cron_schedule += " ";
        cron_schedule += '*';
        cron_schedule += " ";
        cron_schedule += '*';
        cron_schedule += " ";
        cron_schedule += dayofweek;

        // calculate data depending on chosen details
        let data = {};
        if (type === 'reports') {
            data = {
                event: 'generateReport',
                reports : true,
                report : report
            }
        } else {
            data.event_name = event_name;
        }
        

        let query = `UPDATE cron_tasks SET 
                            name = ?, 
                            type = ?,
                            cronDefinition = ?, 
                            data = ? 
                    WHERE id = ?`;
        global.coreDb.run(query, [
            name,
            type,
            cron_schedule,
            JSON.stringify(data),
            id
            ], (err) => {
                if (err) {
                    var msg = 'ERROR editing cronjob at core database: ' + err.message;
                    global.cron.log(msg);
                    return callback(false);
                } else {
                    global.cron.log('Job with name ' + name + ' successfully updated in coredatabase');
                    return callback(true);
                }
            });
    }

    /**
     * save a cronjob at the coredb
     * @param {*} name 
     * @param {*} callback 
     */
    SaveCronJobToCoreDB(name, callback){
        var job = this.jobsArray.filter(j => j.name === name)[0];

        if(job){
            
            let query = "INSERT INTO `cron_tasks` (name, type, running, cronDefinition, data) VALUES (?, ?, ?, ?, ?)";
            global.coreDb.run(query, [
              job.name,
              job.type,
              job.cronjob.running,
              job.cronjob.cronTime.source,
              JSON.stringify(job.data)
            ], (err) => {
                if (err) {
                    var msg = 'ERROR Adding new cronjob to core database: ' + err.message;
                    global.cron.log(msg);
                    return callback(msg);
                } else {
                    global.cron.log('Job with name ' + name + ' saved in coredatabase');
                    return callback(null);
                }
            });
        } else {
            return callback('a job with name ' + name + ' not found!');
        }
    }

    /**
     * Remove a cronjob from the coreDB
     * @param {string} name the unique name of the cronejob 
     * @param {*} callback true at success, false at fail
     */
    RemoveCronJobFromCoreDB(name, callback){
        let query = 'DELETE FROM `cron_tasks` WHERE name = ?';
        global.coreDb.run(query,
            [
                name
            ],
            function (err) {
                if (err) {
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }
}

module.exports = CoreCron;