const EventEmitter = require('events');
const fs = require('fs');
const path = require('path');

/**
 * Core Object for uploads
 * @namespace core/uplaods
 * @class
 */
class uploads extends EventEmitter {
    /**
     * Returns an empty operation object
     * @constructor
     */
    constructor() {
        super();
    }


    /**
     * 
     * @param {string} title descriptive title of file
     * @param {string} filepath full path to file on file system
     * @param {string} filename filename as user uploaded the file
     * @param {string} mimetype mimetype of file
     * @param {string} module name of module this file belongs to
     * @param {integer} objectId id of target object in core database
     * @param {string} objectTable table-name of target object in core database
     * @param {integer} state state (1 == new, 2 == temporary, 3 == deleted)
     * @param {integer} createdBy id of basedata who created the upload
     * @param {function} callback success (true/false), id of new row in table
     */
    addUpload(title, filepath, filename, mimetype, module, objectId, objectTable, state, createdBy, uuid, callback) {
        let query = "INSERT INTO uploads (title, filepath, filename, mimetype, module, objectId, objectTable, state, createdBy, uuid) VALUES (?,?,?,?,?,?,?,?,?,?)";

        global.coreDb.run(query, [
            title,
            filepath,
            filename,
            mimetype,
            module,
            objectId,
            objectTable,
            state,
            createdBy,
            uuid
          ], function (err) {
            if (err) {
              console.log('MODULES-CORE-UPLOADS | Error writing new uplaod to database: ' + err.message);
              global.logger.error('MODULES-CORE-UPLOADS | Error writing new upload to database: ' + err.message);
              return callback(false, null);
            } else {
                var lastID = this.lastID;
                return callback(true, lastID);
            };
            }
          );
    }

    /**
     * Returns an upload-document from core database
     * @param {integer} uId id of uplaod document
     * @param {function} callback rows or null
     */
    getUpload(uId, callback) {
      let query = "SELECT * FROM uploads WHERE id = ?";
      global.coreDb.all(query, [
        uId
      ], (err, rows) => {
        if (err) {
          this._logger.error('MODELS-CORE-UPLOADS | Error loading upload object from core data: ' + err.message);
          return callback(null);
        } else {
          return callback(rows);
        }
      });
    }

    /**
     * Returns list of uploads for the given object in given core database table
     * @param {integer} objectId id of referencing parent object we want to look for existing upload documents for
     * @param {string} objectTable table the upload references for
     * @param {function} callback rows or null
     */
    getUploadsForObjectInTable(objectId, objectTable, callback) {
      let query = "SELECT * FROM uploads WHERE objectId = ? AND objectTable = ?";
      global.coreDb.all(query, [
        objectId,
        objectTable
      ], (err, rows) => {
        if (err) {
          this._logger.error('MODELS-CORE-UPLOADS | Error loading upload object from core data: ' + err.message);
          return callback(null);
        } else {
          return callback(rows);
        }
      });
    }

    /**
     * Returns filepath for given upload id
     * @param {integer} uId id of upload object
     * @param {function} callback string of filepath or null
     */
    getFilePathForId(uId, callback) {
      this.getUpload(uId, (rows) => {
        if (rows != null) {
          return callback(rows[0].filepath);
        } else {
          return callback(null);
        }
      })
    }

    /**
     * Links temporary uploaded files in uploads table now to their parent objectid
     * @param {string} uuid temporaty uuid set at the temp uploads to be processed
     * @param {*} objectId id ob target object the uploads shall be linked to now
     * @param {*} callback success (true/false)
     */
    linkUploadsWithUUIDToObjectId(uuid, objectId, callback) {
      let query = "UPDATE uploads SET objectId = ?, uuid = null WHERE uuid = ?";
      global.coreDb.run(query, [
        objectId,
        uuid
        ], function (err) {
          if (err) {
            console.log('MODULES-CORE-UPLOADS | Error linking temporary uplaoded files to objects at core database: ' + err.message);
            global.logger.error('MODULES-CORE-UPLOADS | Error linking temporary uplaoded files to objects at core database: ' + err.message);
            return callback(false);
          } else {
              return callback(true);
          };
        }
      );
    }

    /**
     * Removes an upload object from core database and storage
     * @param {integer} id id of upload object in database table uploads
     * @param {*} callback success (true/false)
     */
    removeUpload(id, callback) {
      let query = "SELECT * FROM uploads WHERE id = ?";
      global.coreDb.all(query, [
        id
      ], (err, rows) => {
        if (err) {
          this._logger.error('MODELS-CORE-UPLOADS | Error reading upload from core data: ' + err.message);
          return callback(null);
        } else {
          if (rows.length === 1) {
            try {
              var filepath = path.join(global.appRoot, rows[0].filepath);
              fs.unlinkSync(filepath);
              let query2 = "DELETE FROM uploads WHERE id = ?";
              global.coreDb.run(query2, [
                id
                ], function (err) {
                  if (err) {
                    console.log('MODULES-CORE-UPLOADS | Error deleting upload at core database: ' + err.message);
                    global.logger.error('MODULES-CORE-UPLOADS | Error deleting upload at core database: ' + err.message);
                    return callback(false);
                  } else {
                      return callback(true);
                  };
                }
              );
            } catch (err) {
              this._logger.error('MODELS-CORE-UPLOADS | Error removing upload file from storage: ' + err.message);
              console.log('MODELS-CORE-UPLOADS | Error removing upload file from storage: ' + err.message);
              return callback(false);
            }
          } else {
            return callback(true)
          }
        }
      });
    }

}

module.exports = uploads;

