/**
 * 
 *  Handles incoming alarm information and persists them 
 *  at the coredatabase.
 * 
 *  (c) Jens Dinstühler 2020
 * 
 *  Version 1.0.0
 * 
 * 
 */

const EventEmitter = require('events');
const Alarm = require('../alarm');
const FeedbackStore = require('./feedbackstore');
const Ruleengine = require('../ruleengine');
const Operations = require('../operationshelper');
const moment = require('moment');

class CoreAlarmHandler extends EventEmitter {
    constructor() {
        super();
    }

    /**
     * Checks if for the given operationnumber an operation entry exists in database
     * @param {string} operationnumber 
     * @param {function} callback 
     */
    checkOperationNumberExists(operationnumber, callback) {
        global.plgManager.logger.debug('CORE | HANDLERALARM | checking for existing operationnumber: ' + operationnumber);

        if (typeof operationnumber === 'string') {
            var checkSql = "SELECT * FROM operationsStore WHERE operationnumber = ?";
            global.coreDb.all(checkSql, [
                operationnumber
            ], (err, rows) => {
                if (err) {
                    global.plgManager.logger.error('CORE | HANDLERALARM | Error checking for existing operationnumber: ' + err.message);
                    return 0;
                } else {
                    if (rows.length > 0) {
                        global.plgManager.logger.debug('CORE | HANDLERALARM | found existing operation number: ' + rows[0].id);
                        return callback(rows[0].id);
                    } else {
                        global.plgManager.logger.debug('CORE | HANDLERALARM | not an existing operation number');
                        return callback(0);
                    }
                }
            });
        } else {
            // no valid data for operationnumber
            global.plgManager.logger.debug('CORE | HANDLERALARM | not a valid operationnumber (not a string)');
            return callback(0);
        }
    }

    /**
     * Saves new alarm Information into core database
     * @param {alarmInfo} alarmInfo 
     */
    persistNewAlarmInformation(alarmInfo) {
        var that = this;
        if ((typeof alarmInfo != 'undefined')) {
            // Check if operation exists (key is operationnumber)
            console.log('MODELS | CORE | HANDLERALARM | persistNewAlarmInformation | checkOperationNumberExists(' + alarmInfo.operationnumber.value + ')');
            this.checkOperationNumberExists(alarmInfo.operationnumber.value, (opId) => {
                // ZVEIS Calc
                var zveisCalc = '';
                if (typeof alarmInfo.zveisCalc.value !== 'undefined') {
                    try {
                        if (alarmInfo.zveisCalc.value.length > 0) {
                            zveisCalc = JSON.stringify(alarmInfo.zveisCalc.value);
                        }
                    } catch (err) {
                        // Error rendering JSON String
                    }
                }

                // ZVEIS Delta Calc
                var zveisDeltaCalc = null;
                if (typeof alarmInfo.zveisDeltaCalc !== 'undefined') {
                    if (typeof alarmInfo.zveisDeltaCalc.value !== 'undefined') {
                        try {
                            if (alarmInfo.zveisDeltaCalc.value.length > 0) {
                                zveisDeltaCalc = JSON.stringify(alarmInfo.zveisDeltaCalc.value);
                            }
                        } catch (err) {
                            // Error rendering JSON String
                        }
                    }
                }

                // Gear Data Calc
                var gearDataCalc = '';
                if (typeof alarmInfo.gearDataCalc !== 'undefined') {
                    if (typeof alarmInfo.gearDataCalc.value !== 'undefined') {
                        try {
                            if (alarmInfo.gearDataCalc.value.length > 0) {
                                gearDataCalc = JSON.stringify(alarmInfo.gearDataCalc.value);
                            }
                        } catch (err) {
                            // Error rendering JSON String
                        }
                    }
                }

                // GEAR Calc
                var gearCalc = '';
                if (typeof alarmInfo.gearCalc.value !== 'undefined') {
                    try {
                        if (alarmInfo.gearCalc.value.length > 0) {
                            gearCalc = JSON.stringify(alarmInfo.gearCalc.value);
                        }
                    } catch (err) {
                        // Error rendering JSON String
                    }
                }

                if (alarmInfo.operationState.value === null) {
                    alarmInfo.operationState = 0;
                }

                if (alarmInfo.isTest.value === null) {
                    alarmInfo.isTest = 0;
                }

                global.plgManager.logger.info('CORE | HANDLERALARM | operationState reported as: ' + alarmInfo.operationState.value);

                // generate createdAt date and time
                
                var createdAt = moment().format('YYYY-MM-DD HH:mm:ss');
                console.log('CORE | HANDLERALARM | new operation createdAt: ' + createdAt);

                var handlerAlarmRef = this;

                global.coreDb.run(
                    `INSERT INTO operationsStore (` +
                    `alarmdate, ` +
                    `alarmtime, ` +
                    `alarmtimeCalc, ` +
                    `gear, ` +
                    `gearlistCalc, ` +
                    `operationnumber, ` +
                    `location, ` +
                    `message, ` +
                    `community, ` +
                    `rawdata, ` +
                    `object, ` +
                    `district, ` +
                    `keywordId, ` +
                    `keywordCategory, ` +
                    `keywordRaw, ` +
                    `keywordName, ` +
                    `zveis, ` +
                    `zveisListCalc, ` +
                    `subject, ` +
                    `street, ` +
                    `gkx, ` +
                    `gky, ` +
                    `lat, ` +
                    `lon, ` +
                    `operationState, ` +
                    `operationSourceConnector, ` +
                    `subOperationFor, ` +
                    `uuid, ` +
                    `floor, ` +
                    `ils, ` +
                    `section, ` +
                    `isTest, ` + 
                    `createdAt, ` +
                    `zveisDeltaCalc, ` +
                    `gearDataCalc ` +
                    `) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`,
                    [
                        alarmInfo.alarmdate.value,
                        alarmInfo.alarmtime.value,
                        alarmInfo.alarmtimeCalc.value,
                        alarmInfo.gear.value,
                        gearCalc,
                        alarmInfo.operationnumber.value,
                        alarmInfo.location.value,
                        alarmInfo.message.value,
                        alarmInfo.community.value,
                        alarmInfo.rawdata.value,
                        alarmInfo.object.value,
                        alarmInfo.district.value,
                        alarmInfo.keywordId.value,
                        alarmInfo.keywordCategory.value,
                        alarmInfo.keywordRaw.value,
                        alarmInfo.keywordName.value,
                        alarmInfo.zveis.value,
                        zveisCalc,
                        alarmInfo.subject.value,
                        alarmInfo.street.value,
                        alarmInfo.gkx.value,
                        alarmInfo.gky.value,
                        alarmInfo.lat.value,
                        alarmInfo.lon.value,
                        0,
                        alarmInfo.operationSourceConnector.value,
                        opId,
                        alarmInfo.uuid.value,
                        alarmInfo.floor.value,
                        alarmInfo.ils.value,
                        alarmInfo.section.value,
                        alarmInfo.isTest.value,
                        createdAt,
                        zveisDeltaCalc,
                        gearDataCalc
                    ],
                    function (err) {
                        if (err) {
                            global.plgManager.logger.error('CORE | HANDLERALARM | Error inserting new alarm to coredatabase: ' + err.message);
                            console.log('CORE | HANDLERALARM | Error inserting new alarm to coredatabase: ' + err.message);
                        } else {
                            var insertId = this.lastID;
                            global.plgManager.logger.debug(`CORE | HANDLERALARM | New Alarm Info has been inserted to coredatabase (id: ${insertId})`);
                            var operations = new Operations(global.plgManager.logger);
                            operations.getOrganizationsAlarmedByOperation(insertId, (arrOrganizations) => {
                                console.log('Organizations for this Operation with id ' + insertId + ': ');
                                console.log(arrOrganizations);
                                operations.addOrganizationsToOperation(arrOrganizations, insertId, (success) => {
                                    if (success) {
                                        //this.logger('CORE | HANDLERALARM | Successfully added organizations reference for opertation to core database');
                                        console.log('CORE | HANDLERALARM | Successfully added organizations reference for operation to core database');
                                    } else {
                                        //this.logger('CORE | HANDLERALARM | ERROR adding organizations reference for opertation to core database');
                                        console.log('CORE | HANDLERALARM | ERROR adding organizations reference for operation to core database');
                                    }
                                })
                            })
                            
                            // if this is a alarm based on an existing alarm -> close existing one
                            if (opId > 0) {
                                operations.closeOperationCauseOfUpdatedAlarm(alarmInfo.operationnumber.value, insertId, (success) => {
                                    if (success) {
                                        global.plgManager.logger.debug(`CORE | HANDLERALARM | As the alarm was a new alarm for an existing one, the existing one has been closed now (id: ${opId})`);
                                    } else {
                                        global.plgManager.logger.debug(`CORE | HANDLERALARM | closing the existing alarm for this new one could not be closed (id: ${opId})`);
                                    }
                                })
                            }

                            // Tell core that a new alarm has been successfully persisted at core database with given Id
                            global.plgManager.event_new_alarm_persisted(insertId);
                            console.log(`CORE | HANDLERALARM | New Alarm Info has been inserted to coredatabase (id: ${insertId})`);

                            // HandlerAlarmRef instead of this!!!! Scope!
                            // handlerAlarmRef.evaluateRules(insertId, alarmInfo);
                            // alarmInfo.alarmedGroupIds = [4,5,6,7,8];
                            if (alarmInfo.isDefined(alarmInfo._alarmedGroupIds)) {
                                console.log("CORE | HANDLERALARM | alarmedGroupsId available and set!");
                                console.log("CORE | HANDLERALARM | Found groups", alarmInfo._alarmedGroupIds.value);
                                handlerAlarmRef.alarmGroupsDirectly(insertId, alarmInfo);
                            }
                            else {
                                console.log("CORE | HANDLERALARM | alarmedGroupsId unavailable or unset!");
                                handlerAlarmRef.evaluateRules(insertId, alarmInfo);
                            }
                        }
                    });
            })
        } else {
            global.plgManager.logger.error('CORE | HANDLERALARM | Error persisting alarm');
        }
    }

    generateFeedbackEntries(insertId, allUsersToAlarm) {
        if (allUsersToAlarm.length > 0) {
            var uuidArray = [];
            allUsersToAlarm.forEach((alarmableUser, index) => {
                if (alarmableUser.namespace != null){
                    if (alarmableUser.namespace.includes('alarmiatormobile')) {
                        // only add user to array if device-state === 1 (Device is active)
                        if (alarmableUser.deviceState === 1) {
                            uuidArray.push(alarmableUser.basedataUUID);
                        }
                    }
                }                                               
            });
            if (uuidArray.length > 0){
                // Now generate feedback objects for those users 
                console.log('CORE | HANDLERALARM | Adding now feedbackState entries for ' + uuidArray.length + ' basedata members');
                var feedbackstore = new FeedbackStore(global.app.logger);
                feedbackstore.generateFeedbackObjectsForOperation(insertId, uuidArray, (success) => {
                    if (success) {
                        console.log('CORE | HANDLERALARM | Successfully added feedbackstore objects for basedata members');
                    } else {
                        console.log('CORE | HANDLERALARM | Could not add feedbackstore objects for basedata members');
                    }
                });
            }
            else {
                console.log('CORE | HANDLERALARM | No entries added to feedbackstore, no alarm sent to ALARMiator Mobile');
            }
        } else {
            global.plgManager.logger.debug(`CORE | HANDLERALARM | No User to alarm identified by ruleengine for operation (id: ${insertId})`);
            console.log(`CORE | HANDLERALARM | No User to alarm identified by ruleengine for operation (id: ${insertId})`);
        }
    }

    evaluateRules(operationId, alarmInfo) {
        var operations = new Operations(global.plgManager.logger);
        // Now check which users to alarm
        var ruleengine = new Ruleengine();
        // Evaluate all active alarming rules
        ruleengine.evaluateOperationAgainstRules(operationId, (resultAllRules) => {
            // Now load all users to alarm for positive evaluated rules
            ruleengine.loadAllUsersToAlarmByEvaluationResult(resultAllRules, (allUsersToAlarm, allUsers) => {
                // get all users which are active for alarmiator mobile
                if (allUsersToAlarm != null) {
                    // at first fire the event of send out the alarm to plugins
                    global.plgManager.event_new_alarm_with_rules(alarmInfo, allUsersToAlarm);

                    this.generateFeedbackEntries(operationId, allUsersToAlarm);

                } else {
                    global.plgManager.logger.debug(`CORE | HANDLERALARM | Error identifiying users to alarm by ruleengine for operation (id: ${operationId})`);
                    console.log(`CORE | HANDLERALARM | Error identifiying users to alarm by ruleengine for operation (id: ${operationId})`);
                }

                if (resultAllRules != null) {
                    var positiveRulesIds = []

                    resultAllRules.forEach((ruleResult, index) => {
                        if (ruleResult.evaluates === true) {
                            positiveRulesIds.push(ruleResult.alertRuleId);
                        } 
                    });

                    ruleengine.getGroupsToAlarmForRules(positiveRulesIds, (alarmedGroups) => {
                        var groupIds = [];
                        if (alarmedGroups != null) {
                            alarmedGroups.forEach( alarmedGroup => {
                                groupIds.push(alarmedGroup.groupId);
                            })
                        }

                        if (groupIds.length > 0) {
                            
                            operations.addAlarmedGroupsToOperation(operationId, groupIds, (success) => {
                        //         console.log(success);
                            })
                        }
                    })
                }
                else {
                    global.plgManager.logger.debug(`CORE | HANDLERALARM | Error identifiying groups to alarm by ruleengine for operation (id: ${operationId})`);
                    console.log(`CORE | HANDLERALARM | Error identifiying groups to alarm by ruleengine for operation (id: ${operationId})`);
                }
                
            });
        });
    }

    alarmGroupsDirectly(operationId, alarmInfo) {
        var operations = new Operations(global.plgManager.logger);
        var groupIds = alarmInfo._alarmedGroupIds.value;
        operations.addAlarmedGroupsToOperation(operationId, groupIds, (success) => {

        })

        this.getUsersToAlarmForGroups(groupIds,(allUsersToAlarm) => {
            // get all users which are active for alarmiator mobile
            if (allUsersToAlarm != null) {

                console.log(`CORE | HANDLERALARM | Found ${allUsersToAlarm.length} users to alarm directly for groups`);
                global.plgManager.logger.debug(`CORE | HANDLERALARM | Found ${allUsersToAlarm.length} users to alarm directly for groups`);

                // at first fire the event of send out the alarm to plugins
                global.plgManager.event_new_alarm_with_rules(alarmInfo, allUsersToAlarm);

                this.generateFeedbackEntries(operationId, allUsersToAlarm);

            } else {
                global.plgManager.logger.debug(`CORE | HANDLERALARM | Error identifiying users to alarm by groups for operation (id: ${operationId})`);
                console.log(`CORE | HANDLERALARM | Error identifiying users to alarm by groups for operation (id: ${operationId})`);
            }
        })
    }


    /*
        !!! IMPORTANT: !!! 
            Returning properties should be same as ruleengine.getUsersToAlarmForRules 
        !!! IMPORTANT: !!!  
    */
    /**
     * Returns List of Users to alarm for given Array of groupIds
     * @param {array} groupIds group Ids to return users for
     * @param {function} callback rows or null
     */
     getUsersToAlarmForGroups(groupIds, callback) {
        if (Array.isArray(groupIds) && (groupIds.length > 0)) {
            let query_1 = "SELECT DISTINCT " +
                "basedata.id AS basedataId, " +
                "basedata.firstname AS firstName, " +
                "basedata.lastname AS lastName,  " +
                "basedata.uuid AS basedataUUID,  " +
                "basedata.cellphone AS cellphone,  " +
                "basedata.email AS email, " +
                "alertdevices.state AS deviceState, " +
                "fcmtokenStore.fcmToken AS fcmtoken,  " +
                "group_concat(DISTINCT xt_basedata_alarmings.pluginNamespace) AS namespace, " +
                "organizations.name AS organizationname " +
                "FROM groups " +
                "JOIN xt_basedata_groups ON xt_basedata_groups.groupId = groups.id  " +
                "JOIN basedata ON basedata.id = xt_basedata_groups.basedataId " +
                "LEFT JOIN alertdevices ON alertdevices.basedataId = basedata.id  " +
                "LEFT JOIN fcmtokenStore ON fcmtokenStore.deviceId = alertdevices.id  " +
                "LEFT JOIN xt_basedata_alarmings ON xt_basedata_alarmings.basedataId = basedata.id "+
                "LEFT JOIN organizations ON organizations.id = basedata.organizationid ";

            let query_2 = "WHERE ";

            let query_3 = "GROUP BY basedata.id, fcmtoken " +
                "ORDER BY basedata.id";

            let query_2_addendum = '';
            if (Array.isArray(groupIds)) {
                groupIds.forEach((group, index) => {
                    //console.log('LOOP ruleIdArray: ' + index);
                    query_2_addendum = query_2_addendum + " groups.id = " + group + " ";
                    if (index < (groupIds.length - 1)) {
                        query_2_addendum = query_2_addendum + " OR ";
                    }
                });
            }

            let query = query_1 + query_2 + query_2_addendum + query_3;

            console.log('SQL QUERY USERS TO ALARM: ' + query);
            global.coreDb.all(query, [
            ], (err, rows) => {
                if (err) {
                    console.log('MODELS-RULEENGINE | Error loading users to alarm from core data: ' + err.message);
                    return callback(null);
                } else {
                    if (rows != null) {
                        rows.forEach((row) => {
                            if (row.namespace != null) {
                                var nspaceArr = row.namespace.split(',');
                                if (Array.isArray(nspaceArr)) {
                                    row.namespace = nspaceArr;
                                }
                            }
                        });
                    }
                    return callback(rows);
                }
            });
        } else {
            return callback([]);
        }
        
    }
}

module.exports = CoreAlarmHandler;