
const EventEmitter = require('events');
const SqlLite = require('sqlite3');
const { Sequelize } = require('sequelize');
const Umzug = require('umzug');
const { type } = require('os');


class database extends EventEmitter {

    constructor() {
        super();
        this.db = null;
    }

    /**
     * Starts / initializes database connection to core database
     * @param {function} callback true or false
     */
    startDatabase(callback) {
        console.log('MODELS-DATABASE | Starting check for database existence');
        this.checkIfCoreDatabaseExists((successDB) => {
            if (successDB) {
                console.log('MODELS-DATABASE | Starting database migrations');
                this.runMigrations((successMigrations) => {
                    if (successMigrations) {
                        this.openCoreDatabase((successConnection) => {
                            if (successConnection) {
                                console.log('MODELS-DATABASE | Coredabase successfully connected');
                                callback(true);
                            } else {
                                callback(false);
                            }
                        })
                    } else {
                        callback(false);
                    }
                })
            } else {
                callback(false);
            }
        })
    }

    /**
     * Starts / initializes database connection to sessions database
     * @param {function} callback true or false
     */
    startSessionsDatabase(callback) {
        var that = this;
        console.log('MODELS-DATABASE | Starting check for sessions database existence');
        this.checkIfSessionsDatabaseExists((successSessionsDB) => {
            if (successSessionsDB) {
                this.openSessionsDatabase((successOpenSessionsDB) => {
                    if (successOpenSessionsDB) {
                        if (typeof global.developmentMode != 'undefined') {
                            if (global.developmentMode === true) {
                                // Development Mode on --> don't wipe sessions
                                return callback(true);
                            } else {
                                that.wipeSessions((successWipe) => {
                                    return callback(successWipe);
                                })
                            } 
                        }
                    } else {
                        return callback(false);
                    }
                    
                });
            } else {
                return callback(false);
            }
        })
    }

    /**
     * Wipes all existing sessions from sessions store database
     * @param {function} callback success (true/false)
     */
    wipeSessions(callback) {
        let query = "DELETE FROM sessions";
        this.dbSessions.run(query, [], function (err) {
            if (err) {
              console.log('ERROR deleting existing sessions while starting server: ' + err.message);
              return callback(false);
            } else {
              return callback(true);
            }
          })
    }

    /**
     * Runs database migration steps
     * @param {function} callback true or false
     */
    runMigrations(callback) {
        const sequelize = new Sequelize({
            dialect: 'sqlite',
            storage: './store/coredatabase.db',
            logging: msg => {
                if (msg.includes('SequelizeMeta')) {
                    console.log('CORE | DB-MIGRATIONS | SEQUELIZE | Booting up');
                } else {
                    console.log("CORE | DB-MIGRATIONS | SEQUELIZE | " + msg);
                }
            }
        });

        const umzug = new Umzug({
            migrations: {
                path: './migrations',
                params: [
                    sequelize.getQueryInterface()
                ]
            },
            storage: 'sequelize',
            storageOptions: {
                sequelize: sequelize
            }

        });

        (async () => {
            // Checks migrations and run them if they are not already applied. To keep
            // track of the executed migrations, a table (and sequelize model) called SequelizeMeta
            // will be automatically created (if it doesn't exist already) and parsed.
            await umzug.up();

            // HIER MUSS DER CALLBACK SEIN
            callback(true);
        })();
    }

    /**
     * Checks if coredatabase exists. if not -> copy template for initial start.
     * @param {function} callback success (true or false)
     */
    checkIfCoreDatabaseExists(callback) {
        const fs = require('fs');
        const path = require('path');
        try {

            var store_folder = path.join(global.appRoot, 'store');
            var restore_folder = path.join(store_folder, 'restore');
            var templates_folder = path.join(store_folder, 'templates');
            var db_filename = 'coredatabase.db';

            if(fs.existsSync(restore_folder))
            {
                console.log("MODELS | DATABASE | restore folder found.");
                console.log('MODELS | DATABASE | copy '+ db_filename +' from ' + restore_folder + ' to ' + store_folder +'.');
                var src = path.join(restore_folder, db_filename);
                var dst = path.join(store_folder, db_filename);
                fs.copyFileSync(src, dst);

                console.log('MODELS | DATABASE | remove ' + restore_folder + '.');
                fs.rmdirSync(restore_folder, { recursive: true });
            }
            else if (!fs.existsSync(path.join(store_folder, db_filename)))
            {
                console.log('MODELS | DATABASE | coredatabase does not exist. Copying initial template to store.');
                var src = path.join(templates_folder, db_filename);
                var dst = path.join(store_folder, db_filename);
                fs.copyFileSync(src, dst);

            } else 
            {
                console.log("MODELS | DATABASE | coredatabase exists. Ready to open.");
            }



            /* So funktionierts .... */
            callback(true);
            // Aber callback wird aufgerufen, bevor dbmigrate durchläuft

        } catch (err) {
            console.error(err);
            callback(false);
        }
    }

    /**
     * Checks if sessions database exists. if not -> copy template for initial start.
     * @param {function} callback success (true or false)
     */
    checkIfSessionsDatabaseExists(callback) {
        const fs = require('fs');
        const path = require('path');
        try {
            var store_folder = path.join(global.appRoot, 'store');
            var templates_folder = path.join(store_folder, 'templates');
            var db_filename = 'sessions.db';

            if (!fs.existsSync(path.join(store_folder, db_filename))) {
                console.log('MODELS | DATABASE | sessions database does not exist. Copying initial template to store');
                var src = path.join(templates_folder, db_filename);
                var dst = path.join(store_folder, db_filename);
                fs.copyFileSync(src, dst);
            } else {
                console.log("MODELS | DATABASE | sessions database exists. Ready to open.");
            }
            callback(true);
        } catch (err) {
            console.error('MODELS | DATABASE | ERROR: ' + err);
            callback(false);
        }
    }

    /**
     * initializes connection to core database
     * @param {function} callback success (true or false)
     */
    openCoreDatabase(callback) {
        this.db = new SqlLite.Database(global.appRoot + '/store/coredatabase.db', (err) => {
            if (err) {
                console.log('DATABASEMANAGER | Could not connect to database', err);
                callback(false);
            } else {
                console.log('DATABASEMANAGER | Connected');
                callback(true);
            }
        })
    }

    /**
     * initializes connection to sessions database
     * @param {function} callback success (true or false)
     */
    openSessionsDatabase(callback) {
        this.dbSessions = new SqlLite.Database(global.appRoot + '/store/sessions.db', (err) => {
            if (err) {
                console.log('DATABASEMANAGER | Could not connect to Sessions database', err);
                callback(false);
            } else {
                console.log('DATABASEMANAGER | Connected to sessions database');
                callback(true);
            }
        })
    }
}

module.exports = database;