const moment = require('moment');

/**
 * Class handles all methods and models regarding handling email queue
 * @class
 */
class emailqueue {

  /**
   * @constructor
   * @param {object} alogger Logger to be used by class methods (usually req.app.logger)
   */
  constructor(alogger) {
    // always initialize all instance properties
    this.logger = alogger;
  }

  /**
   * Adds a new email to queue
   * @param {string} subject subject Line of eMail
   * @param {string} html html version of email
   * @param {string} plain plan text version of email
   * @param {array} recipientsArray array of recipients of this email
   */
  addToQueue(subject, html, plain, recipientsArray, callback) {
        let query = "INSERT INTO emailQueue (createdAt, subject, html, plain, recipients) VALUES (?, ?, ?, ?, ?)";
        // createdAt calculation
        let createdAt = moment(new Date()).format("YYYY-MM-DD HH:mm:ss");
       
        // flatten recipients in json
        let recipientsFlat = '{}';
        if (Array.isArray(recipientsArray)) {
            recipientsFlat = JSON.stringify(recipientsArray);
        }
        var that = this;

        global.coreDb.run(query,
        [
            createdAt,
            subject,
            html, 
            plain, 
            recipientsFlat
        ],
        function (err) {
            if (err) {
                that.logger.error('MODELS-EMAILQUEUE | Error adding new email to queue: ' + err.message);
                callback(false);
            } else {    
                callback(true);
            }
        });
  }

  /**
   * returns list of emails to be sent out
   * @param {integer} limit optional, limit emails to be sent to. If null, all emails are returned which are waiting to be sent out
   * @param {function} callback rows or null
   */
  getEmailsToBeSent(limit, callback) {
    if (limit === null) {
        let query = "SELECT * FROM `emailQueue` WHERE sendState = 0";
        console.log('MODELS-EMAILQUEUE | loading all emails waiting for delivery from queue');
            global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this._logger.error('MODELS-EMAILQUEUE | Error loading emails from queue from core database:' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    } else {
        let query = "SELECT * FROM `emailQueue` WHERE sendState = 0 LIMIT ?";
        console.log('MODELS-EMAILQUEUE | loading all emails waiting for delivery from queue');
        global.coreDb.all(query, [
            limit
        ], (err, rows) => {
            if (err) {
                this._logger.error('MODELS-EMAILQUEUE | Error loading emails from queue from core database:' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }
  }

  /**
   * returns a single email from queue identified by id
   * @param {integer} id id of email in email queue
   * @param {function} callback rows or null
   */
  getEMailById(id, callback) {
    let query = "SELECT * FROM `emailQueue` WHERE id = ?";
    console.log('MODELS-EMAILQUEUE | loading email from queue with id' + id);
        global.coreDb.all(query, [
            id
        ], (err, rows) => {
        if (err) {
            this._logger.error('MODELS-EMAILQUEUE | Error loading email from queue from core database:' + err.message);
            return callback(null);
        } else {
            return callback(rows);
        }
    }); 
  }

  /**
   * returns list of emails from email queue
   * @param {integer} limit optional, limit emails to be sent to. If null, all emails are returned which are waiting to be sent out
   * @param {integer} filteredState state to be filtered at (null = all, 0 = waiting for sending, 1 = sent successfully, 3 = could not be send out)
   * @param {function} callback rows or null
   */
   getEmailsinQueue(limit, filteredState, callback) {
    if (limit === null) {
        let query = "SELECT * FROM `emailQueue` ORDER BY createdAt DESC";
        if (filteredState != null) {
            query = query + ' WHERE sendState = ' + filteredState;
        }
        console.log('MODELS-EMAILQUEUE | loading all emails from queue');
            global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this._logger.error('MODELS-EMAILQUEUE | Error loading emails from queue from core database:' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    } else {
        let wherePart = '';
        if (filteredState != null) {
            wherePart = " WHERE sendState = " + filteredState;
        }
        let query = "SELECT * FROM `emailQueue` " + wherePart + " LIMIT ? ORDER BY createdAt DESC";
        console.log('MODELS-EMAILQUEUE | loading all emails from queue limited to ' + limit);
        global.coreDb.all(query, [
            limit
        ], (err, rows) => {
            if (err) {
                this._logger.error('MODELS-EMAILQUEUE | Error loading emails from queue from core database:' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }
  }

  /**
   * Sets state for an eMail to "sent" (1)
   * @param {integer} eId id of email state shall be set for to sent
   * @param {*} callback success (true/false)
   */
  setStateSent(eId, callback) {
      let query = "UPDATE emailQueue SET sendState = 1 WHERE id = ?";
      global.coreDb.run(query,
        [
            eId
        ],
        function (err) {
            if (err) {
                this.logger.error('MODELS-EMAILQUEUE | Error setting state to sent for email with id ' + eId + ' at core database: ' + err.message);
                callback(false);
            } else {    
                callback(true);
            }
        });
  }

  /**
   * 
   * @param {array} successIdsArray array of integer ids to set state to success for
   * @param {array} errorIdsArray array of integer ids to set state to error for
   * @param {function} callback success (true/false)
   * @returns 
   */
  setStateSentBulk(successIdsArray, errorIdsArray, callback) {
    global.coreDb.serialize(() => {
        let failState = false;
        successIdsArray.forEach((succId) => {
            let query = "UPDATE emailQueue SET sendState = 1 WHERE id = ?";
            let failState = false;
            global.coreDb.run(query,
                [
                    succId
                ],
                function (err) {
                    if (err) {
                        this.logger.error('MODELS-EMAILQUEUE | Error setting send state for email in queue with id ' + succId + ' at core database: ' + err.message);
                        failState = true;
                    }
                });
        });
        errorIdsArray.forEach((errId) => {
            let query = "UPDATE emailQueue SET sendState = 2 WHERE id = ?";
            global.coreDb.run(query,
                [
                    errId
                ],
                function (err) {
                    if (err) {
                        this.logger.error('MODELS-EMAILQUEUE | Error setting send state for email in queue with id ' + succId + ' at core database: ' + err.message);
                        failState = true;
                    }
                });
        });
    });
    if (failState) {
        return callback(false);
    } else {
        return callback(true);
    }
  }

  /**
   * Sets sendState to cancelled for email with given id
   * @param {integer} eId id of email in queue to set state to canceled for
   * @param {*} callback success (true/false)
   */
  setStateCanceled(eId, callback) {
    let query = "UPDATE emailQueue SET sendState = 2 WHERE id = ?";
    global.coreDb.run(query,
      [
          eId
      ],
      function (err) {
          if (err) {
              this.logger.error('MODELS-EMAILQUEUE | Error setting state to cancelled for email with id ' + eId + ' at core database: ' + err.message);
              callback(false);
          } else {    
              callback(true);
          }
      });
  }

  /**
   * Removes all entries from emailqueue
   * @param {function} callback success (true/false)
   */
  truncateQueue(callback) {
      let query = "DELETE FROM emailQueue";
      global.coreDb.run(query,
        [],
        function (err) {
            if (err) {
                this.logger.error('MODELS-EMAILQUEUE | Error truncating emailqueue at core database: ' + err.message);
                callback(false);
            } else {    
                callback(true);
            }
        });
  }



}

// export the class
module.exports = emailqueue;