const EventEmitter = require('events');

/**
 * Class represents the Event Manager of ALARMiator Server
 * @class
 * @author Jens Dinstühler
 */
class EventManager extends EventEmitter {

    /**
     * @constructor
     * @param {object} logger logger instance to be used by this class instance.
     */
    constructor(logger) {
        super();
        this.logger = logger;
    }

    /**
     * Returns all events from core database
     * @param {function} callback rows or null
     */
    getAllEvents(callback) {
        let query = "SELECT * FROM eventsStore";

        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                console.log(err);
                this.logger.error('MODELS-EVENTMANAGER | Error reading all events from core database: ' + err.message);
                return callback([]);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns an event from core database
     * @param {function} callback rows or null
     */
    getEvent(eventname, callback) {
        let query = "SELECT * FROM eventsStore WHERE eventname = ?";

        global.coreDb.all(query, [
            eventname
        ], (err, rows) => {
            if (err) {
                console.log(err);
                this.logger.error('MODELS-EVENTMANAGER | Error reading event from core database: ' + err.message);
                return callback([]);
            } else {
                return callback(rows);
            }
        });
    }

    
    /**
     * Creates a new event in coredatabase table
     * @param {string} eventname name of new event
     * @param {string} ownermodule 'core' or plugin namespace
     * @param {string} payloadType string, object, json (will be parsed to an object) 
     * @param {function} callback success (true or false)
     */
    addState(eventname, ownermodule, payloadType, callback) {
        let query = "INSERT INTO eventsStore (eventname, ownermodule, payloadType) VALUES (?, ?, ?)";

        global.coreDb.run(query, [
            eventname,
            ownermodule,
            payloadType
        ], function (err) {
            if (err) {
                console.log('MODELS-EVENTMANAGER | Error storing new event to database: ' + err.message);
                callback(false);
            } else {
                callback(true);
            }
        });
    }
    

    /**
     * Removes an event from store in core database
     * @param {string} eventname name of event in core database table eventsStore
     * @param {function} callback success (true or false)
     */
    removeEvent(eventname, callback) {
        let query = "DELETE FROM eventsStore WHERE eventname = ?";

        global.coreDb.run(query, [
            eventname
        ], function (err) {
            if (err) {
                callback(false);
            } else {
                callback(true);
            }
        });
    }


    

    /**
     * Checks, if an event with name exists in database
     * @param {string} eventname 
     * @param {function} callback null or row of existing state
     */
    doesEventExist(eventname, callback) {
        let query = "SELECT * FROM eventsStore WHERE eventname = ?";

        global.coreDb.all(query, [
            eventname
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

}

module.exports = EventManager;