const EventEmitter = require('events');
const moment = require('moment');
/**
 * Class represents fcmtokens in ALARMiator Core
 * @class
 * @author Jens Dinstühler
 */
class Fcmtokens extends EventEmitter {

    /**
     * @constructor
     * @param {object} logger logger instance to be used by this class instance.
     */
    constructor(logger) {
        super();
        this.logger = logger;
    }

    /**
     * Returns all FCM Tokens from core database
     * @param {function} callback rows or null
     */
    getAllFCMTokens(callback) {
        let query = "SELECT * FROM fcmtokenStore";

        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns a single token given with id
     * @param {integer} id tokenid
     * @param {function} callback rows or null
     */
    getTokenWithId(id, callback) {
        let query = "SELECT * FROM fcmtokenStore WHERE id = ?";
        global.coreDb.all(query, [id], (err, rows) => {
            if (err) {
                console.log(err);
                return callback(null);
            } else {
                return callback(rows);
            }
        })
    }

    /**
     * Returns all FCM Tokens from core database enriched with device data for UI Listings
     * @param {function} callback rows or null
     */
    getAllFCMTokensForUI(callback) {
        let query = "SELECT fcmtokenStore.*, " +
            "alertdevices.basedataId AS basedataId, " +
            "basedata.lastname AS lastname, " + 
            "basedata.firstname AS firstname, " + 
            "assets.title AS assetsTitle, " +
            "alertdevices.name AS deviceName, " +
            "alertdevices.assetsId AS assetsId " +
            "FROM fcmtokenStore " +
            "INNER JOIN alertdevices " +
            "ON fcmtokenStore.deviceId = alertdevices.id " +
            "LEFT JOIN basedata " + 
            "ON alertdevices.basedataId = basedata.id " +
            "LEFT JOIN assets " +
            "ON alertdevices.assetsId = assets.id ";

        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                // console.log(rows);
                return callback(rows);
            }
        });
    }

    /**
     * Returns all active FCM Tokens from core database for the given plugin namespace
     * @param {string} pluginNamespace namespace of plugin this tokens belongs to
     * @param {function} callback rows or null 
     */
    getAllFCMTokensForNamespace(pluginNamespace, callback) {
        let query = "SELECT fcmtokenStore.* " +
            "FROM fcmtokenStore " +
            "INNER JOIN alertdevices " +
            "ON fcmtokenStore.deviceId = alertdevices.id " +
            "WHERE fcmtokenStore.pluginNamespace = ? AND " +
            "fcmtokenStore.state = 1 AND " +
            "alertdevices.state = 1";

        global.coreDb.all(query, [
            pluginNamespace
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns rows with tokens for basedata and namespace
     * @param {string} pluginNamespace namespace of plugin, tokens shall be returned for
     * @param {*} basedataId basedata id of member the tokens shall be returned for
     * @param {*} callback rows or null
     */
    getAllFCMTokensForBasedataAndNamespace(pluginNamespace, basedataId, callback) {
        let query = "SELECT " +
            "basedata.id AS basedataId, " +
            "alertdevices.id AS deviceId, " +
            "fcmtokenStore.fcmToken AS fcmToken " +
            "FROM basedata " +
            "INNER JOIN alertdevices ON alertdevices.basedataId = basedata.id " +
            "LEFT JOIN fcmtokenStore ON fcmTokenStore.deviceId = alertdevices.id " +
            "WHERE " +
                "basedata.id = ? AND " +
                "alertdevices.state = 1 AND " +
                "fcmtokenStore.pluginNamespace = ?";

        global.coreDb.all(query, [
            basedataId,
            pluginNamespace
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all fcm tokens for users which have a given access group applied
     * @param {string} accessGroupNamespace namespace for access group from table users_groups
     * @param {function} callback rows or null
     */
    getAllFCMTokensForAccessGroup(accessGroupNamespace, callback) {
        let query = `
                SELECT DISTINCT basedata.id AS basedataId,
                                basedata.firstname AS firstname,
                                basedata.lastname AS lastname,
                                basedata.email AS email,
                                basedata.uuid AS basedataUUID,
                                alertdevices.uuid AS alertdeviceUUID,
                                fcmtokenStore.fcmToken,
                                users.username,
                                users_groups.grantnamespace
                FROM xt_users_users_groups
                    INNER JOIN
                        users_groups ON users_groups.id = xt_users_users_groups.groupId
                    INNER JOIN
                        users ON xt_users_users_groups.userId = users.id
                    INNER JOIN
                        basedata ON users.basedataid = basedata.id
                    INNER JOIN
                        alertdevices ON alertdevices.basedataId = basedata.id
                    INNER JOIN
                        fcmtokenStore ON fcmtokenStore.deviceId = alertdevices.id
                WHERE users_groups.grantnamespace = ? AND 
                    fcmtokenStore.pluginNamespace = 'alarmiatormobile' AND 
                    fcmtokenStore.state = 1 AND 
                    alertdevices.state = 1;
                `;

        global.coreDb.all(query, [
            accessGroupNamespace
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all fcm tokens for users which have a given group applied
     * @param {string} groupShortname shortName for group from table groups
     * @param {function} callback rows or null
     */
    getAllFCMTokensForGroup(groupShortname, callback) {
        let query = "SELECT DISTINCT " + 
                        "basedata.id AS basedataId, " +
                        "basedata.firstname AS firstname, " +
                        "basedata.lastname AS lastname,  " +
                        "basedata.email AS email, " +
                        "basedata.uuid AS basedataUUID, " +
                        "alertdevices.uuid AS alertdeviceUUID, " +
                        "fcmtokenStore.fcmToken, " +
                        "users.username, " +
                        "groups.shortName AS groupShortname " +
                    "FROM basedata " +
                    "INNER JOIN users ON users.basedataid = basedata.id " +
                    "INNER JOIN groups ON groups.id = xt_basedata_groups.groupId " +
                    "INNER JOIN xt_basedata_groups ON xt_basedata_groups.basedataId = basedata.id " +
                    "INNER JOIN alertdevices ON alertdevices.basedataId = basedata.id " +
                    "INNER JOIN fcmtokenStore ON fcmtokenStore.deviceId = alertdevices.id " +
                    "WHERE groups.shortName = ? AND fcmtokenStore.pluginNamespace = 'alarmiatormobile'";

        global.coreDb.all(query, [
            groupShortname
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all fcm tokens for users which have a given role applied
     * @param {integer} roleId id of role users have applied
     * @param {function} callback rows or null
     */
    getAllFCMTokensForRoleId(roleId, callback) {
        let query = "SELECT DISTINCT " +
                        "basedata.id AS basedataId, " +
                        "basedata.firstname AS firstname, " +
                        "basedata.lastname AS lastname,  " +
                        "basedata.email AS email, " +
                        "basedata.uuid AS basedataUUID, " +
                        "alertdevices.uuid AS alertdeviceUUID, " +
                        "fcmtokenStore.fcmToken, " +
                        "users.username, " +
                        "roles.id AS roleId " +
                    "FROM basedata " +
                    "INNER JOIN users ON users.basedataid = basedata.id " +
                    "INNER JOIN roles ON roles.id = xt_basedata_roles.roleId " +
                    "INNER JOIN xt_basedata_roles ON xt_basedata_roles.basedataId = basedata.id " +
                    "INNER JOIN alertdevices ON alertdevices.basedataId = basedata.id " +
                    "INNER JOIN fcmtokenStore ON fcmtokenStore.deviceId = alertdevices.id " +
                    "WHERE roles.id = ? AND fcmtokenStore.pluginNamespace = 'alarmiatormobile'";

        global.coreDb.all(query, [
            roleId
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                return callback(rows);
            }
        });
    }


    /**
     * Returns all fcm tokens for users which have a given qualification applied
     * @param {string} qualificationShortName short name of qualification from table qualifications
     * @param {function} callback rows or null
     */
    getAllFCMTokensForQualification(qualificationShortName, callback) {
        let query = "SELECT DISTINCT " + 
                        "basedata.id AS basedataId, " +
                        "basedata.firstname AS firstname, " +
                        "basedata.lastname AS lastname, " +
                        "basedata.email AS email, " +
                        "basedata.uuid AS basedataUUID, " +
                        "alertdevices.uuid AS alertdeviceUUID, " +
                        "fcmtokenStore.fcmToken, " +
                        "users.username, " +
                        "qualifications.shortName AS qualificationShortname " +
                    "FROM basedata " +
                    "INNER JOIN users ON users.basedataid = basedata.id " +
                    "INNER JOIN qualifications ON qualifications.id = xt_basedata_qualifications.qualificationId " +
                    "INNER JOIN xt_basedata_qualifications ON xt_basedata_qualifications.basedataId = basedata.id " +
                    "INNER JOIN alertdevices ON alertdevices.basedataId = basedata.id " +
                    "INNER JOIN fcmtokenStore ON fcmtokenStore.deviceId = alertdevices.id " +
                    "WHERE qualifications.shortName = ? AND fcmtokenStore.pluginNamespace = 'alarmiatormobile'";

        global.coreDb.all(query, [
            qualificationShortName
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all fcm tokens for users which have permission to see state changes for ISSI
     * @param {number} issi issi
     * @param {function} callback rows or null
     */
    getAllFCMTokensForISSI(issi, callback) {
        let query = `
            SELECT DISTINCT basedata.lastname,
                            basedata.firstname,
                            fcmToken    
            FROM assets
                    JOIN
                        organizations ON assets.ownerOrgId = organizations.id
                    LEFT JOIN
                        groups ON groups.organizationid = organizations.id
                    INNER JOIN
                        xt_basedata_groups ON xt_basedata_groups.groupId = groups.id
                    INNER JOIN
                        alertdevices ON alertdevices.basedataId = xt_basedata_groups.basedataId
                    INNER JOIN
                        basedata ON xt_basedata_groups.basedataId = basedata.id
                    INNER JOIN
                        fcmtokenStore ON fcmtokenStore.deviceId = alertdevices.id
            WHERE issi = ? AND 
                        groups.gearStateAllowed = 1 AND 
                        alertdevices.state = 1 AND 
                        fcmtokenstore.state = 1 AND 
                        fcmtokenStore.pluginNamespace = 'alarmiatormobile';
        `;

        global.coreDb.all(query, [
            issi
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * adds a new fcm token to core database
     * @param {integer} deviceId id of device this token comes from
     * @param {integer} state state (1 active, 0 inactive)
     * @param {string} pluginNamespace namespace of plugin this tokens belongs to
     * @param {integer} predecessorTokenId if this is a new token for an existing (so it is replacing an old one) this is the id of the old token
     * @param {string} fcmToken the actal fcm token
     * @param {function} callback success (true or false)
     */
    createFCMToken(deviceId, state, pluginNamespace, predecessorTokenId, fcmToken, callback) {

        let query = "INSERT INTO fcmtokenStore (deviceId, state, pluginNamespace, predecessorTokenId, fcmToken) VALUES (?, ?, ?, ?, ?)";

        global.coreDb.run(query, [
            deviceId,
            state,
            pluginNamespace,
            predecessorTokenId,
            fcmToken
        ], function (err) {
            if (err) {
                console.log('MODELS-FCMTOKENS | Error storing new device to database: ' + err.message);
                callback(false);
            } else {
                callback(true);
            }
        });
    }

    /**
     * Removes a token from database
     * @param {integer} tokenId id of token in core database
     * @param {function} callback success (true or false)
     */
    removeToken(tokenId, callback) {
        let query = "DELETE FROM fcmtokenStore WHERE id = ?";

        global.coreDb.run(query, [
            tokenId
        ], function (err) {
            if (err) {
                callback(false);
            } else {
                console.log('MODELS-FCMTOKENS | Removed token for id: ' + tokenId);
                callback(true);
            }
        });
    }

    /**
     * Removes all tokens for a given uuid for a device
     * @param {string} uuid 
     * @param {function} callback success (true or false)
     */
    removeTokensForDeviceUUID(uuid, callback) {
        let query = "SELECT * FROM alertdevices WHERE uuid = ?";

        global.coreDb.all(query, [
            uuid
        ], (err, rows) => {
            if (err) {
                console.log(err);
                return callback(null);
            } else {
                if (rows.length > 0) {
                    var deviceId = rows[0].id;
                    let queryTokens = "DELETE FROM fcmtokenStore WHERE deviceId = ?";
                    global.coreDb.run(queryTokens, [
                        deviceId
                    ], function (err) {
                        if (err) {
                            callback(false);
                        } else {
                            console.log('MODELS-FCMTOKENS | Removed token for deviceId: ' + deviceId + ' and uuid: ' + uuid);
                            callback(true);
                        }
                    });
                }
            }
        });
    }

    /**
     * Removes all tokens from store for a given device id
     * @param {integer} deviceId id of device
     * @param {function} callback success (true/false)
     */
    removeTokensForDeviceID(deviceId, callback) {
        let queryTokens = "DELETE FROM fcmtokenStore WHERE deviceId = ?";
        global.coreDb.run(queryTokens, [
            deviceId
        ], function (err) {
            if (err) {
                callback(false);
            } else {
                console.log('MODELS-FCMTOKENS | Removed token for deviceId: ' + deviceId);
                callback(true);
            }
        });
    }

    /**
     * Sets the state of an token to active
     * @param {integer} tokenId id of token in coredatabase to set state for
     * @param {function} callback success (true or false)
     */
    setActive(tokenId, callback) {
        let query = "UPDATE fcmtokenStore SET state = ? WHERE id = ?";
        global.coreDb.run(query, [
            1,
            tokenId
        ], function (err) {
            if (err) {
                console.log(err);
                return callback(false);
            } else {
                return callback(true);
            }
        });
    }

    /**
     * Sets the state of an token to inactive
     * @param {integer} tokenId id of token in coredatabase to set state for
     * @param {function} callback success (true or false)
     */
    setInactive(tokenId, callback) {
        let query = "UPDATE fcmtokenStore SET state = ? WHERE id = ?";
        global.coreDb.run(query, [
            0,
            tokenId
        ], function (err) {
            if (err) {
                console.log(err);
                return callback(false);
            } else {
                return callback(true);
            }
        });
    }

    /**
     * Checks, if a token exists in database
     * @param {string} fcmtoken 
     * @param {function} callback null or row of existing device
     */
    doesTokenExist(fcmtoken, callback) {
        let query = "SELECT * FROM fcmtokenStore WHERE fcmtoken = ?";

        global.coreDb.all(query, [
            fcmtoken
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Increase counter for each failed token + sets timestamp
     * @param {array} tokenList list with all failed tokens
     * @param {function} callback success (true or false)
     */
    updateFailedTokenCounter(tokenList, callback) {
        if (tokenList == null || tokenList.length < 1) {
            return callback(false);
        }
        var tokenString =  "'" + tokenList.join("','") + "'";
        // console.log(tokenString);
        let query = "UPDATE fcmtokenStore SET failCount = failCount +1, lastReportedTimestamp = ?, failState = 1 WHERE fcmToken IN ("+ tokenString +")";
        global.coreDb.run(query, [
            moment().toISOString()
        ], function (err) {
            if (err) {
                console.log(err);
                return callback(false);
            } else {
                return callback(true);
            }
        });
    }

    /**
     * Update failState for successfull tokens
     * @param {array} tokenList list with all successful tokens
     * @param {function} callback success (true or false)
     */
    updateSuccessfullToken(tokenList, callback) {
        if (tokenList == null || tokenList.length < 1) {
            return callback(false);
        }
        var tokenString =  "'" + tokenList.join("','") + "'";
        // console.log(tokenString);
        let query = "UPDATE fcmtokenStore SET failCount = 0, lastReportedTimestamp = ?, failState = 0 WHERE fcmToken IN ("+ tokenString +")";
        global.coreDb.run(query, [
            moment().toISOString()
        ], function (err) {
            if (err) {
                console.log(err);
                return callback(false);
            } else {
                return callback(true);
            }
        });
    }
}

module.exports = Fcmtokens;