
/**
 * Class handles gear groups (alarm) and offers methods dealing with gear groups in core database
 * @class
 */
class gearGroups {
  /**
   * @constructor
   * @param {object} alogger Logger to be used for output of class methods (usually req.app.logger)
   */
  constructor(alogger) {
    // always initialize all instance properties
    this.logger = alogger;
    if (typeof alogger === 'undefined') {
      if (typeof global.logger != 'undefined') {
        this.logger = global.logger;
      }
    }
  }

  /**
   * Returns list of all existing groups
   * @param {function} callback rows
   */
  getListOfGroups(callback) {
    //console.log('MODELS-GEARGROUPS | Reading Groups');
    let query = "SELECT * FROM `gearGroups`";

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        this._logger.error('GROUPS-MODEL | Error loading gearGroups from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Loads a group for a given group id
   * @param {integer} gId group id to load data for from core database
   * @param {function} callback row or null
   */
  getGroupForId(gId, callback) {
    this.logger.debug('MODELS-GEARGROUPS | Reading Group for id ' + gId);
    let query = "SELECT * " +
      "FROM `gearGroups` " +
      "WHERE id = ?";

    global.coreDb.all(query, [
      gId
    ], (err, rows) => {
      if (err) {
        this._logger.error('GROUPS-MODEL | Error loading group from core data for id ' + gId + ': ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns gear referenced to gear group
   * @param {integer} gId id of gear group
   * @param {function} callback rows or null
   */
  getGearForGroupId(gId, callback) {
    this.logger.debug('MODELS-GEARGROUPS | Reading gear for Group with id ' + gId);
    let query = `SELECT 
                    xt_gear_gearGroups.*,
                    
                    assets.title AS gearName
                FROM xt_gear_gearGroups 
                LEFT JOIN assets ON assets.id = xt_gear_gearGroups.assetId
                WHERE groupId = ?`;

    global.coreDb.all(query, [
      gId
    ], (err, rows) => {
      if (err) {
        console.log('GROUPS-MODEL | Error loading gears for group from core data for group id ' + gId + ': ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }


  /**
   * returns all gear groups which can be alarmed from core database
   * @param {function} callback rows or null
   */
  getListOfAlarmableGroups(callback) {
    this.logger.debug('MODELS-GEARGROUPS | Reading alarmable Groups');
    let query = "SELECT " +
      "gearGroups.*, " +
      "organizations.name AS orgName, " +
      "organizations.shortName AS orgShortName " +
      "FROM `gearGroups` " +
      "INNER JOIN organizations ON organizations.id = gearGroups.organizationid " +
      "ORDER BY organizations.shortName ASC";

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-GEARGROUPS | Error loading alarmable gear groups from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }


  /**
   * Returns list of groups for admin UI table view
   * @param {function} callback rows or null
   */
  getlistOfGroupsForUI(callback) {
    //this.logger.debug('MODELS-GEARGROUPS | Reading Groups');
    let query = "SELECT gearGroups.id," +
      "gearGroups.shortName, " +
      "gearGroups.groupname," +
      "organizations.id AS organizationid, " +
      "organizations.name AS organizationname, " +
      "organizations.city AS organizationcity " +
      "FROM `gearGroups` " +
      "INNER JOIN organizations ON gearGroups.organizationid = organizations.id";

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        console.log('GROUPS-MODEL | Error loading groups from core data for UI: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns rows with groups for given organization id
   * @param {integer} orgId Organization id groups to be loaded for
   * @param {*} callback rows or null
   */
  getListOfGroupsForOrganization(orgId, callback) {
    this.logger.debug('MODELS-GEARGROUPS | Reading Groups for organization ' + orgId);
    let query = "SELECT * FROM `gearGroups` WHERE organizationid = ?";

    global.coreDb.all(query, [
      orgId
    ], (err, rows) => {
      if (err) {
        this._logger.error('GROUPS-MODEL | Error loading groups for organization with id ' + orgId + ' from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns rows with groups for given asset id
   * @param {integer} assetId asset id groups should be loaded for
   * @param {function} callback rows or null
   */
  getListOfGroupsForBaseData(assetId, callback) {
    this.logger.debug('MODELS-GEARGROUPS | reading Groups for asset with id ' + basedataId);
    let query = "SELECT groupId FROM xt_gear_gearGroups WHERE assetId = ?";

    global.coreDb.all(query, [
      basedataId
    ], (err, rows) => {
      if (err) {
        this._logger.error('GROUPS-MODEL | Error loading groups for asset with id ' + assetId + ' from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns assets rows for a given group id
   * @param {integer} groupId id of group to load assets for
   * @param {function} callback rows or null
   */
  getAssetsForGroupWithId(groupId, callback) {
    let query = "SELECT " +
      "xt_gear_gearGroups.assetId, " +
      "assets.title, " +
      "assets.radioname, " +
      "assets.issi, "
    "assets.id AS assetId, " +
      "organizations.id AS orgId, " +
      "organizations.name AS orgName " +
      "FROM xt_gear_gearGroups " +
      "INNER JOIN assets ON assets.id = xt_gear_gearGroups.assetId " +
      "INNER JOIN organizations ON organizations.id = assets.ownerOrgId " +
      "WHERE xt_gear_gearGroups.groupId = ?";
    global.coreDb.all(query, [
      groupId
    ], (err, rows) => {
      if (err) {
        this._logger.error('GROUPS-MODEL | Error loading basedata for group with id ' + groupId + ' from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * 
   * @param {Array} groupsArray Array of new groups to be added
   * @param {integer} assetId asset id the new groups shall be added for
   * @param {function} callback true or false (success)
   */
  assignAssetToUser(groupsArray, assetId, callback) {
    this.logger.debug('MODELS-GEARGROUPS | Assigning Group(s) to asset with assetid: ' + assetId);
    let delQuery = "DELETE FROM xt_gear_gearGroups WHERE assetId = ?";
    let valuesPart = '';
    global.coreDb.run(delQuery,
      [
        assetId
      ],
      function (err) {
        if (err) {
          this.logger.error('MODELS-GEARGROUPS | Error removing existing groups for asset with id ' + assetId + ': ' + err.message);
          callback(false);
        } else {
          // successfully removed existing groups
          // Now add the new array as rows to core database
          if (Array.isArray(groupsArray)) {
            let values = [];
            groupsArray.forEach(group => {
              values.push('(' + assetId + ', ' + group[0] + ')');
            });
            valuesPart = values.join(',');
          } else {
            valuesPart = '(' + assetId + ', ' + groupsArray + ') ';
          }

          if (valuesPart.length > 1) {
            let query = "INSERT INTO xt_gear_gearGroups (assetId, groupId) VALUES " + valuesPart + ";";
            global.coreDb.run(query, [], function (err) {
              if (err) {
                console.log('ERROR Adding new groups for asset with id ' + assetId + ' enty to core database: ' + err.message);
                callback(false);
              } else {
                callback(true);
              }
            });
          } else {
            callback(false);
          }
        }
      });
  }

  /**
   * adds / updates gear for gear group
   * @param {integer} groupId id of group to add gear assets to
   * @param {*} gearlistArray array of asset ids of gear to add
   * @param {*} callback success (true/false)
   */
  persistGearForGroup(groupId, gearlistArray, callback) {
    console.log('adding gear to group ' + groupId);
    if (Array.isArray(gearlistArray)) {
      // first make sure, that no old entrys exist for this group
      let query = "DELETE FROM xt_gear_gearGroups WHERE groupId = ?";
      global.coreDb.run(query, [
        groupId
      ], function (err) {
        if (err) {
          console.log('Error deleting existing entries from xt_gear_gearGroups for gear group with id' + groupId);
          return callback(false);
        } else {
          global.coreDb.serialize(() => {
            let query = "INSERT INTO xt_gear_gearGroups (assetId, groupId) VALUES (?, ?)";
            gearlistArray.forEach((gear) => {
              global.coreDb.run(query, [
                gear,
                groupId
              ], function (err) {
                if (err) {
                  console.log('Error inserting new gear asset to table xt_gear_gearGroups:' + err);
                  return callback(false);
                }
              })

            })
          })
          return callback(true);
        }
      })
    } else {
      return callback(true)
    }
  }

  /**
   * Adds a new Gear Group to core database
   * @param {string} groupName Name of the group as group should be displayed in UI
   * @param {integer} orgId organization id the group should be added for
   * @param {array} gearlistArray Array of gear which should be added to this group
   * @param {function} callback success, true or false
   */
  addGroupforOrganization(groupName, orgId, shortName, gearlistArray, callback) {
    var that = this;
    this.doesGroupExistForOrganization(groupName, orgId, (groupExists) => {
      if ((groupExists !== null) && (groupExists !== true)) {
        // group seems not to exist -> add group to core database
        let query = "INSERT INTO `gearGroups` (groupname, organizationid, shortName) VALUES (?, ?, ?)";
        global.coreDb.run(query, [
          groupName,
          orgId,
          shortName
        ], function (err) {
          if (err) {
            console.log('ERROR Adding new gear group for organization with id ' + orgId + 'to core database: ' + err.message);
            callback(false);
          } else {
            console.log('successfully added gear group');
            // now iterate through gear to add them to group
            if (Array.isArray(gearlistArray)) {
              // first make sure, that no old entrys exist for this group

              that.persistGearForGroup(this.lastID, gearlistArray, (success) => {
                if (success) {
                  return callback(true);
                } else {
                  return callback(false);
                }
              })
            } else {
              console.log('gearlistArray is not an array');
              return callback(true);
            }
          }
        });
      } else {
        // can not add group as group exists or check could not be performed
        return callback(false);
      }
    })
  }

  /**
   * 
   * @param {integer} gId Group-ID
   * @param {string} groupName New group name
   * @param {integer} oId organization ID
   * @param {array} gearlistArray array of gear asset ids to be added to the gear group
   * @param {function} callback success, false or true
   */
  updateGroup(gId, groupName, oId, shortName, gearlistArray, callback) {
    var that = this;
    this.doesGroupExistForOrganization(groupName, oId, (bolExists, rows) => {
      var existingGroupId = 0;
      if (rows !== null) {
        existingGroupId = rows[0].id;
        try {
          if (parseInt(existingGroupId) === parseInt(gId)) {
            bolExists = false;
          }
        } catch (err) {
          return callback(false);
        }
      }
      if (bolExists !== true) {
        let query = "UPDATE `gearGroups` SET `groupname` = ?, organizationid = ?, shortName = ?  WHERE `gearGroups`.`id` = ?";
        global.coreDb.run(query, [
          groupName,
          oId,
          shortName,
          gId
        ], function (err) {
          if (err) {
            console.log('ERROR Updating gear Group with id ' + gId + ' in core database: ' + err.message);
            return callback(false);
          } else {
            that.persistGearForGroup(gId, gearlistArray, (success) => {
              if (success) {
                return callback(true);
              } else {
                return callback(false);
              }
            })
          }
        })
      } else {
        return callback(false);
      }
    })
  }

  /**
   * Deletes a group from core database and also removes gear group from alert rules upfront
   * @param {integer} gId group ID of group to be deleted
   * @param {function} callback success, true or false
   */
  deleteGroup(gId, callback) {
    let query = "SELECT * FROM alertrulesGearGroupsStore WHERE groupId = ?";
    global.coreDb.run(query, [
      gId
    ], function (err) {
      if (err) {
        console.log('ERROR Deleting group with id ' + gId + ' from alertrulesGearGroupsStore in core database: ' + err.message);
        return callback(false);
      } else {
        let query = "DELETE from gearGroups WHERE id = ?";
        global.coreDb.run(query, [
          gId
        ], function (err) {
          if (err) {
            console.log('ERROR Deleting group with id ' + gId + ' in core database: ' + err.message);
            return callback(false);
          } else {
            return callback(true);
          }
        })
      }
    })

  }

  /**
   * 
   * @param {string} groupName Name of the group
   * @param {integer} orgId organization id
   * @param {function} callback success true -> exists, false -> not existing, null -> error in query
   */
  doesGroupExistForOrganization(groupName, orgId, callback) {
    let query = "SELECT * FROM gearGroups WHERE groupname = ? AND organizationid = ?";
    global.coreDb.all(query, [
      groupName,
      orgId
    ], (err, rows) => {
      if (err) {
        this._logger.error('GROUPS-MODEL | Error checking for existing gear group in organization: ' + err.message);
        return callback(null);
      } else {
        if (rows.length > 0) {
          return callback(true, rows);
        } else {
          return callback(false, null);
        }
      }
    });
  }
}

// export the class
module.exports = gearGroups;