/**
 * Class represents goods receipts within ALARMiator and offers
 * methods to handle goods receipts related data in core database
 * @class
 */

class GoodsReceipts {

    /**
     * constructor
     */
    constructor() {

    }


    /**
     * Returns all goods receipts as rows from core database
     * @param {function} callback rows or null
     */
    getReceipts(callback) {
        let query = "SELECT assets_goods_receipts.*, " +
            "assets_orders.orderStateId AS orderStateId, " +
            "assets_orders.orderNumber AS orderNumber, " +
            "assets_orders.supplierId AS orderSupplierId, " +
            "assets_orders_states.label AS stateLabel, " +
            "suppliers.companyname AS supplier_companyname, " +
            "suppliers.website AS supplier_website " +
            "FROM assets_goods_receipts " +
            "JOIN assets_orders ON assets_orders.id = assets_goods_receipts.orderId " + 
            "JOIN assets_orders_states ON assets_orders_states.identifier = assets_orders.orderStateId " +
            "JOIN suppliers ON suppliers.id = assets_orders.supplierId";

        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns a goods receipt entry from core database for given receipt id
     * @param {integer} id id of receipt in coredatabase
     * @param {function} callback rows or null
     */
    getReceipt(id, callback) {
        let query = "SELECT assets_goods_receipts.*, " +
            "assets_orders.orderStateId AS orderStateId, " +
            "assets_orders.orderNumber AS orderNumber, " +
            "assets_orders.supplierId AS orderSupplierId, " +
            "assets_orders_states.label AS stateLabel, " +
            "suppliers.companyname AS supplier_companyname, " +
            "suppliers.website AS supplier_website " +
            "FROM assets_goods_receipts " +
            "JOIN assets_orders ON assets_orders.id = assets_goods_receipts.orderId " + 
            "JOIN assets_orders_states ON assets_orders_states.identifier = assets_orders.orderStateId " +
            "JOIN suppliers ON suppliers.id = assets_orders.supplierId";
            "WHERE assets_goods_receipts.id = ?";

        global.coreDb.all(query, [
            id
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * adds a new goods receipt to database
     * @param {object} reqBody request body holding all form fields (usually req.body)
     * @param {function} callback success (true/false), reason (string)
     */
    addReceipt(reqBody, callback) {
        var orderId = reqBody.orderId;
        var receiptNumber = reqBody.receiptNumber;
        var deliveryDate = reqBody.deliveryDate;
        var orderedByBasedataId = reqBody.orderedByBasedataId;
        var notes = reqBody.notes;


        let query = `INSERT INTO assets_goods_receipts (
                        orderId,
                        deliveryDate,
                        quantity,
                        deliveryIsComplete,
                        deliveryIsPartial,
                        inspectedByBasedataId, 
                        notes,
                        receiptNumber
                    ) VALUES (?,?,?,?,?,?,?,?,?,?)`;

        global.coreDb.run(query, [
            orderId,
            deliveryDate,
            quantity,
            deliveryIsComplete,
            deliveryIsPartial,
            orderedByBasedataId, 
            notes,
            receiptNumber
        ], function (err) {
            if (err) {
                console.log(`ERROR Adding new goods receipt to core database: ${err.message}`);
                callback(false, null);
            } else {
                console.log('Added new goods receipt with id: ' + this.lastID);
                var orderId = this.lastID;
                callback(true, orderId);
            }
        });
    }

    /**
     * Removes a receipt permanent from core database
     * @param {integer} id id of receipt which should be deleted
     * @param {*} callback success (true / false)
     */
    removeReceipt(id, callback) {
        let query = "DELETE FROM assets_goods_receipts WHERE id = ?";
        global.coreDb.run(query, [
            id
        ], function (err) {
            if (err) {
                console.log(`ERROR Deleting goods receipt from core database: ${err.message}`);
                callback(false);
            } else {
                callback(true);
            }
        });
    }

    /**
     * Edits a goods receipt with given id in core database 
     * @param {integer} id id of goods receipt to be edited
     * @param {object} reqBody object holding all form fields (usually req.body)
     * @param {function} callback success (true/false)
     */
    editReceipt(id, reqBody, callback) {
        var supplierId = reqBody.supplierId;
        var orderNumber = reqBody.orderNumber;
        var orderDate = reqBody.orderDate;
        var orderStateId = reqBody.orderStateId;
        var initialorderStateId = reqBody.initialorderStateId;
        var notes = reqBody.notes;
        var that = this;

        let query = "UPDATE assets_orders SET " +
            "supplierId = ?, " +
            "orderNumber = ?, " +
            "orderDate = ?, " +
            "orderStateId = ?, " +
            "notes = ? " +
            "WHERE id = ?";
        global.coreDb.run(query, [
            supplierId,
            orderNumber,
            orderDate,
            orderStateId,
            notes,
            id
        ], function (err) {
            if (err) {
                console.log(`ERROR Updating order to core database: ${err.message}`);
                return callback(false);
            } else {
                console.log('initialorderStateId: ' + initialorderStateId);
                if (initialorderStateId === 'registration') {
                    that.addPositions(id, reqBody, (successPositions) => {
                        return callback(true);
                    })
                } else {
                    return callback(true);
                }
            }
        });
    }

    
}

module.exports = GoodsReceipts;