const Operation = require("./core/core_operation");
const Organizations = require('./organizations');
const Configuration = require('./configuration');


// Constructor
class Operations {
    constructor(logger) {
      // always initialize all instance properties
      this.logger = logger;
    }

    /**
     * returns list of all operations
     * @param {function} callback rows or null
     */
    getListOfOperations(callback) {
        //global.plgManager.logger.debug('MODELS-OPERATIONSHELPER | Reading operations list');
        let sql = `SELECT * FROM operationsStore ORDER BY id DESC`;

        global.coreDb.all(sql, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading list of operations from database: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * Returns List of active Operations a member has been alarmed for
     * @param {string} basedataUUID uuid of basedata to return active operations for which have been alarmed for user
     * @param {function} callback rows or null
     */
    getListOfActiveOperationsUserWasAlarmedFor(basedataUUID, callback) {
        let sql = "SELECT DISTINCT " +
                    " operationsStore.* " +
                  "FROM operationsStore " +
                  "JOIN feedbackStore ON feedbackStore.operationId = operationsStore.id " +
                  "WHERE " +
                    " operationsStore.operationState = 0 " +
                  "AND " +
                    " feedbackStore.basedataUUID = ?";

        global.coreDb.all(sql, [
            basedataUUID
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading list of operations from database: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * Returns list of all active operations
     * @param {function} callback rows or null
     */
    getListOfActiveOperations(callback) {
        //global.plgManager.logger.debug('MODELS-OPERATIONSHELPER | Reading active operations list');
        let sql = `SELECT * FROM operationsStore WHERE operationState = 0 ORDER BY id DESC`;

        global.coreDb.all(sql, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading list of active operations from database: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * Returns last operation coming in
     * @param {function} callback 
     */
    getLastOperation(callback) {
        let sql = "SELECT * FROM operationsStore ORDER BY id DESC LIMIT 1";
        //global.plgManager.logger.debug('MODELS-OPERATIONSHELPER | Reading last operation');
        global.coreDb.all(sql, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading last operation from database: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * Returns one single operation from operationsStore in core database
     * @param {integer} oId id of operation to be loaded from core datbase
     * @param {function} callback row or null
     */
    getOperation(oId, callback) {
        let sql = "SELECT * FROM operationsStore WHERE id = ?";
        global.plgManager.logger.debug(`MODELS-OPERATIONSHELPER | Reading operation with id ${oId}`);
        global.coreDb.all(sql, [
            oId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading operation from database: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * Returns one single operation from operationsStore in core database as an operation object
     * @param {integer} oId 
     * @param {function} callback operation object or null
     */
    getOperationAsObject(oId, callback) {
        this.getOperation(oId, (rowsOperation) => {
            if (rowsOperation != null) {
                var operation = this.createOperationFromObject(rowsOperation[0]);
                return callback(operation);
            } else {
                // Operation not found in database
                return callback(null);
            }
        });
    }

    /**
     * Returns one single operation from operationsStore in core database
     * @param {integer} oId id of operation to be loaded from core datbase
     * @param {function} callback row or null
     */
    getOperationForUUID(uuid, callback) {
        let sql = "SELECT * FROM operationsStore WHERE uuid = ?";
        //global.plgManager.logger.debug(`MODELS-OPERATIONSHELPER | Reading operation with uuid ${uuid}`);
        global.coreDb.all(sql, [
            uuid
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading operation from database: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * Returns list of operations ids for the given organization
     * @param {integer} orgId id of organization
     * @param {function} callback rows or null
     */
    getOperationsForOrganization(orgId, callback) {
        let sql = "SELECT * FROM xt_operations_organizations WHERE organizationsId = ?";
        global.coreDb.all(sql, [
            orgId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading operations for organization: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * Returns one single operation from operationsStore in core database as an operation object
     * @param {string} uuid 
     * @param {function} callback operation object or null
     */
    getOperationAsObjectFromUUID(uuid, callback) {
        this.getOperationForUUID(uuid, (rowsOperation) => {
            if (rowsOperation != null) {
                var operation = this.createOperationFromObject(rowsOperation[0]);
                return callback(operation);
            } else {
                // Operation not found in database
                return callback(null);
            }
        });
    }

    /**
     * deletes one single operation from operationsStore in core database
     * @param {integer} oId id of operation to be loaded from core datbase
     * @param {function} callback row or null
     */
    deleteOperation(oId, callback) {
        let sql = "DELETE FROM operationsStore WHERE id = ?";
        global.plgManager.logger.debug(`MODELS-OPERATIONSHELPER | DELETE operation with id ${oId}`);
        global.coreDb.all(sql, [
            oId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error deleteing operation from database: ' + err);
                return callback(null);
            }
            return callback(null);
        });
    }

    /**
     * Returns sub operations for given operation from operationsStore in core database
     * @param {integer} oId id of operation sub operations to be loaded from core datbase for
     * @param {function} callback rows or null
     */
    getSubOperationsForOperation(oId, callback) {
        let sql = "SELECT * FROM operationsStore WHERE subOperationFor = ?";
        global.plgManager.logger.debug(`MODELS-OPERATIONSHELPER | Reading sub operations for operation with id ${oId}`);
        global.coreDb.all(sql, [
            oId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading sub operations from database: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * sets the archived flag for an operation
     * @param {integer} oId id of operation to be archived
     * @param {function} callback success (true/false)
     */
    archiveOperation(oId, callback) {
        let query = "UPDATE operationsStore SET operationState = 2 WHERE id = ?";
        this.logger.debug(`MODELS-OPERATIONSHELPER | archiving operation with id ${oId}`);
        global.coreDb.all(query, [
            oId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error archiving operation: ' + err);
                return callback(false);
            } else {
                global.plgManager.event_alarm_archived(oId);
                return callback(true);
            }
            
        });
    }

    /**
     * sets the archived flag for an operation identified by uuid
     * @param {string} uuid id of operation to be archived
     * @param {function} callback success (true/false)
     */
    archiveOperationByUUID(uuid, callback) {
        let query = "UPDATE operationsStore SET operationState = 2 WHERE uuid = ?";
        this.logger.debug(`MODELS-OPERATIONSHELPER | archiving operation with uuid ${uuid}`);
        global.coreDb.all(query, [
            uuid
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error archiving operation: ' + err);
                return callback(false);
            } else {
                return callback(true);
            }
        });
    }

     /**
     * sets the closed flag for an operation
     * @param {integer} oId id of operation to be archived
     * @param {function} callback success (true/false)
     */
    closeOperation(oId, callback) {
        let query = "UPDATE operationsStore SET operationState = 1 WHERE id = ?";
        this.logger.debug(`MODELS-OPERATIONSHELPER | closing operation with id ${oId}`);
        global.coreDb.all(query, [
            oId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error closing operation: ' + err);
                return callback(false);
            } else {
                global.plgManager.event_alarm_closed(oId);
                return callback(true);
            }
            
        });
    }

    /**
     * Closes an operation in case an updated version of alarm came in
     * @param {string} operationnumber operationnumber of operation to close earlier operations for
     * @param {integer} excludedOpId id if operation not allowed to be closed (as it is the new version)
     * @param {*} callback success (true/false)
     */
    closeOperationCauseOfUpdatedAlarm(operationnumber, excludedOpId, callback) {
        let query = "UPDATE operationsStore SET operationState = 5 WHERE operationnumber = ? AND NOT id = ?";
        this.logger.debug(`MODELS-OPERATIONSHELPER | closing operation as new version came in, id ${operationnumber}`);
        global.coreDb.all(query, [
            operationnumber, 
            excludedOpId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error closing operation as new version came in: ' + err);
                return callback(false);
            } else {
                global.plgManager.event_alarm_closed(operationnumber);
                return callback(true);
            }
            
        });
    }

    /**
     * sets the closed flag for an operation identified by uuid
     * @param {string} uuid uuid of operation to be archived
     * @param {function} callback success (true/false)
     */
    closeOperationByUUID(uuid, callback) {
        let query = "UPDATE operationsStore SET operationState = 1 WHERE uuid = ?";
        this.logger.debug(`MODELS-OPERATIONSHELPER | clsoing operation with uuid ${uuid}`);
        global.coreDb.all(query, [
            uuid
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error closing operation: ' + err);
                return callback(false);
            }
            return callback(true);
        });
    }

    /**
     * Closes all operations which are active
     * @param {function} callback success true /false
     */
    closeAllOperations(callback) {
        let query = "UPDATE operationsStore SET operationState = 1 WHERE operationState = 0";
        this.logger.debug(`MODELS-OPERATIONSHELPER | closing all operations`);
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error closing all operations: ' + err);
                return callback(false);
            }
            global.plgManager.event_alarm_closed(null);
            return callback(true);
        });
    }

    /**
     * Reactivates an operation from state closed or archived to active
     * @param {integer} oId id of operation to reactivate
     * @param {function} callback success (true/false)
     */
    activateOperation(oId, callback) {
        let query = "UPDATE operationsStore SET operationState = 0 WHERE id = ?";
        this.logger.debug(`MODELS-OPERATIONSHELPER | activating operation with id ${oId}`);
        global.coreDb.all(query, [
            oId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error activating operation: ' + err);
                return callback(false);
            } else {
                return callback(true);
            }
            
        });
    }

    
    /**
     * Returns list of operations from core database as JSON for map view
     * @param {function} callback JSON String of operations
     */
    getListOfOperationsJSON(callback) {
        this.logger.info('Reading operations as JSON');
        let query = "SELECT * FROM operationsStore";

        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this.logger.error('OPERATIONS-MODEL | Error loading list of operations from core data: ' + err.message);
                return callback(null);
            } else {
                var counter = 0;
                this.logger.info('OPERATIONS-MODEL | Read ' + rows.length + ' operations successfully');
                var returnJSON = "[";

                rows.forEach(operation => {
                    counter++;
                    var dividerStr = '';
                    if (counter > 0) {
                        dividerStr = ', ';
                    }
                    returnJSON = returnJSON + dividerStr + '{ lat: ' + operation.lat + ', lon: ' + operation.lon + ', keywordId: \'' + operation.keywordId + '\'}';
                });

                returnJSON = returnJSON + ']';
                //TEST
                returnJSON = JSON.stringify(rows);
                return callback(returnJSON);
            }
        });
    }

    /**
     * Returns JSON object with organizations and assigned active operations
     * @param {array} organizationsZveisGearnames Result of methode organizations: getOrganizationsWithZveisAndGearnames
     * @param {array} activeOperationList all active operations as rows
     */
    sortOperationsByOrganizations(organizationsZveisGearnames, activeOperationList){

        var result = {};

        if (organizationsZveisGearnames == null) {
            return result;
        }

        organizationsZveisGearnames.forEach((organization) => {
            result[organization.organizationsName] = [];
        })

        activeOperationList.forEach((operation) => {
            var zveis;
    
            if (operation.zveisListCalc.length > 0) {
                zveis = JSON.parse(operation.zveisListCalc).join('|');
            }
            else {
                // always false
                zveis = '(?=a)b'
            }
            
            var gearList;
        
            if (operation.gearListCalc.length > 0) {
                gearList = JSON.parse(operation.gearListCalc).join('|');
            }
            else {
                // always false
                gearList = '(?=a)b'
            }

            organizationsZveisGearnames.map((organization) => {
    
                var organizationZveis = organization.zveis;
                var organizationGearnames = organization.gearnames;
        
                if (organizationZveis != null && organizationZveis.match(zveis)){
                    result[organization.organizationsName].push(operation);
                    return;
                }
        
                if (organizationGearnames != null && organizationGearnames.match(gearList)){
                    result[organization.organizationsName].push(operation);
                    return;
                }      
            })
        })
        return result
    }  


    /**
     * Returns Organizations which are involved in operation (identified by zvei code)
     * @param {integer} oId operationId
     * @param {function} callback array with organization ids which have been alarmed by operation
     */
    getOrganizationsAlarmedByOperation(oId, callback) {
        this.getOperation(oId, (rowsOperation) => {
            if (rowsOperation != null) {
                //console.log('rowsoperation != null');
                if (rowsOperation.length > 0) {
                    //console.log('rowsoperation.length > 0');
                    // Found operation for given oId
                    var operation = rowsOperation[0];
                    var zveis = [];
                    try {
                        zveis = JSON.parse(operation.zveisListCalc);
                    } catch (err) {
                        this.logger.error('MODELS-OPERATIONSHELEPER | Error parsing zveisListCalc for Operation with id ' + oId);
                        return callback(null);
                    }
                    //console.log('zveis: ');
                    //console.log(zveis);

                    // Now iterate through organizations to get the organizations which have been alarmed with given operation
                    var organizations = new Organizations(this.logger);
                    var alarmedOrganizations = [];
                    organizations.getListOfOrganizations((rowsOrganizations) => {
                        if (rowsOrganizations != null) {
                            //console.log('rowsOrganizations != null');
                            // loaded organizations successfully
                            rowsOrganizations.forEach((organization) => {
                                //console.log('cycle through organization');
                                let orgZveisStr = organization.zveis;
                                if (orgZveisStr != null) {
                                    let orgZveisArr = orgZveisStr.split(',');
                                    if (orgZveisArr.length > 0) {
                                        //console.log('found zveis for organization');
                                        orgZveisArr.forEach((zvei) => {
                                            if (zveis.length > 0) {
                                                //console.log('cycle through zveis');
                                                zveis.forEach((zveiOperation) => {
                                                    //console.log('cycle through operation zveis');
                                                    if (zveiOperation.trim() === zvei.trim()) {
                                                        // we have a match
                                                        this.logger.log('MODELS-OPERATIONSHELEPER | identified alarmed organization with id ' + organization.id + ' for zvei ' + zvei.trim());
                                                        alarmedOrganizations.push(organization.id);
                                                    }
                                                })
                                            }
                                        })
                                    }
                                } else {
                                    this.logger.log('MODELS-OPERATIONSHELEPER | check for zvei in organiuation with ' + organization.id + ' not done as organization has no zveis set.');
                                }
                            });
                            // Now return array of alarmed organization Ids
                            return callback(alarmedOrganizations);
                        } else {
                            this.logger.error('MODELS-OPERATIONSHELEPER | Error reading organizations to check alarmed organizatiosn for Operation with id ' + oId);
                            return callback(null);
                        }
                    })

                } else {
                    // operation could not be found in core database
                    this.logger.error('MODELS-OPERATIONSHELEPER | Error could not find operation with id ' + oId);
                    return callback(null);
                }
            } else {
                // query for operation returned null
                this.logger.error('MODELS-OPERATIONSHELEPER | Error could not find operation with id ' + oId + ' (returned null)');
                return callback(null);
            }
        })
    }

    /**
     * Adds organiaztions to operation in table xt_operations_organizations
     * @param {array} arrOrganizationIds array of organization IDs
     * @param {integer} operationId operation id
     * @param {function} callback success (true/false)
     */
    addOrganizationsToOperation(arrOrganizationIds, operationId, callback) {
        var success = true;
        global.coreDb.serialize(() => {
            if (Array.isArray(arrOrganizationIds)) {
                arrOrganizationIds.forEach((orgId) => {
                    let query = "INSERT INTO xt_operations_organizations (operationsId, organizationsId) VALUES (?, ?);";

                    global.coreDb.run(query, [
                        operationId,
                        orgId
                    ], function (err) {
                        if (err) {
                            this.logger.error('MODELS-OPERATIONSHELPER | Error persisting operation for organization: ' + err.message);
                            success = false;
                        } else {
                            this.logger.log('MODELS-OPERATIONSHELPER | Successfully persisted operation with id ' + operationId + ' for organization with id ' + orgId);
                        }
                    }.bind(this));
                })
            }
        });
        return callback(success);
    }

    /**
     * central handler for maintenance tasks to be started for operations
     */
    doMaintenance() {
        var configuration = new Configuration(this.logger);
        console.log('MODELS-OPERATIONSHELPER | Starting doMaintenance operations');
        configuration.getValueForKeyword('alarmingCloseAfterMinutes', (strAlarmingCloseAfterMinutes) => {
            try {
                var alarmingCloseAfterMinutes = parseInt(strAlarmingCloseAfterMinutes);
                if (isNaN(alarmingCloseAfterMinutes)) {
                    alarmingCloseAfterMinutes = 0;
                    console.log('MODELS-OPERATIONSHELPER | Time to close operations automatically is not set in global server settings. Automatic closing will not be performed. (' + strAlarmingCloseAfterMinutes + ')');
                }
            } catch (err) {
                var alarmingCloseAfterMinutes = 0;
            }
            console.log('MODELS-OPERATIONSHELPER | doMaintenance operations - alarmingCloseAfterMinutes: ' + alarmingCloseAfterMinutes);
            if (alarmingCloseAfterMinutes > 0) {
                this.closeLongRunningOperations(alarmingCloseAfterMinutes, (result) => {
                    console.log('MODELS-OPERATIONSHELPER | doMaintenance operations result: ' + result);
                })
            } else {
                console.log('MODELS-OPERATIONSHELPER | value for automatic closing of operation is set to 0. Doing nothing!')
            }
        })
    }

    /**
     * Close all operations which are older than given age in minutes
     * @param {integer} ageInMinutes 
     * @param {function} callback
     */
    closeLongRunningOperations(ageInMinutes, callback) {
        var param = '-' + ageInMinutes + ' minutes';

        //let query = "UPDATE operationsStore SET operationState = 1 WHERE (createdAt <= datetime('now', '" + param + "', 'localtime')) AND (operationState != 1)";

        let query = "UPDATE operationsStore SET operationState = 1 WHERE (createdAt <= datetime('now', '" + param + "', 'localtime')) AND (operationState != 1)";

        this.logger.debug(`MODELS-OPERATIONSHELPER | closing all operations older than ` + ageInMinutes + ` minutes`);
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error closing all operations older than ' + ageInMinutes + ' minutes');
                console.log(err);
                return callback(false);
            }
            global.plgManager.event_alarm_closed(null);
            return callback(true);
        });
    }

    createOperationFromObject(operationObj){
        var operation = new Operation();
        operation.alarmdate = operationObj.alarmdate;
        operation.alarmtime = operationObj.alarmtime;
        operation.gear = operationObj.gear;
        operation.community = operationObj.community;
        operation.district = operationObj.district;
        operation.location = operationObj.location;
        operation.gkx = operationObj.gkx;
        operation.gky = operationObj.gky;
        operation.keywordCategory = operationObj.keywordCategory;
        operation.keywordId = operationObj.keywordId;
        operation.keywordName = operationObj.keywordName;
        operation.keywordRaw = operationObj.keywordRaw;
        operation.lat = operationObj.lat;
        operation.lon = operationObj.lon;
        operation.message = operationObj.message;
        operation.object = operationObj.object;
        operation.operationSourceConnector = operationObj.operationSourceConnector;
        operation.operationnumber = operationObj.operationnumber;
        operation.rawdata = operationObj.rawdata;
        operation.section = operationObj.section;
        operation.street = operationObj.street;
        operation.subject = operationObj.subject;
        operation.zveis = operationObj.zveis;
        operation.floor = operationObj.floor;
        operation.ils = operationObj.ils;
        operation.uuid = operationObj.uuid;
        operation.isTest = operationObj.isTest;
        operation.createdAt = operationObj.createdAt;
        try {
            if (operationObj.zveisDeltaCalc != null && operationObj.zveisDeltaCalc != "") {
                operation.zveisDeltaCalc = JSON.parse(operationObj.zveisDeltaCalc);
            }
        }
        catch (exception) {
            operation.zveisDeltaCalc = operationObj.zveisDeltaCalc;
        }
        try {
            if (operationObj.gearDataCalc != null && operationObj.gearDataCalc != "") {
                operation.gearDataCalc = JSON.parse(operationObj.gearDataCalc); 
            }
        }
        catch (exception) {
            operation.gearDataCalc = operationObj.gearDataCalc; 
        }
        return operation;
    }


    /**
     * Returns additional infos for one single operation from operationsStore in core database
     * @param {integer} oId id of operation to be loaded from core datbase
     * @param {function} callback row or null
     */
    getAdditionalInfosForOperationUUID(uuid, callback) {
        let sql = `SELECT *
         FROM 
            operationsStore 
         WHERE 
            uuid = ?`;
        //global.plgManager.logger.debug(`MODELS-OPERATIONSHELPER | Reading operation with uuid ${uuid}`);
        global.coreDb.all(sql, [
            uuid
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading operation from database: ' + err);
                return callback(null);
            }
            return callback(rows);
        });
    }

    /**
     * Returns alarmed groups for a given operation
     * @param {string} uuid operationUUID
     * @param {function} callback row or null
     */
    getAlarmedGroupsForOperationUUID(uuid, callback) {
        let sql = `SELECT 
            groups.groupname AS groupname,
            groups.shortName AS shortName,
            organizations.name AS organizationsName,
            organizations.shortName AS organizationsShortName
         FROM 
            xt_operations_usergroups 
         LEFT JOIN 
            groups ON groups.id = xt_operations_usergroups.usersgroupsId
         LEFT JOIN
            organizations ON groups.organizationid = organizations.id
         LEFT JOIN 
            operationsStore ON operationsStore.id = xt_operations_usergroups.operationsId
         WHERE 
            operationsStore.uuid = ?`;
        global.coreDb.all(sql, [
            uuid
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-OPERATIONSHELPER | Error reading alarmed groups from database: ' + err);
                return callback(null);
            }
            if (rows.length > 0){
                var result = {
                    value: rows,
                    label: 'Alarmierte Gruppen'
                }
            return callback(result);
            }
            return callback(null);
        });
    }

    /**
     * Adds alarmed group to operation
     * @param {integer} operationId id of operations object this feedbacks shall be added for
     * @param {array} alarmedGroupsArray array of UUIDs of basedata to add, e.g. ['uuid1', 'uuid2']
     * @param {function} callback success (true/false)
     */
    addAlarmedGroupsToOperation(operationId, alarmedGroupsArray, callback) {
        if (Array.isArray(alarmedGroupsArray)) {

            let query = "INSERT INTO xt_operations_usergroups " +
                "(operationsId, usersgroupsId) ";
            query += 'VALUES ';

            var counter = alarmedGroupsArray.length;
            alarmedGroupsArray.forEach((alarmedGroupId, index) => {
                query += " (" + operationId + ", " + alarmedGroupId + ")";
                if ((index + 1) < counter) {
                    query += ", ";
                }
            })

            console.log(query);

            global.coreDb.run(query, [], function (err) {
                if (err) {
                    console.log(`ERROR Adding new alarmed objects to core database: ${err.message}`);
                    callback(false);
                } else {
                    callback(true);
                }
            });
        } else {
            return callback(false);
        }
    };

  }

  // export the class
  module.exports = Operations;
  