/**
 * Class represents orders within ALARMiator and offers
 * methods to handle order related data in core database
 * @class
 */


class Orders {

    /**
     * constructor
     */
    constructor() {

    }


    /**
     * Returns all orders as rows from core database
     * @param {function} callback rows or null
     */
    getOrders(callback) {
        let query = "SELECT assets_orders.*, " +
            "assets_orders_states.label AS stateLabel, " +
            "suppliers.companyname AS supplier_companyname, " +
            "suppliers.website AS supplier_website " +
            "FROM assets_orders " +
            "JOIN assets_orders_states ON assets_orders_states.identifier = assets_orders.orderStateId " +
            "JOIN suppliers ON suppliers.id = assets_orders.supplierId";

        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * returns all open (not final processed) orders
     * @param {function} callback rows or null
     */
    getNotFulfilledOrders(callback) {
        let query = "SELECT assets_orders.*, " +
            "assets_orders_states.label AS stateLabel, " +
            "suppliers.companyname AS supplier_companyname, " +
            "suppliers.website AS supplier_website " +
            "FROM assets_orders " +
            "JOIN assets_orders_states ON assets_orders_states.identifier = assets_orders.orderStateId " +
            "JOIN suppliers ON suppliers.id = assets_orders.supplierId " + 
            "WHERE assets_orders_states.isFinalState != 1 ORDER BY assets_orders_states.id ASC";

        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns a order entry from core database for given order id
     * @param {integer} id id of order in coredatabase
     * @param {function} callback rows or null
     */
    getOrder(id, callback) {
        let query = "SELECT " +
            "assets_orders.*, " +
            "basedata.firstname, " +
            "basedata.lastname, " +
            "suppliers.companyname " +
            "FROM assets_orders " +
            "JOIN basedata ON basedata.id = assets_orders.orderedByBasedataId " +
            "JOIN suppliers ON suppliers.id = assets_orders.supplierId " +
            "WHERE assets_orders.id = ?";

        global.coreDb.all(query, [
            id
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns all order states from core database
     * @param {function} callback rows or null
     */
    getOrdersStates(callback) {
        let query = "SELECT * FROM assets_orders_states";
        global.coreDb.all(query, [
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns statistics for orders (e.g. for dashboard)
     * @param {function} callback rows or null
     */
    getOrderStatistics(callback) {
        let query = "SELECT COUNT(assets_orders.id) AS openOrders FROM assets_orders JOIN assets_orders_states ON assets_orders_states.identifier = assets_orders.orderStateId WHERE assets_orders_states.isFinalState != 1";
        global.coreDb.all(query, [
        ], (err, rows) => {
            if (err) {
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * adds a new order to database
     * @param {object} reqBody request body holding all form fields (usually req.body)
     * @param {function} callback success (true/false), reason (string)
     */
    addOrder(reqBody, callback) {

        var supplierId = reqBody.supplierId;
        var orderNumber = reqBody.orderNumber;
        var orderDate = reqBody.orderDate;
        var orderedByBasedataId = reqBody.orderedByBasedataId;
        var orderStateId = reqBody.orderStateId;
        var notes = reqBody.notes;

        let query = "INSERT INTO assets_orders (supplierId, orderNumber, orderDate, orderedByBasedataId, orderStateId, notes) VALUES (?,?,?,?,?,?)";

        global.coreDb.run(query, [
            supplierId,
            orderNumber,
            orderDate,
            orderedByBasedataId,
            orderStateId,
            notes
        ], function (err) {
            if (err) {
                console.log(`ERROR Adding new order to core database: ${err.message}`);
                callback(false, null);
            } else {
                console.log('Added new order with id: ' + this.lastID);
                var orderId = this.lastID;
                callback(true, orderId);
            }
        });
    }

    /**
     * Renmoves a order permanent from core database
     * @param {integer} id id of order which should be deleted
     * @param {*} callback success (true / false)
     */
    removeOrder(id, callback) {
        let query = "DELETE FROM assets_orders WHERE id = ?";
        global.coreDb.run(query, [
            id
        ], function (err) {
            if (err) {
                console.log(`ERROR Deleting order from core database: ${err.message}`);
                callback(false);
            } else {
                callback(true);
            }
        });
    }

    /**
     * Sets the state of an existing order
     * @param {integer} id id of order the state shall be set for
     * @param {integer} orderStateId 1 = active, 0 = inactive
     * @param {function} callback success (true/false)
     */
    setOrderState(id, stateId, callback) {
        let query = "UPDATE assets_orders SET orderStateId = ? WHERE id = ?";
        global.coreDb.run(query, [
            stateId,
            id
        ], function (err) {
            if (err) {
                console.log(`ERROR setting state for order in core database: ${err.message}`);
                callback(false);
            } else {
                callback(true);
            }
        });
    }

    /**
     * Edits a order with given id in core database 
     * @param {integer} id id of order to be edited
     * @param {object} reqBody object holding all form fields (usually req.body)
     * @param {function} callback success (true/false)
     */
    editOrder(id, reqBody, callback) {
        var supplierId = reqBody.supplierId;
        var orderNumber = reqBody.orderNumber;
        var orderDate = reqBody.orderDate;
        var orderStateId = reqBody.orderStateId;
        var initialorderStateId = reqBody.initialorderStateId;
        var notes = reqBody.notes;
        var that = this;

        let query = "UPDATE assets_orders SET " +
            "supplierId = ?, " +
            "orderNumber = ?, " +
            "orderDate = ?, " +
            "orderStateId = ?, " +
            "notes = ? " +
            "WHERE id = ?";
        global.coreDb.run(query, [
            supplierId,
            orderNumber,
            orderDate,
            orderStateId,
            notes,
            id
        ], function (err) {
            if (err) {
                console.log(`ERROR Updating order to core database: ${err.message}`);
                return callback(false);
            } else {
                console.log('initialorderStateId: ' + initialorderStateId);
                if (initialorderStateId === 'registration') {
                    that.addPositions(id, reqBody, (successPositions) => {
                        return callback(true);
                    })
                } else {
                    return callback(true);
                }
            }
        });
    }

    /**
     * stores all positions in core database
     * @param {integer} id id of order this positions shall be referenced to
     * @param {object} reqBody request body with all form fields
     * @param {function} callback success (true or false)
     */
    addPositions(id, reqBody, callback) {
        var positions = [];
        // check if we have positions sent by the form
        var i = 0;
        for (i = 1; i < 100; i++) {
            if (typeof reqBody['position[' + i + ']'] != 'undefined') {
                // position exists -> read content into array which will be persisted later
                var positionId = reqBody['positionId[' + i + ']'];
                var sum = reqBody['posSum[' + i + ']'];
                var title = reqBody['posTitle[' + i + ']'];
                var count = reqBody['posCount[' + i + ']'];
                var pos = {
                    position: i,
                    positionId: positionId,
                    title: title,
                    count: count,
                    sum: sum
                }
                positions.push(pos);
            }
        }
        // Now we have all existing positions in positions array
        // console.log(positions);
        if (Array.isArray(positions)) {
            if (positions.length > 0) {
                // We have collected positions, -> persists them now
                var formOrderIds = [];
                global.coreDb.serialize(() => {
                    positions.forEach((position) => {
                        var existing = false;
                        if (position.positionId != null) {
                            if (position.positionId > 0) {
                                existing = true;
                                formOrderIds.push(parseInt(position.positionId));
                            }
                        }

                        if (existing) {
                            let query = "UPDATE assets_orders_positions SET " +
                                "position = ?, " +
                                "title = ?, " +
                                "count = ?, " +
                                "sum = ? " +
                                "WHERE id = ?";
                            global.coreDb.run(query, [
                                position.position,
                                position.title,
                                position.count,
                                position.sum,
                                position.positionId
                            ], function (err) {
                                if (err) {
                                    console.log(`ERROR updating existing order position at core database: ${err.message}`);
                                    //return callback(false);
                                } else {
                                    console.log('updated existing position successfully');
                                }
                            });
                        } else {
                            let query = "INSERT INTO assets_orders_positions (orderId, position, title, count, sum, state) " +
                                "VALUES (?, ?, ?, ?, ?, ?)";
                            global.coreDb.run(query, [
                                id,
                                position.position,
                                position.title,
                                position.count,
                                position.sum,
                                1
                            ], function (err) {
                                if (err) {
                                    console.log(`ERROR inserting order position at core database: ${err.message}`);
                                    //return callback(false);
                                } else {
                                    console.log('inserted new position successfully');
                                }
                            });
                        }
                    })
                });
                this.inactivatePositionsNotInArray(id, formOrderIds, (successHousekeeping) => {
                    if (successHousekeeping) {
                        console.log('positions housekeeping done successfully');
                        return callback(true);
                    } else {
                        console.log('ERROR: positions housekeeping with error');
                        return callback(false);
                    }
                })
            } else {
                // no position handed over -> check if there are some at the database!
                // if so, they need to be set to canceled
                console.log('ERROR: no positions data came from web form');
                this.inactivatePositionsNotInArray(id, [], (successHousekeeping) => {
                    if (successHousekeeping) {
                        console.log('positions housekeeping done successfully');
                        return callback(true);
                    } else {
                        console.log('ERROR: positions housekeeping with error');
                        return callback(false);
                    }
                })
            }
        } else {
            // no position handed over -> check if there are some at the database!
            // if so, they need to be set to canceled
            console.log('ERROR: no positions data came from web form (null)');
            this.inactivatePositionsNotInArray(id, [], (successHousekeeping) => {
                if (successHousekeeping) {
                    console.log('positions housekeeping done successfully');
                    return callback(true);
                } else {
                    console.log('ERROR: positions housekeeping with error');
                    return callback(false);
                }
            })
        }

        //return callback(true);
    }

    /**
     * Returns all positions for an order with given id
     * @param {integer} id id of order to load positions for
     * @param {*} callback rows or null
     */
    getOderPositionsForOrder(id, callback) {
        let query = "SELECT * FROM assets_orders_positions WHERE orderId = ? AND state != 0";
        global.coreDb.all(query, [
            id
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * deactivates existing order positions not in array
     * @param {integer} id id of order to check for
     * @param {*} activePositions one-dimensional array of active order positions
     * @param {*} callback success (true/false)
     */
    inactivatePositionsNotInArray(id, activePositions, callback) {
        console.log('======= acitvePositions ===========');
        console.log(activePositions);
        if (Array.isArray(activePositions)) {
            let query = "SELECT * FROM assets_orders_positions WHERE orderId = ?";
            global.coreDb.all(query, [
                id
            ], (err, rows) => {
                if (err) {
                    return callback(false);
                } else {
                    var existingsIds = [];
                    if (rows.length > 0) {
                        rows.forEach((row) => {
                            existingsIds.push(row.id);
                        })
                        console.log('======= existingsPositionsFromDB ===========');
                        console.log(existingsIds);
                        global.coreDb.serialize(() => {
                            existingsIds.forEach((eId) => {
                                if (activePositions.includes(eId) === false) {
                                    console.log('==found missing position --> inactivate now');
                                    let query = "UPDATE assets_orders_positions SET state = 0 WHERE id = ?";
                                    global.coreDb.run(query, [
                                       eId
                                    ], function (err) {
                                        if (err) {
                                            console.log(`ERROR inserting order position at core database: ${err.message}`);
                                            return callback(false);
                                        }
                                    });
                                }
                            })
                        });
                        return callback(true);
                    } else {
                        // There are no order positions for this order id
                        return callback(true);
                    }
                    
                }
            });
        }
    }
}

module.exports = Orders;