var uuid = require('uuid'); // required for generating timebased guids
const OrganizationsTypes = require('./organizationstypes');



class organizations {
    constructor(logger) {
        // always initialize all instance properties
        this._logger = logger;
    }

    /**
     * returns rows of all organizations in core database
     * @param {*} req 
     * @param {*} res 
     * @param {*} callback 
     */
    getListOfOrganizations(callback) {
        let query = "SELECT " +
                    "organizations.*, " +
                    "organizationsTypes.title AS orgTypeTitle, " +
                    "organizationsTypes.icon AS icon, " +
                    "organizationsTypes.colors AS colors " +
                    "FROM organizations " +
                    "LEFT JOIN organizationsTypes ON organizationsTypes.id = organizations.organizationsType " + 
                    "ORDER BY parent ASC";
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                console.log(err);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }


    /**
     * Returns database row for organization with given orgId
     * @param {integer} orgId 
     * @param {rows} callback 
     */
    getOrganizationWithId(orgId, callback) {
        let query = "SELECT * FROM `organizations` WHERE id = ?";
        global.coreDb.all(query, [
            orgId
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Adds a new organization to core database (sqlite)
     * @param {object} formBody req.form holding the form fields 
     * @param {function} callback success, true or false, null or id of added organization
     */
    addOrganization(formBody, callback) {
        let name = formBody.name;
        let street = formBody.street;
        let zip = formBody.zip;
        let city = formBody.city;
        let country = formBody.country;
        let state = formBody.state;
        let latitude = formBody.latitude;
        let longitude = formBody.longitude;
        let shortName = formBody.shortName;
        let zveis = formBody.zveis;
        let organizationsType = formBody.organizationsType;
        let parent = formBody.parent;
        let imageData = formBody.image_data

        this.doesOrganizationExistwithName(formBody, (exists) => {
            if (!exists) {
                console.log('MODELS-ORGANIZATIONS | Checking if organization of given type ' + organizationsType + ' can be added');
                this.canOrganizationTypeBeAdded(organizationsType, (allowed) => {
                    if (allowed) {
                        // calculate uuid for organization
                        var orgUUID = uuid.v1();
                        let query = 'INSERT INTO `organizations` (name, street, zip, city, country, state, latitude, longitude, uuid, shortName, zveis, organizationsType, parent, image_data) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)';
                        global.coreDb.run(query,
                            [
                                name,
                                street,
                                zip,
                                city,
                                country,
                                state,
                                latitude,
                                longitude,
                                orgUUID,
                                shortName,
                                zveis,
                                organizationsType,
                                parent,
                                imageData == null || imageData === "" ? null : imageData
                            ],
                            function (err) {
                                if (err) {
                                    callback(false, null);
                                } else {
                                    callback(true, this.lastID);
                                }
                            });
                        };
                    });
            } else {
                return callback(false, null);
            }
        })
    }



    /**
     * Updates an organizaton object in core database
     * @param {object} reqBody form data (usually req.body)
     * @param {*} orgId id of organization to update
     * @param {*} callback success (true or false)
     */
    editOrganization(reqBody, orgId, callback) {
        let name = reqBody.name;
        let street = reqBody.street;
        let zip = reqBody.zip;
        let city = reqBody.city;
        let state = reqBody.state;
        let country = reqBody.country;
        let latitude = reqBody.latitude;
        let longitude = reqBody.longitude;
        let shortName = reqBody.shortName;
        let zveis = reqBody.zveis;
        let organizationsType = reqBody.organizationsType;
        let parent = reqBody.parent;

        let query = "UPDATE `organizations` SET `name` = ?," +
            "`street` = ?," +
            "`zip` = ?," +
            "`city` = ?," +
            "`state` = ?," +
            "`country` = ?," +
            "`latitude` = ?," +
            "`longitude` = ?, " +
            "`shortName` = ?, " +
            "`zveis` = ?, " +
            "`organizationsType` = ?, " +
            "`parent` = ? " +
            " WHERE `organizations`.`id` = ?";

        global.coreDb.run(query,
            [
                name,
                street,
                zip,
                city,
                state,
                country,
                latitude,
                longitude,
                shortName,
                zveis,
                organizationsType,
                parent,
                orgId
            ],
            function (err) {
                if (err) {
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Removes an organization from database after a check if it can be deleted
     * @param {integer} orgId id of organization
     * @param {function} callback (success)
     */
    removeOrganizationWithId(orgId, callback) {
        console.log('MODELS-ORGANIZATIONS | Checking if organization with id ' + orgId + ' can be deleted safely');
        
        // object can be deleted, no referencing objects found in core database
        let query = 'DELETE FROM `organizations` WHERE id = ?';
        global.coreDb.run(query,
            [
                orgId
            ],
            function (err) {
                if (err) {
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Checks if an organization can savely be deleted. 
     * @param {integer} orgId id of Organization to check for
     * @param {object} callback object conainting information of can be deleted, if not with reasons
     */
    checkIfCanBeDeleted(orgId, callback) {
        
        return null;
    }


    /**
     * Updates the name of the avatar image for an organization in core database
     * @param {string} image - the image data in base64
     * @param {integer} orgId - the id of the organisation
     * @param {function} callback - a callback with either true or false, depending on the outcome of the operation
     */
    setOrganizationAvatar(image, orgId, callback) {
        // send the user's details to the database
        let query = 'UPDATE `organizations` SET image_data = ?, image = null WHERE id = ?';

        global.coreDb.run(query, [
            image,
            orgId
        ], (err) => {
            if (err) {
                req.app.logger.error('ROUTE-ORGANIZATIONS | error updating avatar-path in database for organization ' + orgId);
                return callback(false);
            }
            return callback(true);
        });
    }

    /**
     * Checks if an organization with the given name or shortName exists in core database
     * @param {object} formBody req.body containing all form fields of organization form
     * @param {*} callback exists (true / false)
     */
    doesOrganizationExistwithName(formBody, callback) {
        let name = formBody.name;
        let shortName = formBody.shortName;

        let query = "SELECT * FROM organizations WHERE name = ? OR shortName = ?";
        global.coreDb.all(query, [
            name,
            shortName
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                if (rows.length > 0) {
                    return callback(true);
                } else {
                    return callback(false);
                }

            }
        });
    }

    /**
     * Checks if an organization with the given type can be added in core database
     * @param {integer} typeId id of the organizationType to be checked for
     * @param {*} callback allowed to add (true / false)
     */
    canOrganizationTypeBeAdded(typeId, callback) {
        var organizationstypes = new OrganizationsTypes();
        organizationstypes.getOrganizationTypeWithId(typeId, (rowsOrgTypes) => {
            if (rowsOrgTypes != null) {
                if (rowsOrgTypes[0].unique != 1) {
                    // Type is not unique, so adding another org with this type is always allowed
                    return callback(true);
                } else {
                    // Type is limited to only one organization with this type
                    // Now check if there is another organization existing with this type set or not
                    this.getListOfOrganizations((rowsOrganizationsExisting) => {
                        if (rowsOrganizationsExisting != null) {
                            var limited = false;
                            rowsOrganizationsExisting.forEach((org) => {
                                if (org.organizationsType === typeId) {
                                    limited = true;
                                }
                            })
                            if (limited === true) {
                                return callback(false);
                            } else {
                                return callback(true);
                            }
                        }
                    })
                }
            } else {
                return callback(true);
            }
        })
    }

    /**
     * Gets owner organizations for zveis and gearnames
     * @param {array} zveisListCalc zveisCalc which zveis should be checked
     * @param {array} gearListCalc gearnamesCalc which gearnames should be checked
     * @param {*} callback rows or null
     */
    getOrganizationsWithZveisAndGearnames(callback) {
        let query = `
            SELECT 
                organizations.name AS organizationsName,
                organizations.zveis AS zveis,
                group_concat(assets.gearname) as gearnames
            FROM 
                organizations
            LEFT JOIN
                assets ON assets.ownerOrgId = organizations.id
            WHERE assets.isGear = 1 OR 
                organizations.zveis IS NOT NULL
            GROUP BY organizationsName
        `;
        global.coreDb.all(query, [
        ], (err, rows) => {
        if (err) {
            console.log(err);
            this._logger.error('MODELS-WALLBOARDS | Error loading wallboard from core data: ' + err.message);
            return callback(null);
        } else {
            return callback(rows);
        }
        });
    }

    /**
     * MARKER_IMAGE_BASE64_MIGRATION
     * TODO: this is a temporary data migration function that will be triggered from
     * the frontend and should be removed in future versions
     */
    getOrganizationsWithPendingImageMigration() {
        return new Promise((resolve, reject) => {
            const query = "SELECT id, name, image FROM organizations WHERE image IS NOT NULL"
            global.coreDb.all(query, [], (err, rows) => {
                if (err) {
                    reject(err)
                    return
                }
                resolve(rows)
            })
        })
    }

    /**
     * MARKER_IMAGE_BASE64_MIGRATION
     * TODO: this is a temporary data migration function that will be triggered from
     * the frontend and should be removed in future versions
     */
    migrateOrganizationImageBase64(organizationId, imageData) {
        return new Promise((resolve, reject) => {
            // set image to null and store the image data as base64
            const query = "UPDATE `organizations` SET image = null, image_data = ? WHERE id = ?"
            global.coreDb.run(query,
                [
                    imageData,
                    organizationId,
                ],
                err => {
                    if (err) {
                        reject(err)
                        return
                    }
                    resolve()
                }
            )
        })
    }

    /**
     * MARKER_IMAGE_BASE64_MIGRATION
     */
    getOrganizationImage(organizationId) {
        return new Promise((resolve, reject) => {
            const query = "SELECT image FROM organizations WHERE id = ?"
            global.coreDb.all(query, [organizationId], (err, rows) => {
                if (err) {
                    resolve(null)
                    return
                } else if (rows.length === 0) {
                    resolve(null)
                } 

                resolve(rows[0].image)
            })
        })
    }
}

// export the class
module.exports = organizations;
