
/**
 * Class handles qualifications (basedata) and offers methods dealing with qualifications in core database
 * @class
 */
class qualifications {
  /**
   * @constructor
   * @param {object} alogger Logger to be used for output of class methods (usually req.app.logger)
   */
  constructor(alogger) {
    // always initialize all instance properties
    this.logger = alogger;
  }

  /**
   * Returns list of all existing qualifications
   * @param {function} callback rows
   */
  getListOfQualifications(callback) {
    this.logger.debug('MODELS-QUALIFICATIONS | Reading qualifications');
    let query = "SELECT * FROM `qualifications`";

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-QUALIFICATIONS | Error loading qualifications from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * returns all qualifications which are active from core database
   * @param {function} callback rows or null
   */
  getListOfActiveQualifications(callback) {
    this.logger.debug('MODELS-QUALIFICATIONS | Reading active qualifications');
    let query = "SELECT * FROM qualifications WHERE state = 1";

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-QUALIFICATIONS | Error loading active qualifications from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Loads a qualification for a given group id
   * @param {integer} qId id of qualification to load data for from core database
   * @param {function} callback row or null
   */
  getQualificationForId(qId, callback) {
    this.logger.debug('MODELS-QUALIFICATIONS | Reading qualification for id ' + qId);
    let query = "SELECT * " +
      "FROM `qualifications` " +
      "WHERE id = ?";

    global.coreDb.all(query, [
      qId
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-QUALIFICATIONS | Error loading qualification from core data for id ' + gqIdId + ': ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Returns all assigned qualifications for basedata with id
   * @param {integer} bid id of basedata to load qualifications for
   * @param {*} callback rows or null
   */
  getQualificationsForBasedata(bid, callback) {
    let query = "SELECT * FROM xt_basedata_qualifications WHERE basedataId = ?";
    global.coreDb.all(query, [
      bid
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-QUALIFICATIONS | Error loading qualification from core data for basedata with id ' + bid + ': ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Adds a new qualification into core database
   * @param {object} reqBody req.form holding all the form fields
   * @param {*} callback success (true/false)
   */
  addQualification(reqBody, callback) {
    let name = reqBody.name;
    let shortName = reqBody.shortName;
    let timeLimited = reqBody.timeLimited;
    let state = reqBody.state;

    if (timeLimited === null) {
      timeLimited = 0;
    }

    if (state === null) {
      state = 1;
    }

    let query = "INSERT INTO qualifications (name, shortName, timeLimited, state) VALUES (?, ?, ?, ?)";
    global.coreDb.run(query, [
      name,
      shortName,
      timeLimited,
      state
    ], function (err) {
      if (err) {
        console.log('ERROR inserting qualification in core database: ' + err.message);
        return callback(false);
      } else {
        return callback(true);
      }
    })
  }

  /**
   * Updates an existing qualification in core database
   * @param {integer} qid id of qualification to be updated
   * @param {object} reqBody object holding the form fields (usually req.form)
   * @param {function} callback success (true/false)
   */
  updateQualification(qid, reqBody, callback) {
    let name = reqBody.name;
    let shortName = reqBody.shortName;
    let timeLimited = reqBody.timeLimited;
    let state = reqBody.state;

    if (timeLimited === null) {
      timeLimited = 0;
    }

    if (state === null) {
      state = 1;
    }

    let query = "UPDATE qualifications SET name = ?, shortName = ?, timeLimited = ?, state = ?  WHERE id = ?";
    global.coreDb.run(query, [
      name,
      shortName,
      timeLimited,
      state,
      qid
    ], function (err) {
      if (err) {
        console.log('ERROR updating qualification in core database: ' + err.message);
        return callback(false);
      } else {
        return callback(true);
      }
    })
  }

  /**
   * Deletes a qualification from core database
   * @param {integer} qId qualification ID of qualification to be deleted
   * @param {function} callback success, true or false
   */
  deleteQualification(qId, callback) {
    let query = "DELETE from qualifications WHERE id = ?";
    global.coreDb.run(query, [
      qId
    ], function (err) {
      if (err) {
        console.log('ERROR Deleting qualification with id ' + qId + ' in core database: ' + err.message);
        return callback(false);
      } else {
        return callback(true);
      }
    })
  }

  /**
   * 
   * @param {string} name Name of the qualification
   * @param {function} callback success true -> exists, false -> not existing, null -> error in query
   */
  doesQualificationExist(name, callback) {
    let query = "SELECT * FROM qualifications WHERE name = ?";
    global.coreDb.all(query, [
      name
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-QUALIFICATIONS | Error checking for existing group in organization: ' + err.message);
        return callback(null);
      } else {
        if (rows.length > 0) {
          return callback(true, rows);
        } else {
          return callback(false, null);
        }
      }
    });
  }

  /**
   * Assigns Qualifications to Wallboards profile
   * @param {Array} qualificationsArray Array of new qualifications to be added
   * @param {integer} wallboardId wallboard id the new groups shall be added for
   * @param {function} callback true or false (success)
   */
  assignQualificationsToWallboard(qualificationsArray, wallboardId, callback) {
    this.logger.debug('MODELS-WALLBOARDS | Assigning Group(s) to wallboard with id: ' + wallboardId);
    let delQuery = "DELETE FROM xt_wallboards_qualifications WHERE wallboardId = ?";
    let valuesPart = '';
    global.coreDb.run(delQuery,
      [
        wallboardId
      ],
      function (err) {
        if (err) {
          console.log('MODELS-WALLBOARDS | Error removing existing qualification for wallboard with id ' + wallboardId + ': ' + err.message);
          callback(false);
        } else {
          // successfully removed existing groups
          // Now add the new array as rows to core database
          if (Array.isArray(qualificationsArray)) {
            let values = [];
            qualificationsArray.forEach(group => {
              values.push('(' + wallboardId + ', ' + group[0] + ')');
            });
            valuesPart = values.join(',');
          } else {
            valuesPart = '(' + wallboardId + ', ' + qualificationsArray + ') ';
          }

          if (valuesPart.length > 1) {
            let query = "INSERT INTO xt_wallboards_qualifications (wallboardId, qualificationId) VALUES " + valuesPart + ";";
            global.coreDb.run(query, [], function (err) {
              if (err) {
                console.log('ERROR Adding new qualification for wallboard with id ' + wallboardId + ' enty to core database: ' + err.message);
                callback(false);
              } else {
                callback(true);
              }
            });
          } else {
            callback(false);
          }
        }
      });
  }

  /**
   * Assigns Qualifications to Basedata
   * @param {Array} qualificationsArray Array of new qualifications to be added
   * @param {integer} basedataId basedata id the new groups shall be added for
   * @param {function} callback true or false (success)
   */
  assignQualificationsToBasedata(qualificationsArray, basedataId, callback) {
    this.logger.debug('MODELS-WALLBOARDS | Assigning Qualifications(s) to basedata with id: ' + basedataId);
    let delQuery = "DELETE FROM xt_basedata_qualifications WHERE basedataId = ?";
    let valuesPart = '';
    let that = this;
    global.coreDb.run(delQuery,
      [
        basedataId
      ],
      function (err) {
        if (err) {
          console.log('MODELS-QUALIFICATIONS | Error removing existing qualification for basedata with id ' + basedataId + ': ' + err.message);
          callback(false);
        } else {
          // successfully removed existing groups
          // Now add the new array as rows to core database
          if (typeof qualificationsArray != 'undefined') {
            if (qualificationsArray.length > 0) {
              if (Array.isArray(qualificationsArray)) {
                let values = [];
                qualificationsArray.forEach(group => {
                  values.push('(' + basedataId + ', ' + group[0] + ')');
                });
                valuesPart = values.join(',');
              } else {
                valuesPart = '(' + basedataId + ', ' + qualificationsArray + ') ';
              }

              if (valuesPart.length > 1) {
                let query = "INSERT INTO xt_basedata_qualifications (basedataId, qualificationId) VALUES " + valuesPart + ";";
                global.coreDb.run(query, [], function (err) {
                  if (err) {
                    console.log('ERROR Adding new qualification for basedata with id ' + basedataId + ' enty to core database: ' + err.message);
                    callback(false);
                  } else {
                    callback(true);
                  }
                });
              } else {
                callback(false);
              }
            } else {
              // No Qualifications have to be added
              that.logger.debug('MODELS-QUALIFICATIONS | Assigning no qualifications(s) to basedata with id: ' + basedataId + ' as no qualifications seem to be left.');
            }
          } else {
            // No Qualifications have to be added
            that.logger.debug('MODELS-QUALIFICATIONS | empty qualifications have been handed over to be set for basedata with id: ' + basedataId);
          }
        }
      });
  }

  getQualificationsForApp(organizationId, callback) {
    let query = "SELECT DISTINCT " +
      "qualifications.shortName " +
      "FROM wallboardsStore " +
      "JOIN xt_wallboards_qualifications ON xt_wallboards_qualifications.wallboardId = wallboardsStore.id " +
      "JOIN qualifications ON qualifications.id = xt_wallboards_qualifications.qualificationId " +
      "WHERE (wallboardsStore.organizationId = ?) " +
      " AND (wallboardsStore.type = 'app' OR wallboardsStore.type = 'both') " +
      " AND (wallboardsStore.state = 1) " +
      "ORDER BY wallboardsStore.state DESC";

    global.coreDb.all(query, [
      organizationId
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-QUALIFICATIONS | Error loading wallboard profile(s) for organization with id ' + organizationId + ': ' + err.message);
        return callback(null);
      } else {
        //console.log("-------------WALLBOARD-PROFILES --------------");
        //console.log(rows);
        //console.log("-------------WALLBOARD-PROFILES --------------");
        if (rows.length > 0) {
          // Found qualififcations... return them now as a result
          //console.log('found wallboard profile -> returning qualifications now');
          return callback(rows);
        } else {
          // Didn't find a profile, check if there is a defailt profile we can use
          //console.log('found no wallboard profile -> checking for default profile now');
          let query = "SELECT DISTINCT " +
            "qualifications.shortName " +
            "FROM wallboardsStore " +
            "JOIN xt_wallboards_qualifications ON xt_wallboards_qualifications.wallboardId = wallboardsStore.id " +
            "JOIN qualifications ON qualifications.id = xt_wallboards_qualifications.qualificationId " +
            "WHERE wallboardsStore.organizationId IS NULL " +
            "  AND (wallboardsStore.type = 'app' OR wallboardsStore.type = 'both') " +
            "  AND (wallboardsStore.isDefault = 1)";

          global.coreDb.all(query, [
          ], (err, rows) => {
            if (err) {
              console.log('MODELS-QUALIFICATIONS | Error loading wallboard profile(s) for organization with id ' + organizationId + ': ' + err.message);
              return callback(null);
            } else {
              // Found default qualififcation... return it now as a result
              return callback(rows);
            }
          });
        }
      }
    });
  }

  getQualificationsForWallboardProfile(wallboardId, callback) {
    let query = "SELECT DISTINCT " +
      "qualifications.shortName " +
      "FROM wallboardsStore " +
      "JOIN xt_wallboards_qualifications ON xt_wallboards_qualifications.wallboardId = wallboardsStore.id " +
      "JOIN qualifications ON qualifications.id = xt_wallboards_qualifications.qualificationId " +
      "WHERE (wallboardsStore.id = ?) " +
      " AND (wallboardsStore.state = 1) " +
      "ORDER BY wallboardsStore.state DESC";
    global.coreDb.all(query, [
      wallboardId
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-QUALIFICATIONS | Error loading wallboard profile(s) for wallboard with id ' + wallboardId + ': ' + err.message);
        return callback(null);
      } else {
        /*
        console.log("-------------WALLBOARD-PROFILES --------------");
        console.log(rows);
        console.log("-------------WALLBOARD-PROFILES --------------");
        */
        if (rows.length > 0) {
          // Found qualififcations... return them now as a result
          console.log('found wallboard profile -> returning qualifications now');
          return callback(rows);
        } else {
          return callback(null)
        }
      }
    });
  }
}

// export the class
module.exports = qualifications;