const Moment = require('moment');
const EmailQueue = require('./emailqueue');
const EmailTemplates = require('./emailtemplates');
const Configuration = require('./configuration');

class respiratory {

    constructor(alogger) {
        // always initialize all instance properties
        this.logger = alogger;
    }

   
    /**
     * Loads list of respiratory entries from core database
     * @param {*} callback 
     */
    loadList(callback) {
        this.logger.debug('MODELS-RESPIRATORY | Reading list from core database');
        let sql = 'SELECT ' +
                    'respiratoryStore.*, ' +
                    'basedata.firstname, ' +
                    'basedata.lastname, ' +
                    'organizations.name as orgName, ' +
                    'organizations.id AS orgId ' +
                    'FROM respiratoryStore ' +
                  'INNER JOIN basedata ON basedata.id = respiratoryStore.basedataId ' +
                  'INNER JOIN organizations ON basedata.organizationid = organizations.id';

        global.coreDb.all(sql, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-RESPIRATORY | Error reading list from database: ' + err);
                console.log(err);
                return callback(null);
            }
            return callback(rows);
        });
    } 


    /**
     * returns basedata ids as rows for which respiratory entries exist in database
     * @param {function} callback rows or null
     */
    getBasedataForWhichRespiratoryEntryExists(callback) {
        let query = "SELECT basedata.id AS basedataId " +
                    "FROM basedata " +
                    "INNER JOIN respiratoryStore ON respiratoryStore.basedataId = basedata.id";
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-RESPIRATORY | Error reading list of basedata from database: ' + err);
                console.log(err);
                return callback(null);
            }
            return callback(rows);
        });
    }


    /**
     * Returns existing entries for basedataId
     * @param {integer} basedataId id of basedata to return entry for
     * @param {*} callback rows or null
     */
    getEntryForBasedata(basedataId, callback) {
        let query = "SELECT * FROM respiratoryStore WHERE basedataId = ?";
        global.coreDb.all(query, [
            basedataId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-RESPIRATORY | Error checking for existing entry from database: ' + err);
                console.log(err);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns respiratory entry from core database
     * @param {integer} respId 
     * @param {function} callback rows or null
     */
    getEntryForId(respId, callback) {
        let query = "SELECT respiratoryStore.*, " +
                        "basedata.firstname AS firstname, " +
                        "basedata.lastname AS lastname " +
                        "FROM respiratoryStore " +
                        "INNER JOIN basedata ON basedata.id = respiratoryStore.basedataId " +
                        "WHERE respiratoryStore.id = ?";

        global.coreDb.all(query, [
            respId
        ], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-RESPIRATORY | Error loading entry from database: ' + err);
                console.log(err);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * returns members with valid respiratory (till today)
     * @param {function} callback rows or null
     */
    getValidMembers(callback) {

        var dateBoundary_day = new Date().getDate();
        var dateBoundary_month = new Date().getMonth() + 1;
        var dateBoundary_year = new Date().getFullYear();
        if (dateBoundary_day < 10) dateBoundary_day = '0' + dateBoundary_day;
        if (dateBoundary_month < 10) dateBoundary_month = '0' + dateBoundary_month;
        var dateBoundary = dateBoundary_year + '-' + dateBoundary_month + '-' + dateBoundary_day;

       

        let query = "SELECT * " +
                    "FROM respiratoryStore " +
                    "WHERE respiratoryStore.capableTill > datetime(?, 'localtime')";
                    global.coreDb.all(query, [
                        dateBoundary
                    ], (err, rows) => {
                        if (err) {
                            this.logger.error('MODELS-RESPIRATORY | Error checking for valid members from database: ' + err);
                            console.log(err);
                            return callback(null);
                        } else {
                            return callback(rows);
                        }
                    });
    }

    /**
     * returns expiring respiratory 
     * @param {integer} months months left boundary
     * @param {function} callback rows or null
     */
    getExpiringMembers(months, callback) {

        var dateBoundary_day = new Date().getDate();
        var dateBoundary_month = new Date().getMonth() + 1;
        var dateBoundary_year = new Date().getFullYear();

        // add months (make sure year overflow is recognised)
        if ((dateBoundary_month + months) > 12) {
            dateBoundary_month = (dateBoundary_month + months) -12;
            dateBoundary_year = dateBoundary_year + 1;
        } else {
            dateBoundary_month = dateBoundary_month + months;
        }


        if (dateBoundary_day < 10) dateBoundary_day = '0' + dateBoundary_day;
        if (dateBoundary_month < 10) dateBoundary_month = '0' + dateBoundary_month;
        var dateBoundary = dateBoundary_year + '-' + dateBoundary_month + '-' + dateBoundary_day;


        let query = "SELECT * " +
                    "FROM respiratoryStore " +
                    "WHERE respiratoryStore.capableTill < datetime(?, 'localtime')";
                    global.coreDb.all(query, [
                        dateBoundary
                    ], (err, rows) => {
                        if (err) {
                            this.logger.error('MODELS-RESPIRATORY | Error checking for expiring members from database: ' + err);
                            console.log(err);
                            return callback(null);
                        } else {
                            return callback(rows);
                        }
                    });
    }

    /**
     * adds a new Entry for a basedata
     * @param {object} reqBody req.body of form sending in the information
     * @param {*} callback success (true/false)
     */
    addRespiratory(reqBody, callback) {
        var basedataId = reqBody.basedataid;
        this.getEntryForBasedata(basedataId, (rowsExisting) => {
            var capable = 1;
            var exists = false;
            if (rowsExisting !== null) {
                if (rowsExisting.length > 0) {
                    exists = true;
                }
            }
            if (exists) {
                // There exists an entry --> return false
                callback(false);
            } else {
                var capableFrom = reqBody.capableFrom;
                var capableTill = reqBody.capableTill;
                var doctorName = reqBody.doctorName;
                var restrictions = reqBody.restrictions;

                if ((capableFrom.length < 10) || (capableTill.length < 10) || (basedataId === null) || (basedataId.length < 1)) {
                    return callback(false);
                }

                // Calculate dates to database format
                // var momenthelper = new Momenthelper();
                // var capableFrom = momenthelper.formDateToMysql(capableFrom);
                // var capableTill = momenthelper.formDateToMysql(capableTill);

                // Check if date is in the past
                var dateTill = Moment(capableTill);
                var dateNow = Moment();
                if (dateTill.diff(dateNow, 'days') <= 1) {
                    capable = 0;
                }
                

                let query = "INSERT INTO respiratoryStore (basedataId, capable, capableFrom, capableTill, doctorName, restrictions) VALUES (?, ?, ?, ?, ?, ?)";
                global.coreDb.run(query,
                    [
                        basedataId,
                        capable,
                        capableFrom,
                        capableTill,
                        doctorName,
                        restrictions
                    ],
                    function (err) {
                        if (err) {
                            console.log('ERROR: ' + err.message);
                            callback(false);
                        } else {
                            callback(true);
                        }
                    });
            }
        })
    }

    /**
     * updates a record in core database
     * @param {integer} respId id of entry 
     * @param {*} reqBody req.body holding form data
     * @param {*} callback success (true/false)
     */
    editRespiratory(respId, reqBody, callback) {
        var capable = reqBody.capable;
        var capableFrom = reqBody.capableFrom;
        var capableTill = reqBody.capableTill;
        var restrictions = reqBody.restrictions;
        var doctorName = reqBody.doctorName;
        var paused = reqBody.paused;

        if (paused === null) paused = 0;

        // Check if date is in the past
        var dateTill = Moment(capableTill);
        var dateNow = Moment();
        if (dateTill.diff(dateNow, 'days') <= 1) {
            capable = 0;
        }

        let query = "UPDATE respiratoryStore SET " +
                    "capable = ?, " +
                    "capableFrom = ?, " +
                    "capableTill = ?, " +
                    "restrictions = ?, " + 
                    "doctorName = ?, " +
                    "paused = ? " +
                    "WHERE id = ?";

        global.coreDb.run(query,
            [
                capable,
                capableFrom,
                capableTill,
                restrictions,
                doctorName,
                paused,
                respId
            ],
            function (err) {
                if (err) {
                    console.log('ERROR: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * central maintenance method for respiratory model (called from core in app.js)
     */
    doMaintenance() {
        var configuration = new Configuration(this._logger);
        configuration.getValueForKeyword('mtncRespiratoryExaminationsDaysInFutureWindow', (mtncRespiratoryExaminationsDaysInFutureWindow) => {
            console.log('MODEL-RESPIRATORY | MAINTENANCE | Starting Maintenance jobs now ...');
            this.maintenance_checkForExpiredRespiratoryExaminations(mtncRespiratoryExaminationsDaysInFutureWindow);
            this.maintenance_checkForExpiredRespiratoryExaminationsSendNotifications(mtncRespiratoryExaminationsDaysInFutureWindow);
        });
    }

    /**
     * Checks for examinations which are expired or will exire in given number of days in the future and sets state to expired.
     * Method shall be ran once a day during maintenance tasks
     * @param {integer} daysFromNow number of days to look into the future
     */
    maintenance_checkForExpiredRespiratoryExaminations(daysFromNow) {
        if (daysFromNow !== null) {
            if (isNaN(daysFromNow)) {
                daysFromNow = 1;
            }
        } else {
            daysFromNow = 1;
        }
        // query for expired examinations
        let queryExpired = `SELECT 
                                respiratoryStore.*,
                                (basedata.firstname || ' ' || basedata.lastname) AS member
                            FROM respiratoryStore
                            INNER JOIN basedata ON basedata.id = respiratoryStore.basedataId
                            WHERE 
                                respiratoryStore.capableTill <= date('now', 'localtime', '+1 days') AND
                                respiratoryStore.capable = 1 AND
                                respiratoryStore.paused = 0
                            ORDER BY
                                respiratoryStore.capableTill DESC`;
        global.coreDb.all(queryExpired, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-RESPIRATORY | MAINTENANCE | Error checking for expired examinations at database: ' + err);
            } else {
                if (rows.length > 0) {
                    // We've identified exired examinations
                    // now set the state to expired (0)
                    console.log('MODELS-RESPIRATORY | MAINTENANCE | Found ' + rows.length + ' expired examinations. Set the state now to expired');
                    var failCounter = 0;
                    global.coreDb.serialize(function() {
                        rows.forEach((examination) => {
                            let query = "UPDATE respiratoryStore SET capable = 0 WHERE id = ?";
                            global.coreDb.run(query,
                                [
                                    examination.id
                                ],
                                function (err) {
                                    if (err) {
                                        console.log('MODELS-RESPIRATORY | MAINTENANCE | Error setting capable state for expired examination at database: ' + err.message);
                                        failCounter++;
                                    } else {
                                    }
                                });
                        })
                    });
                    if (failCounter > 0) {
                        console.log('MODELS-RESPIRATORY | MAINTENANCE | ERROR | There has been an error updating capable state, please see the logs');
                    } else {
                        console.log('MODELS-RESPIRATORY | MAINTENANCE | Successfully finished check for expired examinations');
                    }
                }
            }
        });
    }

    /**
     * Checks for examinations which are expired and creates email reports and puts them into email queue.
     * Method shall be ran once a day during maintenance tasks
     * @param {integer} daysFromNow number of days to look into the future
     */
    maintenance_checkForExpiredRespiratoryExaminationsSendNotifications (daysFromNow) {
        // query for expired examinations
        if (daysFromNow !== null) {
            if(isNaN(daysFromNow)) {
                daysFromNow = 1;
                this.logger.error('MODELS-RESPIRATORY | MAINTENANCE | Days to look into future not a number, using default of 1');
            } else {
                this.logger.log('MODELS-RESPIRATORY | MAINTENANCE | Days to look into future: ' + daysFromNow);
            }
        } else {
            this.logger.error('MODELS-RESPIRATORY | MAINTENANCE | Days to look into future not configured, using default of 1');
            daysFromNow = 1;
        }
        let queryExpired = `SELECT 
                                respiratoryStore.*,
                                (basedata.lastname || ' ' || basedata.firstname) AS member
                            FROM respiratoryStore
                            INNER JOIN basedata ON basedata.id = respiratoryStore.basedataId
                            WHERE 
                                respiratoryStore.capableTill <= date('now', 'localtime', '+${daysFromNow} days') AND
                                respiratoryStore.paused = 0
                            ORDER BY
                                respiratoryStore.capableTill DESC`;
        global.coreDb.all(queryExpired, [], (err, rows) => {
            if (err) {
                this.logger.error('MODELS-RESPIRATORY | MAINTENANCE | Error checking for expired examinations at database: ' + err);
            } else {
                if (rows.length > 0) {
                    // We've identified exired examinations
                    // now set the state to expired (0)
                    console.log('MODELS-RESPIRATORY | MAINTENANCE | Found ' + rows.length + ' expired examinations. Sending email notifications now');
                    var expiredTable = '';
                    var expiredTablePlain = '';
                    rows.forEach((expired) => {
                        if (Moment(expired.capableTill).isValid()) {
                            expired.capableTill = Moment(expired.capableTill).format('DD.MM.YYYY');
                        }
                        expiredTable = expiredTable + `<tr>
                            <td style="text-align: left; border-bottom-width: 1px; border-bottom-color: #E1E1E1; border-bottom-style: solid; padding: 10px 15px 10px 0;" align="left">${expired.member}</td>
                            <td style="text-align: left; border-bottom-color: #E1E1E1; border-bottom-width: 1px; border-bottom-style: solid; padding: 10px 15px;" align="left">${expired.capableTill}</td>
                            <td style="text-align: left; border-bottom-width: 1px; border-bottom-color: #E1E1E1; border-bottom-style: solid; padding: 10px 0 10px 15px;" align="left">${expired.capableTill}</td>
                        </tr>`;
                        expiredTablePlain = expiredTablePlain + `- ` + expired.member + ', ausgelaufen am ' + expired.capableTill + '\n';
                    });
                    let emailtemplates = new EmailTemplates(global.app.logger);
                    let data = {
                        firstname : 'Jens',
                        table: expiredTable,
                        tablePlain: expiredTablePlain
                    }
                    emailtemplates.renderDataToTemplate(data, 'expiredRespiratoryExaminations', (template) => {
                        if (template === null) {
                            console.log('MODEL-RESPIRATORY | MAINTENANCE | Could not send emails as template could not be loaded');
                        } else {
                            let emailqueue = new EmailQueue(global.app.logger);
                            //
                            // TBD - get recipients
                            //
                            let recipients = [];
                            recipients.push('jens@dinstuehler.com');
                            emailqueue.addToQueue('Abgelaufene G26.3', template.html, template.plain, recipients, (success) => {
                                if (success) {
                                    console.log('MODELS-RESPIRATORY | MAINTENANCE | Successfully added email to queue');
                                    emailqueue.getEmailsToBeSent(null, (rowsEmails) => {
                                        if (rowsEmails != null) {
                                            if (rowsEmails.length > 0) {
                                                console.log('MODELS-RESPIRATORY | MAINTENANCE | Informing about new emails by firing event_email_queue_refreshed with ' + rowsEmails.length + ' new emails');
                                                global.plgManager.event_email_queue_refreshed(rowsEmails);
                                            }
                                        }
                                    })
                                } else {
                                    console.log('MODELS-RESPIRATORY | MAINTENANCE | ERROR | Could not add email to queue, please see logs');
                                }
                            })
                        }
                    });
                    
                }
            }
        });
    }


}

// export the class
module.exports = respiratory;