const EventEmitter = require('events');

/**
 * Class represents suppliers within ALARMiator and offers
 * methods to handle supplier related data in core database
 * @class
 */

class Suppliers extends EventEmitter {

    /**
     * constructor
     */
    constructor() {
        super();
    }


    /**
     * Returns all suppliers as rows from core database
     * @param {function} callback rows or null
     */
    getSuppliers(callback) {
        let query = "SELECT * FROM suppliers ORDER BY companyname ASC";
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns a supplier entry from core database for given supplier id
     * @param {integer} sId id of supplier in coredatabase
     * @param {*} callback rows or null
     */
    getSupplier(sId, callback) {
        let query = "SELECT * FROM suppliers WHERE id = ?";
        global.coreDb.all(query, [
            sId
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * adds a new supplier to database
     * @param {object} reqBody request body holding all form fields (usually req.body)
     * @param {function} callback success (true/false), reason (string)
     */
    addSupplier(reqBody, callback) {
        var companyname = reqBody.companyname;
        this.doesSupplierExist(companyname, (exists) => {
            if (exists) {
                // Supplier exists in database for this company name
                return callback(false, 'Lieferant mit diesem Firmennamen existiert bereits');
            } else {
                var department = reqBody.department;
                var street = reqBody.street;
                var zip = reqBody.zip;
                var city = reqBody.city;
                var federalstate = reqBody.federalstate;
                var country = reqBody.country;
                var email = reqBody.email;
                var phone = reqBody.phone;
                var faximile = reqBody.faximile;
                var cellphone = reqBody.cellphone;
                var supportPhone = reqBody.supportPhone;
                var supportEmail = reqBody.supportEmail;
                var website = reqBody.website;
                var supportWebsite = reqBody.supportWebsite;
                var supportWebsiteCredentialsUsername = reqBody.supportWebsiteCredentialsUsername;
                var supportWebsiteCredentialsPassword = reqBody.supportWebsiteCredentialsPassword;
                var contactName = reqBody.contactName;
                var customerNumber = reqBody.customerNumber;
                var state = reqBody.state;

                let query = "INSERT INTO suppliers (" +
                                    "companyname, " +
                                    "department, " +
                                    "street, " +
                                    "zip, " +
                                    "city, " +
                                    "federalstate, " +
                                    "country, " +
                                    "email, " +
                                    "phone, " +
                                    "faximile, " +
                                    "cellphone, " +
                                    "supportPhone, " +
                                    "supportEmail, " +
                                    "website, " +
                                    "supportWebsite, " +
                                    "supportWebsiteCredentialsUsername, " +
                                    "supportWebsiteCredentialsPassword, " +
                                    "contactName, " +
                                    "customerNumber, " + 
                                    "state" +
                                    ") VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
                global.coreDb.run(query, [
                    companyname,
                    department,
                    street,
                    zip,
                    city,
                    federalstate,
                    country,
                    email,
                    phone,
                    faximile,
                    cellphone,
                    supportPhone,
                    supportEmail,
                    website,
                    supportWebsite,
                    supportWebsiteCredentialsUsername,
                    supportWebsiteCredentialsPassword,
                    contactName,
                    customerNumber,
                    state
                ], function (err) {
                    if (err) {
                        console.log(`ERROR Adding new supplier to core database: ${err.message}`);
                        callback(false);
                    } else {
                        callback(true);
                    }
                });
            }
        })
    }

    /**
     * Checks if a supplier with the given name exists in database (needs to be unique)
     * @param {string} name 
     * @param {function} callback exists (true) or not (false)
     */
    doesSupplierExist(name, callback) {
        let query = "SELECT * FROM suppliers WHERE companyname = ?";
        global.coreDb.all(query, [
            name
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                if (rows.length > 0) {
                    return callback(true);
                } else {
                    return callback(false);
                }
            }
        });
    }

     /**
     * Checks if a supplier with the given name exists in database and is not the given supplier with id. Used while editing an existing supplier and to check if the new name of supplier is existing at another supplier.
     * @param {string} name 
     * @param {function} callback exists (true) or not (false)
     */
    doesSupplierExistBeforeEdit(sId, newname, callback) {
        let query = "SELECT * FROM suppliers WHERE companyname = ?";
        global.coreDb.all(query, [
            newname
        ], (err, rows) => {
            if (err) {
                return null;
            } else {
                if (rows.length > 0) {
                    console.log('rows id: ' + rows[0].id + ' - sId: ' + sId);
                    if (parseInt(rows[0].id) === parseInt(sId)) {
                        return callback(false);
                    } else {
                        return callback(true)
                    }
                } else {
                    return callback(false);
                }
            }
        });
    }


    /**
     * Renmoves a supplier permanent from core database, if not referenced assets exist anymore
     * @param {integer} sId id of supplier which should be deleted
     * @param {*} callback success (true / false)
     */
    removeSupplier(sId, callback) {
        let query = "DELETE FROM suppliers WHERE id = ?";
        global.coreDb.run(query, [
            sId
        ], function (err) {
            if (err) {
                console.log(`ERROR Deleting supplier from core database: ${err.message}`);
                callback(false);
            } else {
                callback(true);
            }
        });
    }

    /**
     * Sets the state of an existing supplier to active or inactive
     * @param {integer} sId id of supplier the state shall be set for
     * @param {integer} sState 1 = active, 0 = inactive
     * @param {function} callback success (true/false)
     */
    setSupplierState(sId, sState, callback) {
        let query = "UPDATE suppliers SET state = ? WHERE id = ?";
        global.coreDb.run(query, [
            sState,
            sId
        ], function (err) {
            if (err) {
                console.log(`ERROR setting state for supplier in core database: ${err.message}`);
                callback(false);
            } else {
                callback(true);
            }
        });
    }

    /**
     * Edits a supplier with given id in core database 
     * @param {integer} sId id of supplier to be edited
     * @param {object} reqBody object holding all form fields (usually req.body)
     * @param {*} callback success (true/false)
     */
    editSupplier(sId, reqBody, callback) {
        console.log(reqBody);
        var companyname = reqBody.companyname;
        this.doesSupplierExistBeforeEdit(sId, companyname, (exists) => {
            if (exists) {
                // Supplier exists in database for this company name
                console.log('fehler');
                return callback(false, 'Lieferant mit diesem Firmennamen existiert bereits');
            } else {
                var department = reqBody.department;
                var street = reqBody.street;
                var zip = reqBody.zip;
                var city = reqBody.city;
                var federalstate = reqBody.federalstate;
                var country = reqBody.country;
                var email = reqBody.email;
                var phone = reqBody.phone;
                var faximile = reqBody.faximile;
                var cellphone = reqBody.cellphone;
                var supportPhone = reqBody.supportPhone;
                var supportEmail = reqBody.supportEmail;
                var website = reqBody.website;
                var supportWebsite = reqBody.supportWebsite;
                var supportWebsiteCredentialsUsername = reqBody.supportWebsiteCredentialsUsername;
                var supportWebsiteCredentialsPassword = reqBody.supportWebsiteCredentialsPassword;
                var contactName = reqBody.contactName;
                var customerNumber = reqBody.customerNumber;
                var state = reqBody.state;

                console.log('city: ' + city);

                let query = "UPDATE suppliers SET " +
                                    "companyname = ?, " +
                                    "department = ?, " +
                                    "street = ?, " +
                                    "zip = ?, " +
                                    "city = ?, " +
                                    "federalstate = ?, " +
                                    "country = ?, " +
                                    "email = ?, " +
                                    "phone = ?, " +
                                    "faximile = ?, " +
                                    "cellphone = ?, " +
                                    "supportPhone = ?, " +
                                    "supportEmail = ?, " +
                                    "website = ?, " +
                                    "supportWebsite = ?, " +
                                    "supportWebsiteCredentialsUsername = ?, " +
                                    "supportWebsiteCredentialsPassword = ?, " +
                                    "contactName = ?, " +
                                    "customerNumber = ?, " + 
                                    "state = ? " +
                                    "WHERE id = ?";
                global.coreDb.run(query, [
                    companyname,
                    department,
                    street,
                    zip,
                    city,
                    federalstate,
                    country,
                    email,
                    phone,
                    faximile,
                    cellphone,
                    supportPhone,
                    supportEmail,
                    website,
                    supportWebsite,
                    supportWebsiteCredentialsUsername,
                    supportWebsiteCredentialsPassword,
                    contactName,
                    customerNumber,
                    state,
                    sId
                ], function (err) {
                    if (err) {
                        console.log(`ERROR Updating supplier at core database: ${err.message}`);
                        callback(false);
                    } else {
                        callback(true);
                    }
                });
            }
        })
    }    
}

module.exports = Suppliers;