const EventEmitter = require('events');
const moment = require('moment');
const gk = require('gauss-krueger');
const Operation = require('./core/core_operation');

/**
 * Class offers functionality to manage test alarms
 * @class
 */
class TestAlarmManager extends EventEmitter {

    /**
     * @constructor
     * @param {*} logger 
     */
    constructor(logger) {
        super();

        this.logger = logger;

        this.cronjob_type='testalarm';
    }

    /**
     * logger of TestAlarmManager
     * @param {string} msg 
     */
    Log(msg){
        if (this.logger !== null) {
            this.logger.log('CORE | TESTALARM-MANAGER | ' + msg);
        } 
        
        console.log('CORE | TESTALARM-MANAGER | ' + msg);
        
    }

    /**
     * adds a testalarm to the manager
     * @param {*} name the unipue name of the sample_alarm
     * @param {*} schedule the cron job schedule when that alarm should be thrown
     * @param {Operation} alarm_data the data of the alarm
     * @param {*} callback 
     */
    AddNewTestalarm(name, schedule, alarm_data, callback){

        var cronjob_data = {};
        cronjob_data.event_name = name;
        cronjob_data.event_data = alarm_data;
        var created = global.cron.AddCronJob(name, this.cronjob_type, schedule, cronjob_data);
        if(created){
            global.cron.on(cronjob_data.event_name, (data) =>{
                global.testalarmmanager.FireTestalarm(data);
            });
            global.cron.SaveCronJobToCoreDB(name, (err) => {
                if(err){
                    global.testalarmmanager.Log(err);
                    return callback(err)
                }
                else{
                    return callback(null);
                }
            });
        }
        else{
            var msg = 'creation of testalarm failed! Please have a look at the logs!'; 
            global.testalarmmanager.Log(msg);
            return callback(msg)
        }
    }

    /**
     * removes a testalarm from the manager
     * @param {string} name 
     * @param {*} callback
     */
    RemoveTestalarm(name, callback){

        var removed = global.cron.DeleteCronJob(name);
        if(removed){

            // remove eventhandlers
            var msg = "found " + global.cron.listenerCount(name) + " eventhandler for event " + name + "!";
            global.testalarmmanager.Log(msg);
        
            global.cron.removeAllListeners(name);

            if(global.cron.listenerCount(name) === 0){
                var msg = "all eventhandler for event " + name + "deleted.";
                global.testalarmmanager.Log(msg);
            }
            else{
                var msg = "still found " + global.cron.listenerCount(name) + " eventhandler for event " + name + "!";
                global.testalarmmanager.Log(msg);
            }
            

            global.cron.RemoveCronJobFromCoreDB(name, (success) => {
                if(success){
                    var msg = name + " successfull removed from coredatabase.";
                    global.testalarmmanager.Log(msg);
                    return callback(null);
                }
                else{
                    var msg = "error at remove " + name + " from coredatabase!";
                    global.testalarmmanager.Log(msg);
                    return callback(msg);
                }
            });
        }
        else{
            var msg = 'could not remove cronjob!'
            global.testalarmmanager.Log(msg);
            return callback(msg);
        }
    }

    EditTestalarm(name, schedule, alarm_data, state, callback){
        
        var removed = global.cron.DeleteCronJob(name);
        if(removed){
            var cronjob_data = {};
            cronjob_data.event_name = name;
            cronjob_data.event_data = alarm_data;
            var created = global.cron.AddCronJob(name, this.cronjob_type, schedule, cronjob_data);
            if(created){
                if(state === '1'){
                    global.cron.StartCronJob(name);
                } else {
                    global.cron.StopCronJob(name);
                }
                global.cron.RemoveCronJobFromCoreDB(name, (success) => {
                    if(success){
                        var msg = name + " successfull removed from coredatabase."
                        global.testalarmmanager.Log(msg);
                        global.cron.SaveCronJobToCoreDB(name, (err) => {
                            if(err){
                                global.testalarmmanager.Log(err);
                                return callback(err)
                            }
                            else{
                                return callback(null);
                            }
                        });
                    }
                    else{
                        var msg = "error at remove " + name + " from coredatabase!";
                        global.testalarmmanager.Log(msg);
                        return callback(msg);
                    }
                });
            }
            else{
                var msg = 'creation of cronjob failed!'; 
                global.testalarmmanager.Log(msg);
                return callback(msg)
            }
        }
        else{
            var msg = 'could not remove cronjob!'
            global.testalarmmanager.Log(msg);
            return callback(msg);
        }
    }

    /**
     * start ( active ) an testalarm
     * @param {string} name 
     * @param {*} callback
     */
    StartTestalarm(name, callback){
        var started = global.cron.StartCronJob(name);
        if(started){
            global.cron.UpdateCronJobInCoreDB(name, 'running', true, (err) => {
                if(err){
                    var msg = 'error at writing new value to coredatabase!'
                    global.testalarmmanager.Log(msg);
                    return callback(msg);                  
                }else{
                    return callback(null);
                }
            });
        } else {
            var msg = name + 'could not be started! Please check logs!'
            global.testalarmmanager.Log(msg);
            return callback(msg);
        }
    }

    /**
     * stop ( deactivate ) a Testalarm
     * @param {string} name 
     * @param {*} callback
     */
    StopTestalarm(name, callback){

        var stopped = global.cron.StopCronJob(name);
        if(stopped){
            global.cron.UpdateCronJobInCoreDB(name, 'running', false, (err) => {
                if(err){
                    var msg = 'error at writing new value to coredatabase!'
                    global.testalarmmanager.Log(msg);
                    return callback(msg);                  
                }else{
                    return callback(null);
                }
            });
        } else {
            var msg = name + 'could not be stopped! Please check logs!'
            global.testalarmmanager.Log(msg);
            return callback(msg);
        }
    }

    /**
     * get all testalarms
     * @param {*} callback 
     */
    GetTestalarms(callback){

        global.testalarmmanager.Log('get all testalarms');
        var jobs = global.cron.GetCronjobOfType(this.cronjob_type);
        return callback(jobs);
    }

    /**
     * get one specific Testalarm-job
     * @param {*} name the name of the Testalarm
     * @param {*} callback the Testalarm-object and the error-message if one exists
     */
    GetTestalarm(name, callback){

        global.testalarmmanager.Log('get testalarm with name: ' + name);
        var jobs = global.cron.GetCronjobOfType(this.cronjob_type);
        jobs = jobs.filter(j => j.name === name);
        if(jobs.length > 0){
            var job = jobs[0];
            return callback(job, null);
        } else {
            var msg = 'job \"' + name + '\" not found!';
            global.testalarmmanager.Log(msg);
            return callback(null, msg);
        }
    }

    /**
     * Building an operation-object from a Request Body
     * @param {*} reqBody the reqBody which cointains the data to build an Operation-object from it 
     */
    GetOperationFromReqBody(reqBody){
        const Operation = require('./core/core_operation');
        let operation = new Operation();
        operation.setUUID(); // Calculate new GUID for this operation object as a unique identifier
        global.testalarmmanager.Log('Set new guid for test operation to: ' + operation.uuid.value);
        operation.operationnumber = reqBody.operationnumber;
        operation.subject = reqBody.subject;
        operation.message = reqBody.message;
        operation.object = reqBody.object;
        operation.operationSourceConnector = 'core_'+this.cronjob_type;
        operation.operationState = 0;
        operation.section = reqBody.section;
        operation.street = reqBody.street;
        operation.community = reqBody.community;
        operation.district = reqBody.district;
        operation.gear = reqBody.gear;
        operation.keywordCategory = reqBody.keywordCategory;
        operation.keywordId = reqBody.keywordId;
        operation.keywordName = reqBody.keywordName;
        operation.keywordRaw = reqBody.keywordRaw;
        operation.location = reqBody.location;
        operation.zveis = reqBody.zveis;
        operation.alarmdate = moment().format('DD.MM.YYYY');
        operation.alarmtime = moment().format('HH:mm:ss');
        operation.gkx = reqBody.gkx;
        operation.gky = reqBody.gky;
        operation.lat = 0.0;
        operation.lon = 0.0;
        operation.floor = reqBody.floor;
        operation.ils = reqBody.ils;
        operation.isTest = 1;
        
        if ((operation.gky.value.length > 0) && (operation.gkx.value.length > 0)) {
            // SDS delivered gauss krueger coordinates
            let wgs84 = gk.toWGS({ x: parseFloat(operation.gkx.value), y: parseFloat(operation.gky.value) });
            operation.lat = wgs84.latitude;
            operation.lon = wgs84.longitude;
        } else {
            operation.lat = 0.0;
            operation.lon = 0.0;
        }

        return operation;

    }

    FireTestalarm(alarm_data){
        var operation = new Operation();
        operation.setUUID();
        operation.operationnumber = "P " + moment().format('YYYYMMDDHHmmss');
        operation.createdAt = moment().format();
        operation.subject = alarm_data.event_data.subject.value;
        operation.message = alarm_data.event_data.message.value;
        operation.object = alarm_data.event_data.object.value;
        operation.operationSourceConnector = 'core_'+this.cronjob_type;
        operation.operationState = 0;
        operation.section = alarm_data.event_data.section.value;
        operation.street = alarm_data.event_data.street.value;
        operation.community = alarm_data.event_data.community.value;
        operation.district = alarm_data.event_data.district.value;
        operation.gear = alarm_data.event_data.gear.value;
        operation.keywordCategory = alarm_data.event_data.keywordCategory.value;
        operation.keywordId = alarm_data.event_data.keywordId.value;
        operation.keywordName = alarm_data.event_data.keywordName.value;
        operation.keywordRaw = alarm_data.event_data.keywordRaw.value;
        operation.location = alarm_data.event_data.location.value;
        operation.zveis = alarm_data.event_data.zveis.value;
        operation.alarmdate = moment().format('DD.MM.YYYY');
        operation.alarmtime = moment().format('HH:mm:ss');
        operation.gkx = alarm_data.event_data.gkx.value;
        operation.gky = alarm_data.event_data.gky.value;
        operation.lat = 0.0;
        operation.lon = 0.0;
        operation.floor = alarm_data.event_data.floor.value;
        operation.ils = alarm_data.event_data.ils.value;
        operation.isTest = 1;
        
        if ((operation.gky.value.length > 0) && (operation.gkx.value.length > 0)) {
            // SDS delivered gauss krueger coordinates
            let wgs84 = gk.toWGS({ x: parseFloat(operation.gkx.value), y: parseFloat(operation.gky.value) });
            operation.lat = wgs84.latitude;
            operation.lon = wgs84.longitude;
        } else {
            operation.lat = 0.0;
            operation.lon = 0.0;
        }
        
        global.testalarmmanager.Log('event fired from job ' + alarm_data.event_name + ' --> ' + JSON.stringify(operation));
        
        // Send Event to PluginManager
        global.plgManager.event_new_alarm(operation);
    
    }

    /**
     * converts the cron schedule string in to an object
     * @param {string} crondatastring the cron schedule as string
     */
    GetCronDataObject(crondatastring){
        crondatastring = crondatastring.split(' ');

        var crondata = {};
        crondata.sec = crondatastring[0];
        crondata.min = crondatastring[1];
        crondata.hour = crondatastring[2];
        crondata.day = crondatastring[3];
        crondata.month = crondatastring[4];
        crondata.DayOfWeek = crondatastring[5];

        return crondata;
        
    }
}

module.exports = TestAlarmManager;