const md5 = require('md5');
const QRCode = require('qrcode')
const Usersgroups = require('./usersgroups');
const Basedata = require('./basedata');
const Organizations = require('./organizations');
const Configuration = require('./configuration');
const Pdf = require('./core/pdf');
const labelmake = require('labelmake');
const fs = require('fs');
const Moment = require('moment');

class users {
    constructor(logger) {
        // always initialize all instance properties
        this._logger = logger;
    }

    /**
     * Loads List of users for presentation in UI list from core database
     * @param {function} callback rows or null
     */
    getListOfUsersForUI(callback) {
        let query = "SELECT `users`.*, " +
            "`basedata`.`firstname` AS firstname, " +
            "`basedata`.`lastname` AS lastname, " +
            "organizations.name AS organizationname, " +
            "`organizations`.`id` AS organizationid " +
            "FROM `users` " +
            "LEFT JOIN `basedata` ON basedata.id = `users`.`basedataid` " +
            "LEFT JOIN `organizations` ON `basedata`.`organizationid` = `organizations`.`id` " +
            "ORDER BY `users`.id ASC";

        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this._logger.error('USERS-MODEL | Error loading users from core data: ' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    getExtendedUserInfo(userId) {
        return new Promise((resolve, reject) => {
            // all user info, first + lastname, org id + name
            const query = "SELECT `users`.*, " +
                "`basedata`.`firstname` AS firstname, " +
                "`basedata`.`lastname` AS lastname, " +
                "`organizations`.id as organizationid, " +
                "`organizations`.name " +
                "FROM `users` " +
                "LEFT JOIN `basedata` ON basedata.id = `users`.`basedataid` " +
                "LEFT JOIN `organizations` ON `basedata`.`organizationid` = `organizations`.`id` " +
                "WHERE `users`.id = ?"
            global.coreDb.all(query, [userId], (err, rows) => {
                if (err) {
                    reject(err)
                    return
                } 
                if (rows.length === 0) {
                    resolve(null)
                    return
                }

                resolve(rows[0])
            })
        })
    }

    /**
     * Adds a new User to core database
     * @param {string} username username of new user
     * @param {string} password password of new user
     * @param {integer} basedataId id of basedata entry this user belongs to
     * @param {*} callback success (true or false)
     */
    addUser(username, password, basedataId, state, callback) {
        this.doesUserExist(username, basedataId, (checkResult) => {
            if (checkResult === null) {
                // Error checking core database
                return callback(null);
            } else if (checkResult === true) {
                // Username exists, or for this basedata id another user exists
                return callback(false);
            } else if (checkResult === false) {
                // No user exists for this basedata id and the username does not exist in core database
                // Go on and create user
                

                let query = "INSERT INTO `users` (username, password, startPassword, startPasswordHash, basedataid, state) VALUES (?, ?, ?, ?, ?, ?)";
                global.coreDb.run(query,
                    [
                        username,
                        '',
                        password,
                        md5(password),
                        basedataId,
                        state
                    ],
                    function (err) {
                        if (err) {
                            console.log('ERROR: ' + err.message);
                            callback(false);
                        } else {
                            console.log('created new user');
                            callback(true);
                        }
                    });
            }

        })
    }

    /**
     * Return user entry from core database as row
     * @param {integer} uId user Id of core database table
     * @param {function} callback rows or null
     */
    getUserForId(uId, callback) {
        let query = "SELECT * FROM users WHERE id = ?";

        global.coreDb.all(query, [
            uId
        ], (err, rows) => {
            if (err) {
                this._logger.error('USERS-MODEL | Error loading user from core data: ' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns user for a given basedata id
     * @param {integer} bId 
     * @param {function} callback row or null
     */
    getUserForBasedataId(bId, callback) {
        let query = "SELECT * FROM users WHERE basedataid = ?";

        global.coreDb.all(query, [
            bId
        ], (err, rows) => {
            if (err) {
                this._logger.error('USERS-MODEL | Error loading user from core data for basedataId: ' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });
    }

    /**
     * Returns basedataId of user with username
     * @param {string} username 
     * @param {function} callback null or basedataId of user
     */
    getBasedataIdForUserWithName(username, callback) {
        let query = "SELECT * FROM users WHERE username = ?";

        global.coreDb.all(query, [
            username
        ], (err, rows) => {
            if (err) {
                this._logger.error('USERS-MODEL | Error loading user from core data for basedataId: ' + err.message);
                return callback(null);
            } else {
                if (rows.length > 0) {
                    var basedataId = rows[0].basedataid;
                    return callback(basedataId);
                } else {
                    return callback(null);
                }
            }
        });
    }

    getUserByPasswordResetToken(recoveryToken, callback) {
        if (recoveryToken == null || recoveryToken === "") {
            // we won't search for null
            callback(true, null)
            return
        }

        const query = "SELECT id FROM `users` WHERE recoverytoken = ?"
        global.coreDb.all(query, [md5(recoveryToken)], (err, rows) => {
            if (err) {
                callback(true, null)
                return
            }
            if (rows.length !== 1) {
                this._logger.error(`USERS-MODEL | PASSWORD RECOVERY | Found multiple or no users for token ${recoveryToken}`)
                callback(true, null)
                return
            }
            callback(false, rows[0].id)
        })
    } 

    /**
     * Checks if a username exists or not
     * @param {string} username 
     * @param {string} basedataId basedata Id, check if for this basedataId another user exists
     * @param {function} callback true or false (user exists or not), null if error querying core database
     */
    doesUserExist(username, basedataId, callback) {
        let query = "SELECT * FROM `users` WHERE (username = ?) OR (basedataid = ?)";
        global.coreDb.all(query, [
            username,
            basedataId
        ], (err, rows) => {
            if (err) {
                this._logger.error('USERS-MODEL | Error checking if user / basedataid exists in core data: ' + err.message);
                return callback(null);
            } else {
                if (rows.length > 0) {
                    return callback(true);
                } else {
                    return callback(false);
                }

            }
        });
    }

    /**
     * Checks if a username exists or not
     * @param {string} username 
     * @param {function} callback rows or null
     */
    getUserForUsername(username, callback) {
        let query = "SELECT * FROM `users` WHERE (username = ?)";
        global.coreDb.all(query, [
            username,
        ], (err, rows) => {
            if (err) {
                this._logger.error('USERS-MODEL | Error checking if user / basedataid exists in core data: ' + err.message);
                return callback(null);
            } else {
                if (rows.length > 0) {
                    return callback(rows);
                } else {
                    return callback(false);
                }
            }
        });
    }

    /**
     * Checks if there is a start password set for user (e.g. new password has been set by admin)
     * @param {integer} userId id of user
     * @param {*} callback boolean: true = user has start password set, false = startPassword is not set
     */
    isStartPasswordSetForUser(userId, callback) {
        let query = "SELECT * FROM users WHERE id = ?";
        global.coreDb.all(query, [
            userId,
        ], (err, rows) => {
            if (err) {
                this._logger.error('USERS-MODEL | Error checking if user / basedataid has a startpassword set in core data: ' + err.message);
                return callback(null);
            } else {
                if (rows.length > 0) {
                    if (rows[0].startPassword.length > 0) {
                        return callback(true);
                    } else {
                        return callback(false);
                    }
                } else {
                    return callback(false);
                }
            }
        });
    }

    /**
     * Sets a new startPassword for user and removes the existing hashed password
     * @param {integer} userId id of user the new startPassword shall be set for
     * @param {*} startPassword human readable start password (not hashed!!)
     * @param {*} callback success (true/false)
     */
    setStartPasswordForUser(userId, startPassword, callback) {
        let query = "UPDATE users SET startPassword = ?, startPasswordHash = ?, password = null WHERE id = ?";
        global.coreDb.run(query,
            [
                startPassword,
                md5(startPassword),
                userId
            ],
            function (err) {
                if (err) {
                    console.log('ERROR: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Sets a new password for user and removes a set startPassword
     * @param {integer} userId id of user the password shall be set for
     * @param {*} password new password to be set (not hashed)
     * @param {*} callback success (true / false)
     */
    setPasswordHashForUser(userId, password, callback) {
        let query = "UPDATE users SET startPassword = null, startPasswordHash = null, recoverytoken = null, password = ? WHERE id = ?";
        let hash = md5(password);
        global.coreDb.run(query,
            [
                hash,
                userId
            ],
            function (err) {
                if (err) {
                    console.log('ERROR: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Sets a new password for user based on a md5 hashed password
     * @param {integer} userId id of user
     * @param {*} passwordHash md5 hashed new password
     * @param {*} callback success (true/false)
     */
    setPasswordMD5HashedForUser(userId, passwordHash, callback) {
        let query = "UPDATE users SET startPassword = null, startPasswordHash = null, password = ? WHERE id = ?";
        global.coreDb.run(query,
            [
                passwordHash,
                userId
            ],
            function (err) {
                if (err) {
                    console.log('ERROR: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * 
     * @param {integer} userId id of user to be updated
     * @param {object} reqBody the request body JSON object containing all the user information
     * @param {Array} groupIdRestrictions optional list of groupIds that can be modified by the currently 
     * logged in user. If the currently logged in user is superuser, this list is null
     * @param {function} callback success, true of false
     */
    updateUser(userId, reqBody, groupIdRestrictions, callback) {

        let password = reqBody.password;
        let username = reqBody.username;

        this.getUserForId(userId, (userRow) => {
            this.getUserForUsername(username, (rows) => {
                var validNewUsername = false;
                if (rows === null) {
                    // error during query of core database
                    console.log('ERROR: Error getting user for username ' + username);
                } else if (rows.length > 0) {
                    // found username
                    if (rows[0].id === parseInt(userId)) {
                        validNewUsername = true;
                    }
                } else {
                    // user not found
                    validNewUsername = true;
                }

                if (validNewUsername === true) {
                    // new Username is not existing or belongs to own user Entry
                    // now update the User in core database

                    // Check if password has changed, if not set to old hash
                    var startPassword = null;
                    var startPasswordHash = null;
                    if (password.length > 0) {
                        startPassword = password;
                        startPasswordHash = md5(password);
                        password = '';
                        //password = md5(password);
                    } else {
                        password = userRow[0].password;
                    }

                    // Start Updating the user entry in core database
                    let query = "UPDATE `users` SET password = ?, startPassword = ?, startPasswordHash = ?, username = ? WHERE `users`.`id` = ?";
                    global.coreDb.run(query,
                        [
                            password,
                            startPassword,
                            startPasswordHash,
                            username,
                            userId
                        ],
                        function (err) {
                            if (err) {
                                console.log('ERROR: ' + err.message);
                                callback(false);
                            } else {
                                var usersgroups = new Usersgroups();
                                usersgroups.updateUsersGroups(userId, reqBody, groupIdRestrictions, (success) => {
                                    callback(true);
                                })

                            }
                        });
                } else {
                    console.log('not a valid username');
                    callback(false);
                }
            })
        })
    }



    /**
     * Returns list of all users from core database
     * @param {function} callback rows or null
     */
    getListOfUsers(callback) {
        let query = "SELECT * FROM `users`";
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this._logger.error('USERS-MODEL | Error loading users from core data: ' + err.message);
                return callback(null);
            } else {
                return callback(rows);
            }
        });

        db.query(query, (errUsers, resultUsers) => {
            if (errUsers) {
                // Errror loading organizations
                console.log('Error loading users');
                res.send('Error Reading Userlist');
                return callback(null);
            } else {
                return callback(resultUsers);
            }
        });
    }

    /**
     * Removes a user from core database
     * @param {integer} uID user id of user to be removed from core database
     * @param {function} callback success, true or false
     */
    removeUser(uID, callback) {
        let query = "DELETE FROM users WHERE id = ?";
        global.coreDb.run(query,
            [
                uID
            ],
            function (err) {
                if (err) {
                    console.log('ERROR: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Activates a user in coredatabase
     * @param {integer} uID user id of user to be activated
     * @param {function} callback success (true or false)
     */
    activateUser(uID, callback) {
        let query = "UPDATE `users` SET state = 1 WHERE `users`.`id` = ?";
        global.coreDb.run(query,
            [
                uID
            ],
            function (err) {
                if (err) {
                    console.log('ERROR: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Sets a users Theme in core database
     * @param {integer} uID id of user to set theme for
     * @param {string} theme theme to set 
     * @param {function} callback success (true/false)
     */
    switchThemeTo(uID, theme, callback) {
        let query = "UPDATE `users` SET theme = ? WHERE `users`.`id` = ?";
        global.coreDb.run(query,
            [
                theme,
                uID
            ],
            function (err) {
                if (err) {
                    console.log('ERROR setting theme of user with id ' + uID + ': ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Deactivates a user in coredatabase
     * @param {integer} uID user id of user to be deactivated
     * @param {function} callback success (true or false)
     */
    deactivateUser(uID, callback) {
        let query = "UPDATE `users` SET state = 0 WHERE `users`.`id` = ?";
        global.coreDb.run(query,
            [
                uID
            ],
            function (err) {
                if (err) {
                    console.log('ERROR: ' + err.message);
                    callback(false);
                } else {
                    callback(true);
                }
            });
    }

    /**
     * Generates a pdf document with users credentials
     * @param {integer} userId id of user the credentialreport should be generated for
     * @param {function} callback success (true/false), path to pdf document or null
     */
    generateCredentialsPDF(userId, callback) {
        this.getUserForId(userId, (rowsUser) => {
            if (rowsUser !== null) {
                // Found User in database
                var user = rowsUser[0];
                // Check if User has a startPassword set or he has changed his password
                var password = 'bereits geändert';
                var passwordQr = '';
                if (user.startPassword !== null) {
                    if (user.startPassword.length > 0) {
                        password = user.startPassword;
                        passwordQr = user.startPassword;
                    }
                }
                var basedata = new Basedata(this._logger);
                basedata.getBaseDataForId(user.basedataid, (rowsBasedata) => {
                    if (rowsBasedata !== null) {
                        // Found Basedata Record for user
                        var organizations = new Organizations(this._logger);
                        organizations.getOrganizationWithId(rowsBasedata[0].organizationid, (rowsOrganization) => {
                            if (rowsOrganization !== null) {
                                // Organization found
                                var configuration = new Configuration(this._logger);
                                configuration.getValueForKeyword('externalfqdn', (serverAddress) => {
                                    // Now read the port from plugin config
                                    var port = 5010;
                                    global.plgManager.loadConfigFromStore('restapi', (configRows) => {
                                        if (configRows !== null) {
                                            var plgConfig = configRows[0];
                                            plgConfig = JSON.parse(plgConfig.configstore);
                                            if (typeof plgConfig.fields != 'undefined') {
                                                if (Array.isArray(plgConfig.fields)) {
                                                    plgConfig.fields.forEach(field => {
                                                        if (field.fieldname === 'httpPort') {
                                                            if (field.value.length > 0) {
                                                                port = field.value;
                                                            }
                                                        }
                                                        
                                                    });                             
                                                }
                                            }
                                        } else {
                                            port = 5010;
                                        }
                                        
                                        // create qr code payload
                                        var QrPayload = {
                                            server: serverAddress,
                                            port: port,
                                            username: user.username,
                                            password: password
                                        };
                                        var QrPayloadString = JSON.stringify(QrPayload);


                                        // now start constructing the pdf document
                                        // 1. load the master template from disk
                                        var templateFile = global.appRoot + '/reports/templates/masterlayout_credentials.pdf';
                                        const basePdf = fs.readFileSync(templateFile);
                                        // 2. define template
                                        const template = {
                                            basePdf,
                                            "schemas": [
                                                {
                                                    "welcome": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 11,
                                                            "y": 34.29
                                                        },
                                                        "width": 153.17,
                                                        "height": 7.33,
                                                        "alignment": "left",
                                                        "fontSize": 13,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1
                                                    },
                                                    "introduction": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 11,
                                                            "y": 52.2
                                                        },
                                                        "width": 191.02,
                                                        "height": 12.48,
                                                        "alignment": "left",
                                                        "fontSize": 10,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1
                                                    },
                                                    "serverportLabel": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 11.26,
                                                            "y": 76.89
                                                        },
                                                        "width": 60,
                                                        "height": 5,
                                                        "alignment": "left",
                                                        "fontSize": 11,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "backgroundColor": "#f7dee0"
                                                    },
                                                    "serveraddressLabel": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 11.26,
                                                            "y": 69.89
                                                        },
                                                        "width": 60,
                                                        "height": 5,
                                                        "alignment": "left",
                                                        "fontSize": 11,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "backgroundColor": "#f7dee0"
                                                    },
                                                    "usernameLabel": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 11.26,
                                                            "y": 84.89
                                                        },
                                                        "width": 60,
                                                        "height": 5,
                                                        "alignment": "left",
                                                        "fontSize": 11,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "backgroundColor": "#f7dee0"
                                                    },
                                                    "passwordLabel": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 11.26,
                                                            "y": 91.89
                                                        },
                                                        "width": 60,
                                                        "height": 5,
                                                        "alignment": "left",
                                                        "fontSize": 11,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "backgroundColor": "#f7dee0"
                                                    },
                                                    "hintQrCode": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 11,
                                                            "y": 100
                                                        },
                                                        "width": 144.79,
                                                        "height": 11.48,
                                                        "alignment": "left",
                                                        "fontSize": 10,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1
                                                    },
                                                    "serveraddress": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 71.26,
                                                            "y": 69.89
                                                        },
                                                        "width": 85,
                                                        "height": 4.99,
                                                        "alignment": "left",
                                                        "fontSize": 11,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "fontColor": "#ff0000",
                                                        "backgroundColor": "#e6e6e6"
                                                    },
                                                    "serverport": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 71.26,
                                                            "y": 76.89
                                                        },
                                                        "width": 85,
                                                        "height": 5,
                                                        "alignment": "left",
                                                        "fontSize": 11,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "fontColor": "#ff0000",
                                                        "backgroundColor": "#e6e6e6"
                                                    },
                                                    "username": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 71.26,
                                                            "y": 84.89
                                                        },
                                                        "width": 85,
                                                        "height": 5,
                                                        "alignment": "left",
                                                        "fontSize": 11,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "fontColor": "#ff0000",
                                                        "backgroundColor": "#e6e6e6"
                                                    },
                                                    "password": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 71.26,
                                                            "y": 91.89
                                                        },
                                                        "width": 85,
                                                        "height": 5,
                                                        "alignment": "left",
                                                        "fontSize": 11,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "fontColor": "#ff0000",
                                                        "backgroundColor": "#e6e6e6"
                                                    },
                                                    "appQrCode": {
                                                        "type": "qrcode",
                                                        "position": {
                                                            "x": 161.92,
                                                            "y": 69.89
                                                        },
                                                        "width": 41,
                                                        "height": 41
                                                    },
                                                    "hintQrCodeAndroid": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 35,
                                                            "y": 260
                                                        },
                                                        "width": 37.89,
                                                        "height": 11.48,
                                                        "alignment": "left",
                                                        "fontSize": 10,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1
                                                    },
                                                    "hintQrCodeApple": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 140,
                                                            "y": 260
                                                        },
                                                        "width": 36.83,
                                                        "height": 11.21,
                                                        "alignment": "left",
                                                        "fontSize": 10,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1
                                                    },
                                                    "headlineAppStores": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 12.06,
                                                            "y": 208.27
                                                        },
                                                        "width": 189.51,
                                                        "height": 8.84,
                                                        "alignment": "left",
                                                        "fontSize": 10,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1
                                                    },
                                                    "qrCodeAppApple": {
                                                        "type": "qrcode",
                                                        "position": {
                                                            "x": 35,
                                                            "y": 220
                                                        },
                                                        "width": 35,
                                                        "height": 35
                                                    },
                                                    "qrCodeAppGoogle": {
                                                        "type": "qrcode",
                                                        "position": {
                                                            "x": 140,
                                                            "y": 220
                                                        },
                                                        "width": 35,
                                                        "height": 35
                                                    },
                                                    "timestamp": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 132.45,
                                                            "y": 283.42
                                                        },
                                                        "width": 74.42,
                                                        "height": 6.99,
                                                        "alignment": "left",
                                                        "fontSize": 10,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "fontColor": "#ffffff"
                                                    },
                                                    "divider1": {
                                                        "type": "text",
                                                        "position": {
                                                            "x": 0,
                                                            "y": 205.05
                                                        },
                                                        "width": 210.15,
                                                        "height": 0.99,
                                                        "alignment": "left",
                                                        "fontSize": 12,
                                                        "characterSpacing": 0,
                                                        "lineHeight": 1,
                                                        "backgroundColor": "#808080"
                                                    }
                                                }
                                            ]
                                        };
                                        // 3. define inputs for template fields
                                        const inputs = [
                                            {
                                                "welcome":"Willkommen bei ALARMiator " + rowsBasedata[0].firstname,
                                                "introduction":"Dieses Dokument enthält Deine persönlichen Zugangsdaten für ALARMiator.  Mit diesen Zugangsdaten kannst Du Dich in der ALARMiator Mobile App anmelden. Bitte halte Deine Zugangsdaten geheim. Mit den folgenden Informationen kannst Du Dich in den ALARMiator Mobile Apps anmelden:",
                                                "serverportLabel":"Port",
                                                "serveraddressLabel": "Serveradresse",
                                                "usernameLabel":"Benutzername",
                                                "passwordLabel":"Passwort",
                                                "hintQrCode":"Deine Zugangsdaten sind in dem QR-Code  auf der rechten Seite enthalten. Du kannst mit\n der ALARMiator Mobile  App den QR-Code scannen und damit die Anmeldung sehr einfach durchführen.",
                                                "serveraddress" : serverAddress,
                                                "serverport" : port,
                                                "username" : user.username,
                                                "password" : password,
                                                "appQrCode" : QrPayloadString,
                                                "hintQrCodeAndroid":"Apple iOS im Apple\nAppStore",
                                                "hintQrCodeApple":"Android-Version im \nGoogle PlayStore",
                                                "headlineAppStores":"Hier kannst Du die aktuellste Version der Apps herunterladen und installieren. Scanne dazu den QR-Code für Dein \nBetriebssystem. Alternativ findest Du die Apps auch unter https://www.alarmiator.de/download",
                                                "qrCodeAppGoogle" : "https://play.google.com/store/apps/details?id=de.alarmiator.android",
                                                "qrCodeAppApple" : "https://apps.apple.com/de/app/alarmiator-mobile/id1532501247",
                                                "timestamp" : "generiert am " + Moment().format('DD.MM.YYYY') + ' um ' + Moment().format('HH:mm') + ' Uhr',
                                                "divider1":" "
                                            }
                                        ];
                                        // 4. render PDF Document in Memory
                                        labelmake({ template, inputs }).then(pdf => {
                                            // 5. Write PDF to disk
                                            var outputFile = global.appRoot + '/temp/credentials_' + userId + '_' + rowsBasedata[0].firstname + rowsBasedata[0].lastname + '.pdf';
                                            try {
                                                fs.writeFileSync(outputFile, pdf);
                                                return callback(true, outputFile);
                                            } catch(err) {
                                                return callback(false, null);
                                            }
                                        });
                                        
                                    })
                                });
                            } else {
                                return callback(false, null);
                            }
                        })
                    } else {
                        // could not find Basedata record for user -> return false
                        return callback(false, null);
                    }
                })
            } else {
                // User not found, return false
                return callback(false, null);
            }
        })
    }

    /**
     * Generates a pdf document with users credentials - DEPRECATED
     * @param {integer} userId id of user the credentialreport should be generated for
     * @param {function} callback success (true/false), path to pdf document or null
     */
    generateCredentialsPDF_OLD(userId, callback) {
        this.getUserForId(userId, (rowsUser) => {
            if (rowsUser !== null) {
                // Found User in database
                console.log('rowsUser:');
                console.log(rowsUser);

                var user = rowsUser[0];
                // Check if User has a startPassword set or he has changed his password
                var password = 'bereits geändert';
                var passwordQr = '';
                if (user.startPassword !== null) {
                    if (user.startPassword.length > 0) {
                        password = user.startPassword;
                        passwordQr = user.startPassword;
                    }
                }
                var basedata = new Basedata(this._logger);
                basedata.getBaseDataForId(user.basedataid, (rowsBasedata) => {
                    if (rowsBasedata !== null) {
                        // Found Basedata Record for user
                        var organizations = new Organizations(this._logger);
                        organizations.getOrganizationWithId(rowsBasedata[0].organizationid, (rowsOrganization) => {
                            if (rowsOrganization !== null) {
                                // Organization found
                                var configuration = new Configuration(this._logger);
                                configuration.getValueForKeyword('externalfqdn', (serverAddress) => {
                                    // Now read the port from plugin config
                                    var port = 5010;
                                    global.plgManager.loadConfigFromStore('restapi', (configRows) => {

                                        if (configRows !== null) {
                                            var plgConfig = configRows[0];
                                            plgConfig = JSON.parse(plgConfig.configstore);
                                            if (typeof plgConfig.fields != 'undefined') {
                                                if (Array.isArray(plgConfig.fields)) {
                                                    plgConfig.fields.forEach(field => {
                                                        if (field.fieldname === 'httpPort') {
                                                            if (field.value.length > 0) {
                                                                port = field.value;
                                                            }
                                                        }
                                                        
                                                    });                             
                                                }
                                            }
                                        } else {
                                            port = 5010;
                                        }
                                        // generate QR Code
                                        this.generateCredentialsQR(user.username, passwordQr, serverAddress, port, (pathToQRCode) => {
                                            // now start constructing the pdf document
                                            var pdf = new Pdf(this._logger);
                                            // initialize PDF documebt
                                            pdf.initialize();
                                            // add Header to PDF document
                                            pdf.addHeader('Persönliche Zugangsdaten - ' + rowsBasedata[0].firstname + ' ' + rowsBasedata[0].lastname);
                                            pdf.addText(
                                                'Willkommen bei ALARMiator ' + rowsBasedata[0].firstname,
                                                { 
                                                    fontSize: 16, 
                                                    font: require('pdfjs/font/Helvetica') 
                                                }
                                            );
                                            pdf.addBr();
                                            pdf.addBr();
                                            pdf.addText(
                                                'Dieses Dokument enthält Deine persönlichen Zugangsdaten für ALARMiator. Mit diesen Zugangsdaten kannst Du Dich in der ALARMiator Mobile App anmelden. Bitte halte Deine Zugangsdaten geheim. Folgend findest Du auch eine Übersicht aller über Dich in ALARMiator gespeicherter Infortmationen:',
                                                { 
                                                    fontSize: 12, 
                                                    font: require('pdfjs/font/Helvetica') 
                                                }
                                            );
                                            pdf.addBr();
                                            // Adding user credentials now

                                            pdf.addTable([200, 300], 1);
                                            pdf.addTableHeader(['Information', 'Wert']);
                                            pdf.addTableRow(['Serveradresse', serverAddress]);
                                            pdf.addTableRow(['Port', port]);
                                            pdf.addTableRow(['Benutzername', user.username]);
                                            pdf.addTableRow(['Passwort', password]);
                                            
                                            // Add hint for QR-Code
                                            pdf.addText(
                                                'Deine Zugangsdaten sind in folgendem QR-Code enthalten. Du kannst mit der ALARMiator Mobile App den QR Code scannen und damit die Anmeldung sehr einfach durchführen.',
                                                {
                                                    fontSize: 12,
                                                    font: require('pdfjs/font/Helvetica'),
                                                    paddingTop: 1 * pdf.cm
                                                });
                                            pdf.addBr();

                                            pdf.addImage(pathToQRCode, 0, 0, 100, 'left');

                                            var outputFile = global.appRoot + '/temp/credentials_' + userId + '_' + rowsBasedata[0].firstname + rowsBasedata[0].lastname + '.pdf';
                                            //pdf.closePDF();
                                            pdf.writeDocToFile(outputFile, (success) => {
                                                if (success) {
                                                    return callback(true, outputFile);
                                                } else {
                                                    return callback(false, null);
                                                }
                                            });
                                        })
                                    })

                                });
                            } else {
                                return callback(false, null);
                            }
                        })
                    } else {
                        // could not find Basedata record for user -> return false
                        return callback(false, null);
                    }
                })
            } else {
                // User not found, return false
                return callback(false, null);
            }
        })
    }

    /**
     * Generates a QR Code with credentials, server and port 
     * @param {string} username username of user
     * @param {*} password password of user
     * @param {*} serveraddress address of server to be used
     * @param {*} port port of server
     * @param {*} callback path to jpeg of generated qr code
     */
    generateCredentialsQR(username, password, serveraddress, port, callback) {
        const path = global.appRoot + '/temp/credentials_QR_' + username + '.png';
        const pathjpg = global.appRoot + '/temp/credentials_QR_' + username + '.jpg';

        var payload = {
            server: serveraddress,
            port: port,
            username: username,
            password: password
        };

        QRCode.toFile(path, JSON.stringify(payload), {
            color: {
                dark: '#000', // Blue modules
                light: '#FFF' // Transparent background
            }
        }, function (err) {
            if (err) {
                throw err
            } else {
                console.log('saved.');
                const fs = require("fs");
                const pngToJpeg = require('png-to-jpeg');

                let buffer = fs.readFileSync(path);

                pngToJpeg({ quality: 90 })(buffer)
                    .then(output => {
                        let dest = fs.writeFileSync(pathjpg, output);
                        callback(pathjpg);
                    });
            }
        });
    }
}

// export the class
module.exports = users;
