/**
 * Class ofers methods for the ALARMiator Wallboard (server part)
 * 
 * @author Jens Dinstühler
 * @version 1.0.0
 * 
 */

const EventEmitter = require('events');
const Operations = require('./operationshelper');
const Feedback = require('./core/feedbackstore');
const Feedbackstates = require('./feedbackstates');
const Qualifications = require('./qualifications');

class WallboardHelper extends EventEmitter {

    /**
     * @constructor
     * @param {object} aLogger Logger to be used by class methods
     */
    constructor(aLogger) {
        super();
        this.logger = aLogger;
    }


    /**
     * Returns list of active operations as rows
     * @param {function} callback rows or null
     */
    getActiveOperations(callback) {
        var operations = new Operations(this.logger);
        operations.getListOfActiveOperations((operationsRows) => {
            return callback(operationsRows);
        })
    }

    /**
     * Returns last operation as row
     * @param {function} callback rows or null
     */
    getLastOperation(callback) {
        var operations = new Operations(this.logger);
        operations.getLastOperation((operationsRows) => {
            return callback(operationsRows);
        })
    }

    getFeedbackForOperationUUID(wallboardId, operationUUID, organizationId, callback){

        var qualifications = new Qualifications(global.logger);
        var qualificationsArray = [];
        qualifications.getQualificationsForWallboardProfile(wallboardId, (rowsQualifications) => {
            if (rowsQualifications != null) {
                if (rowsQualifications.length > 0) {
                    rowsQualifications.forEach(quali => {
                        qualificationsArray.push(quali.shortName);
                    });
                }
            }
        })
        
        var feedbackstates = new Feedbackstates(this.logger);
        feedbackstates.getAllStates((rowsFeedbackStates) => {
            var operations = new Operations(this.logger);
            operations.getOperationForUUID(operationUUID, (rowsOperation) => {
                if (rowsOperation.length > 0) {
                    var feedbackstore = new Feedback(this.logger);
                    feedbackstore.getFeedbackForOperationWithIdAndOrganization(rowsOperation[0].id, organizationId, (feedbackRows) => {
                        /**
                         * FOR DEBUGGING
                        console.log('___________________FEEDBACK_____________________');
                        console.log(feedbackRows);
                        console.log('-------------------FEEDBACK---------------------');
                        */


                        /**
                         * make sure that qualifications are distinct
                         */
                        if (feedbackRows != null) {
                            if (feedbackRows.length > 0) {
                                feedbackRows.forEach((fbRow, index) => {
                                    var qualificationsStr = fbRow.qualificationsList;
                                    if (qualificationsStr != null) {
                                        var qualificationsArr = qualificationsStr.split(',');
                                        if (Array.isArray(qualificationsArr)) {
                                            var uniqueArray = qualificationsArr.filter(function (elem, pos) {
                                                return qualificationsArr.indexOf(elem) == pos;
                                            })
                                            feedbackRows[index].qualificationsList = uniqueArray.join(',');
                                        }
                                    }
                                });
                            }
                        }


                        /** NEW kind of feedback */
                        var payloadArray = [];
                        rowsFeedbackStates.forEach((feedbackState) => {

                            var feedbackInState = [];
                            feedbackRows.forEach((feedbackRow) => {
                                if (feedbackRow.feedbackStateLabel === feedbackState.stateLabel) {
                                    feedbackInState.push(feedbackRow);
                                }
                            });

                            var payloadPart = {
                                [feedbackState.stateLabel]: feedbackInState
                            }
                            payloadArray.push(payloadPart);
                        })
                        /** END NEW Kind of feedback */


                        if (feedbackRows !== null) {
                            // filter qualifications to match the wallboard profile settings
                            feedbackRows.forEach((feedback, index) => {
                                var newQualiSting = '';
                                var qualiSting = feedback.qualificationsList;
                                if (qualiSting === null) {
                                    qualiSting = '';
                                }
                                var qualiArray = qualiSting.split(',');
                                if (qualiArray.length > 0) {
                                    qualiArray.forEach((quali) => {
                                        if (qualificationsArray.includes(quali)) {
                                            newQualiSting = newQualiSting + quali + ',';
                                        }
                                    })
                                }
                                if (newQualiSting.length > 0) {
                                    var checkChar = newQualiSting.slice(-1);
                                    if (checkChar == ',') {
                                        newQualiSting = newQualiSting.slice(0, (newQualiSting.length - 1));
                                    }
                                }
                                feedbackRows[index].qualificationsList = newQualiSting;
                            })
                            // End Filter
                            var response =
                                {
                                    feedbackStates: rowsFeedbackStates,
                                    feedback: payloadArray,
                                    qualifications: qualificationsArray,
                                    operationUUID: operationUUID
                                }
                            callback(response);
                        } else {
                            callback(null);
                        }
                    })
                } else {
                    console.log('API | getFeedback | could not find operation for uuid: ' + uuid);
                    response.status(404).send();
                }
            });
        })
    }

    /**
     * renders operation feedback to operations
     * @param {object} operationsRows operations as database rows
     * @param {*} callback rows added with feedback
     */
    compileOperationsFeedback(operationsRows, callback) {
        var operations = [];
        if (operationsRows !== null) {
            var feedback = new Feedback(this.logger);
            operationsRows.forEach((operation, index) => {
                feedback.getFeedbackForOperationWithId(operation.id, (feedbackRows) => {
                    if (feedbackRows != null) {
                        operation.feedback = feedbackRows;
                    } else {
                        operation.feedback = [];
                    }
                    operations.push(operation);
                    if (index === operationsRows.length) {
                        callback(operations);
                    }
                })
            });
        }
    }




}

module.exports = WallboardHelper;