const { v4: uuidv4 } = require('uuid');

/**
 * Class handles wallboard profiles and offers methods dealing with wallboard profiles in core database
 * @class
 */
class wallboards {
  /**
   * @constructor
   * @param {object} alogger Logger to be used for output of class methods (usually req.app.logger)
   */
  constructor(alogger) {
    // always initialize all instance properties
    this.logger = alogger;
  }

  /**
   * Returns list of all existing wallboards
   * @param {function} callback rows
   */
  getListOfWallboards(callback) {
    this.logger.debug('MODELS-WALLBOARDS | Reading wallboards');
    let query = "SELECT wallboardsStore.*, " +
      "organizations.name AS organizationName, " +
      "organizations.id AS organizationId, " +
      "locations.name AS locationName " +
      "FROM wallboardsStore " +
      "LEFT JOIN organizations ON organizations.id = wallboardsStore.organizationId " +
      "LEFT JOIN locations ON locations.id = wallboardsStore.locationId";

    global.coreDb.all(query, [], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-WALLBOARDS | Error loading wallboards from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Loads one wallboard for given id
   * @param {integer} id id of wallboard profile to load from core database
   * @param {*} callback rows or null
   */
  getWallboardForId(id, callback) {
    let query = "SELECT * FROM wallboardsStore WHERE id = ?";
    global.coreDb.all(query, [
      id
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-WALLBOARDS | Error loading wallboard from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Recaclucates a new token for given wallboard
   * @param {integer} wbId id of wallboard
   * @param {function} callback success
   */
  recalculateToken(wbId, callback) {
    let token = uuidv4();
    let query = "UPDATE wallboardsStore SET token = ? WHERE id = ?";
    global.coreDb.run(query, [
      token, 
      wbId
    ], function (err) {
      if (err) {
        console.log('ERROR updating wallboard with new Token in core database: ' + err.message);
        return callback(false);
      } else {
        return callback(true);
      }
    })
  }

  /**
   * Adds a new wallboard into core database
   * @param {object} reqBody req.form holding all the form fields
   * @param {*} callback success (true/false), null or id of inserted wallboard
   */
  addWallboard(reqBody, callback) {

    let name = reqBody.name;
    let organizationId = reqBody.organizationId;
    let namesShownAs = reqBody.namesShownAs;
    let type = reqBody.type;
    let isDefault = reqBody.isDefault;
    let state = reqBody.state;
    let token = uuidv4();
    let configIdle = reqBody.configIdle;
    let configAlarm = reqBody.configAlarm;
    let locationId = reqBody.locationId;

    if (state === null) {
      state = 1;
    }

    let query = "INSERT INTO wallboardsStore (name, organizationId, namesShownAs, type, isDefault, state, token, configIdle, configAlarm, locationId) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    global.coreDb.run(query, [
      name,
      organizationId,
      namesShownAs,
      type,
      isDefault,
      state,
      token,
      configIdle,
      configAlarm,
      locationId
    ], function (err) {
      if (err) {
        console.log('ERROR inserting wallboard in core database: ' + err.message);
        return callback(false, null);
      } else {
        var insertId = this.lastID;
        return callback(true, insertId);
      }
    })
  }


  editWallboard(wallboardId, reqBody, callback) {

    let name = reqBody.name;
    let organizationId = reqBody.organizationId;
    let namesShownAs = reqBody.namesShownAs;
    let type = reqBody.type;
    let state = reqBody.state;
    let configIdle = reqBody.configIdle;
    let configAlarm = reqBody.configAlarm;
    let locationId = reqBody.locationId;

    if (state === null) {
      state = 1;
    }

    this.checkIfDefaultWallboard(wallboardId, (isDefault) => {
      var that = this;
      if (isDefault) {
        name = null;
        organizationId = null;
        type = 'both';
      }
      let query = "UPDATE wallboardsStore SET name = ?, organizationId = ?, namesShownAs = ?, type = ?, state = ?, configIdle = ?, configAlarm = ?, locationId = ? WHERE id = ?";
      global.coreDb.run(query, [
        name,
        organizationId,
        namesShownAs,
        type,
        state,
        configIdle,
        configAlarm,
        locationId,
        wallboardId
      ], function (err) {
        if (err) {
          console.log('ERROR updating wallboard in core database: ' + err.message);
          return callback(false);
        } else {
          console.log('A');
          that.getWallboardForId(wallboardId, (wallboardRow) => {
            console.log('B');
            if (wallboardRow != null) {
              console.log('C');
              if ((wallboardRow[0].token === null) | (wallboardRow[0].token.length < 1)) {
                console.log('Rendering new Token ');
                var token = uuidv4();
                let query = "UPDATE wallboardsStore SET token = ? WHERE id = ?";
                global.coreDb.run(query, [
                  token,
                  wallboardId
                ], function (err) {
                  if (err) {
                    console.log('ERROR updating wallboard in core database: ' + err.message);
                    return callback(false);
                  } else {
                    return callback(true);
                  }
                })
              } else {
                return callback(true);
              }
            } else {
              return callback(true);
            }
          })
        }
      })
    })
  }

  /**
   * Checks if the given wallboard is the default wallboard
   * @param {integer} wid id of wallboard to check for
   * @param {*} callback default (true or false)
   */
  checkIfDefaultWallboard(wid, callback) {
    let query = "SELECT * FROM wallboardsStore WHERE id = ?";
    global.coreDb.all(query, [
      wid
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-WALLBOARDS | Error checking for default wallboard: ' + err.message);
        return callback(null);
      } else {
        if (rows.length > 0) {
          if (rows[0].isDefault === 1) {
            return callback(true);
          } else {
            return callback(false);
          }
        } else {
          return callback(false);
        }
      }
    });
  }


  /**
   * Deletes a wallboard profile from core database
   * @param {integer} id wallboard profile ID of wallboard profile to be deleted
   * @param {function} callback success, true or false
   */
  deleteWallboard(id, callback) {
    this.checkIfDefaultWallboard(id, (isDefaultWB) => {
      if (isDefaultWB != true) {
        let query = "DELETE from xt_wallboards_qualifications WHERE wallboardId = ?";
        global.coreDb.run(query, [
          id
        ], function (err) {
          if (err) {
            console.log('ERROR Deleting qualification with id ' + id + ' in core database: ' + err.message);
            return callback(false);
          } else {
            let queryWallboard = "DELETE from wallboardsStore WHERE id = ?";
            global.coreDb.run(queryWallboard, [
              id
            ], function (err) {
              if (err) {
                console.log('ERROR Deleting qualification with id ' + id + ' in core database: ' + err.message);
                return callback(false);
              } else {
                return callback(true);
              }
            })
          }
        })
      } else {
        return callback(false);
      }
    })
  }


  /**
   * Gets wallboard for token
   * @param {string} token api token for wallboard
   * @param {*} callback rows or null
   */
  getWallboardForToken(token, callback) {
    let query =
      `
        SELECT 
          wallboardsStore.*, 
          organizations.name as organizationsName, 
          organizations.shortName as organizationsShortName 
        FROM 
          wallboardsStore 
        JOIN 
          organizations ON wallboardsStore.organizationId = organizations.id 
        WHERE 
          token =? 
      `;
    global.coreDb.all(query, [
      token
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-WALLBOARDS | Error loading wallboard from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Gets wallboard for organizationsName
   * @param {string} organizationsName
   * @param {*} callback rows or null
   */
  getWallboardForOrganizationName(organizationsName, callback) {
    let query =
      `
        SELECT 
          wallboardsStore.*, 
          organizations.name as organizationsName, 
          organizations.shortName as organizationsShortName 
        FROM 
          wallboardsStore 
        JOIN 
          organizations ON wallboardsStore.organizationId = organizations.id 
        WHERE 
          organizationsName =? 
      `;
    global.coreDb.all(query, [
      organizationsName
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-WALLBOARDS | Error loading wallboard from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
     * Gets wallboard for organization
     * @param {initeger} orgId
     * @param {function} callback rows or null
     */
  getWallboardForOrganization(orgId, callback) {
    let query =
      `
      SELECT 
        wallboardsStore.*  
      FROM 
        wallboardsStore 
      WHERE 
        organizationId =?`;
    global.coreDb.all(query, [
      orgId
    ], (err, rows) => {
      if (err) {
        this.logger.error('MODELS-WALLBOARDS | Error loading wallboards for organization from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }


  /**
   * Returns selected qualifications for an existing wallboard 
   * @param {integer} id wallboard id to load qualifications for
   * @param {function} callback rows or null
   */
  getQualificationsForWallboard(id, callback) {
    let query = "SELECT * FROM xt_wallboards_qualifications WHERE wallboardId = ?";
    global.coreDb.all(query, [
      id
    ], (err, rows) => {
      if (err) {
        this._logger.error('MODELS-WALLBOARDS | Error loading qualifications for wallboard with id ' + id + ' from core data: ' + err.message);
        return callback(null);
      } else {
        return callback(rows);
      }
    });
  }

  /**
   * Sets the state of a wallboard to active
   * @param {integer} id id of wallboard to activate
   * @param {*} callback success (true or false)
   */
  setActive(id, callback) {
    console.log('Set wallboard with id ' + id + ' to active state');
    let query = "UPDATE wallboardsStore SET state = 1 WHERE id = ?";
    global.coreDb.run(query, [
      id
    ], function (err) {
      if (err) {
        console.log('ERROR activating state for wallboard with id ' + id + ' in core database: ' + err.message);
        return callback(false);
      } else {
        return callback(true);
      }
    })
  }

  /**
   * Sets the state of a wallboard to inactive
   * @param {integer} id id of wallboard to set to inactive
   * @param {*} callback success (true or false)
   */
  setInactive(id, callback) {
    let query = "UPDATE wallboardsStore SET state = 0 WHERE id = ?";
    global.coreDb.run(query, [
      id
    ], function (err) {
      if (err) {
        console.log('ERROR activating state for wallboard with id ' + id + ' in core database: ' + err.message);
        return callback(false);
      } else {
        return callback(true);
      }
    })
  }

}


// export the class
module.exports = wallboards;