const fs = require('fs');
const EventEmitter = require('events');
// Generic Plugin-Object
const Plugin = require('./plugin');
const EventManager = require('../models/eventManager');

// Objects used and distibuted by PluginManager
const FCMTokens = require('../models/fctmtokens');

/**
 * Generic ALARMiator Plugin Manager Class
 * 
 * author: Jens Dinstühler
 * vesion: 1.0.0
 * 
 * @class
 */
class ALARMiatorPluginManager extends EventEmitter {

    /**
     * @constructor
     */
    constructor() {
        super();

        // General settings for Plugin Manager
        this.inboundPluginsFolder = global.appRoot + '/plugins/inbound';
        this.outboundPluginsFolder = global.appRoot + '/plugins/outbound';
        this.pluginList = [];
        this.plugins = [];
        this.requireList = [];
        this.instances = [];
        this.pluginObjects = [];
        this.configStore = [];

        // Global Sqlite3 Storage
        this.coreDb = global.coreDb;

        // Logger for Plugin Ecosystem
        this.startLogger();
        this.logger.info('PLUGINMANAGER | Start ----------------------------');

        process.on('uncaughtException', function (error) {
            console.log(error);
        });
    }

    /**
     * Starts Logging which is provided to all plugins
     */
    startLogger() {
        // create a rolling file logger based on date/time that fires process events
        const opts = {
            logDirectory: global.appRoot + '/logs/plgmanager', // NOTE: folder must exist and be writable...
            fileNamePattern: 'ALARMiator-PluginManager-<DATE>.log',
            dateFormat: 'YYYY.MM.DD'
        };
        this.logmanager = require('simple-node-logger');
        this.logger = this.logmanager.createRollingFileLogger(opts);
        this.logger.setLevel('all');
    }

    /**
     * Reads the inbound plugins
     */
    readInboundPluginList() {
        fs.readdirSync(this.inboundPluginsFolder).forEach(file => {
            if (this.isValidPlugin(this.inboundPluginsFolder + '/' + file)) {
                var manifest = this.readPluginManifest(this.inboundPluginsFolder + '/' + file);
                this.logger.debug('PLUGINMANAGER | read Inbound Plugins | valid Plugin found | ' + manifest.name + ' : ' + this.inboundPluginsFolder + '/' + file);
                // make sure every plugin is only added once (issue #116)
                if (this.pluginList.includes(manifest) === false) {
                    this.pluginList.push(manifest);
                    //console.log('added ' + manifest);
                } else {
                    console.log('PLUGINMANAGER | didn\' add ' + manifest + ' as it has been added before');
                }
            } else {
                this.logger.debug('PLUGINMANAGER | read Inbound Plugins | invalid Plugin found: ' + this.inboundPluginsFolder + '/' + file);
            };
        });
    }

    /**
     * Reads the outbound plugins
     */
    readOutboundPluginList() {
        fs.readdirSync(this.outboundPluginsFolder).forEach(file => {
            if (this.isValidPlugin(this.outboundPluginsFolder + '/' + file)) {
                var manifest = this.readPluginManifest(this.outboundPluginsFolder + '/' + file);
                this.logger.debug('PLUGINMANAGER | read outbound Plugins | valid outbound Plugin found | ' + manifest.name + ' : ' + this.outboundPluginsFolder + '/' + file);
                // make sure every plugin is only added once (issue #116)
                if (this.pluginList.includes(manifest) === false) {
                    this.pluginList.push(manifest);
                    //console.log('added ' + manifest);
                } else {
                    console.log('PLUGINMANAGER | didn\' add ' + manifest + ' as it has been added before');
                }
            } else {
                this.logger.debug('PLUGINMANAGER | read outbound Plugins | invalid outbound Plugin found: ' + this.outboundPluginsFolder + '/' + file);
            };
        });
    }

    /**
     * Checks wether a plugin in pluginstore does not exist anymore and wipes it from core database
     * @param {function} callback success (true/false)
     */
    cleanupPluginStore(callback) {
        var sql = "SELECT * FROM pluginstore";
        this.coreDb.all(sql, [], (err, rowsPluginStore) => {
            if (err) {
                this.logger.error('PLUGINMANAGER | ' + err);
                return callback(false);
            }
            var pluginWipeList = [];
            if (rowsPluginStore != null) {
                if (rowsPluginStore.length > 0) {
                    rowsPluginStore.forEach((dbPlugin) => {
                        var pluginStillExisting = false;
                        this.pluginList.forEach((plugin) => {
                            if (plugin.namespace === dbPlugin.namespace) {
                                pluginStillExisting = true;
                            }
                        })
                        if (pluginStillExisting === false) {
                            pluginWipeList.push(dbPlugin.id);
                        }
                    })
                }
            }
            // Now remove found zombies
            if (Array.isArray(pluginWipeList)) {
                if (pluginWipeList.length > 0) {
                    var wipeSuccess = true;
                    this.coreDb.serialize(() => {
                        pluginWipeList.forEach((plgId) => {
                            let query = "DELETE FROM pluginstore WHERE id = ?";
                            this.coreDb.run(query,
                                [
                                    plgId
                                ],
                                function (err) {
                                    if (err) {
                                        wipeSuccess = false;
                                        console.log('PLUGINMANAGER | Error removing zombie plugin entry from pluginstore: ' + err.message);
                                    } else {
                                        console.log('PLUGINMANAGER | Successfully wiped zombie entry for not existing plugin from pluginstore with id ' + plgId);
                                    }
                                });
                        });
                    });
                    return callback(wipeSuccess);
                } else {
                    console.log('PLUGINMANAGER | successfully checked for zombie entrys in pluginstore');
                    return callback(true);
                }
            } else {
                console.log('PLUGINMANAGER | no plugins in wipe list -> everything up to date');
                return callback(true);
            }

        });
    }

    /**
     * Registers Plugin in Core Plugin Store
     * @param {object} manifest manifest object
     */
    registerPluginsInStore(pluginList) {
        // check for missing plugins and remove them from pluginsstore
        this.cleanupPluginStore((success) => {
            pluginList.forEach((manifest) => {
                // check if entry exists for plugin
                var sql = "SELECT * FROM pluginstore WHERE namespace = '" + manifest.namespace + "'";
                this.coreDb.all(sql, [], (err, rows) => {
                    if (err) {
                        this.logger.error('PLUGINMANAGER | ' + err);
                    }
                    //this.logger.debug('PLUGINMANAGER | ' + rows.length + ' configurations in database found for Plugin ' + manifest.name)
                    if (rows.length > 0) {
                        // existing entry
                        // Check if version of installed plugin is different to the manifest version
                        if (manifest.version !== rows[0].version) {

                            // manifest has differen version than plugin config in database
                            // if manifest offers update-method, call this method
                            this.logger.debug('PLUGINMANAGER | Plugin ' + manifest.name + ' shows different version (' + manifest.version + ') than registered in pluginstore (' + rows[0].version + ')');
                            this.migrateConfigStore(manifest.configstore, rows[0].configstore, (migrationSuccess, migratedConfigStore) => {
                                if (migrationSuccess === true) {
                                    manifest.configstore = migratedConfigStore;
                                }
                            });
                            this.logger.debug('PLUGINMANAGER | Updating version information for plugin ' + manifest.name + ' in config store');
                            this.coreDb.run(`UPDATE pluginstore SET version = ?, configstore = ? WHERE namespace = ?`,
                                [
                                    manifest.version,
                                    JSON.stringify(manifest.configstore),
                                    manifest.namespace
                                ],
                                function (err) {
                                    if (err) {
                                        console.log('PLUGINMANAGER | Error updating configuratio for plugin ' + manifest.name + ' in config store: ' + err.message);
                                    } else {
                                        // get the last insert id
                                        console.log(`PLUGINMANAGER | Configuration for plugin ` + manifest.name + ` successfully updated in config store`);
                                    }
                                });
                        }
                    } else {
                        // new plugin, no entry in plugin store
                        this.coreDb.run(`INSERT INTO pluginstore (namespace, version, state, configstore, plgtype, alarmingByUser, fcmtokenconsumer) VALUES(?, ?, ?, ?, ?, ?, ?)`,
                            [
                                manifest.namespace,
                                manifest.version,
                                manifest.state,
                                JSON.stringify(manifest.configstore),
                                manifest.plgtype,
                                manifest.alarmingByUser,
                                manifest.fcmtokenconsumer
                            ],
                            function (err) {
                                if (err) {
                                    console.log('PLUGINMANAGER | Error inserting new plugin to config store: ' + err.message);
                                } else {
                                    //this.logger.info('PLUGINMANAGER | successfully inserted plugin into config store');
                                    // get the last insert id
                                    // check if plugin manfiest offers a setupMethod which needs to be called for the plugin
                                    if (manifest.setupMethod !== null) {
                                        if (typeof manifest.setupMethod !== 'undefined') {
                                            if (manifest.setupMethod.length !== 0) {
                                                //this.logger.info(`PLUGINMANAGER | Executing Setup method for plugin ` + manifest.name);
                                                // There is a setup method definded
                                                // Call this method now
                                                // TBD
                                            }
                                        }
                                    }
                                }
                            });
                    }
                });
            })
        })
    }

    /**
     * Migrates existing configStore from database into new manifest-based configStore and returns the migrated configStore
     * @param {object} manifestConfigStore configStore object from manifest
     * @param {object} databaseConfigStore configStore object from database
     * @param {object} callback migrated configStore object
     */
    migrateConfigStore(manifestConfigStore, databaseConfigStore, callback) {
        var migrationSuccess = false;
        var dbConfigStore = null;
        try {
            dbConfigStore = JSON.parse(databaseConfigStore);
        } catch {
            console.log('PLUGINMANAGER | Error parsing dbConfigStore');
            // Error parsing database config store
        }
        if (dbConfigStore != null) {
            // Extract the fields for the configuration from both configStore properties
            var dbFields = [];
            if (typeof dbConfigStore.fields != 'undefined') {
                dbFields = dbConfigStore.fields;
            }
            var manifestFields = [];
            if (typeof manifestConfigStore.fields != 'undefined') {
                manifestFields = manifestConfigStore.fields;
            }
            // Now iterate over manifest fields an check if for a field a value in database field exists
            // if there is a value set in database, set it in manifest array too
            manifestFields.forEach((manField, manFieldIdx) => {
                var newField = true;
                dbFields.forEach((dbField) => {
                    if (manField.fieldname === dbField.fieldname) {
                        newField = false;
                        if (typeof dbField.value != 'undefined') {
                            manifestFields[manFieldIdx].value = dbField.value;
                        } else {
                            if (typeof manifestFields[manFieldIdx].presetvalue != 'undefined') {
                                manifestFields[manFieldIdx].value = manifestFields[manFieldIdx].presetvalue;
                            }
                        }
                    }
                })
                if (newField === true) {
                    if (typeof manifestFields[manFieldIdx].presetvalue != 'undefined') {
                        manifestFields[manFieldIdx].value = manifestFields[manFieldIdx].presetvalue;
                    }
                }
            })
            manifestConfigStore.fields = manifestFields;
            console.log('PLUGINMANAGER | MIGRATE-PLUGIN | finished migration');
            migrationSuccess = true;
        }
        return callback(migrationSuccess, manifestConfigStore);
    }

    /**
     * Initializes all Plugins in PluginList which are active
     */
    initializePlugins() {
        this.pluginList.forEach((plugin) => {
            if (this.isPluginActive(plugin.namespace)) {
                this.logger.info('PLUGINMANAGER | Initializing plugin ' + plugin.pluginpath);
                var instance = {
                    namespace: plugin.namespace,
                    classPath: plugin.pluginpath + '/base.js'
                }
                this.requireList.push(instance);

                //this.plugins.push(require(plugin.pluginpath + '/' + plugin.main));
            } else {
                this.logger.info('PLUGINMANAGER | Ignoring plugin ' + plugin.pluginpath + ' (not activated)');
            }
        });
        // Now instanziate plugins

        this.requireList.forEach((instance, index) => {
            this.loadClass(instance.classPath, instance.namespace);
        });
        this.logger.info('PLUGINMANAGER | Finished initialization of plugins.');
    }

    /**
     * instanziates a new class object for given namespace
     * @param {string} Path filepath to js-file containing the class
     * @param {*} namespace namespace the new class instance shall be registered for
     */
    loadClass(Path, namespace) {
        this.logger.info(`PLUGINMANAGER | initialising ${Path} for namespace ${namespace}`);
        var requiredClass = require(Path);
        this.pluginObjects[namespace] = new requiredClass;
    }

    /**
     * Checks if plugin is valid (has needed files in path)
     * @param {string} path base path of plugin directory to be checked
     */
    isValidPlugin(path) {
        var retVal = false;
        try {
            if (fs.existsSync(path + '/manifest.json')) {
                retVal = true;
            }
        } catch (err) {
            retVal = false;
        }
        return retVal;
    }

    /**
     * Reads plugin manifest and return its content as an object.
     * @param {strign} path base path of plugin directory to be checked
     */
    readPluginManifest(path) {
        var manifestObj = null;
        try {
            var manifest = JSON.parse(fs.readFileSync(path + '/manifest.json', 'utf8'));
            var manifestObj = new Plugin();
            manifestObj.name = manifest.name;
            manifestObj.namespace = manifest.namespace;
            manifestObj.logidentifier = manifest.logidentifier;
            manifestObj.version = manifest.version;
            manifestObj.description = manifest.description;
            manifestObj.main = manifest.main;
            manifestObj.author = manifest.author;
            manifestObj.plgtype = manifest.plgtype;
            manifestObj.state = 0;
            manifestObj.pluginpath = path;
            manifestObj.isService = manifest.isService;
            manifestObj.configstore = manifest.configstore;
            manifestObj.setupMethod = manifest.setupMethod;
            manifestObj.updateMethod = manifest.updateMethod;
            if (typeof manifest.alarmingByUser !== 'undefined') {
                manifestObj.alarmingByUser = manifest.alarmingByUser;
            }
            if (typeof manifest.fcmtokenconsumer !== 'undefined') {
                manifestObj.fcmtokenconsumer = manifest.fcmtokenconsumer;
            }

        } catch (err) {
            this.logger.error('PLUGINMANAGER | Error reading Manifest for Plugin ' + path);
            this.logger.error('PLUGINMANAGER | ' + err.message);
        }
        return manifestObj;
    }

    /**
     * Resets Plugin in Core Plugin Store
     * @param {object} manifest manifest object
     */
    resetPluginInStore(manifest) {
        // Query for reseting plugin in pluginStore back to manifest settings
        var query = `UPDATE 'pluginstore' SET ` +
            `'version' = ?, ` +
            `'state' = ?, ` +
            `'configstore' = ?, ` +
            `'plgtype' = ? ` +
            "WHERE `pluginstore`.`namespace` = ?";

        this.coreDb.run(query,
            [
                manifest.version,
                manifest.state,
                JSON.stringify(manifest.configstore),
                manifest.plgtype,
                manifest.namespace
            ],
            function (err) {
                if (err) {
                    this.logger.error('PLUGINMANAGER | Error reseting plugin in plugin store: ' + err.message);
                } else {
                    this.logger.debug('PLUGINMANAGER | Reseted plugin [' + manifest.namespace + '] in plugin store');
                }
            }.bind(this));
    }

    /**
     * resets Manifest for given namespace
     * @param {string} namespace namespace the manifest should be reseted for
     */
    resetManifestForNamespace(namespace) {
        // get path for namespace
        var refreshedPluginList = [];
        var bolUpdated = false;

        this.pluginList.forEach((manifest) => {
            if (manifest.namespace === namespace) {
                // we've found the manifest for given namespace
                // now extract path and call readPluginManifest
                var plgPath = manifest.pluginpath;
                var updatedManifest = this.readPluginManifest(plgPath);
                this.resetPluginInStore(updatedManifest);
                refreshedPluginList.push(updatedManifest);
                bolUpdated = true;
            } else
                refreshedPluginList.push(manifest);
        });

        if (bolUpdated === true) {
            // a new manifest has been read
            this.pluginList = refreshedPluginList;
            return true;
        } else {
            return false;
        }
    };

    /**
     * loads config json from configstore and represents it as an object
     * @param {string} namespace 
     */
    loadConfigFromStore(namespace, callback) {
        var sql = "SELECT configstore FROM pluginstore WHERE namespace = '" + namespace + "'";
        this.coreDb.all(sql, [], (err, rows) => {
            if (err) {
                this.logger.error('PLUGINMANAGER | ' + err);
                callback(null);
            }
            if (rows.length > 0) {
                this.logger.debug('PLUGINMANAGER | Anzahl Einträge für Plugin [' + namespace + '] in ConfigStore: ' + rows.length);
                callback(rows);
            } else {
                this.logger.debug('PLUGINMANAGER | Keine Einträge für Plugin [' + namespace + '] im Config Store');
            }
        });
    }

    /**
     * Loads the plugins configuration from core database into class member configStore
     * @param {function} callback success (true or false)
     */
    initializeConfigFromStore(callback) {
        let query = "SELECT * FROM pluginstore";
        global.coreDb.all(query, [], (err, rows) => {
            if (err) {
                this.logger.error('PLUGINMANAGER | Error loading plugin configurations from core database into memory: ' + err.message);
                this.configStore = [];
                return callback(false);
            }
            this.logger.debug('PLUGINMANAGER | loaded plugin configurations from core database into memory');
            this.configStore = rows;
            return callback(true);
        });

    }

    /**
     * returns boolean if plugin is configured as active in config store in core database
     * Check is based on configStore information pulled by initializeConfigFromStore.
     * @param {string} namespace 
     */
    isPluginActive(namespace) {
        var retVal = false;
        if (Array.isArray(this.configStore)) {
            this.configStore.forEach((config) => {
                if (config.namespace === namespace) {
                    if (config.state === 1) {
                        retVal = true;
                    }
                }
            })
        }
        return retVal;
    }

    /**
     * Returns number of active inbound plugins
     * @returns integer
     */
    getCountActivePlugingsInbound() {
        var count = 0;
        if (Array.isArray(this.configStore)) {
            this.configStore.forEach((config) => {
                if (config.state === 1) {
                    if (config.plgtype === 'inbound') {
                        count++;
                    }
                }
            })
        }
        return count;
    }

    /**
     * Returns number of active outbound plugins
     * @returns integer
     */
    getCountActivePlugingsOutbound() {
        var count = 0;
        if (Array.isArray(this.configStore)) {
            this.configStore.forEach((config) => {
                if (config.state === 1) {
                    if (config.plgtype === 'outbound') {
                        count++;
                    }
                }
            })
        }
        return count;
    }

    /**
     * Returns the file path for the plugin identified with namespace
     * @param {string} namespace namespace of plugin the pluginpath shall be returned for
     */
    getPluginPath(namespace) {
        console.log('PLUGINMANAGER | identifying pluginpath for plugin with namespace: ' + namespace);
        var retVal = null;
        this.pluginList.forEach((plugin) => {
            if (plugin.namespace === namespace) {
                retVal = plugin.pluginpath;
            }
        })
        return retVal;
    }

    /**
     * Returns Array with Plugins which offer alarming which can be configured seperately by user
     */
    getPluginsWithAlarmingByUser() {
        var arrPlugins = [];
        if (Array.isArray(this.configStore)) {
            this.configStore.forEach((config) => {
                if (config.alarmingByUser === 1) {
                    arrPlugins.push(config);
                }
            })
        }
        return arrPlugins;
    }






    /**
     * 
     * ------------ Event Handling --------------------
     * 
     * inbound Events, plugin Manager offers to plugins
     */

    /**
     * Sends a notification to a plugin addressed by namespace
     * @param {string} namespace namespace of plugin the message shall be forwarded to
     * @param {string} message payload for the plugin addressed with namespace
     */
    event_plugin_notification(namespace, message) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_plugin_notification');
        this.emit('event_plugin_notification', namespace, message);
    }

    /**
     * the configuration for a plugin with namespace has been changed.
     * The plugin shall itself reload the configuration and restart itself and existing services.
     * @param {string} namespace namespace of the plugin whose config has been changed
     */
    event_pluginConfig_refreshed(namespace) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_pluginConfig_refreshed for ' + namespace);
        this.emit('event_pluginConfig_refreshed', namespace);
    }

    /**
     * a new alarm took place
     * @param {object} alarmInfo alarm object
     */
    event_new_alarm(alarmInfo) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_alarm');
        this.logger.debug(alarmInfo);
        this.emit('event_new_alarm', alarmInfo);
    }

    /**
     * alarm has been archived
     * @param {integer} operationId id of archived operation
     */
    event_alarm_archived(operationId) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_alarm_archived');
        this.emit('event_alarm_archived', operationId);
    }

    /**
     * alarm has been closed
     * @param {integer} operationId id of archived operation
     */
    event_alarm_closed(operationId) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_alarm_closed');
        this.emit('event_alarm_closed', operationId);
    }

    /**
     * a new alarm has been stored to core database
     * @param {integer} operationId id of added operation in database table operationsStore
     */
    event_new_alarm_persisted(operationId) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_alarm_persisted');
        this.logger.debug(operationId);
        this.emit('event_new_alarm_persisted', operationId);
    }


    /**
     * a new alarm took place and rules have been evaluated
     * @param {object} alarmInfo alarm object
     * @param {object} usersToAlarm Users to alarm
     */
    event_new_alarm_with_rules(alarmInfo, usersToAlarm) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_alarm_with_rules');
        this.logger.debug('------------------------------ ALARMINFO ---------------------------------');
        this.logger.debug(alarmInfo);
        this.logger.debug('------------------------------ usersToAlarm ------------------------------');
        this.logger.debug(usersToAlarm);
        this.logger.debug('--------------------------------------------------------------------------');
        this.emit('event_new_alarm_with_rules', alarmInfo, usersToAlarm);
    }

    /**
     * a new testalarm shall be sent
     * @param {object} alarmInfo alarm object
     */
    event_new_test_alarm(basedataId, pluginNamespace, alarmInfo) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_test_alarm | namespace --> ' + pluginNamespace + ' | basedataId: ' + basedataId);
        this.emit('event_new_test_alarm', basedataId, pluginNamespace, alarmInfo);
    }

    /**
     * a new testalarm is requested by the user and needs to be pushed into the queue
     * @param {integer} basedataId id of basedata the test alarm shall be sent for
     * @param {string} pluginNamespace namespace of outbound plugin the test alarm shall be processed by
     * @param {string} token token to be alarmed
     * @param {integer} offset time offset in seconds
     */
    event_new_test_alarm_requested(basedataId, pluginNamespace, token, offset) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_test_alarm_requested | namespace --> ' + pluginNamespace + ' | basedataId: ' + basedataId) + ' | offset: ' + offset;
        this.emit('event_new_test_alarm_requested', basedataId, pluginNamespace, token, offset);
    }

    /**
     * a new testalarm shall be sent out now from the queue
     * @param {string} basedataId basedataId to be alarmed
     * @param {string} pluginNamespace namespace of plugin the test alarm shall be processed by
     * @param {string} token token to be alarmed
     */
    event_new_token_test_alarm(basedataId, pluginNamespace, token) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_token_test_alarm | namespace' + pluginNamespace);
        this.emit('event_new_token_test_alarm', basedataId, pluginNamespace, token);
    }
    /**
     * A new notification to admin shall be sent
     * @param {object} notificationInfo notification object
     */
    event_new_admin_notification(notificationInfo) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_admin_notification');
        this.emit('event_new_admin_notification', notificationInfo);
    }

    /**
     * A new eMail shall be sent
     * @param {object} emailInfo email info object
     */
    event_send_email(emailInfo) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_send_email');
        this.emit('event_send_email', emailInfo);
    }

    /**
     * There was a new eMail added to the queue
     * @param {objects} emailRows 
     */
    event_email_queue_refreshed(emailRows) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_email_queue_refreshed');
        this.emit('event_email_queue_refreshed', emailRows);
    }

    /**
     * a new state has been sent
     * @param {object} stateInfo State object
     */
    event_new_state(stateInfo) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_state');
        this.logger.debug(stateInfo);
        this.emit('event_new_state', stateInfo);
    }

    /**
     * a new alarm via zvei-code took place
     * @param {object} zveiInfo zvei object
     */
    event_new_zveialarm(zveiInfo) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_zveialarm');
        this.logger.debug(zveiInfo);
        this.emit('event_new_zvei_alarm', zveiInfo);
    }

    /**
     * external ip address has changed
     * @param {string} ipInfo 
     */
    event_refresh_extip(ipInfo) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_refresh_extip');
        this.emit('event_refresh_extip', ipInfo);
    }

    /**
     * a pdf document waits for printing to the default printer
     * @param {string} pathToPDF 
     */
    event_print_pdf(pathToPDF) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_print_pdf ');
        this.emit('event_print_pdf', pathToPDF);
    }

    /**
     * Fires a generic event
     * @param {string} eventName name of the event to fire
     * @param {string} payload stringified object or string to be sent as payload
     */
    event_generic(eventName, payload) {
        console.log('PLUGINMANAGER | event_generic | ' + eventName);

        var eventmanager = new EventManager();
        var payloadType = 'string';
        var transmitPayload = null;

        eventmanager.getEvent(eventName, (resultEvent) => {
            if (resultEvent != null) {
                if (resultEvent.length > 0) {
                    payloadType = resultEvent[0].payloadType;
                    console.log('PLUGINMANAGER | event_generic | ' + eventName + ' | type: ' + payloadType);
                    if (payloadType === 'object') {
                        try {
                            transmitPayload = JSON.parse(payload);
                        } catch (err) {
                            console.log('Error: ' + err.message);
                            transmitPayload = payload;
                        }
                    } else if (payloadType === 'multiline') {
                        transmitPayload = payload;
                    }
                    console.log('payload to be sent with event:');
                    console.log(transmitPayload);

                    this.emit(eventName, transmitPayload);
                }
            }
        })
    }

    /**
     * a new feedback response has been received
     * @param {string} operationUUID uuid of operation, this feedback belongs to
     * @param {string} basedataUUID uuid of basedata sending the new state
     * @param {integer} stateId id of feedback state from table
     */
    event_new_feedback_received(operationUUID, basedataUUID, stateId) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_new_feedback_received');
        this.emit('event_new_feedback_received', operationUUID, basedataUUID, stateId);
    }


    /**
     * server configuration has been changed
     * @param {array} emitChangedValuesArray Array of server parameters with information which changed and which not
     */
    event_server_config_changed(emitChangedValuesArray) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_server_config_changed');
        this.emit('event_server_config_changed', emitChangedValuesArray);
    };

    /**
     * wallboard profile configuration has been stored / changed
     * @param {integer} wallboardId id of wallboard profile which configuration has changed
     */
    event_wallboard_config_changed(wallboardId) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_wallboard_config_changed');
        this.emit('event_wallboard_config_changed', wallboardId);
    }

    /**
     * server shutdown
     */
    event_server_shutdown() {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_server_shutdown');
        this.emit('event_server_shutdown');
    };

    /**
     * Received new data from katsys
     * @param {object} data KatSys Payload
     */
    event_katsys_incoming_data(data) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_katsys_incoming_data');
        this.emit('event_katsys_incoming_data', data);
    }

    /**
     * Fcm sending report
     * @param {array} failedFcmTokenList failed fcm token list
     * @param {array} successFcmTokenList success fcm token list
     */
    event_fcm_sending_report(failedFcmTokenList, successFcmTokenList) {
        this.logger.info('PLUGINMANAGER | EVENT | BROADCAST | event_fcm_sending_report');
        this.emit('event_fcm_sending_report', failedFcmTokenList, successFcmTokenList);
    }

    /**
     * General Notification for Core and Plugins to exchange Information about state changes or maintenance information
     * @param {object} notification 
     */
    event_notification(notification) {
        this.emit('event_notification', notification);
    }
}





module.exports = ALARMiatorPluginManager;