const EventEmitter = require('events');
const gk = require('gauss-krueger');
const Operation = require(__dirname + '/../../../../models/core/core_operation.js');
const moment = require('moment');
const ParserUtils = require('./parser-utils');

class ParserLAU extends EventEmitter {

    statusMap = {
        '32770': 'Priorisierter Sprechwunsch',
        '32771': 'Einsatzbereit über Funk',
        '32772': 'Einsatzbereit auf Wache',
        '32773': 'Einsatz übernommen',
        '32774': 'Am Einsatzort eingetroffen',
        '32775': 'Sprechwunsch (einsatzbezogen)',
        '32776': 'Nicht Einsatzbereit',
        '32777': '7',
        '32778': '8',
        '32779': 'Fremdanmeldung'
    }

    statusMappingToStatusShort = {
        '32770': 0,
        '32771': 1,
        '32772': 2,
        '32773': 3,
        '32774': 4,
        '32775': 5,
        '32776': 6,
        '32777': 7,
        '32778': 8,
        '32779': 9
    }

    constructor() {
        super();
        // Parser-Log-Name
        this.parserLogName = 'PARSER-ILSLAU'

        // Status Data
        this.issi = null;
        this.radioStatus = null;
        this.radioStatusHumanReadable = null;
        this.radioStatusShort = null;

        this.parserUtils = new ParserUtils();
    }

    splitZVEIsDelta(sdsSchleifenDelta) {
        let datePart = moment().format('YYYY-MM-DD');
        let datePartYesterday = moment().subtract(1, 'days').format('YYYY-MM-DD');
        let zveiDeltaObj = [];
        if (sdsSchleifenDelta != null) {
            if (sdsSchleifenDelta.length > 0) {
                // split now and extract time

                // for cases where separator is "\n"
                // var sdsSchleifenDeltaArr = sdsSchleifenDelta.split('\n');
                // for cases where separator is ","
                var sdsSchleifenDeltaArr = sdsSchleifenDelta.split(',');
                if (Array.isArray(sdsSchleifenDeltaArr)) {
                    // successfully separated zveis with time into array
                    sdsSchleifenDeltaArr.forEach(zveiWithTime => {
                        zveiWithTime = zveiWithTime.trim();
                        let elementArr = zveiWithTime.split(' ');
                        if (elementArr.length == 2) {
                            // found 2 parts (zvei | time)
                            let timestamp = moment(datePart + ' ' + elementArr[1]).format('YYYY-MM-DDTHH:mm:ss');
                            let sameOrBefore = moment().add(1, 'minutes').isSameOrBefore(timestamp);
                            console.log('PARSING: zvei: ' + elementArr[0] + ' | time: ' + elementArr[1] + ' | timestamp: ' + timestamp + '| same or before now: ' + sameOrBefore);
                            if (sameOrBefore) timestamp = moment(datePartYesterday + ' ' + elementArr[1]).format('YYYY-MM-DDTHH:mm:ss');
                            let zveiDeltaElement = {
                                zvei: elementArr[0],
                                timestamp: timestamp
                            }
                            zveiDeltaObj.push(zveiDeltaElement);
                        } else if (elementArr.length == 1) {
                            console.log('WARNING: only zvei element found but no time: ' + zveiWithTime);
                            let zveiDeltaElement = {
                                zvei: elementArr[0],
                                timestamp: null
                            }
                            zveiDeltaObj.push(zveiDeltaElement);
                        } else {
                            console.log('ERROR: could not parse Delta-Zvei Information for Element: ' + zveiWithTime);
                        }
                    });
                } else {
                    // not an array, do nothing
                    return [];
                }
            }
        }
        return zveiDeltaObj;
    }

    parseRAWAlarmData(rawAlarmDataArray) {
        console.log(this.parserLogName + ' | PARSING RAW ALARM');
        console.log('----------- RAW DATA inside parser ---------- ');
        console.log(rawAlarmDataArray);
        console.log('----------- END RAW DATA inside parser ----- ');
        this.operation = new Operation();
        this.operation.section = rawAlarmDataArray.sds_abschnitt;
        this.operation.alarmdate = rawAlarmDataArray.sds_alarmdatum;
        this.operation.alarmtime = rawAlarmDataArray.sds_alarmuhrzeit;

        if (rawAlarmDataArray.sds_einsatzmittel != undefined) {
            this.operation.gear = rawAlarmDataArray.sds_einsatzmittel;
        }

        this.operation.operationnumber = rawAlarmDataArray.sds_einsatznummer;
        this.operation.location = rawAlarmDataArray.sds_einsatzort;
        this.operation.message = rawAlarmDataArray.sds_freitext;
        this.operation.community = rawAlarmDataArray.sds_gemeinde;
        this.operation.rawdata = rawAlarmDataArray.sds_komplett;
        this.operation.object = rawAlarmDataArray.sds_objekt;
        this.operation.district = rawAlarmDataArray.sds_ortsteil;
        this.operation.keywordId = rawAlarmDataArray.sds_schlagwort_id;
        this.operation.keywordCategory = rawAlarmDataArray.sds_schlagwort_kategorie;
        this.operation.keywordRaw = rawAlarmDataArray.sds_schlagwort_komplett;
        this.operation.keywordName = rawAlarmDataArray.sds_schlagwort_name;

        try {
            // TEMP FIX
            // remove times from zveis list (by extracting only 4-5 digit zveis and zveis with an "F" at the end)
            var re = /[0-9]{4,}[F]*/gm;
            var matchArray = rawAlarmDataArray.sds_schleifen.match(re);
            var newZveis = matchArray.join(', ');
            this.operation.zveis = newZveis;
        }
        catch (exception) {
            console.log(exception);
        }

        try {
            this.operation.zveisDelta = rawAlarmDataArray.sds_schleifen_delta;
            // parse zvei delta
            let sdsSchleifenDelta = rawAlarmDataArray.sds_schleifen_delta;
            if (sdsSchleifenDelta != null) {
                try {
                    //this.operation.zveisDeltaCalc = this.splitZVEIsDelta(sdsSchleifenDelta);
                    this.operation.zveisDeltaCalc = this.parserUtils.calculateZveisGearData("zvei", rawAlarmDataArray.sds_schleifen_alle, sdsSchleifenDelta)
                    if (this.operation.zveis.value == null){
                        try {
                            this.operation.zveis = this.operation.zveisDeltaCalc.value.reduce((acc, currentValue) => {
                                acc.push(currentValue.zvei);
                                return acc;
                            }, []).join(',');
                        }
                        catch (zveiException) {
                            console.log(zveiException);
                        }
                    }
                } catch (err) {
                    console.log('ERROR calling calculateZveisGearData: ' + err);
                }
                /*
                console.log('------------- Calculated ZVEIs Delta ----------------');
                console.log(this.operation.zveisDeltaCalc);
                console.log('.....................................................');
                */
            }
            // end parse zvei delta
        }
        catch (exp) {

        }

        try {
            this.operation.gearData = rawAlarmDataArray.sds_einsatzmittel_delta;
            // parse gear data
            let sdsGearDelta = rawAlarmDataArray.sds_einsatzmittel_delta;
            if (sdsGearDelta != null) {
                try {
                    this.operation.gearDataCalc = this.parserUtils.calculateZveisGearData("gear", rawAlarmDataArray.sds_einsatzmittel_alle, sdsGearDelta);
                    //console.log(this.operation.gearDataCalc);
                    if (this.operation.gear.value == null){
                        try {
                            this.operation.gear = this.operation.gearDataCalc.value.reduce((acc, currentValue) => {
                                acc.push(currentValue.gear);
                                return acc;
                            }, []).join('\n');
                        }
                        catch (gearException) {
                            console.log(gearException);
                        }
                    }                   
                } catch (err) {
                    console.log('ERROR calling calculateZveisGearData: ' + err);
                }
                /*
                console.log('------------- Calculated Gear Data ----------------');
                console.log(this.operation.gearDataCalc);
                console.log('.....................................................');
                */
            }
            // end parse gear data
        }
        catch (exp) {

        }

        this.operation.subject = rawAlarmDataArray.sds_stichwort;
        this.operation.street = rawAlarmDataArray.sds_strasse;
        this.operation.gkx = rawAlarmDataArray.sds_x;
        this.operation.gky = rawAlarmDataArray.sds_y;
        this.operation.floor = rawAlarmDataArray.sds_etage;
        this.operation.ils = rawAlarmDataArray.sds_ils;
        this.calculateLatLon();

        console.log(this.parserLogName + ' | ----------- Operation Object inside parser prio raising event alarm_parsed ---------- ');
        console.log(this.operation);
        console.log(this.parserLogName + ' | ----------- ENDE Operation Object inside parser prior raising event alarm parsed ---- ');

        this.emit('alarm_parsed', null);
    }

    /**
     * injects a test alarm in parser to test following process
     * FOR TESTING
     */
    TestparseRAWAlarmData() {
        console.log(this.parserLogName + ' | TEST | PARSING RAW ALARM');

        this.operation = new Operation();

        this.operation.section = 'Kreuzung';
        this.operation.alarmdate = 'ALARMDATUM';
        this.operation.alarmtime = 'ALARMZEIT';
        this.operation.gear = 'EINSATZMITTEL';
        this.operation.operationnumber = 'EINSATZNUMMER';
        this.operation.location = 'LOCATION';
        this.operation.message = 'MITTEILUNG';
        this.operation.community = 'GEMEINDE';
        this.operation.rawdata = 'KOMPLETTE SDS';
        this.operation.object = 'OBJEKT';
        this.operation.district = 'ORTSTEIL';
        this.operation.keywordId = 'STICHWORT-ID';
        this.operation.keywordCategory = 'STICHWORT KATEGORIE';
        this.operation.keywordRaw = 'STICHWORT RAW';
        this.operation.keywordName = 'SCHLAGWORT NAME';
        this.operation.zveis = 'SCHLEIFEN';
        this.operation.subject = 'STICHWORT';
        this.operation.street = 'STRASSE';
        this.operation.gkx = '4447706.33';
        this.operation.gky = '5485688.77';
        this.operation.floor = '1. OG';
        this.operation.ils = 'ILS Nürnberg';

        this.calculateLatLon();
        this.emit('alarm_parsed', null);
    }

    parseRAWZVEIData(rawZVEIDataArray) {
        console.log(this.parserLogName + ' | parseRAWZVEIData: ');
        this.zveicode = rawZVEIDataArray.data.code;
        this.zveidescription = rawZVEIDataArray.data.description;
        this.emit('zveiParsed', null);
    }

    parseRAWStatusData(rawStatusDataArray) {
        console.log(this.parserLogName + ' | parseRAWStatusData: ' + rawStatusDataArray);
        this.radioStatus = rawStatusDataArray.data.radioStatus;
        this.issi = rawStatusDataArray.data.issi;
        this.translateStatusNumberToHumanReadable();
        this.emit('statusParsed', null);
    }

    translateStatusNumberToHumanReadable() {
        if (typeof this.statusMap[this.radioStatus] != 'undefined') {
            this.radioStatusHumanReadable = this.statusMap[this.radioStatus];

            if (typeof this.statusMappingToStatusShort[this.radioStatus] != 'undefined') {
                this.radioStatusShort = this.statusMappingToStatusShort[this.radioStatus];
            }

            this.emit('statusTranslate_successful', this.radioStatusHumanReadable);
        } else {
            this.emit('statusTranslate_error', null);
        }
    }

    calculateLatLon() {

        if ((this.operation.gky.value.length > 0) && (this.operation.gkx.value.length > 0)) {
            // SDS delivered gauss krueger coordinates
            let wgs84 = gk.toWGS({ x: parseFloat(this.operation.gkx.value), y: parseFloat(this.operation.gky.value) });
            this.operation.lat = wgs84.latitude;
            this.operation.lon = wgs84.longitude;
            this.emit('geocoding_successful', null);
        } else {
            // SDS did not deliver gauss krueger coordinates
            this.operation.lat = 0;
            this.operation.lon = 0;
            this.emit('geocoding_error', null);
        }
    }


}

module.exports = ParserLAU;