// 
// Parser for ILS Nürnberg
//
// Parses incoming Faxes of ILS Nürnberg and
// stores them to the operations table in alarmiator database
//
//
const Tesseract = require('tesseract.js');
const fs = require('fs');
const Path = require('path');
const EventEmitter = require('events');
const OperationsObject = require('./models/operation');
var moment = require('moment');
const geolocationhelper = require(__dirname + '/../../..//models/geolocationhelper');

class Parser extends EventEmitter {

  constructor() {
    super();
    this.initialise();
  }

  LogAtMain(msg) {
    process.send('{"logmessage" : "' + msg + '"}');
  }

  initialise() {
    // required for generating timebased guids
    this.uuid = require('uuid');
    // Require model for operations
    this.operation = new OperationsObject();
    // helper class for geolocation functionality
    this.geolocation = new geolocationhelper();

  }

  writeOCRTextToFile(content, filename) {
    // get filename of new file for naming additional created files the same
    var extension = Path.extname(filename);
    var basename = Path.dirname(filename) + '/../txt';
    var fileName = Path.basename(filename,extension);
    var txtFileName = basename + "/" + fileName;
  
    let data = JSON.stringify(content, null, 4);  
    fs.writeFileSync(txtFileName + '.json', data);  
  }

  getCleanFax(lines){
    this.LogAtMain(`Starting to clean ...`)
    // fix ocr bugs -> remove if there is a dictonary
    var cleanfax = lines.replace(/=/g,':');
  
    cleanfax = cleanfax.replace(/[ ]*[—-]+[—-]+[ ]*/gm,"")
    cleanfax = cleanfax.replace(/EINSATZMTTTEL/g,"EINSATZMITTEL");
    return cleanfax
  }
  
  getSections(lines){
    this.LogAtMain(`Try to get sections for easy setup ...`)
    return lines.match((/\n+[A-Z ]+\n/g)).map(x => x.replace(/\n/g, ''));
  }
  
  getKeywords(lines){
    this.LogAtMain(`Try to get keywords for easy setup ...`)
    return lines.match(/[A-ZÄÜÖ][a-zA-Z.ßäöüÖÄÜ\-—]+:/g).map(x => x.replace(/:/g, ''))
  }

  parseFaxToJSON(lines, faxtemplate) {
    this.LogAtMain(`Starting with Fax to JSON convertion ...`)
    var faxobject = faxtemplate;
    var parsedSecObj;
    // get Sections from Template
    var sections = Object.keys(faxtemplate)
  
    // iterate over all sections
    sections.forEach( section => {
      // parse Fax with section
      parsedSecObj = this.parse(lines, section, sections);
  
      // get Keywords from Template for specific section
      var allkeywords = Object.keys(faxtemplate[section])
  
      var keywordcounter = 0; // Counter to check whether Keywords were found or not
      // iterate over all keywords from this section
      allkeywords.forEach( keyword => {
        // parse section text with keyword
        let parsedObj = this.parse(parsedSecObj.parsedtext, keyword, allkeywords);
  
        // Check if there is a value found for this Keyword
        if (parsedObj.parsedtext !== null) {
          keywordcounter++;
          // Add parsed text to faxobject and remove starting and ending:
          // - whitespaces
          // - :
          // - /
          // - -
          // - —
  
          faxobject[section][keyword]=parsedObj.parsedtext.trim()
          .replace(/(^[:\/—\-]+|[:\/—\-]+$)/gm,'')
          .trim();
        }
      })
      // if no keyword is found add full section text to faxobject
      if (keywordcounter === 0 && parsedSecObj !== null && parsedSecObj.parsedtext !== null) {
        faxobject[section] = parsedSecObj.parsedtext.trim();
      }
    })
    return faxobject;
  }

  JsonToOperation(operationJson, callback) {
    // generate mysql conform timestamp
    var a = moment();
    var timeStampIncome = a.format('YYYY/MM/DD HH:mm');
    this.LogAtMain(`Timestamp incoming: ` + timeStampIncome);
  
    // calculate Latitude / Longitude values out of gauss krueger coordinates
    var latitude = 0.0;
    var longitude = 0.0;
    try {
      this.LogAtMain('Gauss Krueger Coordinates from faximile: ' + operationJson.EINSATZORT.Koordinate);
      this.geolocation.gkToLatLon(operationJson.EINSATZORT.Koordinate, (arrLatLon) => {
        latitude = arrLatLon.latitude;
        longitude = arrLatLon.longitude;
        this.LogAtMain(`calculated Lat/Lon coordinates from GK-coordinates: (` + latitude + `/` + longitude + `)`);
      });
    } catch (err) {
      this.LogAtMain(`Error calculating Lat/lon coordinates from GK-coordinates: ` + err);
      callback();
    }
    
    // link JSON-Values to the general operations-object here
    // if the json-member exists, the value defined will be attached
    // and written to the database into the operations-table
    
    // operation.id 
    var operation = new OperationsObject();
    operation.acknowledged = 0;
    operation.operationguid = this.uuid.v1();
    operation.operationnumber = operationJson.FAXFAXFAXFAX.Nummer;
    operation.timestampincome = timeStampIncome;
    operation.timestampalarm = timeStampIncome; // TODO - Needs to be read from fax!
    operation.messenger = operationJson.MITTEILER.Name;
    operation.comment = operationJson.BEMERKUNG;
    operation.plan = operationJson.OBJEKTINFO["Plan Nr"];
    operation.priority = operationJson.EINSATZGRUND["Prio."]; // Maybe there will be a problem cause of the dot after "Prio"
    operation.einsatzortobject = operationJson.EINSATZORT.Objekt;
    operation.einsatzortzipcode = operationJson.EINSATZORT.PLZ; // not yet available for ILS Nürnberg
    operation.einsatzortcity = operationJson.EINSATZORT.Ort;
    operation.einsatzortstreet = operationJson.EINSATZORT.Straße;
    operation.einsatzortstreetnumber = operationJson.EINSATZORT.Nr;
    operation.einsatzortintersection = operationJson.EINSATZORT.Kreuzung;
    operation.einsatzortproperty = operationJson.EINSATZORT.Objekt;
    operation.einsatzortGKcoordinates = operationJson.EINSATZORT.Koordinate;
    operation.einsatzortlat = latitude; // Needs to be calculated (GK --> LatLon)
    operation.einsatzortlon = longitude; // Needs to be calculated (GK --> LatLon)
    operation.zielortobject = operationJson.ZIELORT.Objekt;
    operation.zielortzipcode = operationJson.ZIELORT.PLZ; // Not availabnle for ILS Nürnberg
    operation.zielortcity = ""; // Not available for ILS Nürnberg
    operation.zielortstreet = operationJson.ZIELORT.Straße;
    operation.zielortstreetnumber = operationJson.ZIELORT.Nr;
    operation.zielortintersection = ""; // Not available for ILS Nürnberg
    operation.zielortlat = ""; // Not available for ILS Nürnberg
    operation.zielortlon = ""; // Not available for ILS Nürnberg
    operation.keyword = operationJson.EINSATZGRUND.Stichwort;
    operation.keywordmisc = operationJson.EINSATZGRUND.Schlagwort;
    operation.rawjson = ""; // TODO - Still Problems in converting JSON to String to be propery escaped for sql statement
    operation.srcservice = "ILSFAX-INBOUND-ilsfax";
    operation.ressources = this.parseOperationRessources(operationJson.EINSATZMITTEL); // Parse operation ressources to the object for later storeage to database table
  
    // Calling callback-function
    callback(operation);
  }

  parseOperationRessources(ressourcesString) {
    var ressourcesArray = ressourcesString.split('\n');
    if (ressourcesArray.length > 1) {
      // remove first element cause it's the headline
      ressourcesArray.splice(0, 1);
      ressourcesArray.forEach(function(ressource, index) {
        // Strip unneeded Information from ressources
        var singleRessourceArray = ressource.split(':');
        if (singleRessourceArray.length > 0) {
          ressourcesArray[index] = singleRessourceArray[0];
        }
      });
    } 
    this.LogAtMain(`Anzahl Einsatzmittel erkannt: ` + ressourcesArray.length);
    ressourcesArray.forEach(element => {
      this.LogAtMain(`Einsatzmittel erkannt: ` + element);
    });
    return ressourcesArray;
  }

  ocr(image){
    Tesseract.recognize(image, {lang: 'deu'})
    .then(function(result) {
  
      var lines = this.getCleanFax(result.text);
  
      // ILS NÜRNBERG Template
      var faxtemplate = {"FAXFAXFAXFAX":{
             "Absender":"",
             "Alarmzeit":"",
             "Fax":"",
             "Termin":"",
             "Nummer":""
          },
          "MITTEILER":{
             "Name":"",
             "Tel":""
          },
          "EINSATZORT":{
             "Straße":"",
             "Nr":"",
             "Abschnitt":"",
             "Kreuzung":"",
             "Ort":"",
             "Objekt":"",
             "Abteilung":"",
             "Zuständig":"",
             "Koordinate":""
          },
          "ZIELORT":{
             "Straße":"",
             "Nr":"",
             "Ort":"",
             "Objekt":"",
             "Station":""
          },
          "PATIENT":"",
          "EINSATZGRUND":{
             "Schlagwort":"",
             "Stichwort":"",
             "Prio.":""
          },
          "EINSATZMITTEL":"",
          "OBJEKTINFO":{
             "Plan Nr":""
          },
          "BEMERKUNG":"",
          "ENDE ALARMFAX":""
       }
  
      var faxdata = this.parseFaxToJSON(lines, faxtemplate)
      this.LogAtMain(`OCR successful, writing fax details to file`);
  
      // Write textfile in archive folder with recognized fax content
  
      this.writeOCRTextToFile(faxdata, image)
      this.JsonToOperation(faxdata, (operation) => {
        // Write data to operation table in databse
        //writeOperationToDatabase(operation);
      });
      this.LogAtMain(`Fax Processing finished.`);
    }.bind(this))
    .catch(err => {
      this.LogAtMain(`Error in OCR: ` + err);
    })
    .finally( msg => {
       //Tesseract.terminate();
    })
  }

  parse(data, keyword, allkeywords) {
    var parsedObj = {};
    parsedObj['keyword'] = keyword;
    var keywordexists = -1;
    // Check if keyword is in data
    if (keyword !== null && data !== null) {
      keywordexists = data.indexOf(keyword);
    }
  
    if (keywordexists !== -1) {
      // Find next keyword
      var indexes = allkeywords.map(x => data.indexOf(x))
      // Remove not included keywords
      var filtered = indexes.filter(x => x !== -1 && x > data.indexOf(keyword));
      // Find next keyword
      var nextkeyword = allkeywords[indexes.indexOf(Math.min.apply(Math, filtered))]
  
      // if there is another keyword
      if (nextkeyword !== undefined) {
        parsedObj['parsedtext'] = data.substring(
          data.indexOf(keyword)+keyword.length,
          data.indexOf(nextkeyword));
        parsedObj['nextindex'] = data.indexOf(nextkeyword);
      }
      else {
        // rest of the string
        parsedObj['parsedtext'] = data.substring(
          data.indexOf(keyword)+keyword.length);
        parsedObj['nextindex'] = -1;
      }
    }
    else {
      parsedObj['parsedtext'] = null;
      parsedObj['nextindex'] = 0;
    }
  
    return parsedObj;
  }

}

module.exports = Parser;