/**
 *  KATSYSPROXY Service
 * 
 *  description:    This class is called in a separate node process 
 *                  in Plugin "KATSYSPROXY". This service opens
 *                  offers a websocket for the katsys plugin
 * 
 *  author:         Stefan Bauer / Nicolas Schmid 2021
 *  version:        1.0.0 
 * 
 */

// required modules for WebSocket

const fs = require('fs');
const path = require('path');
const http = require('http');
const https = require('https');
const WebSocket = require('ws');
const port = process.env.PORT || 8081;
var logmanager = require('simple-node-logger');

global.appRoot = __dirname + '/../../..';

var serviceState = 0;   // 0 == not configured, 1 == configured but not connected, 2 == configured and connected
var config = null;     // holds configuration from plugin config store

/**
 * Start KATSYSPROXY Logger
 */

// create a rolling file logger based on date/time that fires process events
function startLogger() {
    const opts = {
        logDirectory: global.appRoot + "/logs/katsysproxy", // NOTE: folder must exist and be writable...
        fileNamePattern: 'ALARMiator-KATSYSPROXY-<DATE>.log',
        dateFormat: 'YYYY.MM.DD'
    };
    
    global.logger = logmanager.createRollingFileLogger(opts);
    global.logger.setLevel('all');
    global.logger.info('KATSYSPROXY | Started KATSYSPROXY Instance');
    
}


/**
 * Log Function for inter process communication
 */
function LogAtMain(msg) {
    process.send('{"logmessage" : "' + msg + '"}');
    global.logger.info('KATSYSPROXY | Sending logmessage to main: '+ JSON.stringify(msg));
}

function routeIncomingIPCCommands(msg) {
    var payload = [];
    try {
        payload = JSON.parse(msg);
        console.log(payload);
    } catch (err) {
        payload = [];
        console.log('PLUGIN | KATSYSPROXY | WEBSOCKET | Unknown IPC Message from parent process: ' + JSON.stringify(msg));
        global.logger.info('PLUGIN | KATSYSPROXY | WEBSOCKET | Unknown IPC Message from parent process: ' + JSON.stringify(msg));
    }

    // NOW CHECK FOR COMMAND
    if (typeof payload.event_katsys_incoming_data != 'undefined') {
        console.log('KATSYSPROXY | WEBSOCKET | SIGNAL | event_katsys_incoming_data');
        global.logger.info('KATSYSPROXY | WEBSOCKET | SIGNAL | event_katsys_incoming_data');
        var data = payload.event_katsys_incoming_data;
        wss.clients.forEach((client) => {
            client.send(data);
        })

    }
}

/**
 * Wire messages coming from Plugin base.js
 */
process.on('message', (msg) => {
    if (typeof msg.config != 'undefined') {
        config = msg.config;
        serviceState = 1;
        LogAtMain('Recieved Configuration from CORE');
    }
    if (typeof msg.start != 'undefined') {
        if(msg.start === 1) {
            if (serviceState === 1) {
                // KatSysProxy Service is configured, so start it
                LogAtMain('Starting KatSysProxy Websocket');
                
                serviceState = 2;
            } else {
                LogAtMain('Service not in configured and stopped state. Not starting service');
            }
        }
    }
    
    routeIncomingIPCCommands(msg);

});

// GENERIC Error Handling
process.on('unhandledRejection', (reason, p) => {
    console.log('KATSYSPROXY | WEBSOCKET | Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
    LogAtMain('Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
});
process.on('uncaughtException', function (err) {
    console.log('KATSYSPROXY | WEBSOCKET | Caught exception: ' + err);
    process.send('{"error" : "' + err.message + '"}');
});

startLogger();

// ------------------------------------ WSS Https Section -------------------------------------------
// heartbeat callback
function heartbeat() {
    this.isAlive = true;
  }

  //Https Server
  const WsApp = http.createServer(
      /*
    {
    key: fs.readFileSync(keyfile),
    cert: fs.readFileSync(certfile),
   }*/).listen(8081);

  // WebSocket Server;
  const wss = new WebSocket.Server({server: WsApp});
  console.log('WSS Server started, wait for connecton on Port 8081')
  wss.on('connection', (ws, request) => {
    ws.master_token = request.headers.master_token
    ws.sub_token = request.headers.sub_token
    verbindung = ws;

    if (config == null || config.masterToken == "" || config.subToken == "") {
        console.log("No Config available");
        ws.close();
        return;
    }

    // VALIDATE TOKEN
    if (request.headers.master_token != config.masterToken || 
        request.headers.sub_token != config.subToken
        ) {
        LogAtMain("CONNECTION REFUSED! INVALID TOKEN - IP: " + request.socket.remoteAddress);
        ws.close();
        return;
    } else {
        LogAtMain("Received valid token, Access granted - IP: " + request.socket.remoteAddress);
    }

    ws.on('pong', heartbeat);

    ws.on('message', function incoming(message) {
      console.log('received: %s', message);
    });

   ws.on('close', function close(code, reson){
    console.log('socket closed');
    console.log(code);
    console.log(reson);
    });
  });

  wss.on('headers', function incominghead(header, req) {

    // console.log(req.headers.master_token);
    // console.log(req.headers.sub_token);
  });


  // Timer for ping
  const interval = setInterval(function ping() {
    wss.clients.forEach(function each(ws) {
      if (ws.isAlive === false){
        console.log('Connection Terminated');
        return ws.terminate();
      }
      ws.isAlive = false;
      //console.log("Send Ping");
      ws.ping();
    });
  }, 5000);

  wss.on('close', function close() {
    clearInterval(interval);
  });
