const EventEmitter = require('events');
const { del } = require('request');
const EmailQueue = require('../../../models/emailqueue');

/**
 * Class for MailSender
 * @class plugins/outbound/mailsender
 * @namespace plugins/outbound/mailsender
 * @memberof plugins/outbound/mailsender
  */
class mailsender extends EventEmitter {

    /**
     * Constructor
     * 
     * Sets up everything, starts websocket
     */
    constructor() {
        super();
        global.plgManager.logger.info('-------- PLUGIN MailSender -----------');
        this.serviceProcess = null;
        this.registerForEvents();
        this.serviceProcId = null;
        this.loadConfig((success) => {
            if (success) {
                this.initializePlugin((initsuccess) => {
                    global.plgManager.logger.debug('PLUGIN | MAILSENDER | initialized Plugin: ' + initsuccess);
                    if (initsuccess) {
                        this.isConfigured = true;
                        this.prepareService();
                        this.wireServiceCommunication();
                        this.configService();
                        this.startService();
                    }
                })
            } else {
                global.plgManager.logger.error('PLUGIN | MAILSENDER | ERROR | Could not load configuration for plugin');
            }
        })
    }

    /**
     * Registration of Events at the plugin manager (hook in to events provided by the plugin manager)
     */
    registerForEvents() {
        global.plgManager.logger.info('PLUGIN | MAILSENDER | Registering events');
        
        global.plgManager.on('event_pluginConfig_refreshed', (namespace) => {
            if (namespace === 'mailsender') {
                // plugins config has changed --> refresh
                this.loadConfig((success) => {
                    if (success) {
                        global.plgManager.logger.info('PLUGIN | MAILSENDER | successfully reloaded plugin configuration');
                        this.restartService();
                    } else {
                        global.plgManager.logger.error('PLUGIN | MAILSENDER | could not reload plugin config!');
                    }
                });
            }
        })
        
    }

    

    /**
     * Loads configuration for mailsender Service
     * @memberof plugins/outbound/mailsender
     */
    loadConfig(callback) {
        global.plgManager.loadConfigFromStore('mailsender', (configRows) => {
            if (configRows !== null) {
                this.config = configRows[0];
                this.config = JSON.parse(this.config.configstore);
                callback(true);
            } else {
                callback(false);
            }
            global.plgManager.logger.info('PLUGIN | MAILSENDER | Configuration loaded from core database');
        })
    }

    /**
     * returns config parameter for given fieldname
     * @param {string} fieldname 
     * @memberof plugins/outbound/wallboard
     */
    getConfigValue(fieldname) {
        var retVal = null;
        this.config.fields.forEach((field) => {
            if (field.fieldname === fieldname) {
                retVal = field.value;
            }
        });
        return retVal;
    }

    /**
     * reconfigures plugin with new config settings
     * @param {function} callback success, true or false
     * @memberof plugins/outbound/wallboard
     */
    initializePlugin(callback) {
        
        this.mailsenderconfig = {
            host: this.getConfigValue('host'),
            port: this.getConfigValue('port'),
            from: this.getConfigValue('from'),
            senderemail: this.getConfigValue('senderemail'),
            password: this.getConfigValue('password')
        }

        return callback(true);
    }

    /**
     * Wires messages from service to event model of plugin
     * All Messages in service need to be wired here to be recognised in plugin
     * @memberof plugins/outbound/wallboard
     */
    wireServiceCommunication() {
        this.serviceProcess.on('message', (msg) => {
            //console.log('PLUGIN | MAILSENDER | Message: ' + msg);
            var payload = null;
            try {
                payload = JSON.parse(msg);
            } catch (err) {
                payload = err.message;
            }
            
            /**
             * check what payload comes in
             */
            if (typeof payload.error != 'undefined') {
                // child has sent an error
                // try restarting the whole service
                console.log(payload.error);
            } else if (typeof payload.logmessage != 'undefined') {
                // LOGMESSAGE COMING FROM SERVICE
                console.log('PLUGIN | MAILSENDER | SERVICE | ' + payload.logmessage);
                global.plgManager.logger.info('PLUGIN | MAILSENDER | SERVICE | ' + payload.logmessage);
            
            } else {
                if (typeof msg === 'string') {
                    try {
                        let payload = JSON.parse(msg);
                        if (typeof payload.deliverystatus != null) {
                            // We've recieved delivery information from service
                            console.log('PLUGIN | MAILSENDER | Recieved delivery information from service.');
                            global.plgManager.logger.info('PLUGIN | MAILSENDER | Recieved delivery information from service.');
                            this.persistDeliveryStatus(payload.deliverystatus, (success) => {
                                // TBD - See if we need to act regarding the success result ?
                            })
                        }
                    } catch (err) {
                        console.log('PLUGIN | MAILSENDER | ERROR | Recieved data from service but was not able to parse the data.');
                    }
                }
                console.log('PLUGIN | MAILSENDER | SERVICE | UNSOLICITED MESSAGE: ' + msg);
                global.plgManager.logger.info('PLUGIN | MAILSENDER | SERVICE | UNSOLICITED MESSAGE: ' + msg);
            }
        });
    }

    /**
     * Registration of Events at the plugin manager (hook in to events provided by the plugin manager)
     */
    registerForEvents() {
        global.plgManager.logger.info('PLUGIN | MAILSENDER | Registering events')
               
        global.plgManager.on('event_email_queue_refreshed', (data) => {
            global.plgManager.logger.info('PLUGIN | MAILSENDER | EVENT event_email_queue_refreshed received');
            this.informatDaemonAboutNewEMailsInQueue(data);
        })
    }

    /**
     * Creates the service as a seperate node-process (fork)
     * sets the serviceProcId of the forked process
     * @memberof plugins/outbound/mailsender
     */
    prepareService() {
        global.plgManager.logger.info('PLUGIN | MAILSENDER | Starting Service');
        const { fork } = require('child_process');
        this.serviceProcess = fork(__dirname + '/service.js');
        this.serviceProcId = this.serviceProcess.pid;
        this.serviceProcess.title = 'ALARMiator Plugin - MAILSENDER Connector';
        global.plgManager.logger.info('PLUGIN | MAILSENDER | Email Daemon Service started with pid ' + this.serviceProcId);
        this.serviceProcess.on('exit', function (code) {
            this.serviceProcess = null;
            console.log("PLUGIN | MAILSENDER | Daemon Service EXITED " + code);
        });
    }

    /**
     * Sends the service configuration to the process
     */
    configService() {
        this.serviceProcess.send({ config: this.mailsenderconfig });
    }

    /**
     * tells the service to connect
     */
    startService() {
        this.serviceProcess.send({ start: 1 });
    }

    /**
     * restarts the Plugin Service
     */
    restartService() {
        if (this.serviceProcess !== null) {
            global.plgManager.logger.info('PLUGIN | MAILSENDER | Killing Service for restarting ...');
            this.serviceProcess.kill("SIGHUP");
        }
        this.initializePlugin((initsuccess) => {
            if (initsuccess) {
                this.isConfigured = true;
                this.prepareService();
                this.wireServiceCommunication();
                this.configService();
                this.startService();
            }
        })
    }

    /**
     * Methods to control MailSend Daemon
     * 
     */
    informatDaemonAboutNewEMailsInQueue(data) {
        this.serviceProcess.send(JSON.stringify({ event_email_queue_refreshed : data }));
    }

    persistDeliveryStatus(deliveryStatus, callback) {
        let emailqueue = new EmailQueue(global.app.logger);
        emailqueue.setStateSentBulk(deliveryStatus.sent, deliveryStatus.notsend, (success) => {
            if (success) {
                global.plgManager.logger.info('PLUGIN | MAILSENDER | Updated Delivery State successfully in emailQueue');
                return callback(success);
            } else {
                global.plgManager.logger.info('PLUGIN | MAILSENDER | ERROR | Could (partially) not update Delivery State in emailQueue. Please see logs');
                return callback(success);
            }
        })
    }

}

module.exports = mailsender;