/**
 *  MAILSENDER Service
 * 
 *  description:    This class is called in a separate node process 
 *                  in Plugin "MAILSENDER". 
 * 
 *  author:         Jens Dinstühler
 *  version:        1.0.1 
 * 
 */

var nodemailer = require('nodemailer');
const MailDaemon = require('./maildaemon/maildaemon');
var maildaemon = null;

var logmanager = require('simple-node-logger');

global.appRoot = __dirname + '/../../..';

var serviceState = 0;   // 0 == not configured, 1 == configured but not connected, 2 == configured and connected
var config = null;     // holds configuration from plugin config store

/**
 * Start MAILSENDER Logger
 */
// create a rolling file logger based on date/time that fires process events
function startLogger() {
    const opts = {
        logDirectory: global.appRoot + "/logs/mailsender", // NOTE: folder must exist and be writable...
        fileNamePattern: 'ALARMiator-MAILSENDER-<DATE>.log',
        dateFormat: 'YYYY.MM.DD'
    };
    
    global.logger = logmanager.createRollingFileLogger(opts);
    global.logger.setLevel('all');
    global.logger.info('MAILSENDER | Started MAILSENDER Instance');
    
}


/**
 * Log Function for inter process communication
 */
function LogAtMain(msg) {
    process.send('{"logmessage" : "' + msg + '"}');
    //global.logger.info('MAILSENDER | Sending logmessage to main: '+ JSON.stringify(msg));
}

/**
 * prepares delivery information and send back to main thread for further processing
 * @param {object} promisesResult 
 * @returns {object}
 */
function prepareDeliveryInformation(promisesResult) {
    let successful = [];
    let error = [];

    if (promisesResult != null) {
        if (promisesResult.length > 0) {
            promisesResult.forEach((pResult) => {
                switch (pResult.status) {
                    case 'fulfilled':
                        successful.push(pResult.value.emailQueueId);
                        break;
                    case 'rejected':
                        error.push(pResult.reason.emailQueueId);
                        break;
                    default:
                        break;
                }
            })
        }
    }
    let retVal = {
        sent: successful,
        notsend: error
    }
    return retVal;
}

/**
 * Wire messages coming from Plugin base.js
 */
process.on('message', (msg) => {
    if (typeof msg.config != 'undefined') {
        config = msg.config;
        serviceState = 1;
        LogAtMain('DAEMON successfully recieved Configuration from CORE');
        
        // SETUP Mail-Transport DAEMON
        maildaemon = new MailDaemon(global.logger);
        maildaemon.from = config.from;
        maildaemon.host = config.host;
        maildaemon.port = config.port;
        maildaemon.password = config.password;
        maildaemon.senderemail = config.senderemail;
        maildaemon.initialize();
    } else if (typeof msg.start != 'undefined') {
        if(msg.start === 1) {
            if (serviceState === 1) {
                // MailSender Service is configured, so start it
                LogAtMain('Starting MailSender Daemon');
                serviceState = 2;
            } else {
                LogAtMain('Service not in configured and stopped state. Not starting service');
            }
        }
    } else if (typeof msg === 'string') {
        // make object out of recieved string
        var payload = null;
        try {
            payload = JSON.parse(msg);
        } catch (err) {
            payload = null;
        }
        if (typeof payload.event_email_queue_refreshed != null) {
            // We've recieved new eMails, check if we are in test mode or not
            if (config.testmode === 'false') {
                let emailQueue = payload.event_email_queue_refreshed;
                global.logger.info('MAILSENDER | sending now ' + emailQueue.length + ' emails');
                let promises = [];
                emailQueue.forEach((email) => {
                    maildaemon.loadRecipientsFromJSON(email.recipients);
                    maildaemon.htmlBody = email.html;
                    maildaemon.plainBody = email.plain;
                    maildaemon.subject = email.subject;
                    maildaemon.emailQueueId = email.id;
                    maildaemon.composeEmail();
                    
                    let success = new Promise((resolve, reject) => {
                        maildaemon.sendEmail((sentMailState, errormessage) =>{
                            let result = {
                                success: false,
                                emailQueueId: email.id,
                                error: ''
                            }    
                            if (sentMailState) {
                                result.success = true;
                                resolve(result);
                            } else {
                                result.error = errormessage
                                reject(result);
                            }
                        });
                    });
                    promises.push(success);
                })
                Promise.allSettled(promises).then((values) => {
                    /**
                     * TBD - send delivery state of every email back to queue
                     */
                    let payload = prepareDeliveryInformation(values);
                    process.send('{"deliverystatus" : ' + JSON.stringify(payload) + '}');
                    console.log(values);
                })
            } else {
                global.logger.info('MAILSENDER | DAEMON is set to test mode in plugin config. Not sending emails now.');
            }
            
        }

        global.logger.info('MAILSENDER | recieved new Mails from Main');
    } else {
        global.logger.info('MAILSENDER | IPC | recieved unknown data from main thread: ' + JSON.stringify(msg));
    }
});


/**
 * GENERIC Error Handling  
 */ 
process.on('unhandledRejection', (reason, p) => {
    console.log('MAILSENDER | Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
    LogAtMain('Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
});
process.on('uncaughtException', function (err) {
    console.log('MAILSENDER | Caught exception: ' + err);
    process.send('{"error" : "' + err.message + '"}');
});




startLogger();


