
const EventEmitter = require('events');
const SMS77ioVoice = require('./sms77iovoice');

/**
 * Class offers printing functionality for plugin Printer
 * @class
 * @module plugins/outbound/printer
 */
class sms77ioVoice extends EventEmitter {
    constructor() {
        super();
        global.plgManager.logger.info('-------- PLUGIN SMS77IOVOICE -----------');
        this.registerForEvents();
        this.config = null;
        this.credits = 0;
        this.loadConfig((success) => {
            if (success) {
                var httpApiKey = this.getConfigValue('httpApiKey');
                const sms77ioController = new SMS77ioVoice(global.plgManager.logger);
                
                sms77ioController.apikey = httpApiKey;
                sms77ioController.checkSMS77Credits((credits) => {
                    if (typeof credits === 'boolean') {
                        if (credits === false) {
                            console.log('PLUGIN | SMS77IOVOICE | Error requesting credits from sms77.io');
                            this.credits = 0;
                        }
                    } else {
                        this.credits = credits;
                    }
                })
            }
        });
    }

    /**
     * Registration of Events at the plugin manager (hook in to events provided by the plugin manager)
     */
    registerForEvents() {
        global.plgManager.logger.info('PLUGIN | SMS77IOVOICE | Registering events')
        
        global.plgManager.on('event_new_alarm', (alarmInfo) => {
            global.plgManager.logger.info('PLUGIN | SMS77IOVOICE | Recieved new ALARM, initiating Voice-Call now');

            try {
                if (this.getConfigValue('alarmMethode') === "alarmAll"){
                    this.makeAlarmCall(alarmInfo);
                }
            }      
            catch (error){
                console.error("PLUGIN | SMS77IOVOICE | Error caught while sending alarm: " + error.message);
                global.plgManager.logger.error('PLUGIN | SMS77IOVOICE | Error caught while sending alarm: ' + error.message);
            }
        });

        global.plgManager.on('event_new_alarm_with_rules', (alarmInfo, ruleResult) => {
            global.plgManager.logger.info('PLUGIN | SMS77IOVOICE | Received event_new_alarm_with_rules');
            try {
                if (this.getConfigValue('alarmMethode') === "ruleBased"){
                    this.sendAlarmMessageToEveryUser(alarmInfo, ruleResult);
                }
            }      
            catch (error){
                console.error("PLUGIN | SMS77IOVOICE | Error caught while sending alarm: " + error.message);
                global.plgManager.logger.error('PLUGIN | SMS77IOVOICE | Error caught while sending alarm: ' + error.message);
            }

        });

        global.plgManager.on('event_plugin_notification', (namespace, message) => {
            if (namespace === 'sms77iovoice') {
                global.plgManager.logger.info('PLUGIN | SMS77IOVOICE | Recieved Notification from PluginManager: ' + message);
            }
        });
        
        global.plgManager.on('event_pluginConfig_refreshed', (namespace) => {
            if (namespace === 'sms77iovoice') {
                global.plgManager.logger.info('PLUGIN | SMS77IOVOICE | Recieved Info about changed Configuration');
                this.loadConfig((success) => {
                    global.plgManager.logger.info('PLUGIN | SMS77IOVOICE | Successfully reloaded plugin configuration');
                });
            }
        });
    }



    /**
     * Plugin-specific Getters and Setters
     * 
    */
    // --- NONE ---


    /**
     * Plugin-specific implementation
     */

    makeAlarmCall(alarmInfo) {
        var recievers = this.getConfigValue('callRecievers');
        var senderNumber = this.getConfigValue('senderNumber');
        if(senderNumber === null) {
            senderNumber = '0177 1783130';
        }
        var text = 'ALARMIERUNG: Stichwort ' + alarmInfo.subject.value + ', Einsatzort: ' + alarmInfo.location.value + ', Mitteilung: ' + alarmInfo.message.value;
        text = text.substr(0, 160);

        const sms77ioController = new SMS77ioVoice(global.plgManager.logger);
        sms77ioController.apikey = this.getConfigValue('httpApiKey');

        sms77ioController.makeAlarmCall(text, senderNumber, recievers, (success) => {
            console.log('PLUGIN | SMS77IOVOICE | Sent all callouts: ' + success);
        }); 
    }
    
    /**
     * Plugin-specific implementation
     */
    sendAlarmMessageToEveryUser(alarmInfo, ruleResult) {
        var recievers = [];    
        if (ruleResult != null){
            ruleResult.forEach((alarmedUser) => {
                if (alarmedUser.cellphone != undefined && alarmedUser.cellphone != "" && alarmedUser.cellphone != null) {
                    if (alarmedUser.namespace != null){
                        if (alarmedUser.namespace.includes("sms77iovoice")){
                            recievers.push(alarmedUser.cellphone);
                        }
                    }
                }
            })
        }

        var uniqueReceivers = recievers.filter((value, index, self) => {
            return self.indexOf(value) === index;
        });

        var senderNumber = this.getConfigValue('senderNumber');
        if(senderNumber === null) {
            senderNumber = 'ALARMiator';
        }
        var text = alarmInfo.subject.value + ' - ' + alarmInfo.location.value + ' ' + alarmInfo.message.value;

        const sms77ioController = new SMS77ioVoice(global.plgManager.logger);
        sms77ioController.apikey = this.getConfigValue('httpApiKey');

        if (uniqueReceivers.length > 0){
            sms77ioController.makeAlarmCall(text, senderNumber, uniqueReceivers, (success) => {
                console.log('PLUGIN | SMS77IOVOICE | Sent all callouts (user specific, following alarming rules): ' + success);
            }); 
        }        
    }
    
    
    /**
     * Loads configuration for KatSys Service
     * TBD: Needs to be pulled from plugin config store
     */
    loadConfig(callback) {
        global.plgManager.loadConfigFromStore('sms77iovoice', (configRows) => {
            if (configRows !== null) {
                this.config = configRows[0];
                this.config = JSON.parse(this.config.configstore);
                callback(true);
            }
            global.plgManager.logger.info('PLUGIN | SMS77IOVOICE | Configuration reloaded from core database');
        })
    }

    /**
     * returns config parameter for given fieldname
     * @param {string} fieldname 
     */
    getConfigValue(fieldname) {
        var retVal = null;
        this.config.fields.forEach((field) => {
            if (field.fieldname === fieldname) {
                retVal = field.value;
            } 
        });
        return retVal;
    }
}

module.exports = sms77ioVoice;