/**
 *  WALLBOARD Service
 * 
 *  description:    This class is called in a separate node process 
 *                  in Plugin "WALLBOARD". This service opens
 *                  offers a websocket for the wallboard clients
 * 
 *  author:         Jens Dinstühler 2020
 *  version:        1.0.0 
 * 
 */

// required modules for WebSocket
const express = require('express')
const app = express();
const http = require('http');
const server = http.Server(app);
const socketIO = require('socket.io');
const io = socketIO(server);
const port = process.env.PORT || 5020;
var logmanager = require('simple-node-logger');
// required for core integrationd and communication
const Database = require('../../../models/database');
const WallboardHelper = require('../../../models/wallboard');
const Wallboards = require('../../../models/wallboards');
const Organizations = require('../../../models/organizations');
const Operations = require('../../../models/operationshelper');
const Gearstate = require('../../../models/gearstate');
const Locations = require('../../../models/locations');
const Openrouteservice = require('../../../models/openrouteservice');
const Assets = require('../../../models/assets');

const { or } = require('sequelize');
const { exception } = require('console');


// initialization of general properties
var coreDb = null;
global.appRoot = __dirname + '/../../..';
global.coreDb = null;
var serviceState = 0;   // 0 == not configured, 1 == configured but not connected, 2 == configured and connected


/**
 * Start WALLBOARD Logger
 */

// create a rolling file logger based on date/time that fires process events
function startLogger() {
    const opts = {
        logDirectory: global.appRoot + "/logs/wallboard", // NOTE: folder must exist and be writable...
        fileNamePattern: 'ALARMiator-WALLBOARD-<DATE>.log',
        dateFormat: 'YYYY.MM.DD'
    };
    
    global.logger = logmanager.createRollingFileLogger(opts);
    global.logger.setLevel('all');
    global.logger.info('WALLBOARD | Started WALLBOARD Instance');
    
}



/**
* Starts the coredatabase connection
*/
function startCoredatabase() {
    coreDb = new Database();
    coreDb.startDatabase((successDatabase) => {
        if (successDatabase) {
            global.coreDb = coreDb.db;
            LogAtMain('Core Database Connection established');
            openrouteservice = new Openrouteservice(global.logger);
            // Database started, start server now
        } else {
            // Database could not be started. -> Log error to console
            LogAtMain('ERROR: Core Database Connection could not be established');
        }
    });
}

/**
 * Broadcasts an operation object to all sockets
 */
function broadcastNewAlarm() {
    wallboardhelper.getActiveOperations((operationsRows) => {
        if (operationsRows != null){
            if (operationsRows.length > 0){
                organizations.getOrganizationsWithZveisAndGearnames((organizationsRows) => {
                    if (organizationsRows != null) {
                        global.logger.info('WALLBOARD | Get sorted Operations');
                        var sortedOperationsByOrganizations = operations.sortOperationsByOrganizations(organizationsRows,operationsRows);
                        organizationsRows.forEach((organization) => {
                            global.logger.info('WALLBOARD | Check for organization');
                            if (sortedOperationsByOrganizations[organization.organizationsName].length > 0){
                                sendNewAlarmToOrganization(organization.organizationsName, sortedOperationsByOrganizations[organization.organizationsName][0])
                            }
                            else {
                                sendIdleConfigToOrganization(organization.organizationsName);
                            }
                        })
                    }
                })
            }
            else {
                global.logger.info('WALLBOARD | No active operations');

                organizations.getListOfOrganizations((organizationRows) => {

                    if (organizationRows != null) {
                        global.logger.info('WALLBOARD | Found '+organizationRows.length + ' organizations');
                        organizationRows.forEach((organization) => {
                            sendIdleConfigToOrganization(organization.name)
                        })
                    }
                })
            }
        }
        else {
            global.logger.info('WALLBOARD | OperationsRows are null');
        }
    })
}

/**
 * Broadcasts an state object to all rooms
 */
function broadcastNewState() {
    organizations.getListOfOrganizations((organizationRows) => {
        if (organizationRows != null) {
            organizationRows.forEach((organization) => {
                gearstate.getCurrentGearStatesListForOrganization(organization.name, (gearStates) => {
                    if (gearStates != null) {
                        io.to(organization.name).emit('gear-state', gearStates);
                    }
                }) 
            })
        }
    })
}

/**
 * Broadcasts assets maintenance
 */
 function broadcastAssetsMaintenance() {
    organizations.getListOfOrganizations((organizationRows) => {
        if (organizationRows != null) {
            organizationRows.forEach((organization) => {
                assets.getAssetsInMaintenanceWindowForOrganizationName(30, organization.name, (data) => {
                    if (data != null) {
                        io.to(organization.name).emit('maintenance-assets', data);
                    }
                }) 
            })
        }
    })
}


// DO NOT USE FOR PRODUCTION TESTING ONLY
/**
 * Broadcasts an operation object to all sockets
 */
function sendOnConnectPayload(socket) {
    broadcastNewState();
    broadcastNewAlarm();
    broadcastAssetsMaintenance();
}

/**
 * Sends an alarm config to room
 */
function sendNewAlarmConfigToRoom(room) {
    var alarmConfig = [
        {"cols": 2, "rows": 2, "y": 3, "x": 2, "type": "operation-map"},
        {"cols": 2, "rows": 2, "y": 1, "x": 2, "type": "operation-message"},
        {"cols": 4, "rows": 1, "y": 0, "x": 0, "type": "operation-keyword"},
        {"cols": 1, "rows": 1, "y": 0, "x": 4, "type": "operation-stopwatch"},
        {"cols": 2, "rows": 1, "y": 1, "x": 0, "type": "operation-location-info"},
        {"cols": 2, "rows": 2, "y": 2, "x": 0, "type": "operation-gear"},
        {"cols": 2, "rows": 1, "y": 4, "x": 0, "type": "feedback-overview"},
        {"cols": 1, "rows": 4, "y": 1, "x": 4, "type": "gear-states"}
      ]
    io.to(room).emit('config', alarmConfig);
    global.logger.info('WALLBOARD | Send new Alarm Config to room: '+ room);
}

function sendIdleConfigToRoom(room) {
    var idleConfig = [
        {cols: 1, rows: 1, y: 0, x: 0, type: 'idle-screen'},
      ];
    io.to(room).emit('config', idleConfig);
    global.logger.info('WALLBOARD | Send new Idle Config to room: '+ room);
}

function sendIdleConfigToOrganization(organizationName){
    wallboard.getWallboardForOrganizationName(organizationName, (wallboardRows) => {
        if (wallboardRows == null) {
            sendIdleConfigToRoom(organizationName);
        }
        else {
            wallboardRows.forEach((wallboardEntry) => {
                if (wallboardEntry.configIdle != null && wallboardEntry.configIdle != "") {
                    try {
                        var config = JSON.parse(wallboardEntry.configIdle);
                        if (typeof config == "object"){
                            io.to(wallboardEntry.id+"").emit('config',config);
                        }
                        else {
                            sendIdleConfigToRoom(wallboardEntry.id+""); 
                        }
                    }
                    catch (exception) {
                        sendIdleConfigToRoom(wallboardEntry.id+"");    
                    }
                }
                else {
                    sendIdleConfigToRoom(wallboardEntry.id+"");
                }
            })
        }
    })
}

function sendNewAlarmToOrganization(organizationsName, operation){
    wallboard.getWallboardForOrganizationName(organizationsName, (wallboardRows) => {
        if (wallboardRows == null) {
            sendNewAlarmConfigToRoom(organizationsName);
        }
        else {
            wallboardRows.forEach((wallboardEntry) => {
                if (wallboardEntry.configAlarm != null && wallboardEntry.configAlarm != "") {
                    try {
                        var config = JSON.parse(wallboardEntry.configAlarm);
                        if (typeof config == "object"){
                            io.to(wallboardEntry.id+"").emit('config', JSON.parse(wallboardEntry.configAlarm));
                        }
                        else {
                            sendNewAlarmConfigToRoom(wallboardEntry.id+"");    
                        }
                    }
                    catch (exception) {
                        sendNewAlarmConfigToRoom(wallboardEntry.id+"");    
                    }
                }
                else {
                    sendNewAlarmConfigToRoom(wallboardEntry.id+"");
                }

                //console.log(wallboardEntry);
                if (wallboardEntry.locationId != null) {
                    locations.getLocationWithId(wallboardEntry.locationId, (locationRows) => {
                        locationRows.forEach((location) => {
                            console.log("WALLBOARD | checking coordinates")
                            global.logger.info('WALLBOARD | checking coordinates');
                            // check for valid api key
                            if (location.latitude != null 
                                && location.longitude != null 
                                && operation.lat != null 
                                && operation.lon != null){
                                    console.log("WALLBOARD | calculating route")
                                    global.logger.info('WALLBOARD | calculating route');
                                    openrouteservice.calculateRouteGeoJSONWithAPI(
                                        location.latitude,
                                        location.longitude, 
                                        operation.lat, 
                                        operation.lon, 
                                        (result) => {
                                            
                                            result().then((geoJson) => {
                                                if (geoJson != null) {
                                                    console.log("WALLBOARD | Sending Route")
                                                    global.logger.info('WALLBOARD | Sending GeoJSON: ' + JSON.stringify(geoJson));
                                                    io.to(wallboardEntry.id+"").emit('directions', geoJson);

                                                    // Attempt to solve timing issues
                                                    setTimeout(() => {
                                                        io.to(wallboardEntry.id+"").emit('directions', geoJson);
                                                    },2000);
                                                }
                                                else {
                                                    io.to(wallboardEntry.id+"").emit('directions', null);
                                                }
                                            })
                                            .catch((exception) => {
                                                global.logger.error('WALLBOARD | ERROR | GeoJSON exception: ' + JSON.stringify(exception));
                                                console.log(exception);
                                                io.to(wallboardEntry.id+"").emit('directions', null); 
                                            })
                                    })
                            }
                            else {
                                console.log("Missing coordinates");
                            }
                        })
                    }) 
                }
                else {
                    console.log("no location found for wallboard with id: " + wallboardEntry.id)
                    io.to(wallboardEntry.id+"").emit('directions', null); 
                }
                
                wallboardhelper.getFeedbackForOperationUUID(wallboardEntry.id, operation.uuid, wallboardEntry.organizationId ,(feedbackResponse) => {
                    if (feedbackResponse != null) {
                        io.to(wallboardEntry.id+"").emit('feedback',feedbackResponse)
                    }
                    else {
                        io.to(wallboardEntry.id+"").emit('feedback',null)
                    }
                })
            })
        }
        io.to(organizationsName).emit('operation', operation);
    })
}

function updateWallboard() {
    broadcastNewState();
    broadcastNewAlarm();
    broadcastAssetsMaintenance();
}
/**
 * Log Function for inter process communication
 */
function LogAtMain(msg) {
    process.send('{"logmessage" : "' + msg + '"}');
    global.logger.info('WALLBOARD | Sending logmessage to main: '+ JSON.stringify(msg));
}

function routeIncomingIPCCommands(msg) {
    var payload = [];
    try {
        payload = JSON.parse(msg);
    } catch (err) {
        payload = [];
        console.log('PLUGIN | WALLBOARD | WEBSOCKET | Unknown IPC Message from parent process: ' + JSON.stringify(msg));
        global.logger.info('PLUGIN | WALLBOARD | WEBSOCKET | Unknown IPC Message from parent process: ' + JSON.stringify(msg));
    }
    // NOW CHECK FOR COMMAND
    if (typeof payload.event_new_alarm != 'undefined') {
        console.log('WALLBOARD | WEBSOCKET | SIGNAL | event_new_alarm');
        global.logger.info('WALLBOARD | WEBSOCKET | SIGNAL | event_new_alarm');
        broadcastNewAlarm();
    }
    if (typeof payload.event_closed_alarm != 'undefined') {
        console.log('WALLBOARD | WEBSOCKET | SIGNAL | event_closed_alarm');
        global.logger.info('WALLBOARD | WEBSOCKET | SIGNAL | event_closed_alarm');
        broadcastNewAlarm();
    }
    if (typeof payload.event_new_feedback != 'undefined') {
        console.log('WALLBOARD | WEBSOCKET | SIGNAL | event_new_feedback');
        global.logger.info('WALLBOARD | WEBSOCKET | SIGNAL | event_new_feedback');
        broadcastNewAlarm();
    }
    if (typeof payload.event_new_state != 'undefined') {
        console.log('WALLBOARD | WEBSOCKET | SIGNAL | event_new_state');
        global.logger.info('WALLBOARD | WEBSOCKET | SIGNAL | event_new_state');
        broadcastNewState();
    }
    if (typeof payload.event_wallboard_config_changed != 'undefined') {
        console.log('WALLBOARD | WEBSOCKET | SIGNAL | event_wallboard_config_changed');
        global.logger.info('WALLBOARD | WEBSOCKET | SIGNAL | event_wallboard_config_changed');
        updateWallboard();
    }
    if (typeof payload.event_notification != 'undefined') {
        console.log('WALLBOARD | WEBSOCKET | SIGNAL | event_notification');
        global.logger.info('WALLBOARD | WEBSOCKET | SIGNAL | event_notification');
        updateWallboard();
    }
}

/**
 * Wire messages coming from Plugin base.js
 */
process.on('message', (msg) => {
    if (typeof msg.config != 'undefined') {
        config = msg.config;
        serviceState = 1;
        LogAtMain('Recieved Configuration from CORE');
    }
    routeIncomingIPCCommands(msg);

});

// GENERIC Error Handling
process.on('unhandledRejection', (reason, p) => {
    console.log('WALLBOARD | WEBSOCKET | Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
    LogAtMain('Unhandled Rejection at: Promise', p, 'reason:', reason.stack);
});
process.on('uncaughtException', function (err) {
    console.log('WALLBOARD | WEBSOCKET | Caught exception: ' + err);
    process.send('{"error" : "' + err.message + '"}');
});

startLogger();
startCoredatabase();

const wallboard = new Wallboards(global.logger);
const organizations = new Organizations(global.logger);
const operations = new Operations(global.logger);
const gearstate = new Gearstate(global.logger);
const wallboardhelper = new WallboardHelper(global.logger);
const locations = new Locations(global.logger);
const assets = new Assets(global.logger);

var openrouteservice;

io.on('connection', (socket) => {
    console.log('WALLBOARD | WEBSOCKET | Benutzer nun verbunden');
    var ipAddress = socket.handshake.address;
    global.logger.info('WALLBOARD | WEBSOCKET | Benutzer nun verbunden von IP: '+ ipAddress);
    process.send('{"newConnection":"' + ipAddress + '"}');
    socket.on('disconnect', () => {
        console.log('WALLBOARD | WEBSOCKET | Benutzerverbindung abgebaut');
        global.logger.info('WALLBOARD | WEBSOCKET | Benutzerverbindung abgebaut von IP: '+ ipAddress);
        process.send('{"lostConnection":1}');
    });
    socket.on('message', (msg) => {
        console.log('WALLBOARD | WEBSOCKET | incoming message: ' + msg);
        socket.emit('updateData', msg);
    });
    socket.on('authenticate', (token) => {
        console.log('WALLBOARD | WEBSOCKET | incoming auth token: ' + token +' from ' + ipAddress);
        global.logger.info('WALLBOARD | WEBSOCKET | incoming auth token: ' + token +' from ' + ipAddress);
        if (token == null || token.trim() == "") {
            console.log("WALLBOARD | WEBSOCKET | No access token available");
            socket.disconnect(true);
            return;
        }
        wallboard.getWallboardForToken(token, (wallboardRow) => {
            if (wallboardRow != null) {
                if (wallboardRow.length > 0) {
                    console.log("WALLBOARD | WEBSOCKET | Auth Token is Valid!");
                    global.logger.info("WALLBOARD | WEBSOCKET | Auth Token is Valid!");
                    socket.join(wallboardRow[0].organizationsName); // Join organizations channel
                    socket.join(wallboardRow[0].id + ""); // join wallboardId channel
                    // Delay a few ms to boot up wallboard
                    setTimeout(() => {
                        sendOnConnectPayload(socket);
                    },500);
                } else {
                    console.log("WALLBOARD | WEBSOCKET | INVALID TOKEN " + token);
                    global.logger.info("WALLBOARD | WEBSOCKET | INVALID TOKEN " + token);
                    socket.disconnect(true);
                }
            }
            else {
                console.log("WALLBOARD | WEBSOCKET | INVALID TOKEN " + token);
                global.logger.info("WALLBOARD | WEBSOCKET | INVALID TOKEN " + token);
                socket.disconnect(true);
            }
            // console.log(wallboardRow);
            global.logger.info("WALLBOARD | WEBSOCKET | Found wallboard profile for IP: " + ipAddress +": " + JSON.stringify(wallboardRow));
        })
    });
});

server.listen(port, () => {
    console.log(`PLUGIN | WALLBOARD | SERVICE | WEBSOCKET | started at port: ${port}`);
});



