/**
 * Resizes a provided image source according to the parameters provided and converts the image
 * data into base64.
 * @param {String} imageSource - either an object URL or a file path, anything that can be used
 * as <img src="..">
 * @param {Object} maxImageSize - (optional) a JSON object with a `width` and `height` providing maximum
 * dimensions for the parsed image
 * @param {string} canvasId - the HTML id of the HTML canvas element used for resizing
 * @returns {Promise} a promise with the base64 "data" provided anda a "size" element containing the 
 * image size.
 */
const resizeImage = (imageSource, maxImageSize = null, canvasId = "resize-canvas") => new Promise((resolve, reject) => {
    const img = new Image()

    img.onload = () => {
        // fetch original image dimension
        const size = {
            width: img.naturalWidth,
            height: img.naturalHeight,
        }
        // rescale to max dimensions
        if (maxImageSize != null && (size.width > maxImageSize.width || size.height > maxImageSize.height)) {
            const scaleFactor = Math.min(maxImageSize.width / size.width, maxImageSize.height / size.height)
            size.width *= scaleFactor
            size.height *= scaleFactor
        }        
        // rescale canvas
        const canvas = document.getElementById(canvasId)
        canvas.setAttribute("width", size.width)
        canvas.setAttribute("height", size.height)
        
        // draw image onto clear canvas
        const ctx = canvas.getContext("2d")
        ctx.clearRect(0, 0, size.width, size.height)
        ctx.drawImage(img, 0, 0, size.width, size.height)
        // resolve with size and image data
        resolve({ 
            data: canvas.toDataURL(),
            size,
        })        
    }
    img.onerror = () => {
        reject()
    }

    // once the image is loaded, this triggers the onload handler
    img.src = imageSource
})

/**
 * Image upload handler, which will use a file object from a type="file" input (i.e. ev.target.files[0])
 * and convert it to base64 while also resizing it, if it exceeds the dimensions provided
 * @param {Object} file - a file object from an HTML input (selected file for upload)
 * @param {Object} maxImageSize - (optional) a JSON object with a `width` and `height` providing maximum
 * dimensions for the parsed image
 * @param {string} canvasId - the HTML id of the HTML canvas element used for resizing
 * @returns {Promise} a promise that will resolve with the base64 rescaled image data and the size of the 
 * image or reject with an empty payload.
 */
const provideImageFile = (file, maxImageSize = null, canvasId = "resize-canvas") => {
    const objectUrl = window.URL.createObjectURL(file)
    return resizeImage(objectUrl, maxImageSize, canvasId)
        .then(imageData => {
            window.URL.revokeObjectURL(objectUrl)
            return imageData
        })    
}
