
// Includes
const EventEmitter = require('events');
const InboundEmail = require('./inboundEmail');
var { MailListener } = require("mail-listener5");

// Class Definition
class EMailListener extends EventEmitter {
    /**
     * @constructor
     */
    constructor() {
        super();
        this._port = 993;
        this._mailbox = 'INBOX';
        this._queue = [];
        this.logIdentifier = 'PLUGIN | MAILLISTENER | ';
    }

    //
    // - Properties Getter/Setters
    //
    set username(username) {
        this._username = username;
    }
    get username() {
        return this._username;
    }

    set password(password) {
        this._password = password;
    }
    get password() {
        return this._password;
    }

    set host(host) {
        this._host = host;
    }
    get host() {
        return this._host;
    }

    set port(port) {
        this._port = port;
    }
    get port() {
        if (this._port === null) {
            return 993;
        } else {
            return this._port;
        }
    }

    set mailbox(mailbox) {
        this._mailbox = mailbox;
    }
    get mailbox() {
        if (this._mailbox === null) {
            return 'INBOX';
        } else {
            return this._mailbox;
        }
    }

    set allowedSenders(sendersArray) {
        this._allowedSenders = sendersArray;
        console.log(this.logIdentifier + ' allowed Senders of incoming alarm emails ' + this.allowedSenders);
    }
    get allowedSenders() {
        return this._allowedSenders;
    }

    set allowedRecipients(recipientsArray) {
        this._allowedRecipients = recipientsArray;
        console.log(this.logIdentifier + ' allowed Recipients of incoming alarm emails ' + this.allowedRecipients);
    }
    get allowedRecipients() {
        return this._allowedRecipients;
    }

    set allowedAttachments(extensionsArray) {
        this._allowedAttachments = extensionsArray;
        console.log(this.logIdentifier + ' allowed attachments: ' + this._allowedAttachments);
    }
    get allowedAttachments() {
        return this._allowedAttachments;
    }

    set allowedSubjectRegEx(allowedSubjectRegEx) {
        this._allowedSubjectRegEx = allowedSubjectRegEx;
        console.log(this.logIdentifier + ' allowed subject regEx: ' + this.allowedSubjectRegEx);
    }
    get allowedSubjectRegEx() {
        return this._allowedSubjectRegEx;
    }

    /**
     * Checks if seqNo exists in Queue. Returns idx-Number if exists or null if not exists
     * @param {integer} seqNo sequenze Number of incoming mail object
     */
    getQueueIndexForSeqNo(seqNo) {
        var objectIdx = null;
        this._queue.forEach((email, index) => {
            if (email.seqno === seqNo) {
                objectIdx = index;
            }
        });
        return objectIdx;
    }


    connectedToServer() {
        console.log(this.logIdentifier + ' Connected to Server');
    }

    connectedToMailbox(mailbox) {
        console.log(this.logIdentifier + ' Successfully opened mailbox, monitoring new mails now.');
    }

    addNewMailToQueue(seqNo, mail) {
        var attachmentCount = 0;
        if (typeof mail.attachments != 'undefined') {
            for (const [key, value] of Object.entries(mail.attachments)) {
                console.log(`${key}: ${value}`);
            }
            attachmentCount = Object.keys(mail.attachments).length;
        }
        // now register new Mail in queue
        var email = new InboundEmail();
        email.seqno = seqNo;
        email.attachmentCount = attachmentCount;
        this._queue.push(email);
        console.log(this.logIdentifier + ' QUEUE | Objects in queue: ' + this._queue.length);
    }

    addHeaderToQueue(seqNo, headers) {
        var objectIdx = this.getQueueIndexForSeqNo(seqNo);
        if(objectIdx != null) {
            if (Array.isArray(headers.get('from').value)) {
                // more than one from address
                this._queue[objectIdx].from = headers.get('from').value[0].address;
                this._queue[objectIdx].fromName = headers.get('from').value[0].name;
            } else {
                this._queue[objectIdx].from = headers.get('from').value.address;
                this._queue[objectIdx].fromName = headers.get('from').value.name;
            }
            this._queue[objectIdx].to = headers.get('to').value[0].address;
            this._queue[objectIdx].subject = headers.get('subject');
            this._queue[objectIdx].date = headers.get('date');
        }
    }

    addBodyToQueue(seqNo, body) {
        var objectIdx = this.getQueueIndexForSeqNo(seqNo);
        if(objectIdx != null) {
            this._queue[objectIdx].bodyText = body.text;
            this._queue[objectIdx].bodyHtml = body.html;
            this._queue[objectIdx].bodyTextAsHtml = body.textAsHtml;
            
        }
    }

    addAttachmentsToQueue(seqNo, path, attachment) {
        var objectIdx = this.getQueueIndexForSeqNo(seqNo);
    }
    //
    // - Class methods
    //

    initialize() {
        console.log(this.logIdentifier + ' Initializing EMail-Plugin');
        this.options = {
            username: this.username,
            password: this.password,
            host: this.host,
            port: this.port, // imap port
            tls: true,
            connTimeout: 10000, // Default by node-imap
            authTimeout: 5000, // Default by node-imap,
            // debug: console.log, // Or your custom function with only one incoming argument. Default: null
            debug: null,
            tlsOptions: { rejectUnauthorized: false },
            mailbox: this.mailbox, // mailbox to monitor
            searchFilter: ["UNSEEN"], // the search filter being used after an IDLE notification has been retrieved
            markSeen: true, // all fetched email willbe marked as seen and not fetched next time
            fetchUnreadOnStart: true, // use it only if you want to get all unread email on lib start. Default is `false`,
            attachments: true, // download attachments as they are encountered to the project directory
            attachmentOptions: { directory: "attachments/" } // specify a download directory for attachments
        }
        this.mailListener = new MailListener(this.options);
        
    }

    registerHandlers() {
        console.log(this.logIdentifier + ' Registering internal handlers ... ');
        var that = this;
        
        // REGISTER EVENTS
        this.mailListener.on("server:connected", function () {
            that.connectedToServer();
        });

        this.mailListener.on("mailbox", function (mailbox) {
            that.connectedToMailbox(mailbox);
        });

        this.mailListener.on("server:disconnected", function () {
            
        });

        this.mailListener.on("error", function (err) {
            
        });

        this.mailListener.on("headers", function (headers, seqno) {
            that.addHeaderToQueue(seqno, headers);
            console.log(that.logIdentifier + ' EVENT | Header ');
        });

        this.mailListener.on("body", function (body, seqno) {
            that.addBodyToQueue(seqno, body);
            console.log(that.logIdentifier + ' EVENT | Body ');
        })

        this.mailListener.on("attachment", function (attachment, path, seqno) {
            that.addAttachmentsToQueue(seqno, path, attachment);
            console.log(that.logIdentifier + ' EVENT | Attachment ');
        });

        this.mailListener.on("mail", function (mail, seqno) {
            that.addNewMailToQueue(seqno, mail);
            console.log(that.logIdentifier + ' EVENT | New Mail ');
        })
    }

    start() {
        console.log(this.logIdentifier + ' Connecting to Server ... ');
        this.mailListener.start(); // start listening

        // REGISTER EVENTS
        this.registerHandlers();
    }

    stop() {
        this.mailListener.stop();
    }






}


var emaillistener = new EMailListener();
emaillistener.username = 'alarmierung@alarmiator.de';
emaillistener.password = 'AL!2021?TB';
emaillistener.host = 'dinstuehler.eu';
emaillistener.allowedSenders = ['jdi@omninet.de', 'support@alarmiator.de', 'ils@aufgehts.de'];
emaillistener.allowedRecipients = ['alarmierung@alarmiator.de', 'alarm@alarmiator.de'];
emaillistener.allowedSubjectRegEx = '(?i)(?<= |^)ALARM(?= |$)';
emaillistener.allowedAttachments = ['pdf', 'jpg'];
emaillistener.initialize();
emaillistener.start();


